/*
   MNW
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnw.c,v 1.2 1999/03/08 07:17:47 furukawa Exp $
   */

#define MNW_USE_SHORTCUT
#include "mnw.h"
#include <string.h>
#include <stdlib.h>
#include <time.h>

/* -- global variables */
static mnwGraphics gDefaultGraphicsDummy;
mnwGraphics *gDefaultGraphics = &gDefaultGraphicsDummy;
static mnwWindow gRootWindowDummy;
mnwWindow *gRootWindow = &gRootWindowDummy;
static mnwWindow gFunctionGuideDummy;
mnwWindow *gFunctionGuide = &gFunctionGuideDummy;

/* -- local variables */
static mnwRect gScreenRectDummy;
static mnwRect *gScreenRect = &gScreenRectDummy;

/* -- local functions */
static bool
RootWin_Callback(mnwWindow *self, mnwEvent *event);
static bool
FunctionGuide_Callback(mnwWindow *self, mnwEvent *event);

/* -- window system */

/* ɥƥν */
bool
mnwSetup(void)
{
    mnwRect rect;

    /* եåν */
    mnwGraphics_Initialize(gDefaultGraphics);

    /* ٥Ȥν */
    mnwEvent_Setup();

    /*  */
    mnwRect_SetXYWH(gScreenRect,
		    REGION_SCREEN_LEFT, REGION_SCREEN_TOP,
		    REGION_SCREEN_WIDTH, REGION_SCREEN_HEIGHT);

    /* ̤ξõ */
    mnwSetForeColor(kColor_White);
    mnwFillRect(gScreenRect);
    mnwSetForeColor(kColor_Black);

    /* 롼ȥɥ */
    mnwWindow_Register(gRootWindow, RootWin_Callback, nil, gScreenRect);

    /* ե󥯥󥬥ɤ */
    mnwRect_SetXYWH(&rect, 0, 240-20, 640, 20);
    mnwWindow_Register(gFunctionGuide, FunctionGuide_Callback, nil, &rect);

    return true;
}

/* ɥƥդ */
bool
mnwTidy(void)
{
    /* ٥Ȥդ */
    mnwEvent_Tidy();

    /* ̤ξõ */
    mnwSetForeColor(kColor_White);
    mnwFillRect(gScreenRect);
    mnwSetForeColor(kColor_Black);

    /* եåդ */
    mnwGraphics_Finalize(gDefaultGraphics);
    return true;
}

/* -- root window */

#define MNW_ICON_SIZE (16+16/4*16)
#define MNW_TITLEBAR_WIDTH		640
#define MNW_TITLEBAR_SHORT_WIDTH	480

static void
mnw_draw_clock(bool force, bool short_title)
{
    static time_t prev;
    time_t t;

    t = time(&t);
    if (force || t / 60 != prev){
	struct tm *tm;
	mnwRect r;
	char buf[10];
	int x;

	tm = localtime(&t);

	if (short_title){
	    x = MNW_TITLEBAR_SHORT_WIDTH-48;
	}else{
	    x = MNW_TITLEBAR_WIDTH-64;
	}

	mnwRect_SetXYWH(&r, x, 2, 6*5+6, 16);
	mnwDrawFrame3D(&r);
	mnwRect_Inset(&r, 1, 1);
	mnwSetForeColor(kColor_LtGray);
	mnwFillRect(&r);

	sprintf(buf, "%02d:%02d", tm->tm_hour, tm->tm_min);
	mnwSetFont(kFont_12);
	mnwSetForeColor(kColor_Black);
	mnwDrawText(x+3, 4, buf, strlen(buf));
	prev = t / 60;
    }
}

static void
mnw_draw_icon(byte *pixmap)
{
    mnwDrawPixmap(2, 2, 0, 0, pixmap, 16, 16);
}

static bool
RootWin_Callback(mnwWindow *self, mnwEvent *event)
{
    static char title[132];
    static byte icon[MNW_ICON_SIZE];
    static void *icon_pixmap;
    static int style;
    static int bar_width;
    mnwEvent e;
    mnwRect rect;
    byte *data;
    int width, height, size, i;

    /* å */
    switch (event->type){
	case kEvent_Create:		      /* ɥ */
	mnwWindow_Callback(self, event);      /* super */
	/* եʬ */
	mnwEvent_SetFocusWindow(self);
	/* طʿ */
	mnwEvent_SetAll(&e, kEvent_SetBackColor, nil, kColor_LtGray, 0);
	mnwEvent_SendTo(&e, self);
	/* Υꥢ */
	icon_pixmap = mnwCreatePixmap(icon, 16, 16);
	memset(icon, 0, MNW_ICON_SIZE);
	/* Υǥե */
	style = kRootWinStyle_DrawIcon|kRootWinStyle_DrawClock;
	bar_width = 640;
	return true;
	break;

	case kEvent_Destroy:		      /* ɥդ */
	mnwDestroyPixmap(icon_pixmap);
	mnwWindow_Callback(self, event);      /* super */
	/* å롼פνλ */
	mnwEvent_SetAll(&e, kEvent_Quit, nil, 0, 0);
	mnwEvent_Post(&e);
	return true;
	break;

	case kEvent_Redraw:		      /*  */
	    if (self->visible){
		/* طʤ */
		mnwRect_SetXYWH(&rect, 0, 0, bar_width, 20);
		mnwDrawFrame3D(&rect);
		/* clear title */
		mnwSetForeColor(kColor_LtGray);
		mnwRect_SetXYWH(&rect, 1, 1, bar_width-2, 18);
		mnwFillRect(&rect);
		mnwSetForeColor(kColor_Black);
		mnwDrawLine(0, 20, bar_width-1, 20);
		/* draw title */
		mnwSetForeColor(kColor_Black);
		mnwSetFont(kFont_16);
		mnwSetAttr(kAttr_Bold);
		mnwDrawText(22, 2, title, strlen(title));
		mnwSetAttr(kAttr_None);
		/* draw clock */
		if (style & kRootWinStyle_DrawClock){
		    mnw_draw_clock(true, style&kRootWinStyle_ShortTitle);
		}
		/* draw icon */
		if (style & kRootWinStyle_DrawIcon){
		    mnw_draw_icon(icon_pixmap);
		}
		/* send Redraw to the children */
		for (i = 0; i < MNW_MAX_CHILD; i++){
		    if (self->child_valid[i]){
			mnwEvent_SetAll(&e, kEvent_Redraw, nil, 0, 0);
			mnwEvent_SendTo(&e, self->child[i]);
		    }
		}
	    }
	return true;
	break;

	case kEvent_Timer:
	    if (self->visible){
		/* draw clock */
		if (style & kRootWinStyle_DrawClock){
		    mnw_draw_clock(false, style&kRootWinStyle_ShortTitle);
		}
	    }
	break;

	case kEvent_SetText:		      /* ȥ */
	sprintf(title, event->param0);
	return true;
	break;

	case kEvent_SetPixmap:		      /*  */
	data = mnwParseMGR(event->param0, &width, &height, &size);
	memcpy(icon, data, size);
	return true;
	break;

	case kEvent_SetStyle:		      /*  */
	if (event->param2){
	    style |= event->param1;	      /* set */
	}else{
	    style = (style&~(event->param1)); /* clear */
	}
	if (style & kRootWinStyle_ShortTitle){
	    bar_width = MNW_TITLEBAR_SHORT_WIDTH;
	}else{
	    bar_width = MNW_TITLEBAR_WIDTH;
	}
	return true;
	break;

	default:
	return mnwWindow_Callback(self, event);	/* super */
	break;
    }

    /* NOTREACHED */
    return mnwWindow_Callback(self, event);   /* super */
}

/* -- function guide */

#define MNW_MAX_FUCTION_KEY	10
#define MNW_FUNCTION_KEY_WIDTH	(12*4 + 6)
static char mnw_function_name[MNW_MAX_FUCTION_KEY][9];
static bool mnw_function_valid[MNW_MAX_FUCTION_KEY];
static int mnw_function_command[MNW_MAX_FUCTION_KEY];

static void
mnw_draw_function_key(int no)
{
    mnwRect rect;
    int x, y;

    x = no * (MNW_FUNCTION_KEY_WIDTH-1);
    y = 240-18;

    mnwRect_SetXYWH(&rect, x, y, 
		    MNW_FUNCTION_KEY_WIDTH, 18);
    mnwSetForeColor(kColor_LtGray);
    mnwFillRect(&rect);
    mnwSetForeColor(kColor_Black);
    mnwDrawRect(&rect);
    mnwRect_SetXYWH(&rect, x + 1, y + 1,
		    MNW_FUNCTION_KEY_WIDTH-2, 18-2);
    mnwDrawFrame3D(&rect);
    if (mnw_function_valid[no]){
	mnwSetForeColor(kColor_Black);
	mnwSetFont(kFont_12);
	mnwDrawText(x + 3, y + 2, mnw_function_name[no], 8);
    }
}

static bool
FunctionGuide_Callback(mnwWindow *self, mnwEvent *event)
{
    mnwEvent e;
    int i;

    /* å */
    switch (event->type){
	case kEvent_Create:		      /* ɥ */
	mnwWindow_Callback(self, event);      /* super */
	/* ʸν */
	for (i = 0; i < MNW_MAX_FUCTION_KEY; i++){
	    mnw_function_name[i][0] = '\0';
	    mnw_function_valid[i] = false;
	    mnw_function_command[i] = kEvent_None;
	}
	/* طʿ */
	mnwEvent_SetAll(&e, kEvent_SetBackColor, nil, kColor_LtGray, 0);
	mnwEvent_SendTo(&e, self);
	return true;
	break;

	case kEvent_Redraw:		      /*  */
	for (i = 0; i < MNW_MAX_FUCTION_KEY; i++){
	    mnw_draw_function_key(i);
	}
	return true;
	break;

	case kEvent_SetText:		      /* ޥɤ */
	sprintf(mnw_function_name[event->param1], event->param0);
	mnw_function_valid[event->param1] = true;
	mnw_function_command[event->param1] = event->param2;
	mnw_draw_function_key(event->param1);
	return true;
	break;

	case kEvent_UnsetText:		      /* ޥɤβ */
	mnw_function_name[event->param1][0] = '\0';
	mnw_function_valid[event->param1] = false;
	mnw_function_command[event->param1] = 0;
	mnw_draw_function_key(event->param1);
	return true;
	break;

	case kEvent_Command:
	if (mnw_function_valid[event->param1]){
	    mnwEvent_SetAll(&e, kEvent_Command, nil,
			    mnw_function_command[event->param1], 0);
	    mnwEvent_Post(&e);
	}
	return true;
	break;

	default:
	return mnwWindow_Callback(self, event);      /* super */
	break;
    }

    /* NOTREACHED */
    return mnwWindow_Callback(self, event);   /* super */
}
