/*
   MNW dialog
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwdlg.c,v 1.1.1.1 1999/02/28 06:47:46 furukawa Exp $
   */

#define MNW_USE_SHORTCUT
#include "mnw.h"

/* -- Dialog */

static bool
mnwDialog_Initialize(mnwDialog *self)
{
    int i;

    self->prev_focus = mnwEvent_GetFocus();

    self->focus_child = -1;
    for (i = 0; i < MNW_MAX_CHILD; i++){
	self->child_tabstop[i] = false;
    }
    
    return true;
}

static bool
mnwDialog_Finalize(mnwDialog *self)
{
    mnwEvent e;
    mnwEvent_SetAll(&e, kEvent_Hide, nil, 0, 0);
    mnwEvent_SendToDirect(&e, (mnwWindow *)self);
    mnwEvent_SetFocusWindow(self->prev_focus);
    mnwEvent_SetAll(&e, kEvent_Redraw, nil, 0, 0);
    mnwEvent_SendTo(&e, self->parent);
    return true;
}

static int
mnwDialog_find_next_tabstop(mnwDialog *self)
{
    int i, next;
    mnwWindow *focused_window;

    /* first case */
    if (self->focus_child < 0){
	for (i = 0; i < MNW_MAX_CHILD; i++){
	    if (self->child_tabstop[i]){
		self->focus_child = i;
		return i;
	    }
	}
    }

    /* fix focus */
    focused_window = mnwEvent_GetFocus();
    if (focused_window == (mnwWindow *)self){
	mnwDPrintf("next_tab-> fix focus: focus is me");
	for (i = 0; i < MNW_MAX_CHILD; i++){
	    if (self->child_tabstop[i]){
		self->focus_child = i;
		/* do not focus this, focus the next */
		break;
	    }
	}
    }else{
	if (self->focus_child >= 0
	    && self->child[self->focus_child] != focused_window){
	    mnwDPrintf("next_tab-> fix focus: focus is out");
	    for (i = 0; i < MNW_MAX_CHILD; i++){
		if (self->child_tabstop[i]){
		    if (self->child[i] == focused_window){
			self->focus_child = i;
			/* do not focus this, focus the next */
			break;
		    }
		}
	    }
	}
	if (self->focus_child >= 0
	    && self->child[self->focus_child] != focused_window){
	    /* focus is out of my children */
	    mnwDPrintf("next_tab-> oh!");
	    /*return -1;*/
	}
    }
    
    /* find next */
    next = -1;
    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (self->child_tabstop[i]){
	    if (i < self->focus_child){
		if (next < 0){
		    next = i;
		}
	    }else if (i > self->focus_child){
		if (next < 0 || next < self->focus_child){
		    next = i;
		}
	    }
	}
    }
    return next;
}

static bool
mnwDialog_KeyDown(mnwDialog *self, mnwEvent *event)
{
    int next;

    if (event->param1 == kKey_TAB){
	next = mnwDialog_find_next_tabstop(self);
	if (next < 0){
	    return false;
	}
	/* move focus */
	mnwEvent_SetFocusWindow(self->child[next]);
	self->focus_child = next;
    }
    return false;
}

static bool
mnwDialog_SetTabStop(mnwDialog *self, mnwEvent *event)
{
    int i;
    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (self->child[i] == event->param0){
	    self->child_tabstop[i] = true;
	    return true;
	}
    }
    /* NOTE: AddChild, RemoveChildбƤʤ */
    return false;
}

static bool
mnwDialog_DoModal(mnwDialog *self)
{
    mnwEvent e, *event;
    int i;

    /* ɥβĻ벽 */
    mnwEvent_SetAll(&e, kEvent_Show, nil, 0, 0);
    mnwEvent_SendTo(&e, (mnwWindow *)self);
    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (self->child_valid[i]){
	    mnwEvent_SetFocusWindow((mnwWindow *)(self->child[i]));
	    break;
	}
    }

    /* ٥ȥ롼 */
    while ((event = mnwEvent_GetNext(true)) != nil){
	if (!mnwEvent_Translate(event)){
	    if (event->type == kEvent_EndModal){
		break;
	    }else{
#ifdef DEBUG
		mnwPrintEvent(event);
#endif
		mnwEvent_Dispatch(event);
	    }
	}
    }

    return event->param1;
}

bool
mnwDialog_Callback(mnwDialog *self, mnwEvent *event)
{
    switch (event->type){
	case kEvent_Create:
	mnwWindow_Callback((mnwWindow *)self, event); /* super */
	return mnwDialog_Initialize(self);
	break;
	case kEvent_Destroy:
	mnwWindow_Callback((mnwWindow *)self, event); /* super */
	return mnwDialog_Finalize(self);
	break;
	case kEvent_KeyDown:
	return mnwDialog_KeyDown(self, event);
	break;
	case kEvent_SetTabStop:
	return mnwDialog_SetTabStop(self, event);
	break;
	case kEvent_DoModal:
	return mnwDialog_DoModal(self);
	break;
	default:
	/* superdelegate */
	return mnwWindow_Callback((mnwWindow *)self, event);
	break;
    }

    /* ٥Ȥʤ */
    return false;
}
