/*
   MNW edit compo
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwedit.c,v 1.2 1999/08/10 16:37:48 furukawa Exp $
   */

#define MNW_USE_SHORTCUT
#include "mnw.h"
#include <string.h>
#include <ctype.h>

#ifdef linux
#define memcpy(dst, src, len) memmove(dst, src, len)
#endif

static bool
mnwEdit_SetFont(mnwEdit *self, int font);
static bool
mnwEdit_ResetRect(mnwEdit *self, mnwRect *rect);

/* -- Edit */

static bool
mnwEdit_Initialize(mnwEdit *self)
{
    self->type = kWindowType_Compo;
    self->focused = false;
    self->buffer = "";
    self->buffer_len = 0;
    self->text_len = 0;
    self->region_col = 0;
    self->region_row = 0;
    self->origin_col = 0;
    self->origin_row = 0;
    self->cursor_pos = 0;
    self->cursor_len = 0;
    self->multi_line = false;
    mnwEdit_SetFont(self, kFont_16);

    return true;
}

static bool
mnwEdit_Finalize(mnwEdit *self)
{
    return true;
}

static bool
mnwEdit_ResetRect(mnwEdit *self, mnwRect *rect)
{
    self->region_col = (rect->width - 4) / self->font_width;
    self->region_row = (rect->height - 4) / self->font_height;
    if (self->region_col == 0){
	self->region_row = 0;
    }
    return true;
}

static bool
mnwEdit_SetFont(mnwEdit *self, int font)
{
    self->font = font;
    self->font_width = mnwGetFontWidth(self->font);
    self->font_height = mnwGetFontHeight(self->font);
    return mnwEdit_ResetRect(self, &(self->rect));
}

static bool
mnwEdit_IsKanjiFirstByte(mnwEdit *self, int pos)
{
    char *p;
    int cnt;
    bool second_byte_expected;

    p = self->buffer;
    cnt = pos;
    second_byte_expected = false;
    while (cnt-- > 0){
	if (!second_byte_expected){
	    if (mnwIsKanjiFirstByte(*p)){
		second_byte_expected = true;
	    }
	}else{
	    second_byte_expected = false;
	}
	p++;
    }

    if (!second_byte_expected){
	if (mnwIsKanjiFirstByte(*p)){
	    return true;
	}else{
	    return false;
	}
    }else{
	return false;
    }
    /* Ugh!: Is this OK? */
}

static bool
mnwEdit_IsKanjiSecondByte(mnwEdit *self, int pos)
{
    char *p;
    int cnt;
    bool second_byte_expected;

    p = self->buffer;
    cnt = pos;
    second_byte_expected = false;
    while (cnt-- > 0){
	if (!second_byte_expected){
	    if (mnwIsKanjiFirstByte(*p)){
		second_byte_expected = true;
	    }
	}else{
	    second_byte_expected = false;
	}
	p++;
    }

    if (!second_byte_expected){
	return false;
    }else{
	return true;
    }
    /* Ugh!: Is this OK? */
}

static bool
mnwEdit_GetRowPos(mnwEdit *self, int no, int *start, int *end)
{
    int n, pos, col;

    if (*start < 0){
	n = 0;
	pos = 0;
    }else{
	n = no;
	pos = *start;
    }

    for (; n <= no; n++){
	if (pos >= self->text_len){
	    *end = 0;
	    return false;
	}
	col = 0;
	*start = pos;
	while (col < self->region_col && pos < self->text_len){
	    if (self->buffer[pos] == '\n'){
		pos++;
		col++;
		break;
	    }
	    pos++;
	    col++;
	}
	/* ιԤλϤޤ꤬Хܤʤû */
	/* 㳰Ǹɲä */
	if (mnwEdit_IsKanjiSecondByte(self, pos)
	    && pos < self->text_len){
	    pos--;
	    col--;
	}
	*end = pos;
    }
    return true;
}

static int
mnwEdit_CalcRow(mnwEdit *self, int pos)
{
    int row, row_start, row_end;
    row = row_start = row_end = 0;
    while (mnwEdit_GetRowPos(self, row, &row_start, &row_end)){
	if (row_start <= pos && pos <= row_end){
	    return row;
	}
	row_start = row_end;
	row++;
    }
    return 0;
}

static int
mnwEdit_GetCursorRow(mnwEdit *self)
{
    return mnwEdit_CalcRow(self, self->cursor_pos);
}

static int
mnwEdit_GetRow(mnwEdit *self)
{
    int row, row_start, row_end;
    row = row_start = row_end = 0;
    while (mnwEdit_GetRowPos(self, row, &row_start, &row_end)){
	row_start = row_end;
	row++;
    }
    return row;
}

static void
mnwEdit_InvertCursorRow(mnwEdit *self, int row, int row_origin, int pos, int len)
{
    mnwRect r;
    int draw_pos, draw_col, draw_len;

    if (pos - row_origin >= self->region_col){
	return;
    }

    if (len < 1){
	len = 1;			      /* zero length cursor */
    }

    draw_pos = max(row_origin, pos);
    draw_col = draw_pos - row_origin;
    draw_len = min(pos + len - draw_pos,
		   self->region_col - draw_col);
    mnwRect_SetXYWH(&r, 
		    self->rect.x+2 + self->font_width*draw_col,
		    self->rect.y+2 + self->font_height*row,
		    self->font_width*draw_len, self->font_height);
    mnwInvertRect(&r);
}

static void
mnwEdit_InvertCursor(mnwEdit *self)
{
    int pos, len;
    pos = self->cursor_pos;
    len = self->cursor_len;
    
    if (self->multi_line){
	int row, row_start, row_end;
	row_start = row_end = -1;
	for (row = self->origin_row;
	     row < self->origin_row + self->region_row; row++){
	    mnwEdit_GetRowPos(self, row, &row_start, &row_end);
	    if (row_start < 0){
		row_start = 0;
	    }
	    if (pos + len < row_start){
		continue;
	    }
	    if (pos + len == row_end){
		if (mnwEdit_CalcRow(self, pos) == mnwEdit_GetRow(self) - 1){
		    mnwEdit_InvertCursorRow(self,
					    row - self->origin_row,
					    row_start,
					    max(row_start, pos),
					    min(row_end - row_start,
						pos + len - max(row_start, pos)));
		    return;
		}
	    }
	    if (pos < row_end){
		mnwEdit_InvertCursorRow(self,
					row - self->origin_row,
					row_start,
					max(row_start, pos),
					min(row_end - row_start,
					    pos + len - max(row_start, pos)));
	    }
	    if (!mnwEdit_GetRowPos(self, row, &row_start, &row_end)){
		return;
	    }
	    row_start = row_end;
	}
    }else{
	mnwEdit_InvertCursorRow(self, 0, self->origin_col, pos, len);
    }
}

static void
mnwEdit_DrawWhiteRow(mnwEdit *self, int row, int row_origin, int pos, int len)
{
    mnwRect r;
    int draw_pos, draw_col, draw_len;
    
    draw_pos = max(row_origin, pos);
    draw_col = draw_pos - row_origin;
    draw_len = min(self->cursor_pos + len - draw_pos,
		   self->region_col - draw_col);
    mnwRect_SetXYWH(&r, 
		    self->rect.x+2 + self->font_width*draw_col,
		    self->rect.y+2 + self->font_height*row,
		    self->font_width*draw_len, self->font_height);
    mnwSetForeColor(kColor_White);
    mnwFillRect(&r);
}

static void
mnwEdit_DrawWhite(mnwEdit *self, int pos, int len)
{
    if (self->multi_line){
	int row, row_start, row_end;
	row_start = row_end = -1;
	for (row = self->origin_row;
	     row < self->origin_row + self->region_row; row++){
	    if (!mnwEdit_GetRowPos(self, row, &row_start, &row_end)){
		break;
	    }
	    if (pos < row_end){
		if (pos + len < row_start){
		    break;
		}
		mnwEdit_DrawWhiteRow(self,
				     row - self->origin_row, row_start,
				     max(row_start, pos),
				     row_end - max(row_start, pos));
	    }
	    row_start = row_end;
	}
    }else{
	mnwEdit_DrawWhiteRow(self, 0, self->origin_col, pos, len);
    }
}

static void
mnwEdit_DrawTextRow(mnwEdit *self, int row, int row_origin, int pos, int len)
{
    int draw_pos, draw_col, draw_len;
    bool half_kanji_start, half_kanji_end;
    
    draw_pos = max(row_origin, pos);
    draw_col = draw_pos - row_origin;
    draw_len = min(pos + len - draw_pos,
		   self->region_col - draw_col);
    half_kanji_start = mnwEdit_IsKanjiSecondByte(self, draw_pos);
    half_kanji_end = mnwEdit_IsKanjiFirstByte(self, draw_pos+draw_len-1);
    if (!half_kanji_start && !half_kanji_end){
	/* 줤 */
	mnwDrawText(self->rect.x+2 + self->font_width*draw_col,
		    self->rect.y+2 + self->font_height*row,
		    self->buffer + draw_pos,
		    draw_len);
    }else{
	/* Ⱦ礱ʸ */
	int clipx1, clipy1, clipx2, clipy2;
	clipx1 = self->rect.x+2 + self->font_width*draw_col;
	clipy1 = self->rect.y+2;
	clipx2 = clipx1 + self->font_width*draw_len;
	clipy2 = clipy1 + self->font_height;
	if (half_kanji_start){
	    draw_pos--;
	    draw_col--;
	    draw_len++;
	}
	if (half_kanji_end){
	    draw_len++;
	}
	mnwDrawTextWithClipping(self->rect.x+2 + self->font_width*draw_col,
				self->rect.y+2 + self->font_height*row,
				self->buffer + draw_pos,
				draw_len,
				clipx1, clipy1, clipx2, clipy2);
    }
}

static void
mnwEdit_DrawText(mnwEdit *self, int pos, int len)
{
    mnwSetForeColor(self->fore_color);
    mnwSetFont(self->font);

    if (self->multi_line){
	int row, row_start, row_end;
	row_start = row_end = -1;
	for (row = self->origin_row;
	     row < self->origin_row + self->region_row; row++){
	    if (!mnwEdit_GetRowPos(self, row, &row_start, &row_end)){
		break;
	    }
	    if (pos < row_end){
		if (pos + len < row_start){
		    break;
		}
		mnwEdit_DrawTextRow(self,
				    row - self->origin_row, row_start,
				    max(row_start, pos),
				    row_end - max(row_start, pos));
	    }
	    row_start = row_end;
	}
    }else{
	mnwEdit_DrawTextRow(self, 0, self->origin_col, pos, len);
    }
}

static bool
mnwEdit_RedrawTextAll(mnwEdit *self)
{
    mnwRect r;

    if (self->visible){
	/* طʤ */
	mnwRect_Copy(&r, &(self->rect));
	mnwRect_SetXYWH(&r, r.x+2, r.y+2, r.width-4, r.height-4);
	mnwSetForeColor(kColor_White);
	mnwFillRect(&r);

	/* ƥȤ */
	mnwEdit_DrawText(self, 0, self->text_len);
	if (self->focused){
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
    }
    return true;
}

static bool
mnwEdit_Redraw(mnwEdit *self)
{
    mnwRect r;

    /*mnwDPrintf("mnwEdit_Redraw: win(%d)", self->no);*/

    if (self->visible){
	/* طʤ */
	mnwRect_Copy(&r, &(self->rect));
	mnwSetForeColor(kColor_Black);
	mnwDrawRect(&r);
	mnwRect_Inset(&r, 1, 1);
	mnwSetForeColor(kColor_White);
	mnwFillRect(&r);
	if (self->focused){
	    mnwSetForeColor(kColor_Black);       
	    mnwDrawRect(&r);
	}

	/* ƥȤ */
	mnwEdit_RedrawTextAll(self);
    }
    return true;
}

/* ֤ʸõ */
static void
mnwEdit_DeleteBackward(mnwEdit *self, int len)
{
    if (self->cursor_pos <= 0){
	return;
    }
    mnwEdit_InvertCursor(self);		      /* OFF */
    /* Ĺѹ */
    if (self->cursor_len > 0){
	self->cursor_len = 0;
    }
    /* Ⱦʬˤʤʤ褦 */
    if (mnwEdit_IsKanjiSecondByte(self, self->cursor_pos - len)){
	len ++;
    }
    /* θˤˤ餹 */
    memcpy(self->buffer + self->cursor_pos - len,
	   self->buffer + self->cursor_pos,
	   self->text_len - self->cursor_pos);
    self->text_len -= len;
    self->buffer[self->text_len] = '\0';
    self->cursor_pos -= len;

    if (self->multi_line){
	int row;
	row = mnwEdit_GetCursorRow(self);
	if (row < self->origin_row){
	    /* scroll down */
	    self->origin_row--;
	}
	mnwEdit_RedrawTextAll(self);	      /* Redraw & ON */
    }else{
	if (self->cursor_pos - self->origin_col < 0){
	    self->origin_col = self->cursor_pos;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    /* θä */
	    mnwEdit_DrawWhite(self, self->cursor_pos,
			      self->text_len - self->cursor_pos + len);
	    /* θ */
	    mnwEdit_DrawText(self, self->cursor_pos,
			     self->text_len - self->cursor_pos);
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
    }
    self->buffer[self->text_len] = '\0';
}

/* ֤ʸõ */
static void
mnwEdit_DeleteForeward(mnwEdit *self, int len)
{
    mnwEdit_InvertCursor(self);		      /* OFF */
    self->cursor_pos += len;
    /* Ⱦʬˤʤʤ褦 */
    if (mnwEdit_IsKanjiSecondByte(self, self->cursor_pos)){
	self->cursor_pos++;
	len++;
    }
    mnwEdit_InvertCursor(self);		      /* ON */
    mnwEdit_DeleteBackward(self, len);
    self->buffer[self->text_len] = '\0';
}

/* ֤ʸ */
static void
mnwEdit_InsertText(mnwEdit *self, char *text, int len)
{
    if (self->text_len + len - self->cursor_len > self->buffer_len){
	return;
    }

    /* ϰϤõ */
    if (self->cursor_len > 0){
	mnwEdit_DeleteForeward(self, self->cursor_len);
	self->cursor_len = 0;
    }
    mnwEdit_InvertCursor(self);		      /* OFF */

    /* θˤ餹 */
    memcpy(self->buffer + self->cursor_pos + len,
	   self->buffer + self->cursor_pos, self->text_len - self->cursor_pos);

    /* ֤ */
    memcpy(self->buffer + self->cursor_pos, text, len);
    self->text_len += len;
    self->cursor_pos += len;

    if (self->multi_line){
	int row;
	row = mnwEdit_GetCursorRow(self);
	if (row >= self->origin_row + self->region_row){
	    /* scroll up */
	    self->origin_row++;
	}
	/* Ⱦʬʤ */
	if (!mnwEdit_IsKanjiFirstByte(self, self->cursor_pos - len)){
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}
    }else{
	if (self->cursor_pos > self->origin_col + self->region_col){
	    self->origin_col = self->cursor_pos - self->region_col;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    /* Ⱦʬʤ */
	    if (!mnwEdit_IsKanjiFirstByte(self, self->cursor_pos - len)){
		if (!mnwEdit_IsKanjiSecondByte(self, self->cursor_pos - len)){
		/* Υθä */
		    mnwEdit_DrawWhite(self, self->cursor_pos - len,
				      self->text_len - self->cursor_pos);
		    /* Υθ */
		    mnwEdit_DrawText(self, self->cursor_pos - len,
				     self->text_len - self->cursor_pos + len);
		}else{
		    /* Υθä */
		    mnwEdit_DrawWhite(self, self->cursor_pos - len - 1,
				      self->text_len - self->cursor_pos + 1);
		    /* Υθ */
		    mnwEdit_DrawText(self, self->cursor_pos - len - 1,
				     self->text_len - self->cursor_pos + len + 1);
		}
	    }
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
    }
    self->buffer[self->text_len] = '\0';
}

/* ưʱ */
static bool
mnwEdit_MoveRight(mnwEdit *self)
{
    mnwEdit_InvertCursor(self);		      /* OFF */
    if (self->cursor_len > 0){
	self->cursor_pos += self->cursor_len;
    }else{
	if (self->cursor_pos < self->text_len){
	    if (mnwEdit_IsKanjiFirstByte(self, self->cursor_pos)){
		self->cursor_pos += 2;
	    }else{
		self->cursor_pos++;
	    }
	}
    }
    if (self->cursor_len > 0){
	self->cursor_len = 0;
    }

    if (self->multi_line){
	int row;
	row = mnwEdit_GetCursorRow(self);
	if (row >= self->origin_row + self->region_row){
	    /* scroll up */
	    self->origin_row++;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
    }else{
	if (self->cursor_pos > self->origin_col + self->region_col){
	    self->origin_col = self->cursor_pos - self->region_col;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
    }
    return true;
}

/* ưʺ */
static bool
mnwEdit_MoveLeft(mnwEdit *self)
{
    mnwEdit_InvertCursor(self);		      /* OFF */
    if (self->cursor_pos > 0){
	self->cursor_pos--;
	if (mnwEdit_IsKanjiSecondByte(self, self->cursor_pos)){
	    self->cursor_pos--;
	}
    }
    if (self->cursor_len > 0){
	self->cursor_len = 0;
    }

    if (self->multi_line){
	int row;
	row = mnwEdit_GetCursorRow(self);
	if (row < self->origin_row){
	    /* scroll down */
	    self->origin_row--;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
    }else{
	if (self->cursor_pos - self->origin_col < 0){
	    self->origin_col = self->cursor_pos;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
    }
    return true;
}

/* ưʾ */
static bool
mnwEdit_MoveUp(mnwEdit *self)
{
    if (self->multi_line){
	int row, row_start, row_end, col;
	mnwEdit_InvertCursor(self);	      /* OFF */
	if (self->cursor_len > 0){
	    self->cursor_len = 0;
	}
	row = mnwEdit_GetCursorRow(self);
	row_start = row_end = -1;
	mnwEdit_GetRowPos(self, row, &row_start, &row_end);
	if (self->cursor_pos == row_end){
	    col = 0;
	    row_start = row_end = -1;
	    mnwEdit_GetRowPos(self, row, &row_start, &row_end);
	    self->cursor_pos = row_start + min(col, row_end - row_start);
	    if (mnwEdit_IsKanjiSecondByte(self, self->cursor_pos)){
		self->cursor_pos--;
	    }
	}else if (row > 0){
	    col = self->cursor_pos - row_start;
	    row--;
	    row_start = row_end = -1;
	    mnwEdit_GetRowPos(self, row, &row_start, &row_end);
	    self->cursor_pos = row_start + min(col, row_end - row_start);
	    if (mnwEdit_IsKanjiSecondByte(self, self->cursor_pos)){
		self->cursor_pos--;
	    }
	}
	if (self->origin_row > row){
	    self->origin_row = row;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
	return true;
    }else{
	return false;
    }
}

/* ưʲ */
static bool
mnwEdit_MoveDown(mnwEdit *self)
{
    if (self->multi_line){
	int row, row_start, row_end, col;
	mnwEdit_InvertCursor(self);	      /* OFF */
	if (self->cursor_len > 0){
	    self->cursor_pos += self->cursor_len;
	    self->cursor_len = 0;
	}
	row = mnwEdit_GetCursorRow(self);
	row_start = row_end = -1;
	if (mnwEdit_GetRowPos(self, row + 1, &row_start, &row_end)){
	    row_start = row_end = -1;
	    mnwEdit_GetRowPos(self, row, &row_start, &row_end);
	    col = self->cursor_pos - row_start;
	    row++;
	    row_start = row_end = -1;
	    mnwEdit_GetRowPos(self, row, &row_start, &row_end);
	    self->cursor_pos = row_start + min(col, row_end - row_start - 1);
	    if (mnwEdit_IsKanjiSecondByte(self, self->cursor_pos)){
		self->cursor_pos--;
	    }
	}
	if (self->origin_row + self->region_row <= row){
	    self->origin_row = row - self->region_row + 1;
	    mnwEdit_RedrawTextAll(self);      /* Redraw & ON */
	}else{
	    mnwEdit_InvertCursor(self);	      /* ON */
	}
	return true;
    }else{
	return false;
    }
}

/* ̤ʥν */
static bool
mnwEdit_KeyDownSpecial(mnwEdit *self, mnwEvent *event)
{
    mnwEvent e;

    switch (event->param1){
    case kKey_BACKSPACE:
	mnwEdit_DeleteBackward(self, 1);
	mnwEvent_SetAll(&e, kEvent_ChangedNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return true;
	break;
    case kKey_RIGHT:
	return mnwEdit_MoveRight(self);
	break;
    case kKey_LEFT:
	return mnwEdit_MoveLeft(self);
	break;
    case kKey_UP:
	return mnwEdit_MoveUp(self);
	break;
    case kKey_DOWN:
	return mnwEdit_MoveDown(self);
	break;
    }

    if (event->param2 & kKey_CONTROL){
	switch (event->param1){
	case 'f':
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_RIGHT, 0);
	    return mnwEvent_SendToDirect(&e, (mnwWindow *)self);
	    break;
	case 'b':
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_LEFT, 0);
	    return mnwEvent_SendToDirect(&e, (mnwWindow *)self);
	    break;
	case 'p':
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_UP, 0);
	    return mnwEvent_SendToDirect(&e, (mnwWindow *)self);
	    break;
	case 'n':
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_DOWN, 0);
	    return mnwEvent_SendToDirect(&e, (mnwWindow *)self);
	    break;
	case 'j':
	    mnwEdit_InsertText(self, "\n", 1);
	    mnwEvent_SetAll(&e, kEvent_ChangedNotify, self, self->no, self->id);
	    mnwEvent_SendTo(&e, self->parent);
	    return true;
	    break;
	}
    }

    return false;
}

/* ν */
static bool
mnwEdit_KeyDown(mnwEdit *self, mnwEvent *event)
{
    mnwEvent e;

    if (mnwEdit_KeyDownSpecial(self, event)){
	return true;
    }

    switch (event->param1){
    case kKey_TAB:
    case kKey_RETURN:
    case kKey_ESCAPE:
    case kKey_F1:
    case kKey_F2:
    case kKey_F3:
    case kKey_F4:
    case kKey_F5:
    case kKey_F6:
    case kKey_F7:
    case kKey_F8:
    case kKey_F9:
    case kKey_F10:
    case kKey_F11:
    case kKey_F12:
	return false;
	break;
    default:
	if (!(event->param2 & (kKey_CONTROL|kKey_ALT))){
	    /* ʸ */
	    char c = event->param1;
	    if (isprint(c)
		|| mnwIsKanjiFirstByte(c) || mnwIsKanjiSecondByte(c)){
		if (event->param2 & kKey_SHIFT){
		    if (islower(event->param1)){
			c = toupper(event->param1);
		    }
		}
		mnwEdit_InsertText(self, &c, 1);
		mnwEvent_SetAll(&e, kEvent_ChangedNotify, self, self->no, self->id);
		mnwEvent_SendTo(&e, self->parent);
		return true;
	    }
	}
    }
    return false;
}

static bool
mnwEdit_SetText(mnwEdit *self, mnwEvent *event)
{
    mnwEvent e;
    self->text_len = min(event->param1, self->buffer_len);
    memcpy(self->buffer, event->param0, self->text_len);
    self->buffer[self->text_len] = '\0';
    self->cursor_pos = min(self->region_col, self->text_len);
    mnwEvent_SetAll(&e, kEvent_ChangedNotify, self, self->no, self->id);
    mnwEvent_SendTo(&e, self->parent);
    return true;
}

static bool
mnwEdit_SetCursor(mnwEdit *self, mnwEvent *event)
{
    self->cursor_pos = event->param1;
    self->cursor_len = event->param2;
    if (self->cursor_len < 0){
	self->cursor_pos = 0;
	self->cursor_len = self->text_len;
    }
    if (self->cursor_pos - self->origin_col < 0){
	self->origin_col = self->cursor_pos;
    }
    if (self->cursor_pos > self->origin_col + self->region_col){
	self->origin_col = self->cursor_pos - self->region_col;
    }
    mnwEdit_RedrawTextAll(self);
    return true;
}

bool
mnwEdit_Callback(mnwEdit *self, mnwEvent *event)
{
    mnwEvent e;

    switch (event->type){
    case kEvent_Create:
	mnwWindow_Callback((mnwWindow *)self, event); /* super */
	return mnwEdit_Initialize(self);
	break;
    case kEvent_Destroy:
	return mnwEdit_Finalize(self);
	break;
    case kEvent_AddChild:
    case kEvent_RemoveChild:
	return false;
	break;
    case kEvent_GainFocus:
	self->focused = true;
	mnwEvent_SetAll(&e, kEvent_GainFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
#if 1
	mnwEvent_SetAll(&e, kEvent_SetCursor, nil, 0, -1);
	mnwEvent_SendToDirect(&e, (mnwWindow *)self);
#endif
	return mnwEdit_Redraw(self);
	break;
    case kEvent_LostFocus:
	self->focused = false;
	mnwEvent_SetAll(&e, kEvent_LostFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return mnwEdit_Redraw(self);
	break;
    case kEvent_SetBuffer:
	self->buffer = event->param0;
	self->buffer_len = event->param1;
	return true;
	break;
    case kEvent_SetText:
	return mnwEdit_SetText(self, event);
	break;
    case kEvent_SetCursor:
	return mnwEdit_SetCursor(self, event);
	break;
    case kEvent_Redraw:
	return mnwEdit_Redraw(self);
	break;
    case kEvent_SetFont:
	return mnwEdit_SetFont(self, event->param1);
	break;
    case kEvent_SetMultiLine:
	self->multi_line = true;
	return true;
	break;
    case kEvent_KeyDown:
	return mnwEdit_KeyDown(self, event);
	break;
    default:
	/* superdelegate */
	return mnwWindow_Callback((mnwWindow *)self, event);
	break;
    }

    /* ٥Ȥʤ */
    return false;
}
