/*
   MNW event
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwevent.c,v 1.3 1999/03/09 00:20:34 furukawa Exp $
   */

#include "mnw.h"
#ifdef MNW_USE_GGI
#include "ggi/ggi.h"
#endif

/* -- Event */

static mnwEvent gEvent_queue[MNW_MAX_EVENTQUEUE];
static int gEvent_head;			      /* ȯ륤٥ */
static int gEvent_tail;			      /* Υ٥ȤǼ */
static int gEvent_remain;		      /* 塼Υ٥Ȥο */
static mnwEvent gEvent_NoEvent;		      /* ٥ */

static mnwWindow *gFocused_Window;

static mnwTimer mnw_interval_timer[MNW_MAX_TIMER];
static long
mnw_diff_timeval(struct timeval *a, struct timeval *b);
static void
mnw_next_timeval(struct timeval *a, struct timeval *now, long interval_msec);
static long
mnwEvent_InvokeTimer(void);

void
mnwEvent_Setup(void)
{
#ifdef MNW_USE_GGI
    gii_event ev;
    ev.any.size   = sizeof(gii_cmd_event);
    ev.any.type   = evCommand;
    ev.any.origin = GII_EV_ORIGIN_NONE;
    ev.any.target = GII_EV_TARGET_ALL;
    ev.cmd.code   = GII_CMDCODE_GETDEVINFO;

    giiEventSend(ggiJoinInputs(gDefaultVisual, NULL), &ev);
#endif

    gEvent_head = 0;
    gEvent_tail = 0;
    gEvent_remain = 0;
    gFocused_Window = nil;
    mnwEvent_SetAll(&gEvent_NoEvent, kEvent_None, nil, 0, 0);
}

void
mnwEvent_Tidy(void)
{
#ifdef MNW_USE_GGI
#endif
}

void
mnwEvent_Initialize(mnwEvent *self)
{
    self->type = kEvent_None;
    self->target = nil;
    self->param0 = nil;
    self->param1 = 0;
    self->param2 = 0;
}

void
mnwEvent_Finalize(mnwEvent *self)
{
}

void
mnwEvent_SetAll(mnwEvent *self, int type, void *param0, int param1, int param2)
{
    self->type = type;
    self->target = nil;
    self->param0 = param0;
    self->param1 = param1;
    self->param2 = param2;
}

static void
mnwEvent_CopyShallow(mnwEvent *dst, mnwEvent *src)
{
    dst->target = src->target;
    dst->type = src->type;
    dst->param0 = src->param0;
    dst->param1 = src->param1;
    dst->param2 = src->param2;
}

bool
mnwEvent_SendToDirect(mnwEvent *self, mnwWindow *dst)
{
    return (dst->callback)(dst, self);
}

bool
mnwEvent_SendTo(mnwEvent *self, mnwWindow *dst)
{
    if (gEvent_remain < MNW_MAX_EVENTQUEUE){
	mnwEvent_CopyShallow(&(gEvent_queue[gEvent_tail]), self);
	gEvent_queue[gEvent_tail].target = dst;
	gEvent_remain++;
	gEvent_tail = ++gEvent_tail % MNW_MAX_EVENTQUEUE;
	return true;
    }else{
	mnwDPrintf("MNW: warning!! message queue overflow.");
	return false;
    }
}

bool
mnwEvent_Post(mnwEvent *self)
{
    if (gEvent_remain < MNW_MAX_EVENTQUEUE){
	mnwEvent_CopyShallow(&(gEvent_queue[gEvent_tail]), self);
	gEvent_queue[gEvent_tail].target = nil;
	gEvent_remain++;
	gEvent_tail = ++gEvent_tail % MNW_MAX_EVENTQUEUE;
	return true;
    }else{
	mnwDPrintf("MNW: warning!! message queue overflow.");
	return false;
    }
}

void
mnwEvent_SetFocusWindow(mnwWindow *win)
{
    mnwEvent event;
    mnwEvent_Initialize(&event);
    if (gFocused_Window != nil){
	event.type = kEvent_LostFocus;
	mnwEvent_SendToDirect(&event, gFocused_Window);
    }
    gFocused_Window = win;
    if (gFocused_Window != nil){
	event.type = kEvent_GainFocus;
	mnwEvent_SendToDirect(&event, gFocused_Window);
    }
}

void
mnwEvent_SetFocusWindowDelay(mnwWindow *win)
{
    mnwEvent event;
    mnwEvent_Initialize(&event);
    if (gFocused_Window != nil){
	event.type = kEvent_LostFocus;
	mnwEvent_SendTo(&event, gFocused_Window);
    }
    gFocused_Window = win;
    if (gFocused_Window != nil){
	event.type = kEvent_GainFocus;
	mnwEvent_SendTo(&event, gFocused_Window);
    }
}

mnwWindow *
mnwEvent_GetFocus(void)
{
    return gFocused_Window;
}

static mnwEvent *
mnwEvent_GetHead(void)
{
    mnwEvent *event;
    event = &(gEvent_queue[gEvent_head]);
    gEvent_remain--;
    gEvent_head = ++gEvent_head % MNW_MAX_EVENTQUEUE;
    return event;
}

#ifdef MNW_USE_GGI
mnwEvent *
mnwEvent_GetNext(bool block)
{
    gii_event_mask mask;
    gii_event ev;

    while (1){
	long timeout;
	struct timeval tval;

	/* ޥ٥Ȥȯ */
	if (gEvent_remain > 0){
	    timeout = 0;
	}else{
	    timeout = mnwEvent_InvokeTimer();
	}

	/* ٥Ȥȯ */
	if (gEvent_remain > 0){
	    mask = 0;
	}else{
	    if (block){
		tval.tv_sec = 0;
		tval.tv_usec = timeout;
		mask = ggiEventPoll(gDefaultVisual, emAll, &tval);
	    }else{
		tval.tv_sec = 0;
		tval.tv_usec = 0;
		mask = ggiEventPoll(gDefaultVisual, emAll, &tval);
	    }
	}
	while (mask){
	    ggiEventRead(gDefaultVisual, &ev, emAll);
	    if (ev.any.type == evKeyPress){
		mnwEvent event;
		event.type = kEvent_KeyDown;
		event.param0 = nil;
		event.param1 = ev.key.sym;
		event.param2 = ev.key.modifiers;
		mnwEvent_Post(&event);
	    }
	    tval.tv_sec = 0;
	    tval.tv_usec = 0;
	    mask = ggiEventPoll(gDefaultVisual, emAll, &tval);
	}

	/* ٥Ȥ */
	if (gEvent_remain > 0){
	    mnwEvent *event;

	    /* 塼åФ */
	    event = mnwEvent_GetHead();
	    if (event->type == kEvent_Quit){
		return nil;
	    }else{
		/* 桼Ϥ */
		return event;
	    }
	}else if (!block){
	    /* ٥Ȥ桼Ϥ */
	    return &gEvent_NoEvent;
	}
    }
}
#else
mnwEvent *
mnwEvent_GetNext(bool block)
{
    while (1){
	int key;
	long timeout;

	/* ޥ٥Ȥȯ */
	if (gEvent_remain > 0){
	    timeout = 0;
	}else{
	    timeout = mnwEvent_InvokeTimer();
	}

	/* ٥Ȥȯ */
	if (gEvent_remain > 0){
	    key = -1;
	}else{
	    if (block){
		key = get_key_im(timeout/100);
	    }else{
		key = get_key_im(0);
	    }
	}
	if (key >= 0){
	    mnwEvent event;
	    event.type = kEvent_KeyDown;
	    event.param0 = nil;
	    event.param1 = key;
	    event.param2 = 0;
	    mnwEvent_Post(&event);
	}

	/* ٥Ȥ */
	if (gEvent_remain > 0){
	    mnwEvent *event;

	    /* 塼åФ */
	    event = mnwEvent_GetHead();
	    if (event->type == kEvent_Quit){
		return nil;
	    }else{
		/* 桼Ϥ */
		return event;
	    }
	}else if (!block){
	    /* ٥Ȥ桼Ϥ */
	    return &gEvent_NoEvent;
	}
    }
}
#endif

bool
mnwEvent_Dispatch(mnwEvent *event)
{
    mnwWindow *target;

    while (event->type != kEvent_None){
	bool done;
	if (event->target != nil){
	    /* ꥤ٥ */
	    /* ꤵ줿ɥ */
	    done = mnwEvent_SendToDirect(event, event->target);
	    if (event->type != kEvent_Timer){
		mnwGraphics_Flush(gDefaultGraphics);
	    }
	    event->type = kEvent_None;
	    return done;
	}else{
	    /* ̵ꥤ٥ */
	    switch (event->type){
		case kEvent_Redraw:
		event->type = kEvent_None;
		mnwGraphics_Flush(gDefaultGraphics);
		return false;
		break;
		
		case kEvent_KeyDown:
		case kEvent_KeyUp:
		case kEvent_Command:
		if (gFocused_Window != nil){
		    /* եɥ */
		    done = false;
		    target = gFocused_Window;
		    while (!done){
			done = mnwEvent_SendToDirect(event, target);
			if (!done){
			    /* ƤпƤ */
			    if (target->parent == nil){
				done = true;
			    }else{
				target = target->parent;
			    }
			}
		    }
		    event->type = kEvent_None;
		    mnwGraphics_Flush(gDefaultGraphics);
		    return done;
		}else{
		    event->type = kEvent_None;
		    mnwGraphics_Flush(gDefaultGraphics);
		    return false;
		}
		break;
		
		default:
		event->type = kEvent_None;
		mnwGraphics_Flush(gDefaultGraphics);
		return false;
		break;
	    }
	}
    }

    /* NOTREACHED */
    mnwGraphics_Flush(gDefaultGraphics);
    return false;
}

/* -- key event */

bool
mnwEvent_Translate(mnwEvent *event)
{
    mnwEvent e;

    if (event->type == kEvent_KeyDown
	|| event->type == kEvent_KeyDown){
	int key;
	key = event->param1;
	if (0x01 <= key && key <= 0x1a){
	    /* TODO: ??? */
	    if (key != kKey_RETURN
		&& key != kKey_BACKSPACE
		&& key != kKey_TAB){
		event->param1 = key + 0x60;
		event->param2 = kKey_CONTROL;
	    }
	}else if (0x41 <= key && key <= 0x5a){
	    event->param1 = key + 0x20;
	    event->param2 = kKey_SHIFT;
	}else if (0x61 <= key && key <= 0x7a){
	    event->param1 = key;
	    event->param2 = kKey_NONE;
	}else if (kKey_F1 <= key && key <= kKey_F10){
	    mnwEvent_SetAll(&e, kEvent_Command, nil, key - kKey_F1, 0);
	    mnwEvent_SendToDirect(&e, gFunctionGuide);
	    return true;
	}
    }
    return false;
}

bool
mnwEvent_TestModifier(mnwEvent *event, int modifier)
{
    return (event->param2 & modifier);
}

/* -- timer event */

static long
mnw_diff_timeval(struct timeval *a, struct timeval *b)
{
    if (a->tv_sec > b->tv_sec){
	return (((a->tv_sec - b->tv_sec - 1) * 1000)
		+ (a->tv_usec / 1000)
		+ (1000 - (b->tv_usec / 1000)));
    }else if (a->tv_sec == b->tv_sec){
	return (a->tv_usec / 1000) - (b->tv_usec / 1000);
    }else{
	return -(((b->tv_sec - a->tv_sec - 1) * 1000)
		 + (b->tv_usec / 1000)
		 + (1000 - (a->tv_usec / 1000)));
    }
}

static void
mnw_next_timeval(struct timeval *a, struct timeval *now, long interval_msec)
{
    a->tv_sec = now->tv_sec + (interval_msec / 1000);
    a->tv_usec = now->tv_usec + (interval_msec % 1000) * 1000;
    if (a->tv_usec >= 1000*1000){
	a->tv_sec += 1;
	a->tv_usec -= 1000*1000;
    }
}

bool
mnwEvent_SetTimer(int no, mnwWindow *target, long interval_msec)
{
    if (no < 0 && MNW_MAX_TIMER <= no){
	return false;
    }
    mnw_interval_timer[no].valid = true;
    mnw_interval_timer[no].interval = interval_msec;
    mnw_interval_timer[no].target = target;
    gettimeofday(&(mnw_interval_timer[no].next), (struct timezone *)nil);
    return true;
}

bool
mnwEvent_ResetTimer(int no)
{
    if (no < 0 && MNW_MAX_TIMER <= no){
	return false;
    }
    mnw_interval_timer[no].valid = false;
    return true;
}

static long
mnwEvent_InvokeTimer(void)
{
    struct timeval now;
    long next, diff;
    mnwEvent e;
    int i;

    gettimeofday(&now, (struct timezone *)nil);
    next = MNW_MAX_TIMER_WAIT;

    for (i = 0; i < MNW_MAX_TIMER; i++){
	if (mnw_interval_timer[i].valid){
	    diff = mnw_diff_timeval(&(mnw_interval_timer[i].next), &now);
	    /*fprintf(stderr, "checking time ... diff = %ld\n", diff);*/
	    if (diff < 0){
		/* timeout --> invoke event */
		mnwEvent_SetAll(&e, kEvent_Timer, nil, i, 0);
		mnwEvent_SendTo(&e, mnw_interval_timer[i].target);
		/* set next deadline */
		mnw_next_timeval(&(mnw_interval_timer[i].next), &now,
				 mnw_interval_timer[i].interval);
		/* get next interval */
		diff = mnw_diff_timeval(&(mnw_interval_timer[i].next), &now);
	    }
	    next = min(next, diff);
	}
    }

    return next;
}
