/*
   MNW label compo
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwlabel.c,v 1.1.1.1 1999/02/28 06:47:48 furukawa Exp $
   */

#define MNW_USE_SHORTCUT
#include "mnw.h"
#include <string.h>

static bool
mnwLabel_SetFont(mnwLabel *self, int font);
static bool
mnwLabel_ResetRect(mnwLabel *self, mnwRect *rect);

/* -- Label */

static bool
mnwLabel_Initialize(mnwLabel *self)
{
    self->type = kWindowType_Compo;
    self->focused = false;
    self->buffer = "";
    self->buffer_len = 0;
    self->text_len = 0;
    self->align = kAlign_Left;
    self->style = kStyle_None;
    mnwLabel_SetFont(self, kFont_16);

    return true;
}

static bool
mnwLabel_Finalize(mnwLabel *self)
{
    return true;
}

static bool
mnwLabel_ResetRect(mnwLabel *self, mnwRect *rect)
{
    if (self->style == kStyle_None){
	if (rect->height >= self->font_height){
	    self->max_col = (rect->width - 0) / self->font_width;
	}else{
	    self->max_col = 0;
	}
    }else{
	if (rect->height >= self->font_height + 4){
	    self->max_col = (rect->width - 4) / self->font_width;
	}else{
	    self->max_col = 0;
	}
    }
    return true;
}

static bool
mnwLabel_SetFont(mnwLabel *self, int font)
{
    self->font = font;
    self->font_width = mnwGetFontWidth(self->font);
    self->font_height = mnwGetFontHeight(self->font);
    return mnwLabel_ResetRect(self, &(self->rect));
}

static bool
mnwLabel_SetStyle(mnwLabel *self, int style)
{
    self->style = style;
    return mnwLabel_ResetRect(self, &(self->rect));
}

static bool
mnwLabel_Redraw(mnwLabel *self, mnwRect *rect)
{
    int offset_x, offset_y;

    if (self->visible){
	mnwSetForeColor(self->back_color);
	mnwFillRect(&(self->rect));
	if (self->style == kStyle_Border){
	    mnwSetForeColor(self->fore_color);
	    mnwDrawRect(&(self->rect));
	    offset_x = 2;
	    offset_y = 2;
	    if (self->rect.width - 4 - self->text_len*self->font_width > 0){
		if (self->align == kAlign_Right){
		    offset_x += self->rect.width - 4 - self->text_len*self->font_width;
		}else if (self->align == kAlign_Center){
		    offset_x += (self->rect.width - 4 - self->text_len*self->font_width) / 2;
		}
	    }
	}else if (self->style == kStyle_Popup){
	    mnwDrawFrame3D(&(self->rect));
	    offset_x = 2;
	    offset_y = 2;
	    if (self->rect.width - 4 - self->text_len*self->font_width > 0){
		if (self->align == kAlign_Right){
		    offset_x += self->rect.width - 4 - self->text_len*self->font_width;
		}else if (self->align == kAlign_Center){
		    offset_x += (self->rect.width - 4 - self->text_len*self->font_width) / 2;
		}
	    }
	}else{
	    offset_x = 0;
	    offset_y = 0;
	    if (self->rect.width - self->text_len*self->font_width > 0){
		if (self->align == kAlign_Right){
		    offset_x += self->rect.width - self->text_len*self->font_width;
		}else if (self->align == kAlign_Center){
		    offset_x += (self->rect.width - self->text_len*self->font_width) / 2;
		}
	    }
	}
	mnwSetForeColor(self->fore_color);
	mnwSetFont(self->font);
	mnwDrawText(self->rect.x + offset_x, self->rect.y + offset_y,
		    self->buffer, min(self->text_len, self->max_col));
    }
    return true;
}

bool
mnwLabel_Callback(mnwLabel *self, mnwEvent *event)
{
    mnwEvent e;

    switch (event->type){
	case kEvent_Create:
	mnwWindow_Callback((mnwWindow *)self, event); /* super */
	return mnwLabel_Initialize(self);
	break;
	case kEvent_Destroy:
	return mnwLabel_Finalize(self);
	break;
	case kEvent_AddChild:
	case kEvent_RemoveChild:
	return false;
	break;
	case kEvent_GainFocus:
	self->focused = true;
	mnwEvent_SetAll(&e, kEvent_GainFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return mnwLabel_Redraw(self, nil);
	break;
	case kEvent_LostFocus:
	self->focused = false;
	mnwEvent_SetAll(&e, kEvent_LostFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return mnwLabel_Redraw(self, nil);
	break;
	case kEvent_SetBuffer:
	self->buffer = event->param0;
	self->buffer_len = event->param1;
	return true;
	break;
	case kEvent_SetText:
	self->text_len = min(event->param1, self->buffer_len);
	strncpy(self->buffer, event->param0, self->text_len);
	return true;
	break;
	case kEvent_Redraw:
	return mnwLabel_Redraw(self, nil);
	break;
	case kEvent_SetAlign:
	self->align = event->param1;
	return true;
	break;
	case kEvent_SetStyle:
	return mnwLabel_SetStyle(self, event->param1);
	break;
	case kEvent_SetFont:
	return mnwLabel_SetFont(self, event->param1);
	break;
	default:
	/* superdelegate */
	return mnwWindow_Callback((mnwWindow *)self, event);
	break;
    }

    /* ٥Ȥʤ */
    return false;
}
