/*
   MNW list compo
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwlist.c,v 1.1.1.1 1999/02/28 06:47:48 furukawa Exp $
   */

#define MNW_USE_SHORTCUT
#include "mnw.h"
#include <string.h>

static bool
mnwList_SetFont(mnwList *self, int font);
static bool
mnwList_ResetRect(mnwList *self, mnwRect *rect);

/* -- List */

static bool
mnwList_Initialize(mnwList *self)
{
    self->type = kWindowType_Compo;
    self->focused = false;
    self->buffer = "";
    self->items = 0;
    self->max_item = 0;
    self->max_size = 0;
    self->max_col = 0;
    self->max_row = 0;
    self->origin_row = 0;
    self->cursor_row = 0;
    mnwList_SetFont(self, kFont_16);

    return true;
}

static bool
mnwList_Finalize(mnwList *self)
{
    return true;
}

static bool
mnwList_ResetRect(mnwList *self, mnwRect *rect)
{
    self->max_col = (rect->width - 4) / self->font_width;
    self->max_row = (rect->height - 4) / self->font_height;
    if (self->max_col == 0){
	self->max_row = 0;
    }
    return true;
}

static bool
mnwList_SetFont(mnwList *self, int font)
{
    self->font = font;
    self->font_width = mnwGetFontWidth(self->font);
    self->font_height = mnwGetFontHeight(self->font);
    return mnwList_ResetRect(self, &(self->rect));
}

static bool
mnwList_InvertCursor(mnwList *self)
{
    mnwRect r;
    if (self->items > 0){
	mnwRect_Copy(&r, &(self->rect));
	mnwRect_SetXYWH(&r, 
			r.x + 2,
			r.y + 2 + self->cursor_row * self->font_height,
			r.width-4, self->font_height);
	mnwInvertRect(&r);
    }
    return true;
}

static bool
mnwList_Redraw(mnwList *self, mnwRect *rect)
{
    mnwRect r;
    int i, n, x, y;
    char *p;

    if (self->visible){
	mnwRect_Copy(&r, &(self->rect));
	mnwSetForeColor(kColor_Black);
	mnwDrawRect(&r);
	mnwRect_Inset(&r, 1, 1);
	mnwSetForeColor(kColor_LtGray);
	mnwFillRect(&r);
	if (self->focused){
	    mnwSetForeColor(kColor_Black);       
	    mnwDrawRect(&r);
	}

	mnwSetForeColor(self->fore_color);
	mnwSetFont(self->font);
	for (i = 0; i < self->max_row; i++){
	    n = self->origin_row + i;
	    if (n < self->items){
		x = self->rect.x + 2;
		y = self->rect.y + 2 + i * self->font_height;
		p = self->buffer + self->max_size * n;
		mnwDrawText(x, y, p, min(strlen(p), self->max_col));
	    }else{
		break;
	    }
	}
	mnwList_InvertCursor(self);	      /* ON */
    }
    return true;
}

static bool
mnwList_SetText(mnwList *self, mnwEvent *event)
{
    int len, n;
    char *p;

    n = event->param1;
    if (n < self->max_item){
	len = min(event->param2, self->max_size);
	p = self->buffer + self->max_size * n;
	memcpy(p, event->param0, len);
	p[len] = '\0';
	if (self->items < n + 1){
	    self->items = n + 1;
	}
    }
	
    return true;
}

static bool
mnwList_UnsetText(mnwList *self, mnwEvent *event)
{
    int n;
    char *p;

    n = event->param1;
    if (n < self->max_item){
	p = self->buffer + self->max_size * n;
	p[0] = '\0';
    }
	
    return true;
}

static bool
mnwList_CursorUp(mnwList *self)
{
    if (self->cursor_row > 0){
	mnwList_InvertCursor(self);	      /* OFF */
	self->cursor_row--;
	mnwList_InvertCursor(self);	      /* ON */
	return true;
    }else{
	if (self->origin_row <= 0){
	    /* do nothing */
	    return true;
	}else{
	    self->origin_row--;
	    mnwList_Redraw(self, nil);
	    return true;
	}
    }
}

static bool
mnwList_CursorDown(mnwList *self)
{
    if (self->cursor_row < self->max_row - 1){
	mnwList_InvertCursor(self);	      /* OFF */
	self->cursor_row++;
	mnwList_InvertCursor(self);	      /* ON */
	return true;
    }else{
	if (self->origin_row + self->cursor_row + 1 >= self->items){
	    /* do nothing */
	    return true;
	}else{
	    self->origin_row++;
	    mnwList_Redraw(self, nil);
	    return true;
	}
    }
}

static bool
mnwList_KeyDown(mnwList *self, mnwEvent *event)
{
    switch (event->param1){
	case kKey_TAB:
	case kKey_RETURN:
	case kKey_ESCAPE:
	return false;
	break;
	case kKey_UP:
	return mnwList_CursorUp(self);
	break;
	case kKey_DOWN:
	return mnwList_CursorDown(self);
	break;
    }

    if (event->param2 & kKey_CONTROL){
	switch (event->param1){
	    case 'p':
	    return mnwList_CursorUp(self);
	    break;
	    case 'n':
	    return mnwList_CursorDown(self);
	    break;
	}
    }

    return false;
}

static bool
mnwList_GetResult(mnwList *self, mnwEvent *event)
{
    *(int *)event->param0 = self->origin_row + self->cursor_row;
    return true;
}

static bool
mnwList_SetCursor(mnwList *self, mnwEvent *event)
{
    self->origin_row = event->param1;
    while (self->origin_row + self->cursor_row >= self->items){
	self->origin_row--;
    }
    return true;
}

bool
mnwList_Callback(mnwList *self, mnwEvent *event)
{
    mnwEvent e;

    switch (event->type){
	case kEvent_Create:
	mnwWindow_Callback((mnwWindow *)self, event); /* super */
	return mnwList_Initialize(self);
	break;
	case kEvent_Destroy:
	return mnwList_Finalize(self);
	break;
	case kEvent_AddChild:
	case kEvent_RemoveChild:
	return false;
	break;
	case kEvent_GainFocus:
	self->focused = true;
	mnwEvent_SetAll(&e, kEvent_GainFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return mnwList_Redraw(self, nil);
	break;
	case kEvent_LostFocus:
	self->focused = false;
	mnwEvent_SetAll(&e, kEvent_LostFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return mnwList_Redraw(self, nil);
	break;
	case kEvent_SetBuffer:
	self->buffer = event->param0;
	self->max_size = event->param1;
	self->max_item = event->param2;
	memset(self->buffer, 0, self->max_size*self->max_item);
	return true;
	break;
	case kEvent_SetText:
	return mnwList_SetText(self, event);
	break;
	case kEvent_UnsetText:
	return mnwList_UnsetText(self, event);
	break;
	case kEvent_Redraw:
	return mnwList_Redraw(self, nil);
	break;
	case kEvent_SetFont:
	return mnwList_SetFont(self, event->param1);
	break;
	case kEvent_KeyDown:
	return mnwList_KeyDown(self, event);
	break;
	case kEvent_GetResult:
	return mnwList_GetResult(self, event);
	break;
	case kEvent_SetCursor:
	return mnwList_SetCursor(self, event);
	break;
	default:
	/* superdelegate */
	return mnwWindow_Callback((mnwWindow *)self, event);
	break;
    }

    /* ٥Ȥʤ */
    return false;
}
