/*
   XPM to MNW raw graymap format
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: xpm2mgr.c,v 1.2 1999/02/28 07:56:52 furukawa Exp $
   */

#include <stdio.h>
#include <stdlib.h>

/* I'm not sure that this could parse all the XPM files. */

/* MNW raw graymap format is:
     16 bytes header + data

   Header format is:
     sprintf(header, "#MGR%04d%04d%04d", version, width, height);

   Data format is:
     2 bits per pixel
     4 pixel per byte
     right pixel first
   */

/* globals */
#define MAX_BUFSIZE	1024
#define MAX_COLOR_CHAR	256
int num_color_char = 0;
char color_char[MAX_COLOR_CHAR];
unsigned char color_value[MAX_COLOR_CHAR];

FILE *in_fp, *out_fp;
char in_fname[256], out_fname[256];

enum{
    KIND_SIZE,
    KIND_COLOR,
    KIND_DATA,
    KIND_EOF,
};

static int
get_line(char *buf, int size)
{
    static int first = 1;
    char line[MAX_BUFSIZE];

    while (1){
	char *p = fgets(line, MAX_BUFSIZE, in_fp);
	if (p == NULL){
	    return KIND_EOF;
	}
	if (p[0] == '"'){
	    /* trim */
	    p[strlen(p)-1] = '\0';  /* chop */
	    p[strlen(p)-1] = '\0';  /* chop */
	    p[strlen(p)-1] = '\0';  /* chop */
	    memcpy(buf, p + 1, strlen(p) - 1);

	    if (first){
		first = 0;
		return KIND_SIZE;
	    }
	    if (buf[2] == 'c' && buf[3] == ' '){
		return KIND_COLOR;
	    }else{
		return KIND_DATA;
	    }
	}
    }
}

static int
parse_size(char *buf)
{
    char str[10];
    char header[20];
    int i, j, width, height;

    i = 0;
    while (buf[i] != ' '){
	str[i] = buf[i];
	i++;
    }
    str[i] = '\0';
    width = atoi(str);

    i++;
    j = 0;
    while (buf[i] != ' '){
	str[j] = buf[i];
	j++;
	i++;
    }
    str[j] = '\0';
    height = atoi(str);

    if (width % 4 != 0){
	fprintf(stderr, "input image should be 4 pixel aligned\n");
	fclose(in_fp);
	fclose(out_fp);
	exit(1);
    }

    sprintf(header, "#MGR%04d%04d%04d", 1, width, height);
    fwrite(header, 1, 16, out_fp);

    return 0;
}

static int
hexascii2value(char c)
{
    if ('a' <= c && c <= 'f'){
	return c - 'a' + 10;
    }else if ('A' <= c && c <= 'F'){
	return c - 'A' + 10;
    }else{
	return c - '0';
    }
}

static int
parse_color(char *buf)
{
    int n = num_color_char;
    int value;
    char *p = buf;

    color_char[n] = p[0];

    p++;
    while (*p != '#' && *p != '\0'){
	p++;
    }
    if (strlen(p) == 0){
	return -1;
    }
    p++;
    fprintf(stderr, "[%s] ", p);

    value = hexascii2value(p[0])*16
	+ hexascii2value(p[1])
	+ hexascii2value(p[2])*16
	+ hexascii2value(p[3])
	+ hexascii2value(p[4])*16
	+ hexascii2value(p[5]);
    value = value / 3;
    if (value < 1 * 256 / 4){
	color_value[n] = 3;
    }else if (value < 2 * 256 / 4){
	color_value[n] = 2;
    }else if (value < 3 * 256 / 4){
	color_value[n] = 1;
    }else{
	color_value[n] = 0;
    }
    fprintf(stderr, "map color: %d -> %d\n", value, color_value[n]);
    num_color_char++;
    return 0;
}

static unsigned char
get_color(char c)
{
    int i;
    for (i = 0; i < num_color_char; i++){
	if (c == color_char[i]){
	    return color_value[i];
	}
    }

    fprintf(stderr, "unhandled color value [%c] found.\n", c);
    fclose(in_fp);
    fclose(out_fp);
    exit(1);
    return -1;
}

static int
parse_data(char *buf)
{
    char *p = buf;
    unsigned char b;
    int remain;

    remain = strlen(p);
    while (remain >= 4){
	b = (get_color(p[0])&0x3)<<0
	    | (get_color(p[1])&0x3)<<2
	    | (get_color(p[2])&0x3)<<4
	    | (get_color(p[3])&0x3)<<6;
	fwrite(&b, 1, 1, out_fp);
	p += 4;
	remain -= 4;
    }
    return 0;
}

int
main(int argc, char *argv[])
{
    int i, kind;
    char buf[MAX_BUFSIZE];

    if (argc < 2){
	fprintf(stderr, "xpm2mgr xpmfile.xpm");
	exit(-1);
    }

    /* make file name */
    sprintf(in_fname, argv[1]);
    sprintf(out_fname, argv[1]);
    i = 0;
    while (out_fname[i] != '.'){
	i++;
    }
    sprintf(&out_fname[i], ".mgr");
    fprintf(stderr, "in:%s -> out:%s\n", in_fname, out_fname);

    /* open files */
    in_fp = fopen(in_fname, "r");
    out_fp = fopen(out_fname, "w");
    if (in_fp == NULL || out_fp == NULL){
	fprintf(stderr, "cannot open files");
	exit(-1);
    }

    while ((kind = get_line(buf, MAX_BUFSIZE)) != KIND_EOF){
	/*fprintf(stderr, "[%s]\n", buf);*/
	switch (kind){
	case KIND_SIZE:
	    parse_size(buf);
	    break;
	case KIND_COLOR:
	    parse_color(buf);
	    break;
	case KIND_DATA:
	    parse_data(buf);
	    break;
	}
    }

    /* close files */
    fclose(in_fp);
    fclose(out_fp);
    return 0;
}
