/*
   month cache
   Copyright (c) 1997 FURUKAWA Yo.
   $Id: month.c,v 1.1.1.1 1998/12/15 03:42:44 furukawa Exp $
   */

#include "arch.h"
#include "month.h"
#include "index.h"
#include "calendar.h"
#include <stdlib.h>

/* Ȥ˥å夹

   å
   ӥ塼ȤѤΤ
   פȤѤΤ
   ֤°ˤѤΤϡ
     ѹˤɤΤߤ̵Τǡܤ
   */

/* -- macros */
/* ʬΥ쥳ɸ򥭥å夷Ƥ */
#define MonthCache_NUM	3

#define MONTHREC_MAXLABEL	8

/* -- file local variables */
static MonthRec monthCache[MonthCache_NUM];
static bool labelShow[MONTHREC_MAXLABEL];
static bool isSaturdayHoliday;

/* -- file local methods */
static void
MonthRec_Sort(MonthRec *monthRec);
static bool
MonthRec_LE(Record *a, Record *b);

/* -- methods */

/* 쥳ɤν */
void
MonthRec_Init(void)
{
    int i;

    for (i = 0; i < MonthCache_NUM; i++){
	monthCache[i].isValid = false;
    }

    for (i = 0; i < MONTHREC_MAXLABEL; i++){
	labelShow[i] = true;
    }

    isSaturdayHoliday = false;
}

/* 쥳ɤդ */
void
MonthRec_Tidy(void)
{
}

/* 쥳ɤꡧˤϵ٤ */
void
MonthRec_ConfigIsSaturdayHoliday(bool flag)
{
    isSaturdayHoliday = flag;
    MonthRec_Invalidate();
}

/* 쥳ɤꡧ٥ */
void
MonthRec_ConfigSetLabel(uint16 label, bool flag)
{
    if (label < MONTHREC_MAXLABEL){
	labelShow[label] = flag;
    }
    MonthRec_Invalidate();
}

/* 쥳ɤꡧ٥ʵ쥤󥿥ե */
void
MonthRec_SelectLabel(uint16 label)
{
    int i;

    if (label < MONTHREC_MAXLABEL){
	if (label == 0){
	    for (i = 0; i < MONTHREC_MAXLABEL; i++){
		labelShow[i] = true;
	    }
	}else{
	    for (i = 0; i < MONTHREC_MAXLABEL; i++){
		labelShow[i] = false;
	    }
	    labelShow[0] = true;
	    labelShow[label] = true;
	}
    }
    MonthRec_Invalidate();
}

/* 쥳ɤ̵ */
void
MonthRec_Invalidate(void)
{
    int i;
    for (i = 0; i < MonthCache_NUM; i++){
	monthCache[i].isValid = false;
    }
}

/* 쥳ɤμ */
MonthRec *
MonthRec_GetByMonthNo(uint16 monthNo)
{
    int used, hit;
    bool match;
    uint16 year, month, day, date;
    uint16 numOfDays;
    uint16 start_date, end_date;
    uint16 cookie;
    uint8 dayOfWeek, dayOfWeekBit;
    uint8 nthWeekBit;
    MonthRec *monthRec;
    Record *rec;

    monthRec = &monthCache[monthNo % MonthCache_NUM];

    /* Ф֤ */
    if (monthRec->monthNo == monthNo && monthRec->isValid){
	return monthRec;
    }

    /* ʤк */

    /* ɬפ */
    year = 1900 + (monthNo/12);
    month = monthNo % 12 + 1;
    day = 1;
    numOfDays = Calendar_GetNumOfDays(year, month);
    start_date = Calendar_GetDayNo(year, month, day);
    end_date = start_date + numOfDays - 1;

    /* ƥեɤ򥻥å */
    monthRec->isValid = true;
    monthRec->monthNo = monthNo;
    monthRec->numOfDays = numOfDays;
    monthRec->start_date = start_date;
    monthRec->end_date = end_date;

    /* 򸡺 */
    used = 0;
    day = 1;
    dayOfWeek = Calendar_GetDayOfWeekByNo(start_date);
    dayOfWeekBit = gDayOfWeekBit[dayOfWeek];
    nthWeekBit = gNthWeekBit[1];
    for (date = start_date; date <= end_date; date++){
	hit = 0;
	monthRec->rec[day] = &monthRec->pointers[used];
	if (dayOfWeek == 0
	    || (isSaturdayHoliday && dayOfWeek == 6)){
	    monthRec->isHoliday[day] = true;
	}else{
	    monthRec->isHoliday[day] = false;
	}
	/* Ȥ˸ */
	cookie = Index_SetDate(date);
	while (nil != (rec = Index_FindNextByDate(&cookie))){
	    match = false;
	    if (rec->type == RECORD_TYPE_DELETED){
		/* õƤ쥳ɤ̵ */
		continue;
		/* NOTE: ޥåƤ褤 */
	    }
	    if (rec->label < MONTHREC_MAXLABEL
		&& !labelShow[rec->label]){
		/* ɽʤ٥ */
		continue;
	    }
	    if (rec->repeat_type == REPEAT_TYPE_NONE){
		/* NOTE: ®Τ㳰 */
		match = true;
	    }else{
		/* ֤äƤ뤫Ƚꤹ */
		switch (rec->repeat_type){
		    case REPEAT_TYPE_DAILY:
		    if ((date - rec->start_date) % rec->repeat_every == 0){
			match = true;
		    }
		    break;

		    case REPEAT_TYPE_WEEKLY:
		    if (((date/7 - rec->start_date/7) % rec->repeat_every == 0)
			&& (dayOfWeekBit & rec->repeat_arg1)){
			match = true;
		    }
		    break;

		    case REPEAT_TYPE_MONTHLYDAY:
		    if (((monthNo - rec->start_monthNo) % rec->repeat_every == 0)
			&& (day == rec->start_day)){
			match = true;
		    }
		    break;

		    case REPEAT_TYPE_MONTHLYWEEK:
		    if (((monthNo - rec->start_monthNo) % rec->repeat_every == 0)
			&& (nthWeekBit & rec->repeat_arg1)
			&& (dayOfWeekBit & rec->repeat_arg2)){
			match = true;
		    }
		    break;

		    case REPEAT_TYPE_YEARLYDAY:
		    if (((monthNo/12 - rec->start_monthNo/12) % rec->repeat_every == 0)
			&& (month == rec->start_month)
			&& (day == rec->start_day)){
			match = true;
		    }
		    break;

		    case REPEAT_TYPE_YEARLYWEEK:
		    if (((monthNo/12 - rec->start_monthNo/12) % rec->repeat_every == 0)
			&& (month == rec->start_month)
			&& (nthWeekBit & rec->repeat_arg1)
			&& (dayOfWeekBit & rec->repeat_arg2)){
			match = true;
		    }
		    break;

		    default:
		    panic("MonthRec_GetByMonthNo(): Invalid repeat attribute");
		    break;
		}
	    }
	    if (match){
		if (rec->type == RECORD_TYPE_TASK){
		    if (rec->done_len > 0 && rec->done[0] != date)
			continue;
		    /* NOTE: ˤ٤ */
		    /* NOTE: ʣDONEˤ̤б */
		}
		monthRec->rec[day][hit] = rec;
		hit++;
		if (rec->type == RECORD_TYPE_HOLIDAY){
		    monthRec->isHoliday[day] = true;
		}
	    }
	}
	monthRec->num[day] = hit;
	used += hit;
	day++;
	dayOfWeek = (dayOfWeek + 1) % 7;
	dayOfWeekBit = gDayOfWeekBit[dayOfWeek];
	nthWeekBit = gNthWeekBit[day / 7 + ((day%7)?1:0)];
	if (numOfDays - day < 7){
	    nthWeekBit |= gNthWeekBit[5]; /* Last Week */
	}
    }

#if 1
    MonthRec_Sort(monthRec);
#endif

    return monthRec;
}

/* 쥳ɤΥ */
static void
MonthRec_Sort(MonthRec *monthRec)
{
    uint16 day;
    int16 num, i, j;
    Record *rec;

    /* Ȥ˥Ȥ */
    for (day = 1; day <= monthRec->numOfDays; day++){
	num = monthRec->num[day];
	if (num > 1){			      /* Ȥɬפ */
	    /* ʼȤʤ */
	    for (i = 1; i < num; i++){
		rec = monthRec->rec[day][i];
		for (j = i - 1;; j--){
		    if (j < 0 || MonthRec_LE(monthRec->rec[day][j], rec)){
			monthRec->rec[day][j + 1] = rec;
			break;
		    }else{
			monthRec->rec[day][j + 1] = monthRec->rec[day][j];
		    }
		}
	    }
	}
    }
}

/* 쥳ɤνդ */
static bool
MonthRec_LE(Record *a, Record *b)
{
    /* typeǽդ */
    if (a->type < b->type){
	return true;
    }
    if (a->type > b->type){
	return false;
    }

    /* typeƱ */
    switch (a->type){
	case RECORD_TYPE_SCHEDULE:
	/* ϻ */
	if (a->u.sche.start_time_h != b->u.sche.start_time_h)
	    return (bool)(a->u.sche.start_time_h <= b->u.sche.start_time_h);
	if (a->u.sche.start_time_m != b->u.sche.start_time_m)
	    return (bool)(a->u.sche.start_time_m <= b->u.sche.start_time_m);
	/* λ */
	if (a->u.sche.end_time_h != b->u.sche.end_time_h)
	    return (bool)(a->u.sche.end_time_h <= b->u.sche.end_time_h);
	if (a->u.sche.end_time_m != b->u.sche.end_time_m)
	    return (bool)(a->u.sche.end_time_m <= b->u.sche.end_time_m);
	/* NOTE: ʲϴʰץХʡΤ */
	/* դ */
	if (a->start_date != b->start_date)
	    return a->start_date <= b->start_date;
	/* λդεս */
	if (a->end_date != b->end_date)
	    return !(a->end_date <= b->end_date);
	/* ʤΤǥɥ쥹 */
	return (bool)(a <= b);
	break;

	case RECORD_TYPE_TASK:
	/* λ̤λʷ֤̤б */
	if (a->done_len == 0 && b->done_len > 0)
	    return true;
	if (a->done_len > 0 && b->done_len == 0)
	    return false;
	/* 󥯽 */
	if (a->u.task.rank != b->u.task.rank)
	    return (bool)(a->u.task.rank < b->u.task.rank);
	/* ʤΤǥɥ쥹 */
	return (bool)(a <= b);
	break;

	case RECORD_TYPE_CONTACT:
	/* ϻ */
	if (a->u.cont.start_time_h != b->u.cont.start_time_h)
	    return (bool)(a->u.cont.start_time_h <= b->u.cont.start_time_h);
	if (a->u.cont.start_time_m != b->u.cont.start_time_m)
	    return (bool)(a->u.cont.start_time_m <= b->u.cont.start_time_m);
	/* ʤΤǥɥ쥹 */
	return (bool)(a <= b);
	break;

	case RECORD_TYPE_HOLIDAY:
	/* NOTE: ʲϴʰץХʡΤ */
	/* դ */
	if (a->start_date != b->start_date)
	    return a->start_date <= b->start_date;
	/* λդεս */
	if (a->end_date != b->end_date)
	    return !(a->end_date <= b->end_date);
	/* ʤΤǥɥ쥹 */
	return (bool)(a <= b);
	break;

	case RECORD_TYPE_DEADLINE:
	default:
	/* ɤǤ褤Τǥɥ쥹 */
	return (bool)(a <= b);
    }
}
