/*
   token
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: token.c,v 1.1.1.1 1998/12/15 03:42:45 furukawa Exp $
   */

#include "arch.h"
#include "token.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/* -- macros */
#define CHAR_EOL	'\0'

/* -- global variables */

/* -- file local types */

typedef struct _known_token {
    char *str;
    int len;
    uint8 id;
    int16 attr;
} known_token;

typedef struct _known_digit {
    char *str;
    int len;
    int16 val;
} known_digit;

/* -- file local variables */

static known_token knownToken[] = {

{ "January", 7, TOK_MONTH, 1 },
{ "February", 8, TOK_MONTH, 2 },
{ "March", 5, TOK_MONTH, 3 },
{ "April", 5, TOK_MONTH, 4 },
{ "May", 3, TOK_MONTH, 5 },
{ "June", 4, TOK_MONTH, 6 },
{ "July", 4, TOK_MONTH, 7 },
{ "August", 6, TOK_MONTH, 8 },
{ "September", 9, TOK_MONTH, 9 },
{ "October", 7, TOK_MONTH, 10 },
{ "November", 8, TOK_MONTH, 11 },
{ "December", 8, TOK_MONTH, 12 },

{ "JANUARY", 7, TOK_MONTH, 1 },
{ "FEBRUARY", 8, TOK_MONTH, 2 },
{ "MARCH", 5, TOK_MONTH, 3 },
{ "APRIL", 5, TOK_MONTH, 4 },
{ "MAY", 3, TOK_MONTH, 5 },
{ "JUNE", 4, TOK_MONTH, 6 },
{ "JULY", 4, TOK_MONTH, 7 },
{ "AUGUST", 6, TOK_MONTH, 8 },
{ "SEPTEMBER", 9, TOK_MONTH, 9 },
{ "OCTOBER", 7, TOK_MONTH, 10 },
{ "NOVEMBER", 8, TOK_MONTH, 11 },
{ "DECEMBER", 8, TOK_MONTH, 12 },

{ "Jan.", 4, TOK_MONTH, 1 },
{ "Feb.", 4, TOK_MONTH, 2 },
{ "Mar.", 4, TOK_MONTH, 3 },
{ "Apr.", 4, TOK_MONTH, 4 },
{ "May.", 4, TOK_MONTH, 5 },
{ "Jun.", 4, TOK_MONTH, 6 },
{ "Jul.", 4, TOK_MONTH, 7 },
{ "Aug.", 4, TOK_MONTH, 8 },
{ "Sep.", 4, TOK_MONTH, 9 },
{ "Oct.", 4, TOK_MONTH, 10 },
{ "Nov.", 4, TOK_MONTH, 11 },
{ "Dec.", 4, TOK_MONTH, 12 },

{ "Jan", 3, TOK_MONTH, 1 },
{ "Feb", 3, TOK_MONTH, 2 },
{ "Mar", 3, TOK_MONTH, 3 },
{ "Apr", 3, TOK_MONTH, 4 },
{ "May", 3, TOK_MONTH, 5 },
{ "Jun", 3, TOK_MONTH, 6 },
{ "Jul", 3, TOK_MONTH, 7 },
{ "Aug", 3, TOK_MONTH, 8 },
{ "Sep", 3, TOK_MONTH, 9 },
{ "Oct", 3, TOK_MONTH, 10 },
{ "Nov", 3, TOK_MONTH, 11 },
{ "Dec", 3, TOK_MONTH, 12 },

{ "JAN", 3, TOK_MONTH, 1 },
{ "FEB", 3, TOK_MONTH, 2 },
{ "MAR", 3, TOK_MONTH, 3 },
{ "APR", 3, TOK_MONTH, 4 },
{ "MAY", 3, TOK_MONTH, 5 },
{ "JUN", 3, TOK_MONTH, 6 },
{ "JUL", 3, TOK_MONTH, 7 },
{ "AUG", 3, TOK_MONTH, 8 },
{ "SEP", 3, TOK_MONTH, 9 },
{ "OCT", 3, TOK_MONTH, 10 },
{ "NOV", 3, TOK_MONTH, 11 },
{ "DEC", 3, TOK_MONTH, 12 },

{ "am", 2, TOK_AMPM, TOKATTR_AM },
{ "AM", 2, TOK_AMPM, TOKATTR_AM },
{ "pm", 2, TOK_AMPM, TOKATTR_PM },
{ "PM", 2, TOK_AMPM, TOKATTR_PM },
{ "", 4, TOK_AMPM, TOKATTR_AM_J },
{ "", 4, TOK_AMPM, TOKATTR_PM_J },

/* { "", 4, TOK_NENGOU, TOKATTR_SHOUWA}, */
/* { "", 2, TOK_NENGOU, TOKATTR_SHOUWA}, */
{ "ʿ", 4, TOK_NENGOU, TOKATTR_HEISEI},
{ "ʿ", 2, TOK_NENGOU, TOKATTR_HEISEI},
{ "ǯ", 2, TOK_NEN, TOKATTR_NONE},
{ "", 2, TOK_GATU, TOKATTR_NONE},
{ "", 2, TOK_NITI, TOKATTR_NONE},
{ "", 2, TOK_JI, TOKATTR_NONE},
{ "ʬ", 2, TOK_FUN, TOKATTR_NONE},
{ "Ⱦ", 2, TOK_HAN, TOKATTR_NONE},

{ "", 2, TOK_SEPARATE, TOKATTR_BRACKET_OPEN },
{ "", 2, TOK_SEPARATE, TOKATTR_BRACKET_OPEN },
{ "", 2, TOK_SEPARATE, TOKATTR_BRACKET_OPEN },
{ "", 2, TOK_SEPARATE, TOKATTR_BRACKET_CLOSE },
{ "", 2, TOK_SEPARATE, TOKATTR_BRACKET_CLOSE },
{ "", 2, TOK_SEPARATE, TOKATTR_BRACKET_CLOSE },
{ "", 2, TOK_SEPARATE, TOKATTR_COMMA },
{ "", 2, TOK_SEPARATE, TOKATTR_PERIOD },
{ "", 2, TOK_SEPARATE, TOKATTR_SLASH },
{ "", 2, TOK_SEPARATE, TOKATTR_COLON },
{ "", 2, TOK_SEPARATE, TOKATTR_HYPHEN },
{ "", 2, TOK_SEPARATE, TOKATTR_KARA },
{ "", 4, TOK_SEPARATE, TOKATTR_KARA },

{ "(", 1, TOK_SEPARATE, TOKATTR_BRACKET_OPEN },
{ "{", 1, TOK_SEPARATE, TOKATTR_BRACKET_OPEN },
{ "[", 1, TOK_SEPARATE, TOKATTR_BRACKET_OPEN },
{ ")", 1, TOK_SEPARATE, TOKATTR_BRACKET_CLOSE },
{ "}", 1, TOK_SEPARATE, TOKATTR_BRACKET_CLOSE },
{ "]", 1, TOK_SEPARATE, TOKATTR_BRACKET_CLOSE },
{ ",", 1, TOK_SEPARATE, TOKATTR_COMMA },
{ ".", 1, TOK_SEPARATE, TOKATTR_PERIOD },
{ "/", 1, TOK_SEPARATE, TOKATTR_SLASH },
{ ":", 1, TOK_SEPARATE, TOKATTR_COLON },
{ "-", 1, TOK_SEPARATE, TOKATTR_HYPHEN },

{ "", 0, TOK_UNKNOWN, TOKATTR_NONE},
};
/* NOTE: ǽ˥ޥåߤĹΡˤ񤫤ʤФʤʤ */

static known_digit knownDigit[] = {

{ "", 2, 0, },
{ "", 2, 1, },
{ "", 2, 2, },
{ "", 2, 3, },
{ "", 2, 4, },
{ "", 2, 5, },
{ "", 2, 6, },
{ "", 2, 7, },
{ "", 2, 8, },
{ "", 2, 9, },

{ "0", 1, 0, },
{ "1", 1, 1, },
{ "2", 1, 2, },
{ "3", 1, 3, },
{ "4", 1, 4, },
{ "5", 1, 5, },
{ "6", 1, 6, },
{ "7", 1, 7, },
{ "8", 1, 8, },
{ "9", 1, 9, },

{ "", 0, 0 },
};

/* -- file local methods */
static bool
Token_Match(char *str1, char *str2, int len);

/* -- methods */

/* ʸΥޥå */
static bool
Token_Match(char *str1, char *str2, int len)
{
    while (len--){
	if (*str1++ != *str2++){
	    return false;
	}
    }
    return true;
} 

/* Υȡ */
char *
Token_GetNext(char *str, uint8 *token, int16 *token_attr)
{
    /* NOTE: strSJISΣХܤϤƤϤʤʤ */

    char *p;
    int i;
    bool isNum, cont;
    int num;

    p = str;

    /*  */
    if (*p == CHAR_EOL){
	*token = TOK_EOL;
	*token_attr = TOKATTR_NONE;
	return NULL;
    }

    /* ʸ */
    if (isspace(*p)){
	while (*p != CHAR_EOL && isspace(*p)){
	    p++;
	}
	*token = TOK_WHITE;
	*token_attr = TOKATTR_NONE;
	return p;
    }

    /* Τ줿ȡڤ */
    i = 0;
    while (knownToken[i].len > 0){
	if (Token_Match(p, knownToken[i].str, knownToken[i].len)){
	    *token = knownToken[i].id;
	    *token_attr = knownToken[i].attr;
	    return p + knownToken[i].len;
	}
	i++;
    }

    /* ڤ */
    isNum = false;
    num = 0;
    cont = true;
    while (cont){
	cont = false;
	i = 0;
	while (knownDigit[i].len > 0){
	    if (Token_Match(p, knownDigit[i].str, knownDigit[i].len)){
		isNum = true;
		cont = true;
		num = num * 10 + knownDigit[i].val;
		p = p + knownDigit[i].len;
		break;
	    }
	    i++;
	}
    }
    if (isNum){
	*token = TOK_NUMBER;
	*token_attr = num;
	return p;
    }

    /* ʤ */
    *token = TOK_UNKNOWN;
    *token_attr = TOKATTR_NONE;
    return p + 1;
}

#if 0
int main(void)
{
    char buf[1024];
    char *p;
    uint8 token;
    int16 token_attr;

    while (NULL != fgets(buf, sizeof(buf), stdin)){
	fprintf(stdout, buf);
	p = buf;
	while (NULL != (p = Token_GetNext(p, &token, &token_attr))){
	    if (token == TOK_WHITE){
		fprintf(stdout, ".");
	    }else if (token != TOK_UNKNOWN){
		fprintf(stdout, "[%d:%d]", token, token_attr);
	    }else if (token == TOK_UNKNOWN){
		fprintf(stdout, "_");
	    }else{
		fprintf(stdout, "?");
	    }
	}
	fprintf(stdout, "\n");
    }
    return 0;
}
#endif
