/*
   3 months view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: view3m.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "taskid.h"
#include "view.h"
#include "view3m.h"
#include "viewitem.h"
#include "additem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>

/* -- file local macros */
#define CALENDARS		3

#define DAYCELL_COLS		7
#define DAYCELL_ROWS		6
#define DAYCELL_WIDTH		26
#define DAYCELL_HEIGHT		20
#define DAYWEEKCELL_COLS	7
#define DAYWEEKCELL_WIDTH	(DAYCELL_WIDTH)
#define DAYWEEKCELL_HEIGHT	20
#define MONTHCELL_COLS		1
#define MONTHCELL_WIDTH		(DAYCELL_WIDTH*DAYCELL_COLS)
#define MONTHCELL_HEIGHT	20
#define CALENDAR_WIDTH		(DAYCELL_WIDTH*DAYCELL_COLS)
#define CALENDAR_HEIGHT		(DAYCELL_HEIGHT*DAYCELL_ROWS+DAYWEEKCELL_HEIGHT+MONTHCELL_HEIGHT)
#define CALENDAR_INTERVAL	((REGION_USER_WIDTH-(CALENDAR_WIDTH*CALENDARS))/(CALENDARS+1))

#define REGION_CAL1_WIDTH	(CALENDAR_WIDTH)
#define REGION_CAL1_HEIGHT	(CALENDAR_HEIGHT)
#define REGION_CAL1_LEFT	(CALENDAR_INTERVAL)
#define REGION_CAL1_TOP		(REGION_USER_TOP+(REGION_USER_HEIGHT-CALENDAR_HEIGHT)/2)
#define REGION_CAL1_RIGHT	(REGION_CAL1_LEFT+REGION_CAL1_WIDTH-1)
#define REGION_CAL1_BOTTOM	(REGION_CAL1_TOP+REGION_CAL1_HEIGHT-1)
#define REGION_CAL2_WIDTH	(CALENDAR_WIDTH)
#define REGION_CAL2_HEIGHT	(CALENDAR_HEIGHT)
#define REGION_CAL2_LEFT	(CALENDAR_INTERVAL*2+CALENDAR_WIDTH*1)
#define REGION_CAL2_TOP		(REGION_USER_TOP+(REGION_USER_HEIGHT-CALENDAR_HEIGHT)/2)
#define REGION_CAL2_RIGHT	(REGION_CAL2_LEFT+REGION_CAL2_WIDTH-1)
#define REGION_CAL2_BOTTOM	(REGION_CAL2_TOP+REGION_CAL2_HEIGHT-1)
#define REGION_CAL3_WIDTH	(CALENDAR_WIDTH)
#define REGION_CAL3_HEIGHT	(CALENDAR_HEIGHT)
#define REGION_CAL3_LEFT	(CALENDAR_INTERVAL*3+CALENDAR_WIDTH*2)
#define REGION_CAL3_TOP		(REGION_USER_TOP+(REGION_USER_HEIGHT-CALENDAR_HEIGHT)/2)
#define REGION_CAL3_RIGHT	(REGION_CAL3_LEFT+REGION_CAL3_WIDTH-1)
#define REGION_CAL3_BOTTOM	(REGION_CAL3_TOP+REGION_CAL3_HEIGHT-1)

/* -- file local variables */
static uint16 originMonth;
static int cursorM, cursorX, cursorY;
static int region_left[3], region_top[3];

/* -- file local methods */
static void
View3M_RedrawDayWeek(HDC hDC);
static void
View3M_RedrawCalendars(HDC hDC);
static void
View3M_RedrawMonth(HDC hDC, int i);
static void
View3M_InvertCursor(HDC hDC);

/* -- methods */

/* View3M */
bool
View3M_Create(void)
{
    uint16 year, month, day;

    D(mgDPrintf("View3M create\n"));

    /* ViewŗL̕ϐ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
    cursorY = ((day+5) - cursorX) / 7;
    cursorM = 1;
    originMonth = Calendar_GetMonthNo(year, month) - 1;
    cursor_focus = FOCUS_CALENDAR;
    cursor_prev_focus = FOCUS_CALENDAR;
    ViewItem_InitCursor();

    region_left[0] = REGION_CAL1_LEFT;
    region_left[1] = REGION_CAL2_LEFT;
    region_left[2] = REGION_CAL3_LEFT;
    region_top[0] = REGION_CAL1_TOP;
    region_top[1] = REGION_CAL2_TOP;
    region_top[2] = REGION_CAL3_TOP;

    return true;
}

/* View3MЕt */
bool
View3M_Destroy(void)
{
    D(mgDPrintf("View3M destroy\n"));
    return true;
}

/* View3Mĕ` */
void
View3M_Redraw(void)
{
    HDC hDC;
    PAINTSTRUCT ps;

    hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
    View3M_RedrawDayWeek(hDC);
    View3M_RedrawCalendars(hDC);
    View3M_InvertCursor(hDC);		      /* ON */
    mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
}

/* View3Mxĕ` */
void
View3M_DelayedRedraw(void)
{
    /* NOTHING TO DO */
}

/* j̕` */
static void
View3M_RedrawDayWeek(HDC hDC)
{
    int i, dayW;
    int left, top, right, bottom;

    for (i = 0; i < CALENDARS; i++){
	top = region_top[i] + MONTHCELL_HEIGHT;
	bottom = top + DAYWEEKCELL_HEIGHT - 1;

	for (dayW = 0; dayW < DAYCELL_COLS; dayW++){
	    left = region_left[i] + DAYCELL_WIDTH * dayW;
	    right = left + DAYCELL_WIDTH - 1;

	    /* ʃ{^ */
	    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
	    mgMoveTo(hDC, left, top);
	    mgLineTo(hDC, left, bottom);
	    mgMoveTo(hDC, left, top);
	    mgLineTo(hDC, right, top);
	    mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
	    mgMoveTo(hDC, left, bottom);
	    mgLineTo(hDC, right, bottom);
	    mgMoveTo(hDC, right, top);
	    mgLineTo(hDC, right, bottom);

	    /* j */
	    mgSetBkMode(hDC, TRANSPARENT);
	    mgSetTextColor(hDC, CL_BLACK);
	    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F16_FNT);
	    mgTextOut(hDC, left + (DAYWEEKCELL_WIDTH - 16) / 2, top + 2,
		      nameDayWeek[dayW], 2);
	}
    }
}

/* J_[̕` */
static void
View3M_RedrawCalendars(HDC hDC)
{
    int i;
    for (i = 0; i < CALENDARS; i++){
	View3M_RedrawMonth(hDC, i);
    }
}

/* J_[̕` */
static void
View3M_RedrawMonth(HDC hDC, int i)
{
    int left, top, right, bottom;
    int x, y;
    MonthRec *monthRec;
    uint16 year, month, day, date;
    char *nameMonth[13] = {
	"",
	"P", "Q", "R", "S", "T", "U", 
	"V", "W", "X", "PO", "PP", "PQ", 
    };
    char daystr[3];

    /* R[ho */
    monthRec = MonthRec_GetByMonthNo(originMonth + i);
    date = monthRec->start_date;
    Calendar_GetDateByNo(date, &year, &month, &day);

    /* ʒu */
    left = region_left[i];
    right = left + MONTHCELL_WIDTH - 1;
    top = region_top[i];
    bottom = top + MONTHCELL_HEIGHT - 1;

    /* wihԂ */
    mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
    mgSetTextColor(hDC, CL_LTGRAY);
    mgRectangle(hDC, left, top, right, bottom);

    /* ʃ{^ */
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, left, bottom);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, right, top);
    mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, bottom);
    mgLineTo(hDC, right, bottom);
    mgMoveTo(hDC, right, top);
    mgLineTo(hDC, right, bottom);

    /*  */
    mgSetBkMode(hDC, TRANSPARENT);
    mgSetTextColor(hDC, CL_BLACK);
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F16_FNT);
    mgTextOut(hDC,
	      left + (MONTHCELL_WIDTH - strlen(nameMonth[month])*8) / 2, top+2,
	      nameMonth[month], strlen(nameMonth[month]));

    /* ʒu */
    top = region_top[i] + MONTHCELL_HEIGHT + DAYWEEKCELL_HEIGHT;
    bottom = top + DAYCELL_HEIGHT*DAYCELL_ROWS - 1;

    /* wihԂ */
    mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
    mgSetTextColor(hDC, CL_LTGRAY);
    mgRectangle(hDC, left, top, right, bottom);

    /* ̕` */
    x = Calendar_GetDayOfWeekByNo(monthRec->start_date);
    y = 0;
    for (day = 1; day <= monthRec->numOfDays; day++){
	/* ʒu */
	left = region_left[i] + DAYCELL_WIDTH * x;
	right = left + DAYCELL_WIDTH - 1;
	top = region_top[i] + MONTHCELL_HEIGHT + DAYWEEKCELL_HEIGHT + DAYCELL_HEIGHT * y;
	bottom = top + DAYCELL_HEIGHT - 1;

	/* wihԂ */
	if (x == 0 || monthRec->isHoliday[day]){
	    mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
	    mgSetTextColor(hDC, CL_DKGRAY);
	    mgRectangle(hDC, left, top, right, bottom);
	    mgSetTextColor(hDC, CL_WHITE);
	    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN|F16_FNT);
	}else if (date == gTodayDate){
	    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
	    mgSetTextColor(hDC, CL_WHITE);
	    mgRectangle(hDC, left, top, right, bottom);
	    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F16_FNT);
	    mgMoveTo(hDC, left, top);
	    mgLineTo(hDC, left, bottom);
	    mgLineTo(hDC, right, bottom);
	    mgLineTo(hDC, right, top);
	    mgLineTo(hDC, left, top);
	    mgSetTextColor(hDC, CL_BLACK);
	    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F16_FNT);
	}else{
	    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
	    mgSetTextColor(hDC, CL_WHITE);
	    mgRectangle(hDC, left, top, right, bottom);
	    mgSetTextColor(hDC, CL_BLACK);
	    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F16_FNT);
	}

	/*  */
	sprintf(daystr, "%2d", day);
	mgTextOut(hDC, left + (DAYCELL_WIDTH - 16) / 2, top + 2,
		  daystr, 2);

	date++;
	x++;
	if (x == DAYCELL_COLS){
	    x = 0;
	    y++;
	}
    }
}

/* J[\̔] */
static void
View3M_InvertCursor(HDC hDC)
{
    RECT rect;

    rect.left = region_left[cursorM] + DAYCELL_WIDTH * cursorX;
    rect.right = rect.left + DAYCELL_WIDTH - 1;
    rect.top = region_top[cursorM] + MONTHCELL_HEIGHT + DAYWEEKCELL_HEIGHT + DAYCELL_HEIGHT * cursorY;
    rect.bottom = rect.top + DAYCELL_HEIGHT - 1;
    mgInvertRect(hDC, &rect);
}

/* L[R[h */
bool
View3M_KeyDown(WPARAM wParam, LPARAM lParam)
{
    HDC hDC;
    PAINTSTRUCT ps;
    uint16 year, month, day, prev_month;

    switch (wParam){
	case VK_LEFT:			      /*  */
	case UVK_LEFT:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View3M_InvertCursor(hDC);	      /* OFF */
	gCursorDate--;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (day == Calendar_GetNumOfDays(year, month)){
	    /* ŏI */
	    if (cursorM == 0){
		originMonth--;
		View3M_RedrawCalendars(hDC);
	    }else{
		cursorM--;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_RIGHT:			      /* E */
	case UVK_RIGHT:			      /* E */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View3M_InvertCursor(hDC);	      /* OFF */
	gCursorDate++;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (day == 1){
	    /* ŏI */
	    if (cursorM == 2){
		originMonth++;
		View3M_RedrawCalendars(hDC);
	    }else{
		cursorM++;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_UP:			      /*  */
	case UVK_UP:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View3M_InvertCursor(hDC);	      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &prev_month, &day);
	gCursorDate -= 7;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (month != prev_month){
	    /* ŏI */
	    if (cursorM == 0){
		originMonth--;
		View3M_RedrawCalendars(hDC);
	    }else{
		cursorM--;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_DOWN:			      /*  */
	case UVK_DOWN:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View3M_InvertCursor(hDC);	      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &prev_month, &day);
	gCursorDate += 7;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (month != prev_month){
	    /* ŏI */
	    if (cursorM == 2){
		originMonth++;
		View3M_RedrawCalendars(hDC);
	    }else{
		cursorM++;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_PRIOR:			      /* O */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View3M_InvertCursor(hDC);	      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	originMonth -= 3;
	View3M_RedrawCalendars(hDC);
	year = 1900 + ((originMonth + cursorM) / 12);
	month = (originMonth + cursorM) % 12 + 1;
	if (day > Calendar_GetNumOfDays(year, month)){
	    day = Calendar_GetNumOfDays(year, month);
	}
	gCursorDate = Calendar_GetDayNo(year, month, day);
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_NEXT:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View3M_InvertCursor(hDC);	      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	originMonth += 3;
	View3M_RedrawCalendars(hDC);
	year = 1900 + ((originMonth + cursorM) / 12);
	month = (originMonth + cursorM) % 12 + 1;
	if (day > Calendar_GetNumOfDays(year, month)){
	    day = Calendar_GetNumOfDays(year, month);
	}
	gCursorDate = Calendar_GetDayNo(year, month, day);
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}

/* R}h */
bool
View3M_Command(WPARAM wParam, LPARAM lParam)
{
    uint16 year, month, day;

    switch (wParam){
	case ID_FUNC_NOW:
	if (HIWORD(lParam) == BN_CLICKED){
	    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	    cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	    cursorY = ((day+5) - cursorX) / 7;
	    cursorM = 1;
	    originMonth = Calendar_GetMonthNo(year, month) - 1;
	    View_Redraw();
	}else{
	    return false;
	}
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}
