/*
   file browse view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewbr.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "taskid.h"
#include "view.h"
#include "viewbr.h"
#include "viewitem.h"
#include "viewfile.h"
#include "additem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>

/* -- file local macros */
#define FILECELL_ROWS		14
#define FILECELL_COLS		80
#define FILECELL_WIDTH		(6*FILECELL_COLS)
#define FILECELL_HEIGHT		(13)

#define REGION_TITLE_WIDTH	(FILECELL_WIDTH)
#define REGION_TITLE_HEIGHT	(18)
#define REGION_FILE_WIDTH	(FILECELL_WIDTH)
#define REGION_FILE_HEIGHT	(FILECELL_HEIGHT*FILECELL_ROWS)

#define ITEMCELL_ROWS		12
#define ITEMCELL_COLS		1
#define ITEMCELL_WIDTH		(REGION_USER_WIDTH-REGION_TITLE_WIDTH-1)
#define ITEMCELL_HEIGHT		(12+3)

#define REGION_DATE_WIDTH	(ITEMCELL_WIDTH)
#define REGION_DATE_HEIGHT	(16*1+5)
#define REGION_DATE_LEFT	(REGION_USER_LEFT+REGION_TITLE_WIDTH+1)
#define REGION_DATE_TOP		(REGION_USER_TOP)
#define REGION_ITEMS_WIDTH	(ITEMCELL_WIDTH)
#define REGION_ITEMS_HEIGHT	(ITEMCELL_HEIGHT*ITEMCELL_ROWS)

/* -- file local variables */

/* -- file local methods */
static void
ViewBR_RedrawItemRegion(void);

/* -- methods */

/* ViewBR */
bool
ViewBR_Create(void)
{
    D(mgDPrintf("ViewBR create\n"));

    /* ViewŗL̕ϐ */
    cursor_focus = FOCUS_BROWSER;
    cursor_prev_focus = FOCUS_BROWSER;
    ViewItem_Init(REGION_DATE_LEFT, REGION_DATE_TOP,
		  REGION_DATE_WIDTH, REGION_DATE_HEIGHT,
		  REGION_ITEMS_WIDTH, REGION_ITEMS_HEIGHT,
		  ITEMCELL_WIDTH, ITEMCELL_HEIGHT,
		  ITEMCELL_COLS, ITEMCELL_ROWS,
		  F12_FNT, 7+19);
    ViewItem_InitCursor();
    ViewFile_Init(REGION_USER_LEFT, REGION_USER_TOP,
		  REGION_TITLE_WIDTH, REGION_TITLE_HEIGHT,
		  REGION_FILE_WIDTH, REGION_FILE_HEIGHT,
		  FILECELL_WIDTH, FILECELL_HEIGHT,
		  FILECELL_COLS, FILECELL_ROWS,
		  F12_FNT);
    ViewFile_InitCursor();

    return true;
}

/* ViewBRЕt */
bool
ViewBR_Destroy(void)
{
    D(mgDPrintf("ViewBR destroy\n"));
    ViewFile_Tidy();
    return true;
}

/* ViewBRĕ` */
void
ViewBR_Redraw(void)
{
    HDC hDC;
    PAINTSTRUCT ps;

    hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);

    /*  */
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, REGION_USER_LEFT+REGION_TITLE_WIDTH, REGION_USER_TOP);
    mgLineTo(hDC, REGION_USER_LEFT+REGION_TITLE_WIDTH, REGION_USER_BOTTOM);

    ViewFileTitle_Redraw(hDC);
    ViewFileFile_Redraw(hDC);
    ViewItemDate_Redraw(hDC);
    ViewItem_Redraw(hDC);
    ViewFile_InvertCursor(hDC);		      /* ON */
    if (cursor_focus == FOCUS_ITEMLIST)
	ViewItem_InvertCursor(hDC);	      /* ON */
    mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
}

/* ViewBRxĕ` */
void
ViewBR_DelayedRedraw(void)
{
    /* NOTHING TO DO */
}

/* ACë̍ĕ` */
static void
ViewBR_RedrawItemRegion(void)
{
    HDC hDC;
    PAINTSTRUCT ps;

    hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
    ViewItemDate_Redraw(hDC);
    ViewItem_Redraw(hDC);
    if (cursor_focus == FOCUS_ITEMLIST)
	ViewItem_InvertCursor(hDC);
    mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
}

/* L[R[h */
bool
ViewBR_KeyDown(WPARAM wParam, LPARAM lParam)
{
    HDC hDC;
    PAINTSTRUCT ps;

    switch (wParam){
	case VK_LEFT:			      /*  */
	case UVK_LEFT:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewFile_InvertCursor(hDC);	      /* OFF */
	gCursorDate--;
	ViewFile_InvertCursor(hDC);	      /* ON */
	/* tH[JXuEUɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_BROWSER;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewBR_RedrawItemRegion();
	View_SetTimer();
	break;

	case VK_RIGHT:			      /* E */
	case UVK_RIGHT:			      /* E */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewFile_InvertCursor(hDC);	      /* OFF */
	gCursorDate++;
	ViewFile_InvertCursor(hDC);	      /* ON */
	/* tH[JXuEUɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_BROWSER;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewBR_RedrawItemRegion();
	View_SetTimer();
	break;

	case VK_UP:			      /*  */
	case UVK_UP:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewFile_InvertCursor(hDC);	      /* OFF */
	ViewFile_Up(hDC);
	ViewFile_InvertCursor(hDC);	      /* ON */
	/* tH[JXuEUɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InvertCursor(hDC);
	    cursor_focus = FOCUS_BROWSER;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case VK_DOWN:			      /*  */
	case UVK_DOWN:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewFile_InvertCursor(hDC);	      /* OFF */
	ViewFile_Down(hDC);
	ViewFile_InvertCursor(hDC);	      /* ON */
	/* tH[JXuEUɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InvertCursor(hDC);
	    cursor_focus = FOCUS_BROWSER;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case VK_PRIOR:			      /* O */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewFile_PageUp(hDC);
	ViewFile_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* tH[JXuEUɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    cursor_focus = FOCUS_BROWSER;
	}
	break;

	case VK_NEXT:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewFile_PageDown(hDC);
	ViewFile_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* tH[JXACeXgɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    cursor_focus = FOCUS_BROWSER;
	}
	break;

	case UVK_ITEMUP:		      /* ACe */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	if (cursor_focus == FOCUS_BROWSER){
	    D(mgDPrintf("focus ITEMLIST"));
	    ViewItem_InvertCursor(hDC);     /* ON */
	    cursor_focus = FOCUS_ITEMLIST;
	}else{
	    ViewItem_InvertCursor(hDC);     /* OFF */
	    ViewItem_Up(hDC);
	    ViewItem_InvertCursor(hDC);     /* ON */
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case UVK_ITEMDOWN:		      /* ACe */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	if (cursor_focus == FOCUS_BROWSER){
	    D(mgDPrintf("focus ITEMLIST"));
	    ViewItem_InvertCursor(hDC);     /* ON */
	    cursor_focus = FOCUS_ITEMLIST;
	}else{
	    ViewItem_InvertCursor(hDC);     /* OFF */
	    ViewItem_Down(hDC);
	    ViewItem_InvertCursor(hDC);     /* ON */
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}

/* R}h */
bool
ViewBR_Command(WPARAM wParam, LPARAM lParam)
{
    switch (wParam){
	case ID_FUNC_NOW:
	if (HIWORD(lParam) == BN_CLICKED){
	    ViewFile_InitCursor();
	    ViewItem_InitCursor();
	    View_Redraw();
	}else{
	    return false;
	}
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}
