/*
   calender region of view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewcal.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskid.h"
#include "taskmg.h"
#include "record.h"
#include "db.h"
#include "view.h"
#include "viewitem.h"
#include "viewcal.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>

/* -- file local macros */

/* -- file local variables */
static uint16 originDate;
static int cursorX, cursorY; 
static int region_dayweek_width, region_dayweek_height;
static int region_dayweek_left, region_dayweek_right;
static int region_dayweek_top, region_dayweek_bottom;
static int region_weeks_width, region_weeks_height;
static int region_weeks_left, region_weeks_right;
static int region_weeks_top, region_weeks_bottom;
static int cell_width, cell_height;
static int cell_rows, cell_cols;
static int cell_fontwidth, cell_fontheight, cell_font;
static int cell_maxlen;
static int day_rows;
static int day_margin_left, day_margin_top;
static int day_interval;
static int dayweek_fontwidth, dayweek_font;

/* -- file local methods */
static void
ViewCal_RedrawDay(HDC hDC, int x, int y);

/* -- methods */

/* ViewCal̏ */
void
ViewCal_Init(int left, int top,
	     int dayweek_width, int dayweek_height,
	     int weeks_width, int weeks_height,
	     int c_width, int c_height,
	     int c_cols, int c_rows,
	     int d_rows,
	     int c_font, int c_maxlen)
{
    region_dayweek_width = dayweek_width;
    region_dayweek_height = dayweek_height;
    region_dayweek_left = left;
    region_dayweek_top = top;
    region_dayweek_right = region_dayweek_left + region_dayweek_width - 1;
    region_dayweek_bottom = region_dayweek_top + region_dayweek_height - 1;

    region_weeks_width = weeks_width;
    region_weeks_height = weeks_height;
    region_weeks_left = left;
    region_weeks_top = region_dayweek_top + region_dayweek_height;
    region_weeks_right = region_weeks_left + region_weeks_width - 1;
    region_weeks_bottom = region_weeks_top + region_weeks_height - 1;

    cell_width = c_width;
    cell_height = c_height;
    cell_cols = c_cols;
    cell_rows = c_rows;
    cell_font = c_font;
    cell_maxlen = c_maxlen;

    switch (cell_font){
	case F16_8_FNT:
	cell_fontwidth = 8;
	cell_fontheight = 8;
	dayweek_font = F12_FNT;
	dayweek_fontwidth = 6;
	break;
	case F16_FNT:
	cell_fontwidth = 8;
	cell_fontheight = 16;
	dayweek_font = F16_FNT;
	dayweek_fontwidth = 8;
	break;
	case F12_FNT:
	cell_fontwidth = 6;
	cell_fontheight = 12;
	dayweek_font = F12_FNT;
	dayweek_fontwidth = 6;
	break;
    }

    day_rows = d_rows;
    day_margin_left = (cell_width - cell_fontwidth * cell_maxlen) / 2 + 1;
    day_margin_top = cell_fontheight + 1;
    day_interval = (cell_height - day_margin_top) / day_rows;
}

/* J[\̏ */
void
ViewCal_InitCursor(void)
{
    if (gOpt_WeeksStartsFromMonday){
	/* T͌j */
	cursorX = (Calendar_GetDayOfWeekByNo(gCursorDate) + 7 - 1) % 7;
	cursorY = 0;
	originDate = gCursorDate - (uint16)cursorX;
    }else{
	/* T͓j */
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = 0;
	originDate = gCursorDate - (uint16)cursorX;
    }
}

/* j̍ĕ` */
void
ViewCalDayWeek_Redraw(HDC hDC)
{
    int i;
    int left, top, right, bottom;

    top = region_dayweek_top;
    bottom = region_dayweek_bottom;

    for (i = 0; i < cell_cols; i++){
	left = region_dayweek_left + cell_width * i;
	right = left + cell_width - 1;

	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
	mgMoveTo(hDC, left, top);
	mgLineTo(hDC, left, bottom);

	/* ʃ{^ */
	mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
	mgMoveTo(hDC, left+1, top);
	mgLineTo(hDC, left+1, bottom);
	mgMoveTo(hDC, left+1, top);
	mgLineTo(hDC, right, top);
	mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
	mgMoveTo(hDC, left+2, bottom);
	mgLineTo(hDC, right, bottom);
	mgMoveTo(hDC, right, top);
	mgLineTo(hDC, right, bottom);

	/* j */
	mgSetBkMode(hDC, TRANSPARENT);
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|dayweek_font);
	if (gOpt_WeeksStartsFromMonday){
	    /* T͌j */
	    mgTextOut(hDC,
		      left + (cell_width - dayweek_fontwidth*2) / 2, top+1,
		      nameDayWeek[(i+1)%7], 2);
	}else{
	    /* T͓j */
	    mgTextOut(hDC,
		      left + (cell_width - dayweek_fontwidth*2) / 2, top+1,
		      nameDayWeek[i], 2);
	}
    }
}

/* ̍ĕ` */
static void
ViewCal_RedrawDay(HDC hDC, int x, int y)
{
    int left, top, right, bottom;
    uint16 year, month, day, date;
    MonthRec *monthRec;
    char daystr[3];
    int i;

    /* ʒu */
    left = region_weeks_left + cell_width * x;
    right = region_weeks_left + cell_width * (x + 1) - 1;
    top = region_weeks_top + cell_height * y;
    bottom = region_weeks_top + cell_height * (y + 1) - 1;

    /* R[ho */
    date = originDate + (uint16)(x + y * cell_cols);
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* wihԂ */
    if (monthRec->isHoliday[day]){
	/* j͏j */
	mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
	mgSetTextColor(hDC, CL_LTGRAY);
    }else{
	/*  */
	mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
	mgSetTextColor(hDC, CL_WHITE);
    }
    mgRectangle(hDC, left, top, right, bottom);

    /* g */
    mgMoveTo(hDC, left, top);
    if (day == 1){
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
	mgLineTo(hDC, left, bottom);
	mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
	mgLineTo(hDC, right, bottom);
    }else if (monthRec->numOfDays - day < 7){
	mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
	mgLineTo(hDC, left, bottom);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
	mgLineTo(hDC, right, bottom);
    }else{
	mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
	mgLineTo(hDC, left, bottom);
	mgLineTo(hDC, right, bottom);
    }

    if (date != gTodayDate){
	/* tg */
	mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
	mgMoveTo(hDC, left + cell_fontwidth*2 + 2, top);
	mgLineTo(hDC, left + cell_fontwidth*2 + 2, top + cell_fontheight);
	mgLineTo(hDC, left, top + cell_fontheight);

	/* t */
	mgSetBkMode(hDC, TRANSPARENT);
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
	sprintf(daystr, "%2d", day);
	mgTextOut(hDC, left + 1, top, daystr, 2);
    }else{
	/* thԂ */
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
	mgRectangle(hDC, left + 1, top,
		    left + cell_fontwidth*2 + 2, top + cell_fontheight);

	/* t */
	mgSetBkMode(hDC, TRANSPARENT);
	mgSetTextColor(hDC, CL_WHITE);
	mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN|cell_font);
	sprintf(daystr, "%2d", day);
	mgTextOut(hDC, left + 1, top, daystr, 2);
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
    }

    /* \̕\ */
    for (i = 0; i < day_rows && i < (int)monthRec->num[day] && monthRec->rec[day][i]->type != RECORD_TYPE_TASK; i++){
	Record *rec = monthRec->rec[day][i];
	int str_top = top + day_margin_top + i * day_interval;
#if 0
	/* oi[ */
	if (rec->start_date != rec->end_date
	    && rec->repeat_type == RECORD_TYPE_NONE){
	    bool banner;
	    switch (rec->type){
		case RECORD_TYPE_SCHEDULE:
		if (rec->u.sche.start_time_h == 0
		    && rec->u.sche.start_time_m == 0){
		    banner = true;
		}
		break;
		case RECORD_TYPE_HOLIDAY:
		banner = true;
		break;
		default:
		banner = false;
		break;
	    }
	    if (banner){
		mgSetTextColor(hDC, CL_LTGRAY);
		mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
		if (date == rec->start_date){
		    /* oi[Jn */
		    mgRectangle(hDC,
				left + day_margin_left, str_top,
				right, str_top + cell_fontheight);
		    mgSetTextColor(hDC, CL_BLACK);
		    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
		    mgMoveTo(hDC, right, str_top);
		    mgLineTo(hDC, left + day_margin_left, str_top);
		    mgLineTo(hDC, left + day_margin_left, str_top + cell_fontheight);
		    mgLineTo(hDC, right, str_top + cell_fontheight);
		}else if (date == rec->end_date){
		    /* oi[I */
		    mgRectangle(hDC,
				left, str_top,
				right - day_margin_left, str_top + cell_fontheight);
		    mgSetTextColor(hDC, CL_BLACK);
		    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
		    mgMoveTo(hDC, left, str_top);
		    mgLineTo(hDC, right - day_margin_left, str_top);
		    mgLineTo(hDC, right - day_margin_left, str_top + cell_fontheight);
		    mgLineTo(hDC, left, str_top + cell_fontheight);
		}else{
		    /* oi[ */
		    mgRectangle(hDC,
				left, str_top,
				right, str_top + cell_fontheight);
		    mgSetTextColor(hDC, CL_BLACK);
		    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
		    mgMoveTo(hDC, left, str_top);
		    mgLineTo(hDC, right, str_top);
		    mgMoveTo(hDC, left, str_top + cell_fontheight);
		    mgLineTo(hDC, right, str_top + cell_fontheight);
		}
	    }
	}
#endif
	/* \ */
	mgTextOut(hDC,
		  left + day_margin_left, str_top,
		  (char *)(rec->descript),
		  mmin((int)rec->descript_len, cell_maxlen));
    }

    /* \ӂ */
    if ((int)monthRec->num[day] > day_rows && monthRec->rec[day][day_rows]->type != RECORD_TYPE_TASK){
	mgTextOut(hDC, right - 3 + cell_fontwidth*2, top, "", 2);
    }
}

/* T̍ĕ` */
void
ViewCalWeeks_Redraw(HDC hDC)
{
    int x, y;

    for (y = 0; y < cell_rows; y++){
	for (x = 0; x < cell_cols; x++){
	    ViewCal_RedrawDay(hDC, x, y);
	}
    }
}

/* J[\̔] */
void
ViewCal_InvertCursor(HDC hDC)
{
    RECT rect;

    rect.left = region_weeks_left + cell_width * cursorX;
    rect.right = region_weeks_left + cell_width * (cursorX + 1) - 1;
    rect.top = region_weeks_top + cell_height * cursorY;
    rect.bottom = region_weeks_top + cell_height * (cursorY + 1) - 1;
    mgInvertRect(hDC, &rect);
}

/* PTXN[Abv */
void
ViewCal_ScrollUp(HDC hDC)
{
    int x;
    RECT rect;

    rect.left = region_weeks_left;
    rect.right = region_weeks_right;
    rect.top = region_weeks_top;
    rect.bottom = region_weeks_top + cell_height * (cell_rows - 1) - 1;
    mgScrollWindow(ghMainWin, 0, cell_height, &rect);

    for (x = 0; x < cell_cols; x++){
	ViewCal_RedrawDay(hDC, x, 0);
    }
}

/* PTXN[_E */
void
ViewCal_ScrollDown(HDC hDC)
{
    int x;
    RECT rect;

    rect.left = region_weeks_left;
    rect.right = region_weeks_right;
    rect.top = region_weeks_top + cell_height;
    rect.bottom = region_weeks_top + cell_height * cell_rows - 1;
    mgScrollWindow(ghMainWin, 0, -cell_height, &rect);

    for (x = 0; x < cell_cols; x++){
	ViewCal_RedrawDay(hDC, x, (cell_rows-1));
    }
}

/*  */
void
ViewCal_Left(HDC hDC)
{
    gCursorDate--;
    cursorX--;
    if (cursorX < 0){
	cursorX = cell_cols-1;
	cursorY--;
	if (cursorY < 0){
	    cursorY = 0;
	    originDate -= cell_cols;
	    ViewCal_ScrollUp(hDC);
	}
    }
}

/* E */
void
ViewCal_Right(HDC hDC)
{
    gCursorDate++;
    cursorX++;
    if (cursorX > cell_cols-1){
	cursorX = 0;
	cursorY++;
	if (cursorY > (cell_rows-1)){
	    cursorY = (cell_rows-1);
	    originDate += cell_cols;
	    ViewCal_ScrollDown(hDC);
	}
    }
}

/*  */
void
ViewCal_Up(HDC hDC)
{
    gCursorDate -= cell_cols;
    cursorY--;
    if (cursorY < 0){
	cursorY = 0;
	originDate -= cell_cols;
	ViewCal_ScrollUp(hDC);
    }
}

/*  */
void
ViewCal_Down(HDC hDC)
{
    gCursorDate += cell_cols;
    cursorY++;
    if (cursorY > (cell_rows-1)){
	cursorY = (cell_rows-1);
	originDate += cell_cols;
	ViewCal_ScrollDown(hDC);
    }
}

/* T */
void
ViewCal_WeekUp(HDC hDC, int num)
{
    gCursorDate -= cell_cols * num;
    originDate -= cell_cols * num;
    ViewCalWeeks_Redraw(hDC);
}

/* T */
void
ViewCal_WeekDown(HDC hDC, int num)
{
    gCursorDate += cell_cols * num;
    originDate += cell_cols * num;
    ViewCalWeeks_Redraw(hDC);
}

