/*
   itemlist region of view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewitem.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskid.h"
#include "taskmg.h"
#include "record.h"
#include "db.h"
#include "view.h"
#include "viewitem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>

/* -- file local macros */

/* -- file local variables */
static int originI, cursorI;
static int region_date_width, region_date_height;
static int region_date_left, region_date_right;
static int region_date_top, region_date_bottom;
static int region_item_width, region_item_height;
static int region_item_left, region_item_right;
static int region_item_top, region_item_bottom;
static int cell_width, cell_height;
static int cell_rows, cell_cols;
static int cell_fontwidth, cell_fontheight, cell_font;
static int cell_maxlen;

static Record *clippedRecord = NULL;
static uint16 clippedType = RECORD_TYPE_NONE;

/* -- file local methods */
static void
ViewItem_RedrawEach(HDC hDC, int no);

/* -- methods */

/* ViewItem̏ */
void
ViewItem_Init(int left, int top,
	      int date_width, int date_height,
	      int item_width, int item_height,
	      int c_width, int c_height,
	      int c_cols, int c_rows,
	      int c_font, int c_maxlen)
{
    region_date_width = date_width;
    region_date_height = date_height;
    region_date_left = left;
    region_date_top = top;
    region_date_right = region_date_left + region_date_width - 1;
    region_date_bottom = region_date_top + region_date_height - 1;

    region_item_width = item_width;
    region_item_height = item_height;
    region_item_left = left;
    region_item_top = region_date_top + region_date_height;
    region_item_right = region_item_left + region_item_width - 1;
    region_item_bottom = region_item_top + region_item_height - 1;

    cell_width = c_width;
    cell_height = c_height;
    cell_cols = c_cols;
    cell_rows = c_rows;
    cell_font = c_font;
    cell_maxlen = c_maxlen;

    switch (cell_font){
	case F16_FNT:
	cell_fontwidth = 8;
	cell_fontheight = 16;
	break;
	case F12_FNT:
	cell_fontwidth = 6;
	cell_fontheight = 12;
	break;
    }
}

/* ACe̍ĕ` */
static void
ViewItem_RedrawEach(HDC hDC, int no)
{
    int left, top, right, bottom;
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;
    char str[10];

    /* R[ho */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* ǂ̃ACe */
    rec = monthRec->rec[day][originI + no];

    /* ʒu */
    left = region_item_left;
    right = region_item_right;
    top = region_item_top + no * cell_height;
    bottom = region_item_top + (no + 1) * cell_height - 1;

    /* hԂ */
    mgSetTextColor(hDC, CL_WHITE);
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, left, top, right, bottom);

    /* ACe */
    mgSetTextColor(hDC, CL_BLACK);
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
    switch (rec->type){
	case RECORD_TYPE_HOLIDAY:	      /* x */
	/* O[œhԂ */
	mgSetTextColor(hDC, CL_LTGRAY);
	mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
	mgRectangle(hDC, left, top, right, bottom);
	/* ɋx\ */
	mgSetBkMode(hDC, TRANSPARENT);
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
	if (rec->link_len > 0){
	    mgTextOut(hDC, left + 1, top + 1, "<", 1);
	}
	mgTextOut(hDC, left + cell_fontwidth * 1 + 1 + (cell_width - mmin((int)rec->descript_len, cell_maxlen-1)*cell_fontwidth) / 2, top + 1,
		  (char *)(rec->descript),
		  mmin((int)rec->descript_len, cell_maxlen-1));
	break;
	case RECORD_TYPE_SCHEDULE:	      /* XPW[ */
	if (rec->link_len > 0){
	    mgTextOut(hDC, left + 1, top + 1, "<", 1);
	}
	if (rec->u.sche.end_time_h == 0 && rec->u.sche.end_time_m == 0){
	    /* Ȃ */
	    mgTextOut(hDC, left + cell_fontwidth * 7 + 1, top + 1,
		      (char *)(rec->descript), mmin((int)rec->descript_len, cell_maxlen - 7));
	}else{
	    /*  */
	    sprintf(str, "%2d:%02d",
		    rec->u.sche.start_time_h,
		    rec->u.sche.start_time_m);
	    mgTextOut(hDC, left + cell_fontwidth * 1 + 1, top + 1, str, 5);
	    mgTextOut(hDC, left + cell_fontwidth * 7 + 1, top + 1,
		      (char *)(rec->descript), mmin((int)rec->descript_len, cell_maxlen - 7));
	}
	break;
	case RECORD_TYPE_TASK:		      /* ^XN */
	if (rec->link_len > 0){
	    mgTextOut(hDC, left + 1, top + 1, "<", 1);
	}
	sprintf(str, "   %d", rec->u.task.rank);
	mgTextOut(hDC, left, top + 1, str, 6);
	mgTextOut(hDC, left + cell_fontwidth * 7 + 1, top + 1,
		  (char *)(rec->descript), mmin((int)rec->descript_len, cell_maxlen - 7));
	/* ł */
	if (rec->done_len > 0){
	    mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
	    mgMoveTo(hDC, left+8, top+1+cell_fontheight/2-2);
	    mgLineTo(hDC, right-8, top+1+cell_fontheight/2-2);
	    mgMoveTo(hDC, left+8, top+1+cell_fontheight/2+2);
	    mgLineTo(hDC, right-8, top+1+cell_fontheight/2+2);
	}
	break;
	default:			      /* ̑ */
	if (rec->link_len > 0){
	    mgTextOut(hDC, left + 1, top + 1, "<", 1);
	}
	mgTextOut(hDC, left + cell_fontwidth * 1 + 1, top + 1,
		  (char *)(rec->descript),
		  mmin((int)rec->descript_len, cell_maxlen));
	break;
    }

    /* ؂g */
    mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, bottom);
    mgLineTo(hDC, right, bottom);
}

/* ACeXg̍ĕ` */
void
ViewItem_Redraw(HDC hDC)
{
    int left, top, right, bottom;
    uint16 year, month, day;
    MonthRec *monthRec;
    int i;

    /* ʒu */
    left = region_item_left;
    right = region_item_right;
    top = region_item_top;
    bottom = region_item_bottom;

    /* R[ho */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* hԂ */
    mgSetTextColor(hDC, CL_WHITE);
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, left, top, right, bottom);

    /* ACe̕\ */
    for (i = 0; i < cell_rows && i < (int)monthRec->num[day]; i++){
	ViewItem_RedrawEach(hDC, i);
    }
}

/* ACeAbv */
void
ViewItem_Up(HDC hDC)
{
    uint16 year, month, day;
    MonthRec *monthRec;

    /* R[ho */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	if ((int)monthRec->num[day] > cell_rows){
	    if (originI > 0
		&& (cursorI - originI) == 0){
		/* XN[Abv */
		originI--;
		cursorI--;
		ViewItemDate_Redraw(hDC);
		ViewItem_Redraw(hDC);
		/* NOTE: mgScrollWindow()@\Ȃ̂ŁC
		   ׂĂ̍ڂ */
	    }else{
		/* J[\Abv */
		cursorI--;
		if (cursorI < 0)
		    cursorI = 0;
	    }
	}else{
	    /* J[\Abv */
	    cursorI--;
	    if (cursorI < 0)
		cursorI = 0;
	}
    }
}

/* ACe_E */
void
ViewItem_Down(HDC hDC)
{
    uint16 year, month, day;
    MonthRec *monthRec;

    /* R[ho */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	if ((int)monthRec->num[day] > cell_rows){
	    if (originI < (signed)(monthRec->num[day] - cell_rows)
		&& (cursorI - originI) == (cell_rows - 1)){
		/* XN[_E */
		originI++;
		cursorI++;
		ViewItemDate_Redraw(hDC);
		ViewItem_Redraw(hDC);
		/* NOTE: mgScrollWindow()@\Ȃ̂ŁC
		   ׂĂ̍ڂ */
	    }else{
		/* J[\_E */
		cursorI++;
		if (cursorI >= (signed)(monthRec->num[day]))
		    cursorI = monthRec->num[day] - 1;
	    }
	}else{
	    /* J[\_E */
	    cursorI++;
	    if (cursorI >= (signed)(monthRec->num[day]))
		cursorI = monthRec->num[day] - 1;
	}
    }
}

/* J[\̔] */
void
ViewItem_InvertCursor(HDC hDC)
{
    RECT rect;
    uint16 year, month, day;
    MonthRec *monthRec;

    /* R[ho */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	rect.left = region_item_left;
	rect.right = region_item_right;
	rect.top = region_item_top + cell_height * (cursorI - originI);
	rect.bottom = region_item_top + cell_height * ((cursorI - originI)+ 1) - 1;
	mgInvertRect(hDC, &rect);
    }
}

/* J[\̏ */
void
ViewItem_InitCursor(void)
{
    cursorI = 0;
    originI = 0;
}

/* J[\̍Đݒ */
void
ViewItem_ResetCursor(void)
{
    uint16 year, month, day;
    MonthRec *monthRec;

    /* R[ho */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	if (cursorI >= (int)monthRec->num[day]){
	    cursorI = monthRec->num[day] - 1;
	}
	if (cursorI + 1 > cell_rows){
	    /* \ӂ̏ꍇ */
	    originI = cursorI - cell_rows + 1;
	}else{
	    /* \ӂłȂꍇ */
	    originI = 0;
	}
    }else{
	cursorI = 0;
	originI = 0;
	cursor_focus = cursor_prev_focus;
    }
}

/* J[\̏ */
uint16
ViewItem_GetCursor(void)
{
    return cursorI;
}

/* t̍ĕ` */
void
ViewItemDate_Redraw(HDC hDC)
{
    uint16 year, month, day;
    int left, top, right, bottom;
    MonthRec *monthRec;
    char str[80];

    /* R[ho */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* ʒu */
    left = region_date_left;
    right = region_date_right;
    top = region_date_top;
    bottom = region_date_bottom;

    /* hԂ */
    mgSetTextColor(hDC, CL_LTGRAY);
    mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, left, top, right, bottom);

    /* ʃ{^ */
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, left, bottom);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, right, top);
    mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, bottom);
    mgLineTo(hDC, right, bottom);
    mgMoveTo(hDC, right, top);
    mgLineTo(hDC, right, bottom);

    /* j */
    sprintf(str, "%dN%2d%2d(%s)  ", 
	    Calendar_GetYearByNo(gCursorDate),
	    Calendar_GetMonthByNo(gCursorDate),
	    Calendar_GetDayByNo(gCursorDate),
	    nameDayWeek[Calendar_GetDayOfWeekByNo(gCursorDate)]);
    if (region_date_width > (signed)strlen(str)*8){
	mgSetBkMode(hDC, TRANSPARENT);
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F16_FNT);
	mgTextOut(hDC,
		  left + (region_date_width - strlen(str)*8) / 2,
		  top + (region_date_height - 16) / 2,
		  str, strlen(str));
    }else{
	mgSetBkMode(hDC, TRANSPARENT);
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F12_FNT);
	mgTextOut(hDC,
		  left + (region_date_width - strlen(str)*6) / 2,
		  top + (region_date_height - 12) / 2,
		  str, strlen(str));
    }

    /* \ӂ */
    if ((int)monthRec->num[day] > cell_rows){
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F12_FNT);
	if (originI > 0){
	    mgTextOut(hDC, right - 12*1 - 1, top + 2, "", 2);
	}
	if ((int)monthRec->num[day] - originI > cell_rows){
	    mgTextOut(hDC, right - 12*1 - 1, top + 2, "", 2);
	}
	/* NOTE: ʒuɏd˂ĕ` */
    }
}

bool
Misc_CutItem(uint16 date, int no)
{
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;

    D(mgDPrintf("Misc_CutItem()"));

    /* R[ho */
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
    rec = monthRec->rec[day][no];

    /* Nbv{[hɃ}[N */
    clippedRecord = rec;
    clippedType = rec->type;

    /* 폜 */
    Record_Delete(rec);
    /* TODO: Ԃl̃`FbN */
    gDBDirty = true;

    /* LbV̖ */
    MonthRec_Invalidate();

    return true;
}

bool
Misc_CopyItem(uint16 date, int no)
{
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;

    D(mgDPrintf("Misc_CopyItem()"));

    /* R[ho */
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
    rec = monthRec->rec[day][no];

    /* Nbv{[hɃ}[N */
    clippedRecord = rec;
    clippedType = rec->type;

    /* ĕ`悵Ȃ */
    return false;
}

bool
Misc_PasteItem(uint16 date)
{
    Record *rec;
    int r;

    D(mgDPrintf("Misc_PasteItem()"));

    if (clippedRecord == NULL){
	return false;
    }

    D(mgDPrintf("allocate record"));

    /* R[h蓖 */
    rec = Record_AllocateRecord(1);
    if (rec == nil){
	return false;
    }

    /* NbvR[hRs[ */
    memcpy(rec, clippedRecord, sizeof(Record));
    rec->len = sizeof(Record);
    rec->descript = Record_AllocateByte(clippedRecord->descript_len);
    if (rec->descript == nil){
	return false;
    }
    memcpy(rec->descript, clippedRecord->descript, rec->descript_len);
    if (clippedRecord->done_len > 0){
	rec->done = (uint16 *)Record_AllocateByte(clippedRecord->done_len);
	memcpy(rec->done, clippedRecord->done, rec->done_len);
    }
    if (clippedRecord->link_len > 0){
	rec->link = Record_AllocateByte(clippedRecord->link_len);
	memcpy(rec->link, clippedRecord->link, rec->link_len);
    }
    rec->type = clippedType;

    /* ӖRs[ */
    switch (rec->type){
	case RECORD_TYPE_SCHEDULE:
	rec->start_date = date;
	rec->end_date = date + (clippedRecord->end_date - clippedRecord->start_date);
	break;
	case RECORD_TYPE_HOLIDAY:
	rec->start_date = date;
	rec->end_date = date + (clippedRecord->end_date - clippedRecord->start_date);
	break;
	case RECORD_TYPE_TASK:
	rec->start_date = date;
	break;
    }
    if (rec->repeat_type != REPEAT_TYPE_NONE){
	uint16 year, month, day, dayWeek, nthWeek, numOfDays;

	Calendar_GetDateByNo(date, &year, &month, &day);
	dayWeek = Calendar_GetDayOfWeekByNo(date);
	nthWeek = (day / 7) + ((day%7)?1:0);
	numOfDays = Calendar_GetNumOfDays(year, month);
	if (numOfDays - day < 7){
	    nthWeek = 5;
	}
	switch (rec->repeat_type){
	    case REPEAT_TYPE_WEEKLY:
	    rec->repeat_arg1 = gDayOfWeekBit[dayWeek];
	    break;
	    case REPEAT_TYPE_MONTHLYWEEK:
	    case REPEAT_TYPE_YEARLYWEEK:
	    rec->repeat_arg1 = gNthWeekBit[nthWeek];
	    rec->repeat_arg2 = gDayOfWeekBit[dayWeek];
	    break;
	}
    }

    /* R[h̓o^ */
    r = Record_Register(rec);
    if (!r){
	D(mgDPrintf("Record_Register failed"));
	return false;
    }
    gDBDirty = true;

    D(mgDPrintf("Record_Register success"));

    /* LbV̖ */
    MonthRec_Invalidate();

    return true;
}
