/*
   week view (large size)
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewwl.c,v 1.1.1.1 1998/12/15 03:42:49 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "taskid.h"
#include "view.h"
#include "viewwl.h"
#include "viewitem.h"
#include "viewtime.h"
#include "additem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>

/* -- file local macros */
#define HOURCELL_ROWS		7
#define HOURCELL_COLS		7
#define HOURCELL_WIDTH		(16*5+6)
#define HOURCELL_HEIGHT		(16*1+1)
#define ITEM_PER_DAY		3

#define REGION_HOUR_WIDTH	(6*5+2+6)
#define REGION_HOUR_HEIGHT	(HOURCELL_HEIGHT*HOURCELL_ROWS)
#define REGION_DAYWEEK_WIDTH	(HOURCELL_WIDTH*HOURCELL_COLS)
#define REGION_DAYWEEK_HEIGHT	(16*1+4)
#define REGION_TIMETABLE_WIDTH	(HOURCELL_WIDTH*HOURCELL_COLS)
#define REGION_TIMETABLE_HEIGHT	(HOURCELL_HEIGHT*HOURCELL_ROWS)
#define REGION_DAY_WIDTH	(HOURCELL_WIDTH*HOURCELL_COLS)
#define REGION_DAY_HEIGHT	(62)

/* -- file local variables */

/* -- file local methods */

/* -- methods */

/* ViewWL */
bool
ViewWL_Create(void)
{
    D(mgDPrintf("ViewWL create\n"));

    cursor_focus = FOCUS_TIMETABLE;
    cursor_prev_focus = FOCUS_TIMETABLE;
    ViewItem_InitCursor();
    ViewTime_Init(REGION_USER_LEFT, REGION_USER_TOP,
		  REGION_HOUR_WIDTH, REGION_HOUR_HEIGHT,
		  REGION_DAYWEEK_WIDTH, REGION_DAYWEEK_HEIGHT,
		  REGION_TIMETABLE_WIDTH, REGION_TIMETABLE_HEIGHT,
		  REGION_DAY_WIDTH, REGION_DAY_HEIGHT,
		  HOURCELL_WIDTH, HOURCELL_HEIGHT,
		  HOURCELL_COLS, HOURCELL_ROWS,
		  ITEM_PER_DAY,
		  F16_FNT, 10);
    ViewTime_InitCursor();

    return true;
}

/* ViewWLЕt */
bool
ViewWL_Destroy(void)
{
    D(mgDPrintf("ViewWL destroy\n"));
    return true;
}

/* ViewWLĕ` */
void
ViewWL_Redraw(void)
{
    HDC hDC;
    PAINTSTRUCT ps;

    hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
    ViewTimeHour_Redraw(hDC);
    ViewTimeWeek_Redraw(hDC);
    ViewTime_InvertCursor(hDC);		      /* ON */
    mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
}

/* ViewWLxĕ` */
void
ViewWL_DelayedRedraw(void)
{
    /* NOTHING TO DO */
}

/* L[R[h */
bool
ViewWL_KeyDown(WPARAM wParam, LPARAM lParam)
{
    HDC hDC;
    PAINTSTRUCT ps;

    switch (wParam){
	case VK_LEFT:			      /*  */
	case UVK_LEFT:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	gCursorDate--;
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_Left(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_RIGHT:			      /* E */
	case UVK_RIGHT:			      /* E */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	gCursorDate++;
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_Right(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_PRIOR:			      /* O */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_WeekUp(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_NEXT:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_WeekDown(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	/* xAbvf[g */
	View_SetTimer();
	break;

	case VK_UP:			      /*  */
	case UVK_UP:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* J[\ړ */
	if (gCursorHour > 0){
	    gCursorHour--;
	}
	ViewTime_Up(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case VK_DOWN:			      /*  */
	case UVK_DOWN:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* J[\ړ */
	if (gCursorHour < 23){
	    gCursorHour++;
	}
	ViewTime_Down(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case UVK_HOURUP:		      /* ԏ */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\k */
	if (gCursorHourRange > 1){
	    gCursorHourRange--;
	}else{
	    if (gCursorHour > 0){
		gCursorHour--;
	    }
	}
	ViewTime_Up(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case UVK_HOURDOWN:		      /* ԉ */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\g */
	if (gCursorHour + gCursorHourRange < 24){
	    gCursorHourRange++;
	}
	ViewTime_Down(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}

/* R}h */
bool
ViewWL_Command(WPARAM wParam, LPARAM lParam)
{
    switch (wParam){
	case ID_FUNC_NOW:
	if (HIWORD(lParam) == BN_CLICKED){
	    ViewTime_InitCursor();
	    View_Redraw();
	}else{
	    return false;
	}
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}
