/*
   file
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: file.c,v 1.2 1998/12/27 16:26:20 furukawa Exp $
   */

#include "global.h"
#include "taskmg.h"
#include "taskid.h"
#include "file.h"
#include "info.h"
#include "record.h"
#include "month.h"
#include "view.h"
#include "index.h"
#include "db.h"
#include "gendlg.h"
#include "calendar.h"
#include "context.h"
#include <string.h>
#include <stdio.h>
#include <unistd.h>

/* -- file local variables */

/* -- file local methods */
static bool
File_CSVWriteItem(FILE *fp, Record *rec);
static bool
File_EUCConvertItem(Record *rec);

/* -- methods */

/* ǡ١¸ */
bool
File_Save(void)
{
    D(mnwDPrintf("File_Save()"));

    if (gDBDirty){
	File_MakeBackup();
	RecordFile_Save();
	gDBDirty = false;
    }
    Info_MsgBox("ե¸ޤ", kMsgBox_Type_OK);
    return true;
}

/* ǡ١ΥХååץե */
bool
File_MakeBackup(void)
{
    char backup_path[256];

    if (!gDBMadeBackup){
	D(mnwDPrintf("File_MakeBackup(): do backup"));
	sprintf(backup_path, "%s.bak", DB_GetFileName());
	if (rename(DB_GetFileName(), backup_path) < 0){
	    return false;
	}
	gDBMadeBackup = true;
    }
    return true;
}

/* ǡ١Dirtyե饰򤿤Ƥ */
bool
File_SetDBDirty(void)
{
    if (gOpt_SaveDBOnDirty){
	File_MakeBackup();
	RecordFile_Save();
	gDBDirty = false;
    }else{
	gDBDirty = true;
    }
    return true;
}

/* ֻκ */
bool
File_DeleteRegion(void)
{
#ifdef TMP
    bool r;
    uint16 year, month, day, date;
    char msg[80];
    Record *rec;
    uint16 cookie;

    D(mnwDPrintf("File_DeleteRegion()"));

    GenDlg_SetTitle("()");

    r = mgDialogBoxParam(ghTaskInstance,
			 (LPCSTR)"GenDlg",
			 win,
			 (DLGPROC)GenDlgWinProc,
			 (LPARAM)MAKELPARAM(0, GENDLG_DATE));
    if (!r){
	return false;
    }

    date = GenDlg_GetDate();
    Calendar_GetDateByNo(date, &year, &month, &day);

    sprintf(msg, "%dǯ%d%d\nǡõޤ", year, month, day);
    r = (bool)mgMessageBox(win, msg, MB_OKCANCEL);
    if (!r){
	return false;
    }

    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	if (rec->end_date <= date
	    || (rec->type == RECORD_TYPE_TASK && rec->done_len > 0
		&& rec->done[0] <= date)){
	    D(mnwDPrintf("delete %u", rec->start_date));
	    Record_Delete(rec);
	    /* TODO: ֤ͤΥå */
	}
    }
    
    /* å̵ */
    MonthRec_Invalidate();
    View_Redraw();

    mgMessageBox(win, "ǡõޤ", MB_OK);

    File_SetDBDirty();
#endif
    return true;
}

/* ƥCSV񤭤 */
static bool
File_CSVWriteItem(FILE *fp, Record *rec)
{
    uint16 year, month, day, dayWeek, nthWeek, numOfDays;
    char *kanjiNum[] = {
	"", "", "", "", "", "",
    };
    int len;
    byte *p;

    D(mnwDPrintf("File_CSVWriteItem()"));

    switch (rec->type){
	case RECORD_TYPE_TASK:
	/* 1 */
	fprintf(fp, "TASK,");
	/* 2 */
	fprintf(fp, "\"%s\",", Context_GetLabelName(rec->label));
	/* 3 */
	fprintf(fp, "\"");
	len = rec->descript_len;
	p = rec->descript;
	while (len-- > 0){
	    if (*p == '\n'){
		fprintf(fp, "\\n");
	    }else{
		fwrite(p, 1, 1, fp);
	    }
	    p++;
	}
	fprintf(fp, "\",");
	/* 4 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 5 */
	fprintf(fp, ",");
	/* 6 */
	fprintf(fp, ",");
	/* 7 */
	fprintf(fp, "%d,", rec->u.task.rank);
	/* 8 */
	fprintf(fp, "%s,", ((rec->done_len > 0)?"λ":"̤λ"));
	/* 9 */
	fprintf(fp, "\"");
	if (rec->link_len > 0){
	    fprintf(fp, "file:/");
	    fwrite(rec->link, rec->link_len, 1, fp);
	}
	fprintf(fp, "\"\n");
	break;

	case RECORD_TYPE_SCHEDULE:
	/* 1 */
	fprintf(fp, "SCHEDULE,");
	/* 2 */
	fprintf(fp, "\"%s\",", Context_GetLabelName(rec->label));
	/* 3 */
	fprintf(fp, "\"");
	len = rec->descript_len;
	p = rec->descript;
	while (len-- > 0){
	    if (*p == '\n'){
		fprintf(fp, "\\n");
	    }else{
		fwrite(p, 1, 1, fp);
	    }
	    p++;
	}
	fprintf(fp, "\",");
	/* 4 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 5 */
	Calendar_GetDateByNo(rec->end_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 6 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	dayWeek = Calendar_GetDayOfWeekByNo(rec->start_date);
	nthWeek = (day / 7)+((day%7)?1:0);
	numOfDays = Calendar_GetNumOfDays(year, month);
	switch (rec->repeat_type){
	    case REPEAT_TYPE_NONE:
	    fprintf(fp, ",");
	    break;
	    case REPEAT_TYPE_DAILY:
	    if (rec->repeat_every == 1){
		fprintf(fp, ",");
	    }else{
		fprintf(fp, "%d,", rec->repeat_every);
	    }
	    break;
	    case REPEAT_TYPE_WEEKLY:
	    if (rec->repeat_every == 1){
		fprintf(fp, "轵%s,", nameDayWeek[dayWeek]);
	    }else if (rec->repeat_every == 2){
		fprintf(fp, "ֽ%s,", nameDayWeek[dayWeek]);
	    }else{
		fprintf(fp, "%d%s,", rec->repeat_every, nameDayWeek[dayWeek]);
	    }
	    break;
	    case REPEAT_TYPE_MONTHLYDAY:
	    if (rec->repeat_every == 1){
		fprintf(fp, "%d,", day);
	    }else if (rec->repeat_every == 2){
		fprintf(fp, "ַ%d,", day);
	    }else{
		fprintf(fp, "%d%d,", rec->repeat_every, day);
	    }
	    break;
	    case REPEAT_TYPE_MONTHLYWEEK:
	    if (numOfDays-day >= 7){
		if (rec->repeat_every == 1){
		    fprintf(fp, "%s%s,", kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}else if (rec->repeat_every == 2){
		    fprintf(fp, "ַ%s%s,", kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%d%s%s,", rec->repeat_every, kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}
	    }else{
		if (rec->repeat_every == 1){
		    fprintf(fp, "ǽ%s,", nameDayWeek[dayWeek]);
		}else if (rec->repeat_every == 2){
		    fprintf(fp, "ַǽ%s,", nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%dȺǽ%s,", rec->repeat_every, nameDayWeek[dayWeek]);
		}
	    }
	    break;
	    case REPEAT_TYPE_YEARLYDAY:
	    if (rec->repeat_every == 1){
		fprintf(fp, "ǯ%d%d,", month, day);
	    }else{
		fprintf(fp, "%dǯ%d%d,", rec->repeat_every, month, day);
	    }
	    break;
	    case REPEAT_TYPE_YEARLYWEEK:
	    if (numOfDays-day >= 7){
		if (rec->repeat_every == 1){
		    fprintf(fp, "ǯ%d%s%s,", month, kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%dǯ%d%s%s,", rec->repeat_every, month, kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}
	    }else{
		if (rec->repeat_every == 1){
		    fprintf(fp, "ǯ%dǽ%s,", month, nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%dǯ%dǽ%s,", rec->repeat_every, month, nameDayWeek[dayWeek]);
		}
	    }
	}
	/* 7 */
	fprintf(fp, "%02d:%02d,",
		rec->u.sche.start_time_h,
		rec->u.sche.start_time_m);
	/* 8 */
	fprintf(fp, "%02d:%02d,",
		rec->u.sche.end_time_h,
		rec->u.sche.end_time_m);
	/* 9 */
	fprintf(fp, "\"");
	if (rec->link_len > 0){
	    fprintf(fp, "file:/");
	    fwrite(rec->link, rec->link_len, 1, fp);
	}
	fprintf(fp, "\"\n");
	break;

	case RECORD_TYPE_HOLIDAY:
	/* 1 */
	fprintf(fp, "HOLIDAY,");
	/* 2 */
	fprintf(fp, "\"%s\",", Context_GetLabelName(rec->label));
	/* 3 */
	fprintf(fp, "\"");
	len = rec->descript_len;
	p = rec->descript;
	while (len-- > 0){
	    if (*p == '\n'){
		fprintf(fp, "\\n");
	    }else{
		fwrite(p, 1, 1, fp);
	    }
	    p++;
	}
	fprintf(fp, "\",");
	/* 4 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 5 */
	Calendar_GetDateByNo(rec->end_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 6 */
	fprintf(fp, ",");
	/* 7 */
	fprintf(fp, ",");
	/* 8 */
	fprintf(fp, ",");
	/* 9 */
	fprintf(fp, "\"");
	if (rec->link_len > 0){
	    fprintf(fp, "file:/");
	    fwrite(rec->link, rec->link_len, 1, fp);
	}
	fprintf(fp, "\"\n");
	break;
    }
    return true;
}

/* ֻCSV񤭤 */
bool
File_CSVWriteRegion(void)
{
#ifdef TMP
    bool r;
    uint16 year, month, day, date;
    char msg[80];
    Record *rec;
    uint16 cookie;
    FILE *fp;

    D(mnwDPrintf("File_CSVWriteRegion()"));

    GenDlg_SetTitle("񤭤 ");

    r = mgDialogBoxParam(ghTaskInstance,
			 (LPCSTR)"GenDlg",
			 win,
			 (DLGPROC)GenDlgWinProc,
			 (LPARAM)MAKELPARAM(0, GENDLG_DATE));
    if (!r){
	return false;
    }

    date = GenDlg_GetDate();
    Calendar_GetDateByNo(date, &year, &month, &day);

    sprintf(msg, "%dǯ%d%d\nǡ񤭽Фޤ", year, month, day);
    r = (bool)mgMessageBox(win, msg, MB_OKCANCEL);
    if (!r){
	return false;
    }

    fp = fopen(TASK_CSV_FILE, OPENMODE_WRITE_TEXT);
    if (fp == NULL){
	mgMessageBox(win, "Ԥޤ", MB_OK);
	return false;
    }

    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	if (rec->end_date <= date
	    || (rec->type == RECORD_TYPE_TASK && rec->done_len > 0
		&& rec->done[0] <= date)){
	    r = File_CSVWriteItem(fp, rec);
	    if (!r){
		mgMessageBox(win, "Ԥޤ", MB_OK);
		fclose(fp);
		return false;
	    }
	}
    }
    
    fclose(fp);

    mgMessageBox(win, "񤭽Фޤ\nУåեߤ\nԤäƤ(task.csv)", MB_OK);
#endif
    return true;
}

/* ƥCSV񤭤 */
bool
File_CSVWriteAll(void)
{
    bool r;
    char msg[80];
    Record *rec;
    uint16 cookie;
    FILE *fp;

    D(mnwDPrintf("File_CSVWriteAll()"));

    sprintf(msg, "CSVǽ񤭽Фޤ");
    r = (bool)Info_MsgBox(msg, kMsgBox_Type_OKCANCEL);
    if (r != kMsgBox_OK){
	return false;
    }

    fp = fopen(TASK_CSV_FILE, OPENMODE_WRITE_TEXT);
    if (fp == NULL){
	Info_MsgBox("Ԥޤ", kMsgBox_Type_OK);
	return false;
    }

    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	r = File_CSVWriteItem(fp, rec);
	if (!r){
	    Info_MsgBox("Ԥޤ", kMsgBox_Type_OK);
	    fclose(fp);
	    return false;
	}
    }

    fclose(fp);
    
    Info_MsgBox("񤭽Фޤ", kMsgBox_Type_OK);
    return true;
}

/* sjisΣХܤ */
static bool
is_sjis_first_byte(byte c)
{
    return (c > 127);			      /* TODO: ޤȤˤ */
}

/* sjiseucؤѴʾ񤭡 */
static void
convert_sjis_to_euc_overwrite(byte *p1, byte *p2)
{
    byte c1 = *p1;
    byte c2 = *p2;
    int adjust = c2 < 159;
    int rowOffset = c1 < 160 ? 112 : 176;
    int cellOffset = adjust ? (c2 > 127 ? 32 : 31) : 126;
    *p1 = ((c1 - rowOffset) << 1) - adjust;
    *p2 -= cellOffset;
    *p1 += 128;
    *p2 += 128;
}

/* ƥEUCѴ */
static bool
File_EUCConvertItem(Record *rec)
{
    int len;
    byte *p;

    D(mnwDPrintf("File_EUCConvertItem()"));

    len = rec->descript_len;
    p = rec->descript;
    while (len > 0){
	if (is_sjis_first_byte(*p)){
	    convert_sjis_to_euc_overwrite(p, p+1);
	    p += 2;
	    len -= 2;
	}else{
	    p++;
	    len--;
	}
    }

    return true;
}

/* ƥEUC񤭤 */
bool
File_EUCConvertAll(void)
{
    bool r;
    char msg[80];
    Record *rec;
    uint16 cookie;

    D(mnwDPrintf("File_EUCConvertAll()"));

    sprintf(msg, "EUCɤѴޤ");
    r = (bool)Info_MsgBox(msg, kMsgBox_Type_OKCANCEL);
    if (r != kMsgBox_OK){
	return false;
    }

    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	r = File_EUCConvertItem(rec);
	if (!r){
	    Info_MsgBox("Ԥޤ", kMsgBox_Type_OK);
	    return false;
	}
    }
    File_SetDBDirty();

    Info_MsgBox("Ѵޤ", kMsgBox_Type_OK);
    return true;
}

