/*
   find & jump
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: jump.c,v 1.2 1998/12/27 16:26:21 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "taskid.h"
#include "taskmg.h"
#include "gendlg.h"
#include "view.h"
#include "calendar.h"
#include "record.h"
#include "index.h"
#include "db.h"
#include "funckey.h"
#include "info.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* -- file local macros */

#define REGION_DIALOG_WIDTH	160
#define REGION_DIALOG_HEIGHT	70
#define REGION_DIALOG_LEFT	((640-REGION_DIALOG_WIDTH)/2)
#define REGION_DIALOG_TOP	((240-REGION_DIALOG_HEIGHT)/2)

/* -- file local variables */
static char findString[80];
static int findStringLen;
static uint16 findCookie = 0;

/* -- file local methods */
static bool
Jump_FindString(void);
static uint16
Jump_FindFirstOccrence(Record *rec);

/* -- methods */

/*  */
bool
Jump_Find(mnwWindow *self)
{
    mnwRect r;
    mnwEvent e;
    bool ret;

    D(mnwDPrintf("Jump_Find()"));

    GenDlg_SetTitle("");
    GenDlg_SetMode(GENDLG_STRING);

    /*  */
    mnwRect_SetXYWH(&r,
		    REGION_DIALOG_LEFT, REGION_DIALOG_TOP,
		    REGION_DIALOG_WIDTH, REGION_DIALOG_HEIGHT);
    mnwWindow_Register((mnwWindow *)&gGenDialog,
		       (mnwWindowCallback)GenDlg_Callback,
		       (mnwWindow *)gRootWindow, &r);

    /* ե */
    mnwEvent_SetFocusWindow((mnwWindow *)&gGenDialog);

    /* modal dialog */
    mnwEvent_SetAll(&e, kEvent_DoModal, nil, 0, 0);
    ret = mnwEvent_SendToDirect(&e, (mnwWindow *)&gGenDialog);

    /* ξõ */
    mnwWindow_UnRegister((mnwWindow *)&gGenDialog);
    mnwEvent_SetAll(&e, kEvent_Redraw, nil, 0, 0);
    mnwEvent_SendTo(&e, gMainWin);

    if (ret){
	D(mnwDPrintf("Jump_Find() -- got %s", GenDlg_GetString()));
	sprintf(findString, GenDlg_GetString());
	findStringLen = strlen(findString);
    }else{
	D(mnwDPrintf("Jump_Find() -- dialog failure"));
	return false;
    }

    /* Ƭ饵 */
    findCookie = Index_Rewind();
    ret = Jump_FindString();
    if (ret){
	return true;
    }else{
	Info_MsgBox("ĤޤǤ", kMsgBox_Type_OK);
	findCookie = Index_Rewind();
	return false;
    }
}

/*  */
bool
Jump_FindNext(mnwWindow *self)
{
    bool ret;

    D(mnwDPrintf("Jump_FindNext()"));

    if (strlen(findString) == 0 || findCookie == 0){
	return false;
    }

    /* ³饵 */
    ret = Jump_FindString();
    if (ret){
	return true;
    }else{
	Info_MsgBox("ĤޤǤ", kMsgBox_Type_OK);
	findCookie = Index_Rewind();
	return false;
    }
}

/* ƸĤä饸 */
static bool
Jump_FindString(void)
{
    Record *rec;
    int i;
    uint16 date;

    while (nil != (rec = Index_FindNext(&findCookie))){
	if (rec->type == RECORD_TYPE_DELETED){
	    /* õƤ쥳ɤ̵ */
	    continue;
	}
	if (gLabel != 0 && rec->label != 0
	    && rec->label != gLabel){
	    /* ٥뤬ʤ */
	    continue;
	}
	if (findStringLen > (int)rec->descript_len){
	    continue;
	}
	for (i = 0; i <= ((int)rec->descript_len - findStringLen); i++){
	    if (0 == memcmp((rec->descript)+i, findString, findStringLen)){
		/* Ĥä */
		date = Jump_FindFirstOccrence(rec);
		D(mnwDPrintf("found %s (continue:%u,date:%u)",
			     findString, findCookie-1, date));
		if (date > 0){
		    View_Destroy();
		    gCursorDate = date;
		    View_Create();
		    View_Redraw();
		    return true;
		}
	    }
	}
    }

    /* Ĥʤä */
    return false;
}

/* ǽνиõ */
static uint16
Jump_FindFirstOccrence(Record *rec)
{
    uint16 year, month, day, date;
    uint16 monthNo;
    uint8 dayOfWeek, dayOfWeekBit;
    uint8 nthWeek, nthWeekBit;

    /* ξ */
    if (rec->type == RECORD_TYPE_TASK){
	if (rec->done_len > 0){
	    return rec->done[0];
	}else{
	    return rec->start_date;
	}
    }

    /* ֤ʤ */
    if (rec->repeat_type == REPEAT_TYPE_NONE){
	return rec->start_date;
    }
    
    /* ֤ */
    for (date = rec->start_date; date <= rec->end_date; date++){
	/* ɬפ */
	Calendar_GetDateByNo(date, &year, &month, &day);
	dayOfWeek = Calendar_GetDayOfWeekByNo(date);
	dayOfWeekBit = gDayOfWeekBit[dayOfWeek];
	nthWeek = (day / 7) + ((day%7)?1:0);
	nthWeekBit = gNthWeekBit[nthWeek];
	monthNo = Calendar_GetMonthNo(year, month);
	/* NOTE: 󤳤׻Τ٤ */

	/* ֤äƤ뤫Ƚꤹ */
	switch (rec->repeat_type){
	    case REPEAT_TYPE_DAILY:
	    if ((date - rec->start_date) % rec->repeat_every == 0){
		return date;
	    }
	    break;

	    case REPEAT_TYPE_WEEKLY:
	    if (((date/7 - rec->start_date/7) % rec->repeat_every == 0)
		&& (dayOfWeekBit & rec->repeat_arg1)){
		return date;
	    }
	    break;

	    case REPEAT_TYPE_MONTHLYDAY:
	    if (((monthNo - rec->start_monthNo) % rec->repeat_every == 0)
		&& (day == rec->start_day)){
		return date;
	    }
	    break;

	    case REPEAT_TYPE_MONTHLYWEEK:
	    if (((monthNo - rec->start_monthNo) % rec->repeat_every == 0)
		&& (nthWeekBit & rec->repeat_arg1)
		&& (dayOfWeekBit & rec->repeat_arg2)){
		return date;
	    }
	    break;

	    case REPEAT_TYPE_YEARLYDAY:
	    if (((monthNo/12 - rec->start_monthNo/12) % rec->repeat_every == 0)
		&& (month == rec->start_month)
		&& (day == rec->start_day)){
		return date;
	    }
	    break;

	    case REPEAT_TYPE_YEARLYWEEK:
	    if (((monthNo/12 - rec->start_monthNo/12) % rec->repeat_every == 0)
		&& (month == rec->start_month)
		&& (nthWeekBit & rec->repeat_arg1)
		&& (dayOfWeekBit & rec->repeat_arg2)){
		return date;
	    }
	    break;
	    
	    default:
	    panic("MonthRec_GetByMonthNo(): Invalid repeat attribute");
	    break;
	}
    }
	
    return 0;
    /* NOTE: month.cȤۤƱ */
}

/*  */
bool
Jump_Jump(mnwWindow *self)
{
    mnwRect r;
    mnwEvent e;
    bool ret;

    D(mnwDPrintf("Jump_Jump()"));

    GenDlg_SetTitle("˥");
    GenDlg_SetMode(GENDLG_DATE);

    /*  */
    mnwRect_SetXYWH(&r,
		    REGION_DIALOG_LEFT, REGION_DIALOG_TOP,
		    REGION_DIALOG_WIDTH, REGION_DIALOG_HEIGHT);
    mnwWindow_Register((mnwWindow *)&gGenDialog,
		       (mnwWindowCallback)GenDlg_Callback,
		       (mnwWindow *)gRootWindow, &r);

    /* ե */
    mnwEvent_SetFocusWindow((mnwWindow *)&gGenDialog);

    /* modal dialog */
    mnwEvent_SetAll(&e, kEvent_DoModal, nil, 0, 0);
    ret = mnwEvent_SendToDirect(&e, (mnwWindow *)&gGenDialog);

    /* ξõ */
    mnwWindow_UnRegister((mnwWindow *)&gGenDialog);
    mnwEvent_SetAll(&e, kEvent_Redraw, nil, 0, 0);
    mnwEvent_SendTo(&e, gMainWin);

    if (ret){
	View_Destroy();
	gCursorDate = GenDlg_GetDate();
	View_Create();
	View_Redraw();
	return true;
    }else{
	return false;
    }
}

bool
Jump_Command(mnwWindow *self, mnwEvent *event)
{
    switch (event->param1){
    case ID_FUNC_JUMP:
    case ID_MENUITEM_JUMP:
	Jump_Jump(self);
	return true;
	break;
    case ID_FUNC_FIND:
    case ID_MENUITEM_FIND:
	Jump_Find(self);
	return true;
	break;
    case ID_FUNC_FINDNEXT:
    case ID_MENUITEM_FINDNEXT:
	Jump_FindNext(self);
	return true;
	break;
    }
    return false;
}
