/*
   view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: view.c,v 1.2 1998/12/27 16:26:22 furukawa Exp $
   */

#define VIEW
#include "global.h"
#include "taskid.h"
#include "taskmg.h"
#include "view.h"
#include "view3m.h"
#include "view5w.h"
#include "view3w.h"
#include "view2w.h"
#include "viewdl.h"
#include "viewws.h"
#include "viewwl.h"
#include "viewitem.h"
#include "viewtime.h"
#include "viewcal.h"
#include "additem.h"
#include "gendlg.h"
#include "calendar.h"
#include "month.h"
#include "info.h"
#include "file.h"
#include "jump.h"
#include "db.h"
#include "context.h"
#include "util.h"
#include <stdio.h>

/* -- file local macros */

#define DELAYED_UPDATE_DELAY	2
#define VIEW_SHORTCUT

/* -- file local variables */
static uint16 view_timer = 0;

/* -- file local methods */
static bool
View_ChangeSize(void);
bool
AddDlg_ToggleDoneItem(uint16 date, int no);

/* -- methods */

/* View */
bool
View_Create(void)
{
    view_timer = 0;

    switch (gCurrentView){
	case VIEW_3MONTH:
	return View3M_Create();
	break;
	case VIEW_5WEEK:
	return View5W_Create();
	break;
	case VIEW_3WEEK:
	return View3W_Create();
	break;
	case VIEW_2WEEK:
	return View2W_Create();
	break;
	case VIEW_WEEK_S:
	return ViewWS_Create();
	break;
	case VIEW_WEEK_L:
	return ViewWL_Create();
	break;
	case VIEW_DAY_L:
	return ViewDL_Create();
	break;
    }
    return true;
}

/* Viewդ */
bool
View_Destroy(void)
{
    mnwRect rect;
    switch (gCurrentView){
	case VIEW_3MONTH:
	View3M_Destroy();
	break;
	case VIEW_5WEEK:
	View5W_Destroy();
	break;
	case VIEW_3WEEK:
	View3W_Destroy();
	break;
	case VIEW_2WEEK:
	View2W_Destroy();
	break;
	case VIEW_WEEK_S:
	ViewWS_Destroy();
	break;
	case VIEW_WEEK_L:
	ViewWL_Destroy();
	break;
	case VIEW_DAY_L:
	ViewDL_Destroy();
	break;
    }

    /* ߤ̵ */
    /* TODO: should be invalidate */
    mnwSetForeColor(kColor_LtGray);
    mnwRect_SetXYXY(&rect,
		    REGION_USER_LEFT, REGION_USER_TOP,
		    REGION_USER_RIGHT, REGION_USER_BOTTOM);
    mnwFillRect(&rect);

    return true;
}

/* ȥι */
void
View_UpdateTitle(void)
{
    uint16 year, month, day, dayWeek;
    char str[80];

    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    dayWeek = Calendar_GetDayOfWeekByNo(gCursorDate);

    if (gLabel > 0){
	sprintf(str, "%4dǯ%2d%2d(%s) %8s",
		year, month, day, nameDayWeek[dayWeek],
		Context_GetLabelName(gLabel));
    }else{
	sprintf(str, "%4dǯ%2d%2d(%s)",
		year, month, day, nameDayWeek[dayWeek]);
    }
    MainWin_SetTitle(str);
}


/* View޽ */
bool
View_Timer(void)
{
    if (view_timer > 0){
	view_timer--;
	if (view_timer == 0 && gMode == MODE_VIEW){
	    View_UpdateTitle();
	    switch (gCurrentView){
		case VIEW_3MONTH:
		View3M_DelayedRedraw();
		return true;
		break;
		case VIEW_5WEEK:
		View5W_DelayedRedraw();
		return true;
		break;
		case VIEW_3WEEK:
		View3W_DelayedRedraw();
		return true;
		break;
		case VIEW_2WEEK:
		View2W_DelayedRedraw();
		return 0;
		break;
		case VIEW_WEEK_S:
		ViewWS_DelayedRedraw();
		return 0;
		break;
		case VIEW_WEEK_L:
		ViewWL_DelayedRedraw();
		return 0;
		case VIEW_DAY_L:
		ViewDL_DelayedRedraw();
		return 0;
		break;
	    }
	}
    }

    /* ʤ */
    return false;
}

/* View޽ */
bool
View_SetTimer(void)
{
    view_timer = DELAYED_UPDATE_DELAY;
    return true;
}

/* View */
void
View_Redraw(void)
{
    /*D(mnwDPrintf("View_Redraw():"));*/

    View_UpdateTitle();

    switch (gCurrentView){
	case VIEW_3MONTH:
	View3M_Redraw();
	break;
	case VIEW_5WEEK:
	View5W_Redraw();
	break;
	case VIEW_3WEEK:
	View3W_Redraw();
	break;
	case VIEW_2WEEK:
	View2W_Redraw();
	break;
	case VIEW_WEEK_S:
	ViewWS_Redraw();
	break;
	case VIEW_WEEK_L:
	ViewWL_Redraw();
	break;
	case VIEW_DAY_L:
	ViewDL_Redraw();
	break;
    }
}

/* ɽ */
bool
View_KeyDown(mnwEvent *event)
{
    bool r;
    mnwEvent e;

    switch (event->param1){
	case 'q':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* Q */
	    mnwEvent_SetAll(&e, kEvent_Destroy, nil, 0, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 's':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* S */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_ADD_SCHEDULE, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+S */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_MENUITEM_SAVE, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 't':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* T */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_ADD_TASK, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'h':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* H */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_ADD_HOLIDAY, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'l':
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+L */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_NOW, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'p':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* P */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_ITEMUP, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)
	    && mnwEvent_TestModifier(event, kKey_SHIFT)){ /* SC+P */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_HOURUP, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+P */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_UP, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'n':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* N */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_ITEMDOWN, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)
	    && mnwEvent_TestModifier(event, kKey_SHIFT)){ /* SC+N */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_HOURDOWN, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+N */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_DOWN, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'b':
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+B */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_LEFT, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'f':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* F */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_FIND, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+F */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_RIGHT, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'g':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* G */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_FINDNEXT, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'm':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* M */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_MAILER, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'x':
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+X */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_MENUITEM_CUT, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'c':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* C */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_MONTH, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+C */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_MENUITEM_COPY, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'v':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* V */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_MENUITEM_ABOUT, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+V */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_MENUITEM_PASTE, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'd':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* D */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_SIZE, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+D */
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kKey_DELETE, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
	case 'j':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* J */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_JUMP, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	if (mnwEvent_TestModifier(event, kKey_CONTROL)){ /* CTRL+J */
	    switch (gCurrentView){
		case VIEW_3MONTH:
		/* ľΥӥ塼Τ˰ܹ */
		View_Destroy();
		gCurrentView = gPreviousView;
		gPreviousView = VIEW_3MONTH;
		r = View_Create();
		View_Redraw();
		return r;
		break;
		/* ȥ */
		case VIEW_5WEEK:
		case VIEW_3WEEK:
		case VIEW_2WEEK:
		mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_WEEK, 0);
		mnwEvent_SendTo(&e, gMainWin);
		return true;
		break;
		case VIEW_WEEK_S:
		case VIEW_WEEK_L:
		mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_DAY, 0);
		mnwEvent_SendTo(&e, gMainWin);
		return true;
		break;
		case VIEW_DAY_L:
		mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_WEEKS, 0);
		mnwEvent_SendTo(&e, gMainWin);
		return true;
		break;
	    }
	}
	break;
	case kKey_RETURN:
	D(mnwDPrintf("got RETURN at Common View"));
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* RETURN */
	    if (cursor_focus == FOCUS_ITEMLIST){
		/* ιܤԽ */
		r = AddDlg_EditItem(gCursorDate, ViewItem_GetCursor());
		if (r){
		    ViewItem_ResetCursor();
		    View_Redraw();
		}
		return true;
	    }else{
		switch (gCurrentView){
		    case VIEW_3MONTH:
		    /* ľΥӥ塼Τ˰ܹ */
		    View_Destroy();
		    gCurrentView = gPreviousView;
		    gPreviousView = VIEW_3MONTH;
		    r = View_Create();
		    View_Redraw();
		    return r;
		    break;
		    /* ȥ */
		    case VIEW_5WEEK:
		    case VIEW_3WEEK:
		    case VIEW_2WEEK:
		    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_WEEK, 0);
		    mnwEvent_SendTo(&e, gMainWin);
		    return true;
		    break;
		    case VIEW_WEEK_S:
		    case VIEW_WEEK_L:
		    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_DAY, 0);
		    mnwEvent_SendTo(&e, gMainWin);
		    return true;
		    break;
		    case VIEW_DAY_L:
		    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_WEEKS, 0);
		    mnwEvent_SendTo(&e, gMainWin);
		    return true;
		    break;
		}
	    }
	}
	break;

	case kKey_INSERT:		      /* BS */ /* TODO: use bs key */
	D(mnwDPrintf("got BS at Common View"));
	switch (gCurrentView){
	    case VIEW_3MONTH:
	    /* ľΥӥ塼Τ˰ܹ */
	    View_Destroy();
	    gCurrentView = gPreviousView;
	    gPreviousView = VIEW_3MONTH;
	    r = View_Create();
	    View_Redraw();
	    return r;
	    break;

	    /* եȥ */
	    case VIEW_5WEEK:
	    case VIEW_3WEEK:
	    case VIEW_2WEEK:
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_DAY, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	    break;
	    case VIEW_WEEK_S:
	    case VIEW_WEEK_L:
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_WEEKS, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	    break;
	    case VIEW_DAY_L:
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_WEEK, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	    break;
	}
	break;

	case kKey_ESCAPE:			      /* ESC */
	D(mnwDPrintf("got ESC at Common View"));
	/* TODO: */
	return true;
	break;

	case kKey_DELETE:			      /* DEL */
	D(mnwDPrintf("got DEL at View"));
	mnwEvent_SetAll(&e, kEvent_Command, nil, ID_MENUITEM_DELETE, 0);
	mnwEvent_SendTo(&e, gMainWin);
	return true;
	break;

	case kKey_SPACE:			      /* SPACE */
	D(mnwDPrintf("got SPACE at View"));
	if (cursor_focus == FOCUS_ITEMLIST){
	    /* λ̤λȥ */
	    r = AddDlg_ToggleDoneItem(gCursorDate, ViewItem_GetCursor());
	    if (r){
		ViewItem_ResetCursor();
		View_Redraw();
	    }
	}else{
	    /* ⤷ʤ */
	    return false;
	}
	break;

	case kKey_UP:
	if (mnwEvent_TestModifier(event, kKey_SHIFT)){
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_HOURUP, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}else if (mnwEvent_TestModifier(event, kKey_CONTROL)){
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_ITEMUP, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	/* դĤ̤ */
	break;
	
	case kKey_DOWN:
	if (mnwEvent_TestModifier(event, kKey_SHIFT)){
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_HOURDOWN, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}else if (mnwEvent_TestModifier(event, kKey_CONTROL)){
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_ITEMDOWN, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	/* դĤ̤ */
	break;

	case kKey_PRIOR:
	if (gOpt_SelectItemByPageUpDown){
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_ITEMUP, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	/* դĤ̤ */
	break;
	
	case kKey_NEXT:
	if (gOpt_SelectItemByPageUpDown){
	    mnwEvent_SetAll(&e, kEvent_KeyDown, nil, kUKey_ITEMDOWN, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	/* դĤ̤ */
	break;

    	case '1':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* 1 */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_SEL_LABEL1, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
    	case '2':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* 2 */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_SEL_LABEL2, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
    	case '3':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* 3 */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_SEL_LABEL3, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
    	case '4':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* 4 */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_SEL_LABEL4, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
    	case '9':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* 9 */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_SEL_LABEL_CUST, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
    	case '0':
	if (!mnwEvent_TestModifier(event, kKey_CONTROL)
	    && !mnwEvent_TestModifier(event, kKey_SHIFT)){ /* 0 */
	    mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_SEL_LABEL0, 0);
	    mnwEvent_SendTo(&e, gMainWin);
	    return true;
	}
	break;
    }

    /* ƤʤΤcurrent viewdelegate */
    switch (gCurrentView){
	case VIEW_3MONTH:
	return View3M_KeyDown(event);
	break;
	case VIEW_5WEEK:
	return View5W_KeyDown(event);
	break;
	case VIEW_3WEEK:
	return View3W_KeyDown(event);
	break;
	case VIEW_2WEEK:
	return View2W_KeyDown(event);
	break;
	case VIEW_WEEK_S:
	return ViewWS_KeyDown(event);
	break;
	case VIEW_WEEK_L:
	return ViewWL_KeyDown(event);
	break;
	case VIEW_DAY_L:
	return ViewDL_KeyDown(event);
	break;
    }
    return 0;
}

/* ޥɽ */
bool
View_Command(mnwWindow *self, mnwEvent *event)
{
    int r = true;
    uint8 hour, min;

    switch (event->param1){
	case ID_MENUITEM_SAVE:
	File_Save();
	return true;
	break;
	case ID_MENUITEM_RENDEL:
	File_DeleteRegion();
	return true;
	break;
	case ID_MENUITEM_WRITE_CSV_REGION:
	File_CSVWriteRegion();
	break;
	case ID_MENUITEM_WRITE_CSV_ALL:
	File_CSVWriteAll();
	break;
	case ID_MENUITEM_OPEN:
	Info_NotImplemented();
	break;

	case ID_MENUITEM_DELETE:
	if (cursor_focus == FOCUS_ITEMLIST){
	    /* ǧ */
	    r = Info_MsgBox("򤵤줿ܤ\nޤ",
			  kMsgBox_Type_OKCANCEL);
	    if (r == kMsgBox_OK){
		/* ιܤõ */
		r = AddDlg_DeleteItem(gCursorDate, ViewItem_GetCursor());
		if (r){
		    ViewItem_ResetCursor();
		    View_Redraw();
		}
	    }
	    return true;
	}else{
	    /* ⤷ʤ */
	    return false;
	}
	break;
	case ID_MENUITEM_CUT:
	if (cursor_focus == FOCUS_ITEMLIST){
	    /* ιܤ򥫥å */
	    r = Misc_CutItem(gCursorDate, ViewItem_GetCursor());
	    if (r){
		ViewItem_ResetCursor();
		View_Redraw();
	    }
	}
	return true;
	break;
	case ID_MENUITEM_COPY:
	if (cursor_focus == FOCUS_ITEMLIST){
	    /* ιܤ򥳥ԡ */
	    r = Misc_CopyItem(gCursorDate, ViewItem_GetCursor());
	    if (r){
		ViewItem_ResetCursor();
		View_Redraw();
	    }
	}
	return true;
	break;
	case ID_MENUITEM_PASTE:
	/* ιܤڡ */
	r = Misc_PasteItem(gCursorDate);
	if (r){
	    ViewItem_ResetCursor();
	    View_Redraw();
	}
	return true;
	break;

	case ID_FUNC_ADD_SCHEDULE:
	case ID_MENUITEM_ADD_SCHEDULE:
	return AddDlg_AddItem(ID_FUNC_ADD_SCHEDULE);
	break;

	case ID_FUNC_ADD_TASK:
	case ID_MENUITEM_ADD_TASK:
	return AddDlg_AddItem(ID_FUNC_ADD_TASK);
	break;

	case ID_FUNC_ADD_HOLIDAY:
	case ID_MENUITEM_ADD_HOLIDAY:
	return AddDlg_AddItem(ID_FUNC_ADD_HOLIDAY);
	break;

	case ID_FUNC_JUMP:
	case ID_FUNC_FIND:
	case ID_MENUITEM_FIND:
	case ID_FUNC_FINDNEXT:
	case ID_MENUITEM_FINDNEXT:
	return Jump_Command(self, event);
	break;

	case ID_FUNC_NOW:
	gTodayDate = DB_GetDate(&gTodayYear, &gTodayMonth, &gTodayDay,
				&hour, &min);
	gCursorDate = gTodayDate;
	gCursorHour = hour;
	/* current viewή */
	break;

	case ID_FUNC_CONVERT_SJIS:
	    return File_EUCConvertAll();
	break;

	case ID_FUNC_SIZE:
	case ID_MENUITEM_SIZE:
	return View_ChangeSize();
	break;

	case ID_FUNC_MONTH:
	case ID_MENUITEM_MONTH:
	if (gCurrentView != gCurrentViewMonth){
	    View_Destroy();
	    gPreviousView = gCurrentView;
	    gCurrentView = gCurrentViewMonth;
	    r = View_Create();
	    View_Redraw();
	}
	return (bool)r;
	break;

	case ID_FUNC_WEEKS:
	case ID_MENUITEM_WEEKS:
	if (gCurrentView != gCurrentViewWeeks){
	    View_Destroy();
	    gPreviousView = gCurrentView;
	    gCurrentView = gCurrentViewWeeks;
	    r = View_Create();
	    View_Redraw();
	}
	return (bool)r;
	break;

	case ID_FUNC_WEEK:
	case ID_MENUITEM_WEEK:
	if (gCurrentView != gCurrentViewWeek){
	    View_Destroy();
	    gPreviousView = gCurrentView;
	    gCurrentView = gCurrentViewWeek;
	    r = View_Create();
	    View_Redraw();
	}
	return (bool)r;
	break;

	case ID_FUNC_DAY:
	case ID_MENUITEM_DAY:
	if (gCurrentView != gCurrentViewDay){
	    View_Destroy();
	    gPreviousView = gCurrentView;
	    gCurrentView = gCurrentViewDay;
	    r = View_Create();
	    View_Redraw();
	}
	return (bool)r;
	break;

	case ID_FUNC_LIST:
	case ID_MENUITEM_LIST:
	Info_NotImplemented();
	break;

	case ID_MENUITEM_ABOUT:
	Info_DisplayAbout();
	return true;
	break;

	case ID_FUNC_SEL_LABEL0:
	gLabel = 0;
	MonthRec_SelectLabel(gLabel);
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_ResetCursor();
	}
	View_Redraw();
	return true;
	break;
	case ID_FUNC_SEL_LABEL1:
	gLabel = 1;
	MonthRec_SelectLabel(gLabel);
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_ResetCursor();
	}
	View_Redraw();
	return true;
	break;
	case ID_FUNC_SEL_LABEL2:
	gLabel = 2;
	MonthRec_SelectLabel(gLabel);
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_ResetCursor();
	}
	View_Redraw();
	return true;
	break;
	case ID_FUNC_SEL_LABEL3:
	gLabel = 3;
	MonthRec_SelectLabel(gLabel);
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_ResetCursor();
	}
	View_Redraw();
	return true;
	break;
	case ID_FUNC_SEL_LABEL4:
	gLabel = 4;
	MonthRec_SelectLabel(gLabel);
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_ResetCursor();
	}
	View_Redraw();
	return true;
	break;
	case ID_FUNC_SEL_LABEL_CUST:
	gLabel = 5;
	Context_SelectCustomLabel();
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_ResetCursor();
	}
	View_Redraw();
	return true;
	break;
    }

    /* ƤʤΤcurrent viewdelegate */
    switch (gCurrentView){
	case VIEW_3MONTH:
	return View3M_Command(event);
	break;
	case VIEW_5WEEK:
	return View5W_Command(event);
	break;
	case VIEW_3WEEK:
	return View3W_Command(event);
	break;
	case VIEW_2WEEK:
	return View2W_Command(event);
	break;
	case VIEW_WEEK_S:
	return ViewWS_Command(event);
	break;
	case VIEW_WEEK_L:
	return ViewWL_Command(event);
	break;
	case VIEW_DAY_L:
	return ViewDL_Command(event);
	break;
    }
    return 0;
}

/* ѹ */
static bool
View_ChangeSize(void)
{
    bool r = true;

    /* Viewѹ */
    switch (gCurrentView){
	case VIEW_5WEEK:
	View_Destroy();
	gCurrentView = VIEW_3WEEK;
	gCurrentViewWeeks = VIEW_3WEEK;
	break;
	case VIEW_3WEEK:
	View_Destroy();
	gCurrentView = VIEW_2WEEK;
	gCurrentViewWeeks = VIEW_2WEEK;
	break;
	case VIEW_2WEEK:
	View_Destroy();
	gCurrentView = VIEW_5WEEK;
	gCurrentViewWeeks = VIEW_5WEEK;
	break;
	case VIEW_WEEK_S:
	View_Destroy();
	gCurrentView = VIEW_WEEK_L;
	gCurrentViewWeek = VIEW_WEEK_L;
	break;
	case VIEW_WEEK_L:
	View_Destroy();
	gCurrentView = VIEW_WEEK_S;
	gCurrentViewWeek = VIEW_WEEK_S;
	break;

	default:
	return false;
	break;
    }

    gPreviousView = gCurrentView;
    r = View_Create();
    View_Redraw();
    return r;
}

/* View˥ơ֥뤬뤫 */
bool
View_HasTimeTable(void)
{
    if (gCurrentView == VIEW_WEEK_S
	|| gCurrentView == VIEW_WEEK_L
	|| gCurrentView == VIEW_DAY_L){
	return true;
    }else{
	return false;
    }
}

#if 0
/* μ */
static char *
View_GetDialog(void)
{
    if (gCurrentView == VIEW_BROWSER){
	D(mnwDPrintf("use SMALL DIALOG"));
	return "AddItemS";
    }else if (gOpt_LargeDialog){
	D(mnwDPrintf("use LARGE DIALOG"));
	return "AddItemL";
    }else{
	D(mnwDPrintf("use NORMAL DIALOG"));
	return "AddItem";
    }
}
#endif
