/*
   calender region of view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewcal.c,v 1.2 1999/02/01 04:51:06 furukawa Exp $
   */

#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "record.h"
#include "view.h"
#include "viewitem.h"
#include "viewcal.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* -- file local macros */

/* -- file local variables */
static uint16 originDate;
static int cursorX, cursorY; 
static int region_dayweek_width, region_dayweek_height;
static int region_dayweek_left, region_dayweek_right;
static int region_dayweek_top, region_dayweek_bottom;
static int region_weeks_width, region_weeks_height;
static int region_weeks_left, region_weeks_right;
static int region_weeks_top, region_weeks_bottom;
static int cell_width, cell_height;
static int cell_rows, cell_cols;
static int cell_fontwidth, cell_fontheight, cell_font;
static int cell_maxlen;
static int day_rows;
static int day_margin_left, day_margin_top;
static int day_interval;
static int dayweek_fontwidth, dayweek_font;

/* -- file local methods */
static void
ViewCal_RedrawDay(int x, int y);

/* -- methods */

/* ViewCalν */
void
ViewCal_Init(int left, int top,
	     int dayweek_width, int dayweek_height,
	     int weeks_width, int weeks_height,
	     int c_width, int c_height,
	     int c_cols, int c_rows,
	     int d_rows,
	     int c_font, int c_maxlen)
{
    region_dayweek_width = dayweek_width;
    region_dayweek_height = dayweek_height;
    region_dayweek_left = left;
    region_dayweek_top = top;
    region_dayweek_right = region_dayweek_left + region_dayweek_width - 1;
    region_dayweek_bottom = region_dayweek_top + region_dayweek_height - 1;

    region_weeks_width = weeks_width;
    region_weeks_height = weeks_height;
    region_weeks_left = left;
    region_weeks_top = region_dayweek_top + region_dayweek_height;
    region_weeks_right = region_weeks_left + region_weeks_width - 1;
    region_weeks_bottom = region_weeks_top + region_weeks_height - 1;

    cell_width = c_width;
    cell_height = c_height;
    cell_cols = c_cols;
    cell_rows = c_rows;
    cell_font = c_font;
    cell_maxlen = c_maxlen;

    switch (cell_font){
	case kFont_10x8:
	cell_fontwidth = 5;
	cell_fontheight = 8;
	dayweek_font = kFont_12;
	dayweek_fontwidth = 6;
	break;
	case kFont_16x8:
	cell_fontwidth = 8;
	cell_fontheight = 8;
	dayweek_font = kFont_12;
	dayweek_fontwidth = 6;
	break;
	case kFont_16:
	cell_fontwidth = 8;
	cell_fontheight = 16;
	dayweek_font = kFont_16;
	dayweek_fontwidth = 8;
	break;
	case kFont_12:
	cell_fontwidth = 6;
	cell_fontheight = 12;
	dayweek_font = kFont_12;
	dayweek_fontwidth = 6;
	break;
    }

    day_rows = d_rows;
    day_margin_left = (cell_width - cell_fontwidth * cell_maxlen) / 2 + 1;
    day_margin_top = cell_fontheight + 1;
    day_interval = (cell_height - day_margin_top) / day_rows;
}

/* ν */
void
ViewCal_InitCursor(void)
{
    if (gOpt_WeeksStartsFromMonday){
	/* Ϸˤ */
	cursorX = (Calendar_GetDayOfWeekByNo(gCursorDate) + 7 - 1) % 7;
	cursorY = 0;
	originDate = gCursorDate - (uint16)cursorX;
    }else{
	/* ˤ */
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = 0;
	originDate = gCursorDate - (uint16)cursorX;
    }
}

/* κ */
void
ViewCalDayWeek_Redraw(void)
{
    int i;
    int left, top, right, bottom;
    mnwRect rect;

    top = region_dayweek_top;
    bottom = region_dayweek_bottom;

    for (i = 0; i < cell_cols; i++){
	left = region_dayweek_left + cell_width * i;
	right = left + cell_width - 1;

	/*  */
	mnwSetForeColor(kColor_Black);
	mnwDrawLine(left, top, left, bottom);

	/* ̥ܥ */
	mnwRect_SetXYXY(&rect, left+1, top, right, bottom);
	mnwSetForeColor(kColor_LtGray);
	mnwFillRect(&rect);
	mnwDrawFrame3D(&rect);

	/*  */
	mnwSetForeColor(kColor_Black);
	mnwSetFont(dayweek_font);
	if (gOpt_WeeksStartsFromMonday){
	    /* Ϸˤ */
	    mnwDrawText(left + (cell_width - dayweek_fontwidth*2) / 2, top+1,
		     nameDayWeek[(i+1)%7], 2);
	}else{
	    /* ˤ */
	    mnwDrawText(left + (cell_width - dayweek_fontwidth*2) / 2, top+1,
		     nameDayWeek[i], 2);
	}
    }
}

/* κ */
static void
ViewCal_RedrawDay(int x, int y)
{
    int left, top, right, bottom;
    uint16 year, month, day, date;
    MonthRec *monthRec;
    char daystr[3];
    int i;
    mnwRect rect;

    /*  */
    left = region_weeks_left + cell_width * x;
    right = region_weeks_left + cell_width * (x + 1) - 1;
    top = region_weeks_top + cell_height * y;
    bottom = region_weeks_top + cell_height * (y + 1) - 1;

    /* 쥳ɼФ */
    date = originDate + (uint16)(x + y * cell_cols);
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* طɤĤ֤ */
    if (monthRec->isHoliday[day]){
	/* ˤ⤷Ͻ */
	mnwSetForeColor(kColor_LtGray);
    }else{
	/* ʿ */
	mnwSetForeColor(kColor_White);
    }
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /*  */
    if (day == 1){
	mnwSetForeColor(kColor_Black);
	mnwDrawLine(left, top, left, bottom);
	mnwSetForeColor(kColor_LtGray);
	mnwDrawLine(left+1, top, left+1, bottom);
	mnwSetForeColor(kColor_DkGray);	      /* DOT */
	mnwDrawLine(left, bottom, right, bottom);
    }else if (monthRec->numOfDays - day < 7){
	mnwSetForeColor(kColor_DkGray);	      /* DOT */
	mnwDrawLine(left, top, left, bottom);
	mnwSetForeColor(kColor_LtGray);
	mnwDrawLine(left, bottom-1, right, bottom-1);
	mnwSetForeColor(kColor_Black);
	mnwDrawLine(left, bottom, right, bottom);
    }else{
	mnwSetForeColor(kColor_DkGray);	      /* DOT */
	mnwDrawLine(left, top, left, bottom);
	mnwDrawLine(left, bottom, right, bottom);
    }

    if (date != gTodayDate){
	/*  */
	mnwSetForeColor(kColor_DkGray);	      /* DOT */
	mnwDrawLine(left + cell_fontwidth*2 + 2, top,
		 left + cell_fontwidth*2 + 2, top + cell_fontheight);
	mnwDrawLine(left + cell_fontwidth*2 + 2, top + cell_fontheight,
		 left, top + cell_fontheight);

	/*  */
	mnwSetForeColor(kColor_Black);
	mnwSetFont(cell_font);
	sprintf(daystr, "%2d", day);
	mnwDrawText(left + 1, top, daystr, 2);
    }else{
	/* ɤĤ֤ */
	mnwSetForeColor(kColor_Black);
	mnwRect_SetXYXY(&rect, left + 1, top,
			left + cell_fontwidth*2 + 2, top + cell_fontheight);
	mnwFillRect(&rect);

	/*  */
	mnwSetForeColor(kColor_White);
	mnwSetFont(cell_font);
	sprintf(daystr, "%2d", day);
	mnwDrawText(left + 1, top, daystr, 2);
	mnwSetForeColor(kColor_Black);
    }

    /* ͽɽ */
    for (i = 0; i < day_rows && i < (int)monthRec->num[day] && monthRec->rec[day][i]->type != RECORD_TYPE_TASK; i++){
	Record *rec = monthRec->rec[day][i];
	int str_top = top + day_margin_top + i * day_interval;
	mnwDrawText(left + day_margin_left, str_top,
		 (char *)(rec->descript),
		 mmin((int)rec->descript_len, cell_maxlen));
    }

    /* ɽդ */
    if ((int)monthRec->num[day] > day_rows && monthRec->rec[day][day_rows]->type != RECORD_TYPE_TASK){
	mnwDrawText(right - 3 + cell_fontwidth*2, top, "", 2);
    }
}

/* κ */
void
ViewCalWeeks_Redraw(void)
{
    int x, y;

    for (y = 0; y < cell_rows; y++){
	for (x = 0; x < cell_cols; x++){
	    ViewCal_RedrawDay(x, y);
	}
    }
}

/* ȿž */
void
ViewCal_InvertCursor(void)
{
    mnwRect rect;

    mnwRect_SetXYXY(&rect,
		    region_weeks_left + cell_width * cursorX,
		    region_weeks_top + cell_height * cursorY,
		    region_weeks_left + cell_width * (cursorX + 1) - 1,
		    region_weeks_top + cell_height * (cursorY + 1) - 1);
    mnwInvertRect(&rect);
}

/* ʬ륢å */
void
ViewCal_ScrollUp(void)
{
    int x;
    mnwRect rect;

    mnwRect_SetXYXY(&rect,
		    region_weeks_left,
		    region_weeks_top,
		    region_weeks_right,
		    region_weeks_top + cell_height * (cell_rows - 1) - 1);
    mnwCopyRect(&rect, 0, cell_height);

    for (x = 0; x < cell_cols; x++){
	ViewCal_RedrawDay(x, 0);
    }
}

/* ʬ */
void
ViewCal_ScrollDown(void)
{
    int x;
    mnwRect rect;

    mnwRect_SetXYXY(&rect,
		    region_weeks_left,
		    region_weeks_top + cell_height,
		    region_weeks_right,
		    region_weeks_top + cell_height * cell_rows - 1);
    mnwCopyRect(&rect, 0, -cell_height);

    for (x = 0; x < cell_cols; x++){
	ViewCal_RedrawDay(x, (cell_rows-1));
    }
}

/*  */
void
ViewCal_Left(void)
{
    gCursorDate--;
    cursorX--;
    if (cursorX < 0){
	cursorX = cell_cols-1;
	cursorY--;
	if (cursorY < 0){
	    cursorY = 0;
	    originDate -= cell_cols;
	    ViewCal_ScrollUp();
	}
    }
}

/*  */
void
ViewCal_Right(void)
{
    gCursorDate++;
    cursorX++;
    if (cursorX > cell_cols-1){
	cursorX = 0;
	cursorY++;
	if (cursorY > (cell_rows-1)){
	    cursorY = (cell_rows-1);
	    originDate += cell_cols;
	    ViewCal_ScrollDown();
	}
    }
}

/*  */
void
ViewCal_Up(void)
{
    gCursorDate -= cell_cols;
    cursorY--;
    if (cursorY < 0){
	cursorY = 0;
	originDate -= cell_cols;
	ViewCal_ScrollUp();
    }
}

/*  */
void
ViewCal_Down(void)
{
    gCursorDate += cell_cols;
    cursorY++;
    if (cursorY > (cell_rows-1)){
	cursorY = (cell_rows-1);
	originDate += cell_cols;
	ViewCal_ScrollDown();
    }
}

/*  */
void
ViewCal_WeekUp(int num)
{
    gCursorDate -= cell_cols * num;
    originDate -= cell_cols * num;
    ViewCalWeeks_Redraw();
}

/*  */
void
ViewCal_WeekDown(int num)
{
    gCursorDate += cell_cols * num;
    originDate += cell_cols * num;
    ViewCalWeeks_Redraw();
}
