/*
   itemlist region of view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewitem.c,v 1.4 1999/02/03 05:09:54 furukawa Exp $
   */

#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "record.h"
#include "view.h"
#include "viewitem.h"
#include "file.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* -- file local macros */

/* -- file local variables */
static int originI, cursorI;
static int region_date_width, region_date_height;
static int region_date_left, region_date_right;
static int region_date_top, region_date_bottom;
static int region_item_width, region_item_height;
static int region_item_left, region_item_right;
static int region_item_top, region_item_bottom;
static int cell_width, cell_height;
static int cell_rows, cell_cols;
static int cell_fontwidth, cell_fontheight, cell_font;
static int cell_maxlen;
static bool display_end_time;

static Record *clippedRecord = NULL;
static uint16 clippedType = RECORD_TYPE_NONE;

/* -- file local methods */
static void
ViewItem_RedrawEach(int no);

/* -- methods */

/* ViewItemν */
void
ViewItem_Init(int left, int top,
	      int date_width, int date_height,
	      int item_width, int item_height,
	      int c_width, int c_height,
	      int c_cols, int c_rows,
	      int c_font, int c_maxlen,
	      bool d_end_time)
{
    region_date_width = date_width;
    region_date_height = date_height;
    region_date_left = left;
    region_date_top = top;
    region_date_right = region_date_left + region_date_width - 1;
    region_date_bottom = region_date_top + region_date_height - 1;

    region_item_width = item_width;
    region_item_height = item_height;
    region_item_left = left;
    region_item_top = region_date_top + region_date_height;
    region_item_right = region_item_left + region_item_width - 1;
    region_item_bottom = region_item_top + region_item_height - 1;

    cell_width = c_width;
    cell_height = c_height;
    cell_cols = c_cols;
    cell_rows = c_rows;
    cell_font = c_font;
    cell_maxlen = c_maxlen;

    display_end_time = d_end_time;

    switch (cell_font){
	case kFont_16:
	cell_fontwidth = 8;
	cell_fontheight = 16;
	break;
	case kFont_12:
	cell_fontwidth = 6;
	cell_fontheight = 12;
	break;
    }
}

/* ƥκ */
static void
ViewItem_RedrawEach(int no)
{
    int left, top, right, bottom;
    int descript_start;
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;
    char str[10];
    mnwRect rect;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* ɤΥƥफ */
    rec = monthRec->rec[day][originI + no];

    /*  */
    left = region_item_left;
    right = region_item_right;
    top = region_item_top + no * cell_height;
    bottom = region_item_top + (no + 1) * cell_height - 1;

    /* ɤĤ֤ */
    mnwSetForeColor(kColor_White);
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /* ƥ */
    mnwSetForeColor(kColor_Black);
    mnwSetFont(cell_font);
    switch (rec->type){
	case RECORD_TYPE_HOLIDAY:	      /*  */
	/* 졼ɤĤ֤ */
	mnwSetForeColor(kColor_LtGray);
	mnwRect_SetXYXY(&rect, left, top, right, bottom);
	mnwFillRect(&rect);
	/* ˵ɽ */
	mnwSetForeColor(kColor_Black);
	if (rec->link_len > 0){
	    mnwDrawText(left + 1, top + 1, "<", 1);
	}
	mnwDrawText(left + cell_fontwidth * 1 + 1 + (cell_width - mmin((int)rec->descript_len, cell_maxlen-1)*cell_fontwidth) / 2, top + 1,
		 (char *)(rec->descript),
		 mmin((int)rec->descript_len, cell_maxlen-1));
	break;
	case RECORD_TYPE_SCHEDULE:	      /* 塼 */
	if (display_end_time){
	    descript_start = 13;
	}else{
	    descript_start = 7;
	}
	if (rec->link_len > 0){
	    mnwDrawText(left + 1, top + 1, "<", 1);
	}
	if (rec->u.sche.end_time_h == 0 && rec->u.sche.end_time_m == 0){
	    /* ʤ */
	    /*  */
	    mnwDrawText(left + cell_fontwidth * descript_start + 1, top + 1,
		     (char *)(rec->descript), mmin((int)rec->descript_len, cell_maxlen - 7));
	}else{
	    /* 濫 */
	    if (display_end_time
		&& (rec->u.sche.start_time_h != rec->u.sche.end_time_h
		    || rec->u.sche.start_time_m != rec->u.sche.end_time_m)){
		/* ϻ */
		sprintf(str, "%2d:%02d-",
			rec->u.sche.start_time_h,
			rec->u.sche.start_time_m);
		mnwDrawText(left + cell_fontwidth * 1 + 1, top + 1, str, 6);
		/* λ */
		sprintf(str, "%2d:%02d",
			rec->u.sche.end_time_h,
			rec->u.sche.end_time_m);
		mnwDrawText(left + cell_fontwidth * 7 + 1, top + 1, str, 5);
		/*  */
		mnwDrawText(left + cell_fontwidth * 13 + 1, top + 1,
			    (char *)(rec->descript), mmin((int)rec->descript_len, cell_maxlen - 7));
	    }else{
		/* ϻ֤Τ */
		sprintf(str, "%2d:%02d",
			rec->u.sche.start_time_h,
			rec->u.sche.start_time_m);
		mnwDrawText(left + cell_fontwidth * 1 + 1, top + 1, str, 5);
		/*  */
		mnwDrawText(left + cell_fontwidth * descript_start + 1, top + 1,
			    (char *)(rec->descript), mmin((int)rec->descript_len, cell_maxlen - 7));
	    }
	}
	break;
	case RECORD_TYPE_TASK:		      /*  */
	if (rec->link_len > 0){
	    mnwDrawText(left + 1, top + 1, "<", 1);
	}
	sprintf(str, "   %d", rec->u.task.rank);
	mnwDrawText(left, top + 1, str, 6);
	mnwDrawText(left + cell_fontwidth * 7 + 1, top + 1,
		    (char *)(rec->descript), mmin((int)rec->descript_len, cell_maxlen - 7));
	/* Ǥä */
	if (rec->done_len > 0){
	    mnwSetForeColor(kColor_DkGray);   /* DOT */
	    mnwDrawLine(left+8, top+1+cell_fontheight/2-2,
		     right-8, top+1+cell_fontheight/2-2);
	    mnwDrawLine(left+8, top+1+cell_fontheight/2+2,
		     right-8, top+1+cell_fontheight/2+2);
	}
	break;
	default:			      /* ¾ */
	if (rec->link_len > 0){
	    mnwDrawText(left + 1, top + 1, "<", 1);
	}
	mnwDrawText(left + cell_fontwidth * 1 + 1, top + 1,
		    (char *)(rec->descript),
		    mmin((int)rec->descript_len, cell_maxlen));
	break;
    }

    /* ڤ */
    mnwSetForeColor(kColor_DkGray);	      /* DOT */
    mnwDrawLine(left, bottom, right, bottom);
}

/* ƥꥹȤκ */
void
ViewItem_Redraw(void)
{
    int left, top, right, bottom;
    uint16 year, month, day;
    MonthRec *monthRec;
    int i;
    mnwRect rect;

    /*  */
    left = region_item_left;
    right = region_item_right;
    top = region_item_top;
    bottom = region_item_bottom;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* ɤĤ֤ */
    mnwSetForeColor(kColor_White);
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /* ƥɽ */
    for (i = 0; i < cell_rows && i < (int)monthRec->num[day]; i++){
	ViewItem_RedrawEach(i);
    }
}

/* ƥॢå */
void
ViewItem_Up(void)
{
    uint16 year, month, day;
    MonthRec *monthRec;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	if ((int)monthRec->num[day] > cell_rows){
	    if (originI > 0
		&& (cursorI - originI) == 0){
		/* 륢å */
		originI--;
		cursorI--;
		ViewItemDate_Redraw();
		ViewItem_Redraw();
		/* NOTE: mgScrollWindow()ǽʤΤǡ
		   ٤Ƥιܤľ */
	    }else{
		/* 륢å */
		cursorI--;
		if (cursorI < 0)
		    cursorI = 0;
	    }
	}else{
	    /* 륢å */
	    cursorI--;
	    if (cursorI < 0)
		cursorI = 0;
	}
    }
}

/* ƥ */
void
ViewItem_Down(void)
{
    uint16 year, month, day;
    MonthRec *monthRec;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	if ((int)monthRec->num[day] > cell_rows){
	    if (originI < (signed)(monthRec->num[day] - cell_rows)
		&& (cursorI - originI) == (cell_rows - 1)){
		/*  */
		originI++;
		cursorI++;
		ViewItemDate_Redraw();
		ViewItem_Redraw();
		/* NOTE: mgScrollWindow()ǽʤΤǡ
		   ٤Ƥιܤľ */
	    }else{
		/*  */
		cursorI++;
		if (cursorI >= (signed)(monthRec->num[day]))
		    cursorI = monthRec->num[day] - 1;
	    }
	}else{
	    /*  */
	    cursorI++;
	    if (cursorI >= (signed)(monthRec->num[day]))
		cursorI = monthRec->num[day] - 1;
	}
    }
}

/* ȿž */
void
ViewItem_InvertCursor(void)
{
    uint16 year, month, day;
    MonthRec *monthRec;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	mnwRect rect;
	mnwRect_SetXYXY(&rect,
			region_item_left,
			region_item_top + cell_height * (cursorI - originI),
			region_item_right,
			region_item_top + cell_height * ((cursorI - originI)+ 1) - 1);
	mnwInvertRect(&rect);
    }
}

/* ν */
void
ViewItem_InitCursor(void)
{
    cursorI = 0;
    originI = 0;
}

/* κ */
void
ViewItem_ResetCursor(void)
{
    uint16 year, month, day;
    MonthRec *monthRec;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    if (monthRec->num[day] > 0){
	if (cursorI >= (int)monthRec->num[day]){
	    cursorI = monthRec->num[day] - 1;
	}
	if (cursorI + 1 > cell_rows){
	    /* ɽդξ */
	    originI = cursorI - cell_rows + 1;
	}else{
	    /* ɽդǤʤ */
	    originI = 0;
	}
    }else{
	cursorI = 0;
	originI = 0;
	cursor_focus = cursor_prev_focus;
    }
}

/* ν */
uint16
ViewItem_GetCursor(void)
{
    return cursorI;
}

/* դκ */
void
ViewItemDate_Redraw(void)
{
    uint16 year, month, day;
    int left, top, right, bottom;
    MonthRec *monthRec;
    char str[80];
    mnwRect rect;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /*  */
    left = region_date_left;
    right = region_date_right;
    top = region_date_top;
    bottom = region_date_bottom;

    /* ɤĤ֤ */
    mnwSetForeColor(kColor_LtGray);
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /* ̥ܥ */
    mnwDrawFrame3D(&rect);

    /*  */
    sprintf(str, "%dǯ%2d%2d(%s)  ", 
	    Calendar_GetYearByNo(gCursorDate),
	    Calendar_GetMonthByNo(gCursorDate),
	    Calendar_GetDayByNo(gCursorDate),
	    nameDayWeek[Calendar_GetDayOfWeekByNo(gCursorDate)]);
    if (region_date_width > (signed)strlen(str)*8){
	mnwSetForeColor(kColor_Black);
	mnwSetFont(kFont_16);
	mnwDrawText(left + (region_date_width - strlen(str)*8) / 2,
		 top + (region_date_height - 16) / 2,
		 str, strlen(str));
    }else{
	mnwSetForeColor(kColor_Black);
	mnwSetFont(kFont_12);
	mnwDrawText(left + (region_date_width - strlen(str)*6) / 2,
		 top + (region_date_height - 12) / 2,
		 str, strlen(str));
    }

    /* ɽդ */
    if ((int)monthRec->num[day] > cell_rows){
	mnwSetFont(kFont_12);
	if (originI > 0){
	    mnwDrawText(right - 12*1 - 1, top + 2, "", 2);
	}
	if ((int)monthRec->num[day] - originI > cell_rows){
	    mnwDrawText(right - 12*1 - 1, top + 2, "", 2);
	}
	/* NOTE: Ʊ֤˽Ťͤ */
    }
}

bool
Misc_CutItem(uint16 date, int no)
{
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;

    D(mnwDPrintf("Misc_CutItem()"));

    /* 쥳ɼФ */
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
    rec = monthRec->rec[day][no];

    /* åץܡɤ˥ޡ */
    clippedRecord = rec;
    clippedType = rec->type;

    /*  */
    Record_Delete(rec);
    /* TODO: ֤ͤΥå */
    File_SetDBDirty();

    /* å̵ */
    MonthRec_Invalidate();

    return true;
}

bool
Misc_CopyItem(uint16 date, int no)
{
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;

    D(mnwDPrintf("Misc_CopyItem()"));

    /* 쥳ɼФ */
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
    rec = monthRec->rec[day][no];

    /* åץܡɤ˥ޡ */
    clippedRecord = rec;
    clippedType = rec->type;

    /* 褷ʤ */
    return false;
}

bool
Misc_PasteItem(uint16 date)
{
    Record *rec;
    int r;

    D(mnwDPrintf("Misc_PasteItem()"));

    if (clippedRecord == NULL){
	return false;
    }

    D(mnwDPrintf("allocate record"));

    /* 쥳ɳ */
    rec = Record_AllocateRecord(1);
    if (rec == nil){
	return false;
    }

    /* åץ쥳ɤ򥳥ԡ */
    memcpy(rec, clippedRecord, sizeof(Record));
    rec->len = sizeof(Record);
    rec->descript = Record_AllocateByte(clippedRecord->descript_len);
    if (rec->descript == nil){
	return false;
    }
    memcpy(rec->descript, clippedRecord->descript, rec->descript_len);
    if (clippedRecord->done_len > 0){
	rec->done = (uint16 *)Record_AllocateByte(clippedRecord->done_len);
	memcpy(rec->done, clippedRecord->done, rec->done_len);
    }
    if (clippedRecord->link_len > 0){
	rec->link = Record_AllocateByte(clippedRecord->link_len);
	memcpy(rec->link, clippedRecord->link, rec->link_len);
    }
    rec->type = clippedType;

    /* ̣ԡ */
    switch (rec->type){
	case RECORD_TYPE_SCHEDULE:
	D(mnwDPrintf("hoge"));
	rec->start_date = date;
	rec->end_date = date + (clippedRecord->end_date - clippedRecord->start_date);
	break;
	case RECORD_TYPE_HOLIDAY:
	D(mnwDPrintf("hoge"));
	rec->start_date = date;
	rec->end_date = date + (clippedRecord->end_date - clippedRecord->start_date);
	break;
	case RECORD_TYPE_TASK:
	D(mnwDPrintf("hoge"));
	rec->start_date = date;
	break;
    }
    if (rec->repeat_type != REPEAT_TYPE_NONE){
	uint16 year, month, day, dayWeek, nthWeek, numOfDays;

	Calendar_GetDateByNo(date, &year, &month, &day);
	dayWeek = Calendar_GetDayOfWeekByNo(date);
	nthWeek = (day / 7) + ((day%7)?1:0);
	numOfDays = Calendar_GetNumOfDays(year, month);
	if (numOfDays - day < 7){
	    nthWeek = 5;
	}
	switch (rec->repeat_type){
	    case REPEAT_TYPE_WEEKLY:
	    rec->repeat_arg1 = gDayOfWeekBit[dayWeek];
	    break;
	    case REPEAT_TYPE_MONTHLYWEEK:
	    case REPEAT_TYPE_YEARLYWEEK:
	    rec->repeat_arg1 = gNthWeekBit[nthWeek];
	    rec->repeat_arg2 = gDayOfWeekBit[dayWeek];
	    break;
	}
    }

    /* 쥳ɤϿ */
    r = Record_Register(rec);
    if (!r){
	D(mnwDPrintf("Record_Register failed"));
	return false;
    }
    File_SetDBDirty();

    D(mnwDPrintf("Record_Register success"));

    /* å̵ */
    MonthRec_Invalidate();

    return true;
}
