/*
   timetable region of view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewtime.c,v 1.1.1.1 1998/12/15 03:42:50 furukawa Exp $
   */

#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "view.h"
#include "viewtime.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* -- file local macros */

/* -- file local variables */
static uint16 originDate, originHour;
static int cursorX, cursorY; 
static int region_hour_width, region_hour_height;
static int region_hour_left, region_hour_right;
static int region_hour_top, region_hour_bottom;
static int region_dayweek_width, region_dayweek_height;
static int region_dayweek_left, region_dayweek_right;
static int region_dayweek_top, region_dayweek_bottom;
static int region_timetable_width, region_timetable_height;
static int region_timetable_left, region_timetable_right;
static int region_timetable_top, region_timetable_bottom;
static int region_day_width, region_day_height;
static int region_day_left, region_day_right;
static int region_day_top, region_day_bottom;
static int cell_width, cell_height;
static int cell_rows, cell_cols;
static int cell_fontwidth, cell_fontheight, cell_font;
static int cell_maxlen;
static int day_rows;

/* -- file local methods */
static void
ViewTimeDayWeek_Redraw(void);
static void
ViewTimeTable_Redraw(int x);
static void
ViewTimeDay_Redraw(int x);

/* -- methods */

/* ViewTimeν */
void
ViewTime_Init(int left, int top,
	      int hour_width, int hour_height,
	      int dayweek_width, int dayweek_height,
	      int timetable_width, int timetable_height,
	      int day_width, int day_height,
	      int c_width, int c_height,
	      int c_cols, int c_rows,
	      int d_rows,
	      int c_font, int c_maxlen)
{
    region_hour_width = hour_width;
    region_hour_height = hour_height;
    region_hour_left = left;
    region_hour_top = top + dayweek_height;
    region_hour_right = region_hour_left + region_hour_width - 1;
    region_hour_bottom = region_hour_top + region_hour_height - 1;

    region_dayweek_width = dayweek_width;
    region_dayweek_height = dayweek_height;
    region_dayweek_left = region_hour_right + 1;
    region_dayweek_top = top;
    region_dayweek_right = region_dayweek_left + region_dayweek_width - 1;
    region_dayweek_bottom = region_dayweek_top + region_dayweek_height - 1;

    region_timetable_width = timetable_width;
    region_timetable_height = timetable_height;
    region_timetable_left = region_hour_right + 1;
    region_timetable_top = region_dayweek_bottom + 1;
    region_timetable_right = region_timetable_left + region_timetable_width - 1;
    region_timetable_bottom = region_timetable_top + region_timetable_height - 1;

    region_day_width = day_width;
    region_day_height = day_height;
    region_day_left = region_hour_right + 1;
    region_day_top = region_timetable_bottom + 1;
    region_day_right = region_day_left + region_day_width - 1;
    region_day_bottom = region_day_top + region_day_height - 1;

    cell_width = c_width;
    cell_height = c_height;
    cell_cols = c_cols;
    cell_rows = c_rows;
    cell_font = c_font;
    cell_maxlen = c_maxlen;
    day_rows = d_rows;

    switch (cell_font){
	case kFont_16:
	cell_fontwidth = 8;
	cell_fontheight = 16;
	break;
	case kFont_12:
	cell_fontwidth = 6;
	cell_fontheight = 12;
	break;
    }

    /* ץ󡧥ơ֥γϤ򣹻˸ */
    if (gOpt_TimeTableStartsNine){
	gCursorHour = 9;
    }
}

/* ν */
void
ViewTime_InitCursor(void)
{
    if (cell_cols > 1){
	if (gOpt_WeeksStartsFromMonday){
	    /* Ϸˤ */
	    cursorX = (Calendar_GetDayOfWeekByNo(gCursorDate) + 7 - 1) % 7;
	}else{
	    /* ˤ */
	    cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	}
    }else{
	cursorX = 0;
    }
    originDate = gCursorDate - (uint16)cursorX;
    if (gCursorHour + cell_rows > 24){
	originHour = 24 - cell_rows;
	cursorY = gCursorHour - originHour;
    }else{
	originHour = gCursorHour;
	cursorY = 0;
    }
    gCursorHourRange = 1;
}

/* ּκ */
void
ViewTimeHour_Redraw(void)
{
    int i;
    int left, top, right, bottom;
    mnwRect rect;

    /*  */
    top = region_hour_top;
    bottom = region_hour_bottom;
    left = region_hour_left;
    right = region_hour_right;

    /* ɤĤ֤ */
    mnwSetForeColor(kColor_LtGray);
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /*  */
    mnwSetForeColor(kColor_Black);
    mnwSetFont(kFont_12);
    for (i = 0; i < cell_rows; i++){
	char buf[6];
	sprintf(buf, "%2d:00", originHour + i);
	mnwDrawText(left+1, top + cell_height * i, buf, 5);
    }
}

/* κ */
static void
ViewTimeDayWeek_Redraw(void)
{
    int i;
    int left, top, right, bottom;
    uint16 year, month, day;
    char str[20];
    mnwRect rect;

    top = region_dayweek_top;
    bottom = region_dayweek_bottom;

    /* ɤĤ֤ */
    mnwSetForeColor(kColor_LtGray);
    mnwRect_SetXYXY(&rect, region_dayweek_left, top, region_dayweek_right, bottom);
    mnwFillRect(&rect);

    for (i = 0; i < cell_cols; i++){
	left = region_dayweek_left + cell_width * i;
	right = left + cell_width - 1;

	/*  */
	mnwSetForeColor(kColor_Black);
	mnwDrawLine(left, top, left, bottom);

	/* ̥ܥ */
	mnwRect_SetXYXY(&rect, left+1, top, right, bottom);
	mnwDrawFrame3D(&rect);

	/*  */
	mnwSetForeColor(kColor_Black);
	mnwSetFont(cell_font);
	Calendar_GetDateByNo(originDate+i, &year, &month, &day);
	if (gOpt_WeeksStartsFromMonday){
	    /* Ϸˤ */
	    sprintf(str, "%2d  %s", day, nameDayWeek[(i+1)%7]);
	}else{
	    /* ˤ */
	    sprintf(str, "%2d  %s", day, nameDayWeek[i]);
	}
	mnwDrawText(left + (cell_width - strlen(str)*cell_fontwidth) / 2, top+1,
		 str, strlen(str));
    }
}

/* ơ֥ */
static void
ViewTimeTable_Redraw(int x)
{
    int left, top, right, bottom;
    uint16 year, month, day;
    MonthRec *monthRec;
    int i;
    mnwRect rect;

    /*  */
    left = region_timetable_left + cell_width * x;
    right = region_timetable_left + cell_width * (x + 1) - 1;
    top = region_timetable_top;
    bottom = region_timetable_bottom;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(originDate +(uint16)x, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* طɤĤ֤ */
    mnwSetForeColor(kColor_White);
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /*  */
    mnwSetForeColor(kColor_DkGray);		      /* DOT */
    mnwDrawLine(left, top, left, bottom);

    /*  */
    for (i = 0; i < cell_rows; i++){
	mnwDrawLine(left, top + cell_height * i, right, top + cell_height * i);
    }

    /* ƥ */
    for (i = 0; i < (int)monthRec->num[day]; i++){
	Record *rec = monthRec->rec[day][i];
	if (rec->type == RECORD_TYPE_SCHEDULE
	    && !(rec->u.sche.end_time_h == 0 && rec->u.sche.end_time_m == 0)){
	    /* ֤Τ륢ƥ */
	    int16 start_min, end_min;
	    int16 start_y, end_y;
	    bool head_in, tail_in, body_in, desc_in;
	    start_min = rec->u.sche.start_time_h*60 + rec->u.sche.start_time_m;
	    end_min = rec->u.sche.end_time_h*60 + rec->u.sche.end_time_m;
	    start_y = ((start_min - (int16)originHour*60) * cell_height) / 60;
	    end_y = ((end_min - (int16)originHour*60) * cell_height) / 60;
	    /* Ƚ */
	    head_in = tail_in = body_in = desc_in = false;
	    if (start_y < 0 && end_y > region_timetable_height){
		body_in = true;
	    }
	    if (start_y >= 0 && start_y <= region_timetable_height){
		/* ƬäƤ */
		head_in = true;
		if (region_timetable_height - start_y > cell_fontheight){
		    /* ʸ */
		    desc_in = true;
		}
	    }else{
		start_y = 0;
	    }
	    if (end_y >= 0 && end_y <= region_timetable_height){
		/* äƤ */
		tail_in = true;
	    }else{
		end_y = region_timetable_height - 1;
	    }
	    /* äƤΤ */
	    if (head_in || tail_in || body_in){
		start_y += region_timetable_top;
		end_y += region_timetable_top;
		/* طɤĤ֤ */
		mnwSetForeColor(kColor_LtGray);
		mnwRect_SetXYXY(&rect, left, start_y, right, end_y);
		mnwFillRect(&rect);
		/*  */
		mnwSetForeColor(kColor_Black);
		mnwDrawLine(left, start_y, left, end_y);
		if (tail_in){
		    mnwDrawLine(left, end_y, right, end_y);
		    mnwDrawLine(right, end_y, right, start_y);
		}else{
		    mnwDrawLine(right, end_y, right, start_y);
		}
		if (head_in){
		    mnwDrawLine(right, start_y, left, start_y);
		}
		if (desc_in){
		    /* ʸ */
		    mnwSetForeColor(kColor_Black);
		    mnwSetFont(cell_font);
		    mnwDrawText(left + 2, start_y + 1, 
			     (char *)rec->descript, mmin((int)rec->descript_len, cell_maxlen));
		}
	    }
	}
    }
}

/* κ */
static void
ViewTimeDay_Redraw(int x)
{
    int left, top, right, bottom;
    uint16 year, month, day;
    MonthRec *monthRec;
    uint16 y;
    int i;
    mnwRect rect;

    /*  */
    left = region_day_left + cell_width * x;
    right = region_day_left + cell_width * (x + 1) - 1;
    top = region_day_top;
    bottom = region_day_bottom;

    /* 쥳ɼФ */
    Calendar_GetDateByNo(originDate +(uint16)x, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* طɤĤ֤ */
    if (monthRec->isHoliday[day]){
	/* ˤ⤷Ͻ */
	mnwSetForeColor(kColor_LtGray);
    }else{
	/* ʿ */
	mnwSetForeColor(kColor_White);
    }
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /*  */
    mnwSetForeColor(kColor_DkGray);	      /* DOT */
    mnwDrawLine(left, top, left, bottom);
    /*  */
    mnwSetForeColor(kColor_Black);
    mnwDrawLine(left, top, right, top);

    /* ͽɽ */
    mnwSetForeColor(kColor_Black);
    mnwSetFont(cell_font);
    for (i = 0, y = 0; i < day_rows && i < (int)monthRec->num[day] && monthRec->rec[day][i]->type != RECORD_TYPE_TASK; i++){
	Record *rec = monthRec->rec[day][i];
	if (rec->type == RECORD_TYPE_HOLIDAY
	    || ((rec->type == RECORD_TYPE_SCHEDULE)
		&& (rec->u.sche.end_time_h == 0 && rec->u.sche.end_time_m == 0))){
	    mnwDrawText(left + 2, top + 2 + y * (cell_fontheight+1), 
		     (char *)(rec->descript),
		     mmin((int)rec->descript_len, cell_maxlen));
	    y++;
	}
    }
}

/* κ */
void
ViewTimeWeek_Redraw(void)
{
    int x;

    if (cell_cols > 1){
	ViewTimeDayWeek_Redraw();
    }
    for (x = 0; x < cell_cols; x++){
	ViewTimeTable_Redraw(x);
	if (cell_cols > 1){
	    ViewTimeDay_Redraw(x);
	}
    }
}

/* ȿž */
void
ViewTime_InvertCursor(void)
{
    mnwRect rect;
    int topY;

    topY = cursorY - (gCursorHourRange - 1);
    if (topY < 0){
	topY = 0;
    }

    mnwRect_SetXYXY(&(rect),
		    region_timetable_left + cell_width * cursorX,
		    region_timetable_top + cell_height * topY,
		    region_timetable_left + cell_width * (cursorX + 1) - 1,
		    region_timetable_top + cell_height * (cursorY + 1) - 1);
    mnwInvertRect(&rect);
}

/*  */
void
ViewTime_Up(void)
{
    int x;

    if (originHour > 0 && cursorY == 0){
	/* 륢å */
	originHour--;
	ViewTimeHour_Redraw();
	for (x = 0; x < cell_cols; x++){
	    ViewTimeTable_Redraw(x);
	}
    }else{
	/* 륢å */
	cursorY--;
	if (cursorY < 0){
	    cursorY = 0;
	}
    }
}

/*  */
void
ViewTime_Down(void)
{
    int x;

    if (originHour + cell_rows < 24 && cell_rows - 1 == cursorY){
	/*  */
	originHour++;
	ViewTimeHour_Redraw();
	for (x = 0; x < cell_cols; x++){
	    ViewTimeTable_Redraw(x);
	}
    }else{
	/*  */
	cursorY++;
	if (cursorY >= cell_rows){
	    cursorY = cell_rows - 1;
	}
    }
}

/*  */
void
ViewTime_Left(void)
{
    if (cell_cols > 1){
	cursorX--;
	if (cursorX < 0){
	    cursorX = cell_cols-1;
	    originDate -= cell_cols;
	    ViewTimeWeek_Redraw();
	}
    }else{
	originDate = gCursorDate;
    }
}

/*  */
void
ViewTime_Right(void)
{
    if (cell_cols > 1){
	cursorX++;
	if (cursorX > cell_cols-1){
	    cursorX = 0;
	    originDate += cell_cols;
	    ViewTimeWeek_Redraw();
	}
    }else{
	originDate = gCursorDate;
    }
}

/*  */
void
ViewTime_WeekUp(void)
{
    originDate -= cell_cols;
    gCursorDate -= cell_cols;
    ViewTimeWeek_Redraw();
}

/*  */
void
ViewTime_WeekDown(void)
{
    originDate += cell_cols;
    gCursorDate += cell_cols;
    ViewTimeWeek_Redraw();
}

