/*
   week view (large size)
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewwl.c,v 1.2 1998/12/27 16:26:24 furukawa Exp $
   */

#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "taskid.h"
#include "view.h"
#include "viewwl.h"
#include "viewtime.h"
#include "viewitem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* -- file local macros */
#define HOURCELL_ROWS		7
#define HOURCELL_COLS		7
#define HOURCELL_WIDTH		(16*5+6)
#define HOURCELL_HEIGHT		(16*1+1)
#define ITEM_PER_DAY		3

#define REGION_HOUR_WIDTH	(6*5+2+6)
#define REGION_HOUR_HEIGHT	(HOURCELL_HEIGHT*HOURCELL_ROWS)
#define REGION_DAYWEEK_WIDTH	(HOURCELL_WIDTH*HOURCELL_COLS)
#define REGION_DAYWEEK_HEIGHT	(16*1+4)
#define REGION_TIMETABLE_WIDTH	(HOURCELL_WIDTH*HOURCELL_COLS)
#define REGION_TIMETABLE_HEIGHT	(HOURCELL_HEIGHT*HOURCELL_ROWS)
#define REGION_DAY_WIDTH	(HOURCELL_WIDTH*HOURCELL_COLS)
#define REGION_DAY_HEIGHT	(62)

/* -- file local variables */

/* -- file local methods */

/* -- methods */

/* ViewWL */
bool
ViewWL_Create(void)
{
    D(mnwDPrintf("ViewWL create\n"));

    cursor_focus = FOCUS_TIMETABLE;
    cursor_prev_focus = FOCUS_TIMETABLE;
    ViewItem_InitCursor();
    ViewTime_Init(REGION_USER_LEFT, REGION_USER_TOP,
		  REGION_HOUR_WIDTH, REGION_HOUR_HEIGHT,
		  REGION_DAYWEEK_WIDTH, REGION_DAYWEEK_HEIGHT,
		  REGION_TIMETABLE_WIDTH, REGION_TIMETABLE_HEIGHT,
		  REGION_DAY_WIDTH, REGION_DAY_HEIGHT,
		  HOURCELL_WIDTH, HOURCELL_HEIGHT,
		  HOURCELL_COLS, HOURCELL_ROWS,
		  ITEM_PER_DAY,
		  kFont_16, 10);
    ViewTime_InitCursor();

    return true;
}

/* ViewWLդ */
bool
ViewWL_Destroy(void)
{
    D(mnwDPrintf("ViewWL destroy\n"));
    return true;
}

/* ViewWL */
void
ViewWL_Redraw(void)
{
    ViewTimeHour_Redraw();
    ViewTimeWeek_Redraw();
    ViewTime_InvertCursor();		      /* ON */
}

/* ViewWLٱ */
void
ViewWL_DelayedRedraw(void)
{
    /* NOTHING TO DO */
}

/* ɽ */
bool
ViewWL_KeyDown(mnwEvent *event)
{
    switch (event->param1){
	case kKey_LEFT:			      /*  */
	case kUKey_LEFT:		      /*  */
	ViewTime_InvertCursor();	      /* OFF */
	gCursorDate--;
	/* 򥫡򸵤 */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_Left();
	ViewTime_InvertCursor();	      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_RIGHT:		      /*  */
	case kUKey_RIGHT:		      /*  */
	ViewTime_InvertCursor();	      /* OFF */
	gCursorDate++;
	/* 򥫡򸵤 */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_Right();
	ViewTime_InvertCursor();	      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_PRIOR:		      /*  */
	/* 򥫡򸵤 */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_WeekUp();
	ViewTime_InvertCursor();	      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_NEXT:			      /*  */
	/* 򥫡򸵤 */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	ViewTime_WeekDown();
	ViewTime_InvertCursor();	      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_UP:			      /*  */
	case kUKey_UP:			      /*  */
	ViewTime_InvertCursor();	      /* OFF */
	/* 򥫡򸵤 */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* ư */
	if (gCursorHour > 0){
	    gCursorHour--;
	}
	ViewTime_Up();
	ViewTime_InvertCursor();	      /* ON */
	break;

	case kKey_DOWN:			      /*  */
	case kUKey_DOWN:		      /*  */
	ViewTime_InvertCursor();	      /* OFF */
	/* 򥫡򸵤 */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* ư */
	if (gCursorHour < 23){
	    gCursorHour++;
	}
	ViewTime_Down();
	ViewTime_InvertCursor();	      /* ON */
	break;

	case kUKey_HOURUP:		      /* ־ */
	ViewTime_InvertCursor();	      /* OFF */
	/* 򥫡̾ */
	if (gCursorHourRange > 1){
	    gCursorHourRange--;
	}else{
	    if (gCursorHour > 0){
		gCursorHour--;
	    }
	}
	ViewTime_Up();
	ViewTime_InvertCursor();	      /* ON */
	break;

	case kUKey_HOURDOWN:		      /* ֲ */
	ViewTime_InvertCursor();	      /* OFF */
	/* 򥫡 */
	if (gCursorHour + gCursorHourRange < 24){
	    gCursorHourRange++;
	}
	ViewTime_Down();
	ViewTime_InvertCursor();	      /* ON */
	break;

	default:
	/* ʤ */
	return false;
	break;
    }

    /*  */
    return true;
}

/* ޥɽ */
bool
ViewWL_Command(mnwEvent *event)
{
    switch (event->param1){
	case ID_FUNC_NOW:
	ViewTime_InitCursor();
	View_Redraw();
	break;

	default:
	/* ʤ */
	return false;
	break;
    }

    /*  */
    return true;
}
