/*
   MNW header
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnw.h,v 1.4 1999/03/08 07:17:48 furukawa Exp $
   */

#ifndef _MNW_H_
#define _MNW_H_
#define MNW_ROTATED_TEXT
#define MGL_PROTOTYPE

#include "arch.h"
#include "mgl.h"			      /* mgl library */
#include <stdio.h>
#include <sys/time.h>

/* -- macros */

#define REGION_SCREEN_WIDTH	640
#define REGION_SCREEN_HEIGHT	240
#define REGION_SCREEN_LEFT	0
#define REGION_SCREEN_TOP	0
#define REGION_SCREEN_RIGHT	(REGION_SCREEN_LEFT+REGION_SCREEN_WIDTH-1)
#define REGION_SCREEN_BOTTOM	(REGION_SCREEN_TOP+REGION_SCREEN_HEIGHT-1)

#define kColor_Black	3
#define kColor_DkGray	2
#define kColor_LtGray	1	
#define kColor_White	0

#define kFont_Default	0
#define kFont_12	1
#define kFont_16	2
#define kFont_24	3
#define kFont_16x8	4
#define kFont_10x8	5

#define kAttr_None	0
#define kAttr_Bold	1
#define kAttr_Italic	2

#define MNW_MAX_CHILD		32
#define MNW_MAX_EVENTQUEUE	128

#define MNW_MAX_TIMER		5
#define MNW_MAX_TIMER_WAIT	(60*1000)

/* for speed */
#ifdef MNW_INLINING_FOR_SPEED
#define mnwGraphics_SetForeColor(self, color) self->fore_color = (color); set_color(color)
#endif

/* -- key code */

enum{
    kKey_RETURN = 0x0d,
    kKey_TAB = 0x09,
    kKey_BACKSPACE = 0x08,
    kKey_ESCAPE = 0x1b,
    kKey_SPACE = 0x20,
    kKey_F1 = 0x101,
    kKey_F2,
    kKey_F3,
    kKey_F4,
    kKey_F5,
    kKey_F6,
    kKey_F7,
    kKey_F8,
    kKey_F9,
    kKey_F10,
    kKey_F11,
    kKey_F12,
    kKey_HOME = 0x111,
    kKey_END,
    kKey_UP,
    kKey_DOWN,
    kKey_RIGHT,
    kKey_LEFT,
    kKey_PRIOR,
    kKey_NEXT,
    kKey_INSERT,
    kKey_DELETE,
};

enum{
    kKey_NONE = 0,
    kKey_CONTROL = 0x01,
    kKey_SHIFT = 0x02,
    kKey_ALT = 0x04,
};

/* -- Direction */

enum{
    kDIR_NORTH,
    kDIR_WEST,
    kDIR_SOUTH,
    kDIR_EAST,
};

/* -- Rect */

typedef struct _mnwRect {
    int x;
    int y;
    int width;
    int height;
} mnwRect;

/* -- Graphics */

typedef struct _mnwGraphics {
    int fore_color;
    int back_color;
    int pen_x;
    int pen_y;
    int pen_color;
    int pen_size;
    int font;
    int font_width;
    int font_height;
} mnwGraphics;

/* -- Event */

enum{
    /* generic */
    kEvent_None,
    kEvent_Create,
    kEvent_Destroy,
    kEvent_Show,
    kEvent_Hide,
    kEvent_SetFocus,
    kEvent_GainFocus,
    kEvent_LostFocus,
    kEvent_Timer,
    kEvent_KeyDown,
    kEvent_KeyUp,
    kEvent_Command,
    kEvent_Redraw,
    kEvent_Quit,
    kEvent_SetRect,
    kEvent_SetParent,
    kEvent_AddChild,
    kEvent_RemoveChild,
    kEvent_SetForeColor,
    kEvent_SetBackColor,
    kEvent_GetRect,
    kEvent_SetID,
    kEvent_GetID,
    kEvent_SetOption,

    /* dialog */
    kEvent_SetTabStop,
    kEvent_DoModal,
    kEvent_EndModal,

    /* clipboard */
    kEvent_Cut,
    kEvent_Paste,
    kEvent_Copy,
    kEvent_Clear,

    /* compo */
    kEvent_SetFont,
    kEvent_SetAlign,
    kEvent_SetStyle,
    kEvent_SetBuffer,
    kEvent_SetText,
    kEvent_UnsetText,
    kEvent_InsertText,
    kEvent_SetCommand,
    kEvent_GetResult,
    kEvent_SetCursor,
    kEvent_SetMultiLine,
    kEvent_SetPixmap,
    kEvent_GainFocusNotify,
    kEvent_LostFocusNotify,
    kEvent_ChangedNotify,

    /* user */
    kEvent_User0 = 1000,
};

typedef struct _mnwEvent {
    struct _mnwWindow *target;
    int type;
    void *param0;
    int param1;
    int param2;
} mnwEvent;

typedef struct _mnwTimer{
    bool valid;
    long interval;
    struct _mnwWindow *target;
    struct timeval next;
} mnwTimer;

/* -- Style */

enum{
    kAlign_Left,
    kAlign_Center,
    kAlign_Right,
};

enum{
    kStyle_None = 0x00,
    kStyle_Border = 0x01,
    kStyle_Popup = 0x02,
};

enum{
    kRootWinStyle_DrawIcon = 0x01,
    kRootWinStyle_DrawClock = 0x02,
    kRootWinStyle_ShortTitle = 0x04,
};

/* -- Window */

enum{
    kWindowType_Generic,
    kWindowType_Dialog,
    kWindowType_Compo,
};

typedef bool (*mnwWindowCallback)(struct _mnwWindow *win, mnwEvent *event);

typedef struct _mnwWindow {
    int no;
    int id;
    int type;
    mnwRect rect;
    bool visible;
    int fore_color;
    int back_color;
    struct _mnwWindow *parent;
    int child_num;
    int child_valid[MNW_MAX_CHILD];
    struct _mnwWindow *child[MNW_MAX_CHILD];
    mnwWindowCallback callback;
} mnwWindow;

/* -- Dialog */

typedef struct _mnwDialog {
    int no;
    int id;
    int type;
    mnwRect rect;
    bool visible;
    int fore_color;
    int back_color;
    struct _mnwWindow *parent;
    int child_num;
    int child_valid[MNW_MAX_CHILD];
    struct _mnwWindow *child[MNW_MAX_CHILD];
    mnwWindowCallback callback;
    /* dialog */
    int child_tabstop[MNW_MAX_CHILD];
    int focus_child;
    mnwWindow *prev_focus;
    int userdata0;
    int userdata1;
    int userdata2;
} mnwDialog;

/* -- Label Compo */

typedef struct _mnwLabel {
    int no;
    int id;
    int type;
    mnwRect rect;
    bool visible;
    int fore_color;
    int back_color;
    struct _mnwWindow *parent;
    int child_num;
    int child_valid[MNW_MAX_CHILD];
    struct _mnwWindow *child[MNW_MAX_CHILD];
    mnwWindowCallback callback;
    /* label */
    bool focused;
    char *buffer;
    int buffer_len;
    int text_len;
    int max_col;
    int style;
    int align;
    int font;
    int font_width;
    int font_height;
} mnwLabel;

/* -- Button Compo */

typedef struct _mnwButton {
    int no;
    int id;
    int type;
    mnwRect rect;
    bool visible;
    int fore_color;
    int back_color;
    struct _mnwWindow *parent;
    int child_num;
    int child_valid[MNW_MAX_CHILD];
    struct _mnwWindow *child[MNW_MAX_CHILD];
    mnwWindowCallback callback;
    /* button */
    bool focused;
    char *buffer;
    int buffer_len;
    int text_len;
    int max_col;
    int font;
    int font_width;
    int font_height;
    int command;
} mnwButton;

/* -- List Compo */

typedef struct _mnwList {
    int no;
    int id;
    int type;
    mnwRect rect;
    bool visible;
    int fore_color;
    int back_color;
    struct _mnwWindow *parent;
    int child_num;
    int child_valid[MNW_MAX_CHILD];
    struct _mnwWindow *child[MNW_MAX_CHILD];
    mnwWindowCallback callback;
    /* list */
    bool focused;
    char *buffer;
    int max_item;			      /* max item number */
    int max_size;			      /* max size of each item */
    int max_col;			      /* display col */
    int max_row;			      /* display row */
    int origin_row;
    int cursor_row;
    int items;
    int font;
    int font_width;
    int font_height;
} mnwList;

/* -- Edit Compo */

typedef struct _mnwEdit {
    int no;
    int id;
    int type;
    mnwRect rect;
    bool visible;
    int fore_color;
    int back_color;
    struct _mnwWindow *parent;
    int child_num;
    int child_valid[MNW_MAX_CHILD];
    struct _mnwWindow *child[MNW_MAX_CHILD];
    mnwWindowCallback callback;
    /* edit */
    bool focused;
    char *buffer;
    int buffer_len;
    int text_len;
    int region_col;
    int region_row;
    int origin_col;
    int origin_row;
    int cursor_pos;
    int cursor_len;
    int font;
    int font_width;
    int font_height;
    bool multi_line;
} mnwEdit;

/* -- MsgBox */

enum{
    kMsgBox_Type_OK = 0x01,
    kMsgBox_Type_OKNG = 0x02,
    kMsgBox_Type_OKCANCEL = 0x04,
    kMsgBox_Type_OKNGCANCEL = 0x08,

    kMsgBox_Default_OK = 0x10,
    kMsgBox_Default_NG = 0x20,
    kMsgBox_Default_CANCEL = 0x40,
};

enum{
    kMsgBox_FAIL = 0x00,
    kMsgBox_OK = 0x01,
    kMsgBox_NG = 0x02,
    kMsgBox_CANCEL = 0x04,
};

/* -- global variables */

extern mnwGraphics *gDefaultGraphics;
extern mnwWindow *gRootWindow;
extern mnwWindow *gFunctionGuide;
#ifdef MNW_USE_GGI
#include <ggi/ggi.h>
extern ggi_visual_t gDefaultVisual;
#endif

bool
mnwWindow_Callback(mnwWindow *self, mnwEvent *event);
bool
mnwDialog_Callback(mnwDialog *self, mnwEvent *event);
bool
mnwLabel_Callback(mnwLabel *self, mnwEvent *event);
bool
mnwButton_Callback(mnwButton *self, mnwEvent *event);
bool
mnwList_Callback(mnwList *self, mnwEvent *event);
bool
mnwEdit_Callback(mnwEdit *self, mnwEvent *event);

/* -- prototype */

bool
mnwSetup(void);
bool
mnwTidy(void);
void
mnwRect_SetXYXY(mnwRect *self, int x1, int y1, int x2, int y2);
void
mnwRect_SetXYWH(mnwRect *self, int x1, int y1, int w, int h);
void
mnwRect_Inset(mnwRect *self, int x_offset, int y_offset);
void
mnwRect_Offset(mnwRect *self, int x_offset, int y_offset);
void
mnwRect_Copy(mnwRect *dst, mnwRect *src);

void
mnwGraphics_Initialize(mnwGraphics *graph);
void
mnwGraphics_Finalize(mnwGraphics *graph);
void
mnwGraphics_Flush(mnwGraphics *graph);
#ifndef MNW_INLINING_FOR_SPEED
void
mnwGraphics_SetForeColor(mnwGraphics *graph, int color);
#endif
void
mnwGraphics_SetBackColor(mnwGraphics *graph, int color);
void
mnwGraphics_DrawPixel(mnwGraphics *graph, int x, int y);
void
mnwGraphics_DrawLine(mnwGraphics *graph,
		     int x1, int y1, int x2, int y2);
void
mnwGraphics_DrawRect(mnwGraphics *graph, mnwRect *rect);
void
mnwGraphics_DrawFrame3D(mnwGraphics *graph, mnwRect *rect);
void
mnwGraphics_FillRect(mnwGraphics *graph, mnwRect *rect);
void
mnwGraphics_InvertRect(mnwGraphics *graph, mnwRect *rect);
void
mnwGraphics_CopyRect(mnwGraphics *graph, mnwRect *rect, int dx, int dy);
void
mnwGraphics_BitBlt(mnwGraphics *self,
		   int dst_x, int dst_y, int src_x, int src_y,
		   int width, int height,
		   void *pixmap, int pixmapwidth, int pixmapheight);
void *
mnwGraphics_CreatePixmap(mnwGraphics *self,
			 void *pixmap_buf, int pixmapwidth, int pixmapheight);
void
mnwGraphics_DestroyPixmap(mnwGraphics *self, void *pixmap);
void
mnwGraphics_DrawPixmap(mnwGraphics *self,
		       int dst_x, int dst_y, int src_x, int src_y,
		       void *pixmap, int width, int height);
void
mnwGraphics_SetFont(mnwGraphics *graph, int font);
void
mnwGraphics_SetAttr(mnwGraphics *self, int attr);
void
mnwGraphics_DrawText(mnwGraphics *graph, int x, int y, char *text, int len);
void
mnwGraphics_DrawTextWithClipping(mnwGraphics *self, int x, int y,
				 char *text, int len,
				 int clipx1, int clipy1,
				 int clipx2, int clipy2);
#ifdef MNW_ROTATED_TEXT
void
mnwGraphics_DrawTextRotated(mnwGraphics *self, int x, int y,
			    char *text, int len, int direction);
#endif

void
mnwEvent_Setup(void);
void
mnwEvent_Tidy(void);
void
mnwEvent_Initialize(mnwEvent *self);
void
mnwEvent_Finalize(mnwEvent *self);
void
mnwEvent_SetAll(mnwEvent *self, int type, void *param0, int param1, int param2);
bool
mnwEvent_SendToDirect(mnwEvent *self, mnwWindow *dst);
bool
mnwEvent_SendTo(mnwEvent *self, mnwWindow *dst);
bool
mnwEvent_Post(mnwEvent *self);
void
mnwEvent_SetFocusWindow(mnwWindow *win);
mnwWindow *
mnwEvent_GetFocus(void);
mnwEvent *
mnwEvent_GetNext(bool block);
bool
mnwEvent_Dispatch(mnwEvent *event);
bool
mnwEvent_Translate(mnwEvent *event);
bool
mnwEvent_TestModifier(mnwEvent *event, int modifier);
bool
mnwEvent_SetTimer(int no, mnwWindow *target, long interval_msec);
bool
mnwEvent_ResetTimer(int no);

bool
mnwWindow_Register(mnwWindow *self, mnwWindowCallback callback,
		   mnwWindow *parent, mnwRect *rect);
bool
mnwWindow_UnRegister(mnwWindow *self);

void
mnwDPrintf(char *format, ...);
char *
mnwEventToString(mnwEvent *event);
void
mnwPrintEvent(mnwEvent *event);
int
mnwGetFontWidth(int font_id);
int
mnwGetFontHeight(int font_id);
bool
mnwIsKanjiFirstByte(byte c);
bool
mnwIsKanjiSecondByte(byte c);
bool
mnwIsIMEOn(void);
bool
mnwToggleIME(void);
byte *
mnwParseMGR(byte *mrp, int *width, int *height, int *size);

int
mnwMsgBox(char *msg, int flags);

/* -- shotcut */
#ifdef MNW_USE_SHORTCUT
#define mnwFlush() mnwGraphics_Flush(gDefaultGraphics)
#define mnwSetForeColor(color) mnwGraphics_SetForeColor(gDefaultGraphics, (color))
#define mnwSetBackColor(color) mnwGraphics_SetBackColor(gDefaultGraphics, (color))
#define mnwDrawLine(x1, y1, x2, y2) mnwGraphics_DrawLine(gDefaultGraphics, (x1), (y1), (x2), (y2))
#define mnwDrawPixel(x, y) mnwGraphics_DrawPixel(gDefaultGraphics, (x), (y))
#define mnwDrawRect(rect) mnwGraphics_DrawRect(gDefaultGraphics, (rect))
#define mnwDrawFrame3D(rect) mnwGraphics_DrawFrame3D(gDefaultGraphics, (rect))
#define mnwFillRect(rect) mnwGraphics_FillRect(gDefaultGraphics, (rect))
#define mnwInvertRect(rect) mnwGraphics_InvertRect(gDefaultGraphics, (rect))
#define mnwCopyRect(rect, dx, dy) mnwGraphics_CopyRect(gDefaultGraphics, (rect), (dx), (dy))
#define mnwBitBlt(dst_x, dst_y, src_x, src_y, width, height, pixmap, pixmapwidth, pixmapheight) mnwGraphics_BitBlt(gDefaultGraphics, (dst_x), (dst_y), (src_x), (src_y), (width), (height), (pixmap), (pixmapwidth), (pixmapheight))
#define mnwCreatePixmap(pixmap_buf, pixmapwidth, pixmapheight) mnwGraphics_CreatePixmap(gDefaultGraphics, (pixmap_buf), (pixmapwidth), (pixmapheight))
#define mnwDestroyPixmap(pixmap) mnwGraphics_DestroyPixmap(gDefaultGraphics, (pixmap))
#define mnwDrawPixmap(dst_x, dst_y, src_x, src_y, pixmap, width, height) mnwGraphics_DrawPixmap(gDefaultGraphics, (dst_x), (dst_y), (src_x), (src_y), (pixmap), (width), (height))
#define mnwSetFont(font) mnwGraphics_SetFont(gDefaultGraphics, (font))
#define mnwSetAttr(attr) mnwGraphics_SetAttr(gDefaultGraphics, (attr))
#define mnwDrawText(x, y, text, len) mnwGraphics_DrawText(gDefaultGraphics, (x), (y), (text), (len))
#define mnwDrawTextWithClipping(x, y, text, len, clipx1, clipy1, clipx2, clipy2) mnwGraphics_DrawTextWithClipping(gDefaultGraphics, (x), (y), (text), (len), (clipx1), (clipy1), (clipx2), (clipy2))
#ifdef MNW_ROTATED_TEXT
#define mnwDrawTextRotated(x, y, text, len, direction) mnwGraphics_DrawTextRotated(gDefaultGraphics, (x), (y), (text), (len), (direction))
#endif
#endif

/* -- optional prototype */

#ifdef MGL_PROTOTYPE
int get_key_im(int time_out);
int get_key(int time_out);
int refresh(void);
int open_graph(void);
int close_graph(void);
int set_color(int col);
int draw_line(int x1, int y1, int x2, int y2);
int draw_rect(int x, int y, int xs, int ys);
int fill_rect(int x, int y, int xs, int ys);
int bitblt(struct screen *dst, int dx, int dy,
	   struct screen *src, int sx,int sy,
	   int xsize, int ysize,int op);
int free_screen(struct screen *s);
int set_font(int size, int attr);
int draw_font(int x, int y, int code);
int draw_font_clipping(int x, int y, int code, int x1, int y1, int x2, int y2);
int draw_font_rotated(int x, int y, int code, int direction);
int toggle_im_mode(void);
int get_im_mode(void);
int draw_pixel(int x, int y);
int load_font(char *fname, int width, int height);
#endif

#ifdef MNW_USE_TRUETYPE
unsigned short
conv_JISX0201toUCS2(unsigned short src);
unsigned short
conv_JISX0208toUCS2(unsigned short src);
unsigned short
conv_JISX0212toUCS2(unsigned short src);
bool
mnwTT_Initialize(char *file_name);
bool
mnwTT_Finalize(void);
int
mnwTTFont_Create(int point_size, int res, bool gray_render);
void
mnwTT_DrawText(int id, int x, int y, char *text, int len);
void
mnwTT_DrawFontRotated(int id, int x, int y, int code, int direction);
void
mnwTT_DrawTextRotated(int id, int x, int y,
		      char *text, int len, int direction);
int
mnwTT_GetFontWidth(int id);
int
mnwTT_GetFontHeight(int id);
#endif

#endif
