/*
   MNW button compo
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwbutt.c,v 1.1.1.1 1999/02/28 06:47:46 furukawa Exp $
   */

#define MNW_USE_SHORTCUT
#include "mnw.h"
#include <string.h>

static bool
mnwButton_SetFont(mnwButton *self, int font);

/* -- Button */

static bool
mnwButton_Initialize(mnwButton *self)
{
    self->type = kWindowType_Compo;
    self->focused = false;
    self->buffer = "";
    self->buffer_len = 0;
    self->text_len = 0;
    self->command = 0;
    mnwButton_SetFont(self, kFont_16);

    return true;
}

static bool
mnwButton_Finalize(mnwButton *self)
{
    return true;
}

static bool
mnwButton_ResetRect(mnwButton *self, mnwRect *rect)
{
    if (rect->height > self->font_height + 6){
	self->max_col = (rect->width - 6) / self->font_width;
    }else{
	self->max_col = 0;
    }
    return true;
}

static bool
mnwButton_SetFont(mnwButton *self, int font)
{
    self->font = font;
    self->font_width = mnwGetFontWidth(self->font);
    self->font_height = mnwGetFontHeight(self->font);
    return mnwButton_ResetRect(self, &(self->rect));
}

static bool
mnwButton_SetCommand(mnwButton *self, int command)
{
    self->command = command;
    return true;
}

static bool
mnwButton_KeyDown(mnwButton *self, mnwEvent *event)
{
    mnwEvent e;

    if (event->param1 == kKey_RETURN){
	mnwEvent_SetAll(&e, kEvent_Command, nil, self->command, 0);
	mnwEvent_SendTo(&e, self->parent);
	return true;
    }
    return false;
}

static bool
mnwButton_Redraw(mnwButton *self, mnwRect *rect)
{
    int offset_x, offset_y;
    mnwRect r;

    if (self->visible){
	mnwSetForeColor(self->back_color);
	mnwFillRect(&(self->rect));
	mnwSetForeColor(kColor_Black);
	mnwDrawRect(&(self->rect));
	mnwRect_SetXYWH(&r,
			self->rect.x + 1, self->rect.y + 1,
			self->rect.width - 2, self->rect.height - 2);
	mnwDrawFrame3D(&r);
	offset_x = 3;
	offset_y = 3;
	if (self->rect.width - 6 - self->text_len*self->font_width > 0){
	    offset_x += (self->rect.width - 6 - self->text_len*self->font_width) / 2;
	}
	mnwSetForeColor(self->fore_color);
	mnwSetFont(self->font);
	mnwDrawText(self->rect.x + offset_x, self->rect.y + offset_y + 1,
		    self->buffer, min(self->text_len, self->max_col));
	if (self->focused){
	    mnwRect_SetXYWH(&r,
			    self->rect.x + 3, self->rect.y + 3,
			    self->rect.width - 6, self->rect.height - 6);
	    mnwInvertRect(&r);
	}
    }
    return true;
}

bool
mnwButton_Callback(mnwButton *self, mnwEvent *event)
{
    mnwEvent e;

    switch (event->type){
	case kEvent_Create:
	mnwWindow_Callback((mnwWindow *)self, event); /* super */
	return mnwButton_Initialize(self);
	break;
	case kEvent_Destroy:
	return mnwButton_Finalize(self);
	break;
	case kEvent_AddChild:
	case kEvent_RemoveChild:
	return false;
	break;
	case kEvent_GainFocus:
	self->focused = true;
	mnwEvent_SetAll(&e, kEvent_GainFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return mnwButton_Redraw(self, nil);
	break;
	case kEvent_LostFocus:
	self->focused = false;
	mnwEvent_SetAll(&e, kEvent_LostFocusNotify, self, self->no, self->id);
	mnwEvent_SendTo(&e, self->parent);
	return mnwButton_Redraw(self, nil);
	break;
	case kEvent_SetBuffer:
	self->buffer = event->param0;
	self->buffer_len = event->param1;
	return true;
	break;
	case kEvent_SetText:
	self->text_len = min(event->param1, self->buffer_len);
	strncpy(self->buffer, event->param0, self->text_len);
	return true;
	break;
	case kEvent_Redraw:
	return mnwButton_Redraw(self, nil);
	break;
	case kEvent_SetFont:
	return mnwButton_SetFont(self, event->param1);
	break;
	case kEvent_SetCommand:
	return mnwButton_SetCommand(self, event->param1);
	break;
	case kEvent_KeyDown:
	return mnwButton_KeyDown(self, event);
	break;
	default:
	/* superdelegate */
	return mnwWindow_Callback((mnwWindow *)self, event);
	break;
    }

    /* ٥Ȥʤ */
    return false;
}
