/*
   MNW graphics
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwgraph.c,v 1.3 1999/03/08 07:17:52 furukawa Exp $
   */

#include "mnw.h"

#ifdef MNW_USE_GGI
ggi_visual_t gDefaultVisual;
static ggi_mode vis_mode;
static ggi_color black  = { 0x0000, 0x0000, 0x0000 };
static ggi_color dkgray = { 0x5555, 0x5555, 0x5555 };
static ggi_color ltgray = { 0xAAAA, 0xAAAA, 0xAAAA };
static ggi_color white  = { 0xFFFF, 0xFFFF, 0xFFFF };
static int tt_id[10];
#endif

/* -- Rect */

/* -- method */

void
mnwRect_SetXYXY(mnwRect *self, int x1, int y1, int x2, int y2)
{
    self->x = x1;
    self->y = y1;
    self->width = x2 - x1 + 1;
    self->height = y2 - y1 + 1;
}

void
mnwRect_SetXYWH(mnwRect *self, int x1, int y1, int w, int h)
{
    self->x = x1;
    self->y = y1;
    self->width = w;
    self->height = h;
}

void
mnwRect_Inset(mnwRect *self, int x_offset, int y_offset)
{
    self->x += x_offset;
    self->y += y_offset;
    self->width -= x_offset*2;
    self->height -= y_offset*2;
}

void
mnwRect_Offset(mnwRect *self, int x_offset, int y_offset)
{
    self->x += x_offset;
    self->y += y_offset;
}

void
mnwRect_Copy(mnwRect *dst, mnwRect *src)
{
    dst->x = src->x;
    dst->y = src->y;
    dst->width = src->width;
    dst->height = src->height;
}

/* -- Graphics */

/* -- method */

void
mnwGraphics_Initialize(mnwGraphics *self)
{
#ifdef MNW_USE_GGI
    /* GGIν */
    if (ggiInit() != 0){
	return;
    }
    gDefaultVisual = ggiOpen(NULL, NULL);
    if (gDefaultVisual == NULL){
	ggiExit();
	return;
    }
    ggiSetFlags(gDefaultVisual, GGIFLAG_ASYNC);
    ggiCheckMode(gDefaultVisual, &vis_mode);
    if (ggiSetMode(gDefaultVisual, &vis_mode) < 0){
	ggiClose(gDefaultVisual);
	ggiExit();
	return;
    }
    if (GT_SCHEME(vis_mode.graphtype) == GT_PALETTE){
	ggiSetColorfulPalette(gDefaultVisual);
    }
    ggiSetGCForeground(gDefaultVisual, ggiMapColor(gDefaultVisual, &white));
    ggiFillscreen(gDefaultVisual);

    mnwTT_Initialize("/home/furukawa/loft/font/dyna/DFGsk7.ttc");
    tt_id[kFont_12] = mnwTTFont_Create(12, 72, true);
    tt_id[kFont_16] = mnwTTFont_Create(16, 72, true);
    tt_id[kFont_24] = mnwTTFont_Create(24, 72, true);
#else
    /* mglν */
    open_graph();
    /* 10x8 եȤΥ */
    load_font("k10x8.fnt", 10, 8);
#endif

    self->fore_color = kColor_Black;
    self->back_color = kColor_White;
    self->pen_x = 0;
    self->pen_y = 0;
    self->pen_color = kColor_Black;
    self->pen_size = 1;
    self->font = kFont_12;
    self->font_width = 12;
    self->font_height = 12;
}

void
mnwGraphics_Finalize(mnwGraphics *self)
{
#ifdef MNW_USE_GGI
    /* GGIνλ */
    ggiClose(gDefaultVisual);
    ggiExit();
#else
    /* mglνλ */
    close_graph();
#endif
}

void
mnwGraphics_Flush(mnwGraphics *self)
{
#ifdef MNW_USE_GGI
    ggiFlush(gDefaultVisual);
#else
    refresh();
#endif
}

void
mnwGraphics_SetForeColor(mnwGraphics *self, int color)
{
    self->fore_color = color;
    /* NOTE: ΤȤͤƱ */
#ifdef MNW_USE_GGI
    switch (color){
    case kColor_White:
	ggiSetGCForeground(gDefaultVisual, ggiMapColor(gDefaultVisual, &white)); break;
    case kColor_LtGray:
	ggiSetGCForeground(gDefaultVisual, ggiMapColor(gDefaultVisual, &ltgray)); break;
    case kColor_DkGray:
	ggiSetGCForeground(gDefaultVisual, ggiMapColor(gDefaultVisual, &dkgray)); break;
    case kColor_Black:
	ggiSetGCForeground(gDefaultVisual, ggiMapColor(gDefaultVisual, &black)); break;
    }
#else
    set_color(color);
#endif
}

void
mnwGraphics_SetBackColor(mnwGraphics *self, int color)
{
    self->back_color = color;
    /* NOTE: ߤΤȤ̣ʤ */
#ifdef MNW_USE_GGI
    switch (color){
    case kColor_White:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &white)); break;
    case kColor_LtGray:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &ltgray)); break;
    case kColor_DkGray:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &dkgray)); break;
    case kColor_Black:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &black)); break;
    }
#endif
}

void
mnwGraphics_DrawPixel(mnwGraphics *self, int x, int y)
{
#ifdef MNW_USE_GGI
    ggiDrawPixel(gDefaultVisual, x, y);
#else
    draw_pixel(x, y);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
}

void
mnwGraphics_DrawLine(mnwGraphics *self,
		     int x1, int y1, int x2, int y2)
{
#ifdef MNW_USE_GGI
    ggiDrawLine(gDefaultVisual, x1, y1, x2, y2);
#else
    draw_line(x1, y1, x2, y2);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}

void
mnwGraphics_DrawRect(mnwGraphics *self, mnwRect *rect)
{
#ifdef MNW_USE_GGI
    ggiDrawLine(gDefaultVisual, rect->x, rect->y, rect->x + rect->width - 1, rect->y);
    ggiDrawLine(gDefaultVisual, rect->x, rect->y, rect->x, rect->y + rect->height - 1);
    ggiDrawLine(gDefaultVisual, rect->x, rect->y + rect->height - 1, rect->x + rect->width - 1, rect->y + rect->height - 1);
    ggiDrawLine(gDefaultVisual, rect->x + rect->width - 1, rect->y, rect->x + rect->width - 1, rect->y + rect->height - 1);
    /* TODO: Use HLine and VLine */
#else
    draw_rect(rect->x, rect->y, rect->width, rect->height);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}

void
mnwGraphics_DrawFrame3D(mnwGraphics *self, mnwRect *rect)
{
#ifdef MNW_USE_GGI
    mnwGraphics_SetForeColor(self, kColor_White);
    ggiDrawLine(gDefaultVisual, rect->x, rect->y, rect->x + rect->width - 1, rect->y);
    ggiDrawLine(gDefaultVisual, rect->x, rect->y, rect->x, rect->y + rect->height - 1);
    mnwGraphics_SetForeColor(self, kColor_DkGray);
    ggiDrawLine(gDefaultVisual, rect->x, rect->y + rect->height - 1, rect->x + rect->width - 1, rect->y + rect->height - 1);
    ggiDrawLine(gDefaultVisual, rect->x + rect->width - 1, rect->y, rect->x + rect->width - 1, rect->y + rect->height - 1);
#else
    mnwGraphics_SetForeColor(self, kColor_White);
    draw_line(rect->x, rect->y, rect->x + rect->width - 1, rect->y);
    draw_line(rect->x, rect->y, rect->x, rect->y + rect->height - 1);
    mnwGraphics_SetForeColor(self, kColor_DkGray);
    draw_line(rect->x, rect->y + rect->height - 1, rect->x + rect->width - 1, rect->y + rect->height - 1);
    draw_line(rect->x + rect->width - 1, rect->y, rect->x + rect->width - 1, rect->y + rect->height - 1);
#endif
    mnwGraphics_Flush(self);
}

void
mnwGraphics_FillRect(mnwGraphics *self, mnwRect *rect)
{
#ifdef MNW_USE_GGI
    switch (self->fore_color){
    case kColor_White:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &white)); break;
    case kColor_LtGray:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &ltgray)); break;
    case kColor_DkGray:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &dkgray)); break;
    case kColor_Black:
	ggiSetGCBackground(gDefaultVisual, ggiMapColor(gDefaultVisual, &black)); break;
    }
    ggiDrawBox(gDefaultVisual, rect->x, rect->y, rect->width, rect->height);
#else
    fill_rect(rect->x, rect->y, rect->width, rect->height);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}

void
mnwGraphics_InvertRect(mnwGraphics *self, mnwRect *rect)
{
#ifdef MNW_USE_GGI
    /* TODO */
#else
    set_color(4);			      /* invert */
    fill_rect(rect->x, rect->y, rect->width, rect->height);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
    set_color(kColor_Black);
#endif
}

void
mnwGraphics_CopyRect(mnwGraphics *self, mnwRect *rect, int dx, int dy)
{
#ifdef MNW_USE_GGI
    ggiCopyBox(gDefaultVisual, rect->x, rect->y, rect->width, rect->height,
	       rect->x + dx, rect->y + dy);
#else
    bitblt((struct screen *)0, rect->x + dx, rect->y + dy,
	   (struct screen *)0, rect->x, rect->y, rect->width, rect->height, 0);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}

void
mnwGraphics_BitBlt(mnwGraphics *self,
		   int dst_x, int dst_y, int src_x, int src_y,
		   int width, int height,
		   void *pixmap, int pixmapwidth, int pixmapheight)
{
#ifdef MNW_USE_GGI
    /* TODO */
#else
    struct screen *offscreen;
    char *src = pixmap;
    int i;

    offscreen = create_memscreen(pixmapwidth, pixmapheight, NULL);
    
    for (i = 0; i < offscreen->wbytes*offscreen->height; i++){
	(offscreen->bitmap)[i] = ((src[i]&(0x03<<0))<<6)|((src[i]&(0x03<<2))<<2)|((src[i]&(0x03<<4))>>2)|((src[i]&(0x03<<6))>>6);
    }

    bitblt((struct screen *)0, dst_x, dst_y,
	   offscreen, src_x, src_y,
	   width, height, 0);
    free_screen(offscreen);

    /* NOTE: this is internal function. do not use */
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}


void *
mnwGraphics_CreatePixmap(mnwGraphics *self,
			 void *pixmap_buf, int pixmapwidth, int pixmapheight)
{
#ifdef MNW_USE_GGI
    /* TODO */
#else
    struct screen *offscreen;
    offscreen = create_memscreen(pixmapwidth, pixmapheight, pixmap_buf);
    return (void *)offscreen;
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
}

void
mnwGraphics_DestroyPixmap(mnwGraphics *self, void *pixmap)
{
#ifdef MNW_USE_GGI
    /* TODO */
#else
    free_screen(pixmap);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
}

void
mnwGraphics_DrawPixmap(mnwGraphics *self,
		       int dst_x, int dst_y, int src_x, int src_y,
		       void *pixmap, int width, int height)
{
#ifdef MNW_USE_GGI
    /* TODO */
#else
    bitblt((struct screen *)0, dst_x, dst_y,
	   (struct screen *)pixmap, src_x, src_y,
	   width, height, 0);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}


void
mnwGraphics_SetFont(mnwGraphics *self, int font)
{
#ifdef MNW_USE_GGI
    self->font = font;
    switch (font){
	default:
	case kFont_Default:
	case kFont_12:
	self->font_width = 12;
	self->font_height = 12;
	break;
	case kFont_16:
	self->font_width = 16;
	self->font_height = 16;
	break;
	case kFont_24:
	self->font_width = 24;
	self->font_height = 24;
	break;
	case kFont_16x8:
	self->font_width = 16;
	self->font_height = 8;
	break;
	case kFont_10x8:
	self->font_width = 10;
	self->font_height = 8;
	break;
    }
#else
    self->font = font;
    switch (font){
	default:
	case kFont_Default:
	case kFont_12:
	set_font(12, 0);
	self->font_width = 12;
	self->font_height = 12;
	break;
	case kFont_16:
	set_font(16, 0);
	self->font_width = 16;
	self->font_height = 16;
	break;
	case kFont_24:
	set_font(24, 0);
	self->font_width = 24;
	self->font_height = 24;
	break;
	case kFont_16x8:
	set_font(7, 0);
	self->font_width = 16;
	self->font_height = 8;
	break;
	case kFont_10x8:
	set_font(8, 0);
	self->font_width = 10;
	self->font_height = 8;
	break;
    }
#endif
}

void
mnwGraphics_SetAttr(mnwGraphics *self, int attr)
{
#ifdef MNW_USE_GGI
    /* TODO */
#else
    switch (attr){
	case kAttr_Bold:
	set_font(self->font_width, FA_BOLD);
	break;
	case kAttr_Italic:
	set_font(self->font_width, FA_ITALIC);
	break;
	default:
	set_font(self->font_width, 0);
	break;
    }
#endif
}

void
mnwGraphics_DrawText(mnwGraphics *self, int x, int y, char *text, int len)
{
    int remain;
    int code;
    char *p;

    remain = len;
    p = text;

    while (*p && remain > 0){
	if (*p & 0x80) {
	    /* EUC 2byte char */
	    if (remain < 2){
		return;
	    }
	    code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
#ifdef MNW_USE_GGI
	    mnwTT_DrawFontRotated(tt_id[self->font], x, y, code, kDIR_NORTH);
#else
	    draw_font(x, y, code);
#endif
	    if ((p[0]&0xff) == 0x8E){
		/* EUC x0201 kana */
		x += self->font_width / 2;
	    }else{
		x += self->font_width;
	    }
	    p += 2;
	    remain -= 2;
	}else{
	    /* 1byte char */
#ifdef MNW_USE_GGI
	    mnwTT_DrawFontRotated(tt_id[self->font], x, y, *p & 0xff, kDIR_NORTH);
#else
	    draw_font(x, y, *p & 0xff);
#endif
	    x += self->font_width / 2;
	    p++;
	    remain--;
	}
    }
    mnwGraphics_Flush(self);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
}

void
mnwGraphics_DrawTextWithClipping(mnwGraphics *self, int x, int y,
				 char *text, int len,
				 int clipx1, int clipy1,
				 int clipx2, int clipy2)
{
#ifdef MNW_USE_GGI
    /* TODO */
#else
    int remain;
    int code;
    char *p;

    remain = len;
    p = text;

    while (*p && remain > 0){
	if (*p & 0x80) {
	    /* EUC 2byte char */
	    if (remain < 2){
		return;
	    }
	    code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
	    draw_font_clipping(x, y, code, clipx1, clipy1, clipx2, clipy2);
	    if ((p[0]&0xff) == 0x8E){
		/* EUC x0201 kana */
		x += self->font_width / 2;
	    }else{
		x += self->font_width;
	    }
	    p += 2;
	    remain -= 2;
	}else{
	    /* 1byte char */
	    draw_font_clipping(x, y, *p&0xff, clipx1, clipy1, clipx2, clipy2);
	    x += self->font_width / 2;
	    p++;
	    remain--;
	}
    }
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}


/* --- ʲʸβžɽΤΥ󥿥ե */
/* --- ¿Υץꥱΰܿΰ٤ˤɬܤǤϤʤ */
#ifdef MNW_ROTATED_TEXT

static bool mnw_is_zurasi_char(char *text);
static bool mnw_is_a_bit_zurasi_char(char *text);
static bool mnw_is_rotate_char(char *text);

/* ʸβžɽ
   ܸʣХʸˤϽĽ񤭡ѻʣХʸˤϲ񤭤

   (x,y)̸ξκꤹ

   åԥ󥰤ϥݡȤʤ
   Τ餷ɽ뤿ᡤΰˤϽʬդΤ
   */
void
mnwGraphics_DrawTextRotated(mnwGraphics *self, int x, int y,
			    char *text, int len, int direction)
{
#ifdef MNW_USE_GGI
    mnwTT_DrawTextRotated(tt_id[self->font], x, y, text, len, direction);
#else
    int remain;
    int code;
    char *p;

    remain = len;
    p = text;

    switch (direction){
	default:
	case kDIR_NORTH:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    draw_font_rotated(x + self->font_width/2,
				      y - self->font_height/2,
				      code, kDIR_NORTH);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    draw_font_rotated(x + self->font_width/4,
				      y - self->font_height/4,
				      code, kDIR_NORTH);
		}else if (mnw_is_rotate_char(p)){
		    draw_font_rotated(x, y, code, kDIR_EAST);
		}else{
		    draw_font_rotated(x, y, code, kDIR_NORTH);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    y += self->font_height/2;
		}else{
		    y += self->font_height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		draw_font_rotated(x, y, *p & 0xff, kDIR_EAST);
		y += self->font_width/2;
		p++;
		remain--;
	    }
	}
	break;

	case kDIR_SOUTH:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    draw_font_rotated(x - self->font_width - self->font_width/2,
				      y - self->font_height + self->font_height/2,
				      code, kDIR_SOUTH);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    draw_font_rotated(x  - self->font_width - self->font_width/4,
				      y - self->font_height + self->font_height/4,
				      code, kDIR_SOUTH);
		}else if (mnw_is_rotate_char(p)){
		    draw_font_rotated(x - self->font_width,
				      y - self->font_height,
				      code, kDIR_WEST);
		}else{
		    draw_font_rotated(x - self->font_width,
				      y - self->font_height,
				      code, kDIR_SOUTH);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    y -= self->font_height/2;
		}else{
		    y -= self->font_height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		draw_font_rotated(x - self->font_height,
				  y - self->font_width,
				  *p & 0xff, kDIR_WEST);
		y -= self->font_width/2;
		p++;
		remain--;
	    }
	}
	break;

	case kDIR_WEST:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    draw_font_rotated(x - self->font_height/2,
				      y - self->font_width - self->font_width/2,
				      code, kDIR_WEST);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    draw_font_rotated(x - self->font_height/4,
				      y - self->font_width  - self->font_width/4,
				      code, kDIR_WEST);
		}else if (mnw_is_rotate_char(p)){
		    draw_font_rotated(x, y - self->font_width,
				      code, kDIR_NORTH);
		}else{
		    draw_font_rotated(x, y - self->font_width,
				      code, kDIR_WEST);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    x += self->font_height/2;
		}else{
		    x += self->font_height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		draw_font_rotated(x, y - self->font_height,
				  *p & 0xff, kDIR_NORTH);
		x += self->font_width/2;
		p++;
		remain--;
	    }
	}
	break;

	case kDIR_EAST:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    draw_font_rotated(x - self->font_height + self->font_height/2,
				      y + self->font_width/2,
				      code, kDIR_EAST);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    draw_font_rotated(x - self->font_height + self->font_height/4,
				      y + self->font_width/4,
				      code, kDIR_EAST);
		}else if (mnw_is_rotate_char(p)){
		    draw_font_rotated(x - self->font_height, y,
				      code, kDIR_SOUTH);
		}else{
		    draw_font_rotated(x - self->font_height, y,
				      code, kDIR_EAST);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    x -= self->font_height/2;
		}else{
		    x -= self->font_height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		draw_font_rotated(x - self->font_width,
				  y,
				  *p & 0xff, kDIR_SOUTH);
		x -= self->font_width/2;
		p++;
		remain--;
	    }
	}
	break;
    }

    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
#endif
    mnwGraphics_Flush(self);
}

#ifdef MNW_USE_TRUETYPE
/* ʸβžɽ
   ܸʣХʸˤϽĽ񤭡ѻʣХʸˤϲ񤭤

   (x,y)̸ξκꤹ

   åԥ󥰤ϥݡȤʤ
   Τ餷ɽ뤿ᡤΰˤϽʬդΤ
   */
void
mnwTT_DrawTextRotated(int id, int x, int y,
		      char *text, int len, int direction)
{
    int remain;
    int code;
    char *p;
    int width, height;

    remain = len;
    p = text;

    width = mnwTT_GetFontWidth(id);
    height = mnwTT_GetFontHeight(id);

    switch (direction){
	default:
	case kDIR_NORTH:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x + width/2,
				      y - height/2,
				      code, kDIR_NORTH);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x + width/4,
				      y - height/4,
				      code, kDIR_NORTH);
		}else if (mnw_is_rotate_char(p)){
		    mnwTT_DrawFontRotated(id, x, y, code, kDIR_EAST);
		}else{
		    mnwTT_DrawFontRotated(id, x, y, code, kDIR_NORTH);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    y += height/2;
		}else{
		    y += height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		mnwTT_DrawFontRotated(id, x, y, *p & 0xff, kDIR_EAST);
		y += width/2;
		p++;
		remain--;
	    }
	}
	break;

	case kDIR_SOUTH:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x - width - width/2,
				      y - height + height/2,
				      code, kDIR_SOUTH);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x  - width - width/4,
				      y - height + height/4,
				      code, kDIR_SOUTH);
		}else if (mnw_is_rotate_char(p)){
		    mnwTT_DrawFontRotated(id, x - width,
				      y - height,
				      code, kDIR_WEST);
		}else{
		    mnwTT_DrawFontRotated(id, x - width,
				      y - height,
				      code, kDIR_SOUTH);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    y -= height/2;
		}else{
		    y -= height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		mnwTT_DrawFontRotated(id, x - height,
				  y - width,
				  *p & 0xff, kDIR_WEST);
		y -= width/2;
		p++;
		remain--;
	    }
	}
	break;

	case kDIR_WEST:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x - height/2,
				      y - width - width/2,
				      code, kDIR_WEST);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x - height/4,
				      y - width  - width/4,
				      code, kDIR_WEST);
		}else if (mnw_is_rotate_char(p)){
		    mnwTT_DrawFontRotated(id, x, y - width,
				      code, kDIR_NORTH);
		}else{
		    mnwTT_DrawFontRotated(id, x, y - width,
				      code, kDIR_WEST);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    x += height/2;
		}else{
		    x += height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		mnwTT_DrawFontRotated(id, x, y - height,
				  *p & 0xff, kDIR_NORTH);
		x += width/2;
		p++;
		remain--;
	    }
	}
	break;

	case kDIR_EAST:
	while (*p && remain > 0){
	    if (*p & 0x80) {
		/* EUC 2byte char */
		if (remain < 2){
		    return;
		}
		code = ((p[0]<<8)&0xff00)|(p[1]&0xff);
		if (mnw_is_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x - height + height/2,
				      y + width/2,
				      code, kDIR_EAST);
		}else if (mnw_is_a_bit_zurasi_char(p)){
		    mnwTT_DrawFontRotated(id, x - height + height/4,
				      y + width/4,
				      code, kDIR_EAST);
		}else if (mnw_is_rotate_char(p)){
		    mnwTT_DrawFontRotated(id, x - height, y,
				      code, kDIR_SOUTH);
		}else{
		    mnwTT_DrawFontRotated(id, x - height, y,
				      code, kDIR_EAST);
		}
		if ((p[0]&0xff) == 0x8E){
		    /* EUC x0201 kana */
		    x -= height/2;
		}else{
		    x -= height;
		}
		p += 2;
		remain -= 2;
	    }else{
		/* 1byte char */
		mnwTT_DrawFontRotated(id, x - width,
				  y,
				  *p & 0xff, kDIR_SOUTH);
		x -= width/2;
		p++;
		remain--;
	    }
	}
	break;
    }

    mnwGraphics_Flush(gDefaultGraphics);
    /* NOTE: ꡼ڤؤϥݡȤƤʤ */
}
#endif


#define mnw_wcharcmp(text1, text2) (((text1)[0]==(text2)[0])&&((text1)[1]==(text2)[1]))

/* 餷ɽʸ
   ʤ
   */
static bool
mnw_is_zurasi_char(char *text)
{
    return mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text);
}

static bool
mnw_is_a_bit_zurasi_char(char *text)
{
    return mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text);
}

/* ٲžɽʸ
 */
static bool
mnw_is_rotate_char(char *text)
{
    return mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text)
	|| mnw_wcharcmp("", text);
}

#endif
