/*
   MNW utility
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwutil.c,v 1.2 1999/03/08 07:17:56 furukawa Exp $
   */

#include "mnw.h"
#include <stdio.h>
#include <stdarg.h>

/* -- utility */

void
mnwDPrintf(char *format, ...)
{
    char msg[128];
    va_list ap;

    va_start(ap, format);
    vsprintf(msg, format, ap);
    fprintf(stderr, "%s\n", msg);
    va_end(ap);
}

char *
mnwEventToString(mnwEvent *event)
{
    static char name[80];

    switch (event->type){
	case kEvent_None:
	return "None";
	case kEvent_Create:
	return "Create";
	case kEvent_Destroy:
	return "Destroy";
	case kEvent_Show:
	return "Show";
	case kEvent_Hide:
	return "Hide";
	case kEvent_SetFocus:
	return "SetFocus";
	case kEvent_GainFocus:
	return "GainFocus";
	case kEvent_LostFocus:
	return "LostFocus";
	case kEvent_Timer:
	return "Timer";
	case kEvent_KeyDown:
	return "KeyDown";
	case kEvent_KeyUp:
	return "KeyUp";
	case kEvent_Command:
	return "Command";
	case kEvent_Redraw:
	return "Redraw";
	case kEvent_Quit:
	return "Quit";
	case kEvent_SetRect:
	return "SetRect";
	case kEvent_SetParent:
	return "SetParent";
	case kEvent_AddChild:
	return "AddChild";
	case kEvent_RemoveChild:
	return "RemoveChild";
	case kEvent_SetForeColor:
	return "SetForeColor";
	case kEvent_SetBackColor:
	return "SetBackColor";
	case kEvent_GetRect:
	return "GetRect";
	case kEvent_SetID:
	return "SetID";
	case kEvent_GetID:
	return "GetID";
	case kEvent_SetTabStop:
	return "SetTabStop";
	case kEvent_DoModal:
	return "DoModal";
	case kEvent_EndModal:
	return "EndModal";
	case kEvent_Cut:
	return "Cut";
	case kEvent_Paste:
	return "Paste";
	case kEvent_Copy:
	return "Copy";
	case kEvent_Clear:
	return "Clear";
	case kEvent_SetFont:
	return "SetFont";
	case kEvent_SetAlign:
	return "SetAlign";
	case kEvent_SetStyle:
	return "SetStyle";
	case kEvent_SetBuffer:
	return "SetBuffer";
	case kEvent_SetText:
	return "SetText";
	case kEvent_UnsetText:
	return "UnsetText";
	case kEvent_InsertText:
	return "InsertText";
	case kEvent_SetCommand:
	return "SetCommand";
	case kEvent_GetResult:
	return "GetResult";
	case kEvent_SetCursor:
	return "SetCursor";
	case kEvent_GainFocusNotify:
	return "GainFocusNotify";
	case kEvent_LostFocusNotify:
	return "LostFocusNotify";
    }
    sprintf(name, "(unknown)%d", event->type);
    return name;
}

int
mnwGetFontWidth(int font_id)
{
    switch (font_id){
	case kFont_12:
	return 6;
	break;
	case kFont_16:
	return 8;
	break;
	case kFont_24:
	return 12;
	break;
	case kFont_16x8:
	return 8;
	break;
    }
    return 0;
}

int
mnwGetFontHeight(int font_id)
{
    switch (font_id){
	case kFont_12:
	return 12;
	break;
	case kFont_16:
	return 16;
	break;
	case kFont_24:
	return 24;
	break;
	case kFont_16x8:
	return 8;
	break;
    }
    return 0;
}

void
mnwPrintEvent(mnwEvent *event)
{
    fprintf(stderr, "e:%-12s to:win(%d) 0:%X 1:%X 2:%X\n",
	    mnwEventToString(event),
	    (event->target==nil)?-1:event->target->no,
	    (unsigned int)event->param0, event->param1, event->param2);
}

bool
mnwIsKanjiFirstByte(byte c)
{
    return (0xA0 < c && c < 0xFF);
}

bool
mnwIsKanjiSecondByte(byte c)
{
    return (0xA0 < c && c < 0xFF);
}

bool
mnwIsIMEOn(void)
{
#ifdef MNW_USE_GGI
    /* TODO */
    return true;
#else
    return get_im_mode();
#endif
}

bool
mnwToggleIME(void)
{
#ifdef MNW_USE_GGI
    /* TODO */
    return true;
#else
    toggle_im_mode();
    return get_im_mode();
#endif
}

static int
mnw_hex2value(byte c)
{
    if ('a' <= c && c <= 'f'){
	return c - 'a' + 10;
    }else if ('A' <= c && c <= 'A'){
	return c - 'A' + 10;
    }else{
	return c - '0';
    }
}

byte *
mnwParseMGR(byte *mgr, int *width, int *height, int *size)
{
    *width = mnw_hex2value(mgr[8]) * 1000
	+ mnw_hex2value(mgr[8+1]) * 100
	+ mnw_hex2value(mgr[8+2]) * 10
	+ mnw_hex2value(mgr[8+3]) * 1;
    *height = mnw_hex2value(mgr[12]) * 1000
	+ mnw_hex2value(mgr[12+1]) * 100
	+ mnw_hex2value(mgr[12+2]) * 10
	+ mnw_hex2value(mgr[12+3]) * 1;
    *size = (*width / 4) * *height;
    return mgr + 16;
}
