/*
   MNW window
   Copyright (c) 1998 FURUKAWA Yo.
   $Id: mnwwin.c,v 1.1.1.1 1999/02/28 06:47:49 furukawa Exp $
   */

#define MNW_USE_SHORTCUT
#include "mnw.h"

/* -- Window (generic) */

bool
mnwWindow_Register(mnwWindow *self, mnwWindowCallback callback,
		   mnwWindow *parent, mnwRect *rect)
{
    int i;
    static int serial_no = 0;
    mnwEvent event;

    self->no = serial_no++;
    self->id = 0;
    self->type = kWindowType_Generic;
    self->visible = false;
    self->fore_color = kColor_Black;
    self->back_color = kColor_LtGray;
    self->callback = callback;
    self->child_num = 0;
    for (i = 0; i < MNW_MAX_CHILD; i++){
	self->child_valid[i] = false;
	self->child[i] = nil;
    }
    mnwEvent_SetAll(&event, kEvent_SetParent, parent, 0, 0);
    mnwEvent_SendToDirect(&event, self);
    if (parent != nil){
	mnwEvent_SetAll(&event, kEvent_AddChild, self, 0, 0);
	mnwEvent_SendToDirect(&event, parent);
    }
    mnwEvent_SetAll(&event, kEvent_SetRect, rect, 0, 0);
    mnwEvent_SendToDirect(&event, self);
    mnwEvent_SetAll(&event, kEvent_Create, nil, 0, 0);
    mnwEvent_SendToDirect(&event, self);
    return true;
}

bool
mnwWindow_UnRegister(mnwWindow *self)
{
    mnwEvent event;
    mnwEvent_SetAll(&event, kEvent_RemoveChild, self, 0, 0);
    mnwEvent_SendToDirect(&event, self->parent);
    mnwEvent_SetAll(&event, kEvent_Destroy, nil, 0, 0);
    mnwEvent_SendToDirect(&event, self);
    return true;
}

/* -- Window (generic) */

static bool
mnwWindow_Initialize(mnwWindow *self)
{
    return true;
}

static bool
mnwWindow_Finalize(mnwWindow *self)
{
    self->visible = false;
    return true;
}

static bool
mnwWindow_SetRect(mnwWindow *self, mnwRect *rect)
{
    mnwRect_Copy(&(self->rect), rect);
    if (self->parent != nil){
	mnwRect_Offset(&(self->rect),
		       self->parent->rect.x, self->parent->rect.y);
    }
    return true;
}

static bool
mnwWindow_GetRect(mnwWindow *self, mnwRect *rect)
{
    mnwRect_Copy(rect, &(self->rect));
    return true;
}

static bool
mnwWindow_SetParent(mnwWindow *self, mnwWindow *parent)
{
    self->parent = parent;
    return true;
}

static bool
mnwWindow_AddChild(mnwWindow *self, mnwWindow *child)
{
    int i;

    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (!self->child_valid[i]){
	    self->child_valid[i] = true;
	    self->child[i] = child;
	    self->child_num++;
	    return true;
	}
    }
    return false;
}

static bool
mnwWindow_RemoveChild(mnwWindow *self, mnwWindow *child)
{
    int i;

    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (self->child_valid[i] && self->child[i] == child){
	    self->child_valid[i] = false;
	    self->child_num--;
	    return true;
	}
    }
    return false;
}

static bool
mnwWindow_SetFocus(mnwWindow *self)
{
    mnwEvent_SetFocusWindow(self);
    return true;
}

static bool
mnwWindow_Show(mnwWindow *self)
{
    int i;
    mnwEvent event;
    self->visible = true;
    /* send Show to the children */
    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (self->child_valid[i]){
	    mnwEvent_SetAll(&event, kEvent_Show, nil, 0, 0);
	    mnwEvent_SendTo(&event, self->child[i]);
	}
    }
#if 1
    mnwEvent_SetAll(&event, kEvent_Redraw, nil, 0, 0);
    mnwEvent_SendToDirect(&event, self);
#endif
    return true;
}

static bool
mnwWindow_Hide(mnwWindow *self)
{
    int i;
    mnwEvent event;
    self->visible = false;
    /* send Hide to the children */
    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (self->child_valid[i]){
	    mnwEvent_SetAll(&event, kEvent_Hide, nil, 0, 0);
	    mnwEvent_SendTo(&event, self->child[i]);
	}
    }
#if 1
    mnwEvent_SetAll(&event, kEvent_Redraw, nil, 0, 0);
    mnwEvent_SendToDirect(&event, self);
#endif
    return true;
}

static void
mnwWindow_DrawFrame(mnwWindow *self)
{
    mnwRect rect;
    mnwSetForeColor(kColor_Black);
    mnwDrawRect(&(self->rect));
    mnwRect_SetXYWH(&rect,
		    self->rect.x + 1,
		    self->rect.y + 1,
		    self->rect.width - 2,
		    self->rect.height - 2);
    mnwDrawFrame3D(&rect);
}

static bool
mnwWindow_Redraw(mnwWindow *self, mnwRect *rect)
{
    mnwEvent e;
    int i;

    if (self->visible){
	mnwSetForeColor(self->back_color);
	mnwFillRect(&(self->rect));
	mnwSetForeColor(kColor_Black);
	mnwWindow_DrawFrame(self);
    }else{
	mnwSetForeColor(self->parent->back_color);
	mnwFillRect(&(self->rect));
	mnwSetForeColor(kColor_Black);
    }
    /* send Redraw to the children */
    for (i = 0; i < MNW_MAX_CHILD; i++){
	if (self->child_valid[i]){
	    mnwEvent_SetAll(&e, kEvent_Redraw, nil, 0, 0);
	    mnwEvent_SendTo(&e, self->child[i]);
	}
    }
    return true;
}

bool
mnwWindow_Callback(mnwWindow *self, mnwEvent *event)
{
    switch (event->type){
	case kEvent_Create:
	return mnwWindow_Initialize(self);
	break;
	case kEvent_Destroy:
	return mnwWindow_Finalize(self);
	break;
	case kEvent_SetRect:
	return mnwWindow_SetRect(self, event->param0);
	break;
	case kEvent_GetRect:
	return mnwWindow_GetRect(self, event->param0);
	break;
	case kEvent_SetParent:
	return mnwWindow_SetParent(self, event->param0);
	break;
	case kEvent_AddChild:
	return mnwWindow_AddChild(self, event->param0);
	break;
	case kEvent_RemoveChild:
	return mnwWindow_RemoveChild(self, event->param0);
	break;
	case kEvent_Show:
	return mnwWindow_Show(self);
	break;
	case kEvent_Hide:
	return mnwWindow_Hide(self);
	break;
	case kEvent_SetForeColor:
	self->fore_color = event->param1;
	return true;
	break;
	case kEvent_SetBackColor:
	self->back_color = event->param1;
	return true;
	break;
	case kEvent_SetFocus:
	return mnwWindow_SetFocus(self);
	break;
	case kEvent_Redraw:
	return mnwWindow_Redraw(self, nil);
	break;
	case kEvent_SetID:
	self->id = event->param1;
	return true;
	break;
	case kEvent_GetID:
	*(int *)(event->param0) = self->id;
	return true;
	break;
    }

    /* ٥Ȥʤ */
    return false;
}
