/*
   calendar routines
   Copyright (c) 1997 FURUKAWA Yo.
   $Id: calendar.c,v 1.1.1.1 1998/12/15 03:42:44 furukawa Exp $
   */

#include "arch.h"
#include "calendar.h"
#include <stdio.h>
#include <stdlib.h>

/* NOTE: Years before 1900 are not supported. */

/* NOTE: dayNo must be 1 or greater */

/* -- global variables */

uint8 gDayOfWeekBit[7] = {
    0x01,
    0x02,
    0x04,
    0x08,
    0x10,
    0x20,
    0x40
};

uint8 gNthWeekBit[6] = {
    0x00,
    0x01,
    0x02,
    0x04,
    0x08,
    0x10,
};

/* -- file local variables */
static char *nameOfMonth[13] = {
    "",
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December",
};

static char *nameOfWeek[7] = {
    "sun",
    "mon",
    "tue",
    "wed",
    "thu",
    "fri",
    "sat",
};

static uint16 daysInMonth[2][13] = {
	{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
	{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
};

/* -- file local methods */

/* -- methods */

uint16
Calendar_GetDayNo(uint16 year, uint16 month, uint16 day)
{
    uint16 numOfYears;
    uint16 leapYearsSince1900;
    uint16 daysSince1Jan1900;
    uint16 i;

    numOfYears = year - 1900;
    leapYearsSince1900 = 
	((year - 1900) / 4) - ((year / 100) - 19) + ((year / 400) - 4);
    if (IsLeap(year)){
	leapYearsSince1900--;
    }
    daysSince1Jan1900 = numOfYears * 365 + leapYearsSince1900;
    for (i = 1; i < month; i++){
	if (IsLeap(year)){
	    daysSince1Jan1900 += daysInMonth[0][i];
	}else{
	    daysSince1Jan1900 += daysInMonth[1][i];
	}
    }
    daysSince1Jan1900 += day;

    return daysSince1Jan1900;
}

uint16
Calendar_GetDayOfWeek(uint16 year, uint16 month, uint16 day)
{
    uint16 daysSince1Jan1900;

    daysSince1Jan1900 = Calendar_GetDayNo(year, month, day);
    return (DayOfWeek_1Jan1900 + (daysSince1Jan1900 - 1)) % 7;
}

uint16
Calendar_GetYearByNo(uint16 dayNo)
{
    uint16 year;

    year = 1900 + dayNo / 365; /* may over */
    while (dayNo < Calendar_GetDayNo(year, 1, 1)){
	year--;
    }

    return year;
}

uint16
Calendar_GetMonthByNo(uint16 dayNo)
{
    uint16 year;
    uint16 begin;
    uint16 month;

    year = Calendar_GetYearByNo(dayNo);
    begin = Calendar_GetDayNo(year, 1, 1);
    for (month = 1; month <= 12; month++){
	if (IsLeap(year)){
	    begin += daysInMonth[0][month];
	}else{
	    begin += daysInMonth[1][month];
	}
	if (dayNo < begin)
	    break;
    }
    return month;
}

uint16
Calendar_GetDayByNo(uint16 dayNo)
{
    uint16 year;
    uint16 month;
    uint16 begin;

    year = Calendar_GetYearByNo(dayNo);
    month = Calendar_GetMonthByNo(dayNo);
    begin = Calendar_GetDayNo(year, month, 1);

    return dayNo - begin + 1;
}

void
Calendar_GetDateByNo(uint16 dayNo, 
		     uint16 *out_year, uint16 *out_month, uint16 *out_day)
{
    uint16 year;
    uint16 month;
    uint16 begin;

    /* year */
    year = 1900 + dayNo / 365; /* may over */
    while (dayNo < (begin = Calendar_GetDayNo(year, 1, 1))){
	year--;
    }

    /* month */
    for (month = 1; month <= 12; month++){
	if (IsLeap(year)){
	    begin += daysInMonth[0][month];
	}else{
	    begin += daysInMonth[1][month];
	}
	if (dayNo < begin)
	    break;
    }

    /* day */
    if (IsLeap(year)){
	begin -= daysInMonth[0][month];
    }else{
	begin -= daysInMonth[1][month];
    }

    *out_year = year;
    *out_month = month;
    *out_day = dayNo - begin + 1;
}

void
Calendar_GetMonthDayByNo(uint16 dayNo, 
			 uint16 *out_month, uint16 *out_day)
{
    uint16 year;
    uint16 month;
    uint16 begin;

    /* year */
    year = 1900 + dayNo / 365; /* may over */
    while (dayNo < (begin = Calendar_GetDayNo(year, 1, 1))){
	year--;
    }

    /* month */
    for (month = 1; month <= 12; month++){
	if (IsLeap(year)){
	    begin += daysInMonth[0][month];
	}else{
	    begin += daysInMonth[1][month];
	}
	if (dayNo < begin)
	    break;
    }

    /* day */
    if (IsLeap(year)){
	begin -= daysInMonth[0][month];
    }else{
	begin -= daysInMonth[1][month];
    }

    *out_month = (year - 1900) * 12 + month - 1;
    *out_day = dayNo - begin + 1;
}

uint16
Calendar_GetNumOfDays(uint16 year, uint16 month)
{
    if (IsLeap(year)){
	return daysInMonth[0][month];
    }else{
	return daysInMonth[1][month];
    }
}

uint16
Calendar_GetNumOfDaysByNo(uint16 dayNo)
{
    uint16 year, month, day;

    Calendar_GetDateByNo(dayNo, &year, &month, &day);
    if (IsLeap(year)){
	return daysInMonth[0][month];
    }else{
	return daysInMonth[1][month];
    }
}

char *
Calendar_GetNameOfMonth(uint16 month)
{
    return nameOfMonth[month];
}

char *
Calendar_GetNameOfWeek(uint16 dayOfWeek)
{
    return nameOfWeek[dayOfWeek];
}

#if 0
#include <locale.h>
#include <time.h>

void
Calendar_Print(uint16 year, uint16 month, uint16 day)
{
    uint16 dayofweek;
    uint16 numOfDays;
    uint16 origin;
    int i, j;

    for (i = 0; i < (20 - (strlen(nameOfMonth[month]) + 5)) / 2; i++){
	printf(" ");
    }
    printf("%s %4d\n", nameOfMonth[month], year);
    printf("Su Mo Tu We Th Fr Sa\n");

    origin = Calendar_GetDayNo(year, month, day);
    /* Sunday of the last week */
    origin -= (Calendar_GetDayOfWeekByNo(origin) + 7);

    for (j = 0; j < 4; j++){
	for (i = 0; i < 7; i++){
	    uint16 year, month, day;
	    Calendar_GetDateByNo(origin, &year, &month, &day);
	    printf("%2d ", day);
	    origin++;
	}
	printf("\n");
    }
}

uint16 main(int argc, char *argv[])
{
    uint16 year, month, day;

    if (argc < 3){
	struct tm *local_time;
	time_t now;

	setlocale(LC_TIME, "");
	time(&now);
	local_time = localtime(&now);
	year = local_time->tm_year + 1900;
	month = local_time->tm_mon + 1;
	day = local_time->tm_mday;
    }else{
	year = atoi(argv[1]);
	month = atoi(argv[2]);
	day = atoi(argv[3]);
    }
    Calendar_Print(year, month, day);

    return 0;
}
#endif

