/*
   record file
   Copyright (c) 1997 FURUKAWA Yo.
   $Id: recfile.c,v 1.2 1998/12/27 16:26:16 furukawa Exp $
   */

#include "arch.h"
#include "record.h"
#include "index.h"
#include "db.h"
#include "util.h"
#include <stdio.h>
#include <sys/stat.h>
#ifdef ARCH_POSIX
#include <unistd.h>
#endif
#include <string.h>

/* TODO: ХȥθʤФʤʤ */

/* -- file local variables */

/* -- file local methods */

/* -- methods */

/* 쥳ɥեɤ߹ */
bool
RecordFile_Load(void)
{
    FILE *fp;
    int fd, r, count;
    struct stat file_stat;
    uint32 remain;
    bool state;

    /* open file */
    fp = fopen((char *)DB_GetFileName(), OPENMODE_READ_BINARY);
    if (NULL == fp)
	return false;

    /* get file length */
    fd = fileno(fp);
    r = fstat(fd, &file_stat);
    if (r < 0){
	fclose(fp);
	return false;
    }
    remain = file_stat.st_size;

    /* read records one by one */
    state = true;
    count = 0;
    while (remain > 0){
	byte dummy;
	Record *rec = Record_AllocateRecord(1);
	if (rec == nil){
	    state = false;
	    break;
	}

	/* 飴ХȤθĹ */
	fread(&rec->len, 2, 1, fp);
	if (rec->len > remain){
	    state = false;
	    break;
	}
	fread(&rec->type, 2, 1, fp);
	if (rec->type == RECORD_TYPE_TAIL){
	    state = true;
	    /* 齪 */
	    break;
	}
	fread(&rec->flag, 2, 1, fp);
	fread(&rec->id, 4, 1, fp);
	fread(&rec->user, 2, 1, fp);
	fread(&rec->label, 2, 1, fp);
	fread(&rec->start_date, 2, 1, fp);
	fread(&rec->end_date, 2, 1, fp);
	fread(&rec->update_user, 2, 1, fp);
	fread(&rec->update_date, 2, 1, fp);
	fread(&rec->update_time_h, 1, 1, fp);
	fread(&rec->update_time_m, 1, 1, fp);
	fread(&rec->repeat_type, 1, 1, fp);
	fread(&rec->repeat_every, 1, 1, fp);
	fread(&rec->repeat_arg1, 1, 1, fp);
	fread(&rec->repeat_arg2, 1, 1, fp);
	fread(&rec->except_len, 2, 1, fp);
	fread(&rec->done_len, 2, 1, fp);
	fread(&rec->descript_len, 2, 1, fp);
	fread(&rec->link_len, 2, 1, fp);

	switch (rec->type){
	    case RECORD_TYPE_TASK:
	    fread(&rec->u.task.request_user, 2, 1, fp);
	    fread(&rec->u.task.rank, 1, 1, fp);
	    fread(&dummy, 1, 1, fp);
	    break;

	    case RECORD_TYPE_DEADLINE:
	    fread(&rec->u.dead.request_user, 2, 1, fp);
	    fread(&rec->u.dead.end_time_h, 1, 1, fp);
	    fread(&rec->u.dead.end_time_m, 1, 1, fp);
	    break;

	    case RECORD_TYPE_CONTACT:
	    fread(&rec->u.cont.start_time_h, 1, 1, fp);
	    fread(&rec->u.cont.start_time_m, 1, 1, fp);
	    fread(&dummy, 1, 1, fp);
	    fread(&dummy, 1, 1, fp);
	    break;

	    case RECORD_TYPE_SCHEDULE:
	    fread(&rec->u.sche.start_time_h, 1, 1, fp);
	    fread(&rec->u.sche.start_time_m, 1, 1, fp);
	    fread(&rec->u.sche.end_time_h, 1, 1, fp);
	    fread(&rec->u.sche.end_time_m, 1, 1, fp);
	    break;

	    case RECORD_TYPE_HOLIDAY:
	    fread(&dummy, 1, 1, fp);
	    fread(&dummy, 1, 1, fp);
	    fread(&dummy, 1, 1, fp);
	    fread(&dummy, 1, 1, fp);
	    break;

	    default:
	    break;
	}
	
	/* ĹΰĹ򸡾 */
	if (rec->len != 40 + rec->except_len + rec->done_len
	    + rec->descript_len + rec->link_len){
	    /* ĹȤʤ */
	    state = false;
	    break;
	}

	/* 顤Ĺ */
	if (rec->except_len > 0){
	    rec->except = Record_AllocateUint16(rec->except_len / 2);
	    if (rec->except == nil){
		state = false;
		break;
	    }
	    fread(rec->except, rec->except_len, 1, fp);
	}else{
	    rec->except = nil;
	}
	if (rec->done_len > 0){
	    rec->done = Record_AllocateUint16(rec->done_len / 2);
	    if (rec->done == nil){
		state = false;
		break;
	    }
	    fread(rec->done, rec->done_len, 1, fp);
	}else{
	    rec->done = nil;
	}
	if (rec->descript_len > 0){
	    rec->descript = Record_AllocateByte(rec->descript_len);
	    if (rec->descript == nil){
		state = false;
		break;
	    }
	    fread(rec->descript, rec->descript_len, 1, fp);
	}else{
	    rec->descript = nil;
	}
	if (rec->link_len > 0){
	    rec->link = Record_AllocateByte(rec->link_len);
	    if (rec->link == nil){
		state = false;
		break;
	    }
	    fread(rec->link, rec->link_len, 1, fp);
	}else{
	    rec->link = nil;
	}

	remain -= rec->len;
	count++;

	/* 쥳ɤϿ */
	state = Record_Register(rec);
	if (!state){
	    break;
	}
    }
#ifdef DEBUG
    if (state == false){
	fprintf(stderr, "illegal record found\n");
	fflush(stderr);
    }
#endif
    D(fprintf(stderr, "%d records read\n", count); fflush(stderr););

    /* close file */
    fclose(fp);
    return state;
}

/* 쥳ɥեؤ¸ */
bool
RecordFile_Save(void)
{
    Record *rec;
    FILE *fp;
    int count;
    uint16 cookie;
    uint16 mode;
    uint16 tail_len;
    uint16 tail_type;

    mode = DB_GetMode();

    /* open file */
    fp = fopen((char *)DB_GetFileName(), OPENMODE_WRITE_BINARY);
    if (NULL == fp)
	return false;

    /* write to file */
    count = 0;
    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	byte dummy;
	
	/* Х⡼ɤΤȤ */
	if ((rec->type == RECORD_TYPE_DELETED)
	    && (mode != DB_MODE_CLIENT)){
	    /* ¸ʤʺ */
	    continue;
	}

	/* 򹹿 */
	rec->len = 40 + rec->except_len + rec->done_len
	    + rec->descript_len + rec->link_len;

	/* 飴ХȤθĹ */
	fwrite(&rec->len, 2, 1, fp);
	fwrite(&rec->type, 2, 1, fp);
	fwrite(&rec->flag, 2, 1, fp);
	fwrite(&rec->id, 4, 1, fp);
	fwrite(&rec->user, 2, 1, fp);
	fwrite(&rec->label, 2, 1, fp);
	fwrite(&rec->start_date, 2, 1, fp);
	fwrite(&rec->end_date, 2, 1, fp);
	fwrite(&rec->update_user, 2, 1, fp);
	fwrite(&rec->update_date, 2, 1, fp);
	fwrite(&rec->update_time_h, 1, 1, fp);
	fwrite(&rec->update_time_m, 1, 1, fp);
	fwrite(&rec->repeat_type, 1, 1, fp);
	fwrite(&rec->repeat_every, 1, 1, fp);
	fwrite(&rec->repeat_arg1, 1, 1, fp);
	fwrite(&rec->repeat_arg2, 1, 1, fp);
	fwrite(&rec->except_len, 2, 1, fp);
	fwrite(&rec->done_len, 2, 1, fp);
	fwrite(&rec->descript_len, 2, 1, fp);
	fwrite(&rec->link_len, 2, 1, fp);

	switch (rec->type){
	    case RECORD_TYPE_TASK:
	    fwrite(&rec->u.task.request_user, 2, 1, fp);
	    fwrite(&rec->u.task.rank, 1, 1, fp);
	    fwrite(&dummy, 1, 1, fp);
	    break;

	    case RECORD_TYPE_DEADLINE:
	    fwrite(&rec->u.dead.request_user, 2, 1, fp);
	    fwrite(&rec->u.dead.end_time_h, 1, 1, fp);
	    fwrite(&rec->u.dead.end_time_m, 1, 1, fp);
	    break;

	    case RECORD_TYPE_CONTACT:
	    fwrite(&rec->u.cont.start_time_h, 1, 1, fp);
	    fwrite(&rec->u.cont.start_time_m, 1, 1, fp);
	    fwrite(&dummy , 1, 1, fp);
	    fwrite(&dummy , 1, 1, fp);
	    break;

	    case RECORD_TYPE_SCHEDULE:
	    fwrite(&rec->u.sche.start_time_h, 1, 1, fp);
	    fwrite(&rec->u.sche.start_time_m, 1, 1, fp);
	    fwrite(&rec->u.sche.end_time_h, 1, 1, fp);
	    fwrite(&rec->u.sche.end_time_m, 1, 1, fp);
	    break;

	    case RECORD_TYPE_HOLIDAY:
	    fwrite(&dummy , 1, 1, fp);
	    fwrite(&dummy , 1, 1, fp);
	    fwrite(&dummy , 1, 1, fp);
	    fwrite(&dummy , 1, 1, fp);
	    break;

	    default:
	    break;
	}
	
	/* 顤Ĺ */
	if (rec->except_len > 0){
	    fwrite((byte *)rec->except, rec->except_len, 1, fp);
	}
	if (rec->done_len > 0){
	    fwrite((byte *)rec->done, rec->done_len, 1, fp);
	}
	if (rec->descript_len > 0){
	    fwrite((byte *)rec->descript, rec->descript_len, 1, fp);
	}	
	if (rec->link_len > 0){
	    fwrite((byte *)rec->link, rec->link_len, 1, fp);
	}

	count++;
    }
    D(fprintf(stderr, "%d records written\n", count); fflush(stderr););

    /* write tail */
    tail_len = 4;
    tail_type = RECORD_TYPE_TAIL;
    fwrite(&tail_len, 2, 1, fp);
    fwrite(&tail_type, 2, 1, fp);

    /* close file */
    fclose(fp);
    return true;
}
