/*
   record
   Copyright (c) 1997 FURUKAWA Yo.
   $Id: record.c,v 1.2 1998/12/27 16:26:16 furukawa Exp $
   */

#include "arch.h"
#include "record.h"
#include "index.h"
#include "calendar.h"
#include "month.h"
#include "db.h"
#include <stdio.h>
#include <sys/stat.h>
#ifdef ARCH_POSIX
#include <unistd.h>
#endif
#include <string.h>

/* -- file local variables */

/* 쥳ɥ֥å */
static Record *recordBlock[BLOCK_NUM_MAX];
static Record *recordBlockNext;
static uint16 recordBlockRemain;
static uint16 recordBlockNum;
static uint16 recordAllocatedNum;

/* BYTE֥å */
static byte *byteBlock[BLOCK_NUM_MAX];
static byte *byteBlockNext;
static uint16 byteBlockRemain;
static uint16 byteBlockNum;

/* UINT16֥å */
static uint16 *uint16Block[BLOCK_NUM_MAX];
static uint16 *uint16BlockNext;
static uint16 uint16BlockRemain;
static uint16 uint16BlockNum;

#ifdef DEBUG
/* 쥳ɽ缡ɤ߽Ф */
static uint16 seqRecordNo;
#endif

/* -- file local methods */
static bool
Record_AllocateRecordBlock(void);
static bool
Record_AllocateUint16Block(void);
static bool
Record_AllocateByteBlock(void);
static bool
Record_FreeBlocks(void);

/* -- methods */

/* 쥳ɤν */
bool
Record_Init(void)
{
    /* ֥åν */
    recordBlockNum = 0;
    recordBlockRemain = 0;
    recordAllocatedNum = 0;
    uint16BlockNum = 0;
    uint16BlockRemain = 0;
    byteBlockNum = 0;
    byteBlockRemain = 0;

    return true;
}

/* 쥳ɤդ */
bool
Record_Tidy(void)
{
    /* 쥳ɥ֥åβ */
    Record_FreeBlocks();

    return true;
}

/* 쥳ɥ֥åγ */
static bool
Record_AllocateRecordBlock(void)
{
    if (recordBlockNum < BLOCK_NUM_MAX){
	/* ֥å */
	recordBlock[recordBlockNum] =
	    (Record *)malloc((uint16)sizeof(Record) * RECORD_NUM_IN_BLOCK);
	if (recordBlock[recordBlockNum] == NULL){
	    return false;
	}
	recordBlockNext = recordBlock[recordBlockNum];
	recordBlockRemain = RECORD_NUM_IN_BLOCK;
	recordBlockNum++;
	D(fprintf(stderr, "allocate record block No.%d\n", recordBlockNum); fflush(stderr);)
	return true;
    }else{
	return false;
    }
}

/* UINT16֥åγ */
static bool
Record_AllocateUint16Block(void)
{
    if (uint16BlockNum < BLOCK_NUM_MAX){
	/* ֥å */
	uint16Block[uint16BlockNum] =
	    (uint16 *)malloc((uint16)sizeof(uint16) * UINT16_NUM_IN_BLOCK);
	if (uint16Block[uint16BlockNum] == NULL){
	    return false;
	}
	uint16BlockNext = uint16Block[uint16BlockNum];
	uint16BlockRemain = UINT16_NUM_IN_BLOCK;
	uint16BlockNum++;
	D(fprintf(stderr, "allocate uint16 block No.%d\n", uint16BlockNum); fflush(stdout););
	return true;
    }else{
	return false;
    }
}

/* BYTE֥åγ */
static bool
Record_AllocateByteBlock(void)
{
    if (byteBlockNum < BLOCK_NUM_MAX){
	/* ֥å */
	byteBlock[byteBlockNum] =
	    (byte *)malloc((uint16)sizeof(byte) * BYTE_NUM_IN_BLOCK);
	if (byteBlock[byteBlockNum] == NULL){
	    return false;
	}
	byteBlockNext = byteBlock[byteBlockNum];
	byteBlockRemain = BYTE_NUM_IN_BLOCK;
	byteBlockNum++;
	D(fprintf(stderr, "allocate byte block No.%d\n", byteBlockNum); fflush(stdout););
	return true;
    }else{
	return false;
    }
}

/* ֥åդ */
static bool
Record_FreeBlocks(void)
{
    uint16 i;

    for (i = 0; i < recordBlockNum; i++){
	free(recordBlock[i]);
    }
    for (i = 0; i < uint16BlockNum; i++){
	free(uint16Block[i]);
    }
    for (i = 0; i < byteBlockNum; i++){
	free(byteBlock[i]);
    }

    return true;
}

/* 쥳ΰγ */
Record *
Record_AllocateRecord(uint16 num)
{
    Record *save;

    if (recordBlockRemain < num){
	if (!Record_AllocateRecordBlock()){
	    return (Record *)nil;
	}
    }
    save = recordBlockNext;
    recordBlockNext += num;
    recordBlockRemain -= num;
    recordAllocatedNum += num;
    return save;
}

/* UINT16ΰγ */
uint16 *
Record_AllocateUint16(uint16 num)
{
    uint16 *save;

    if (uint16BlockRemain < num){
	if (!Record_AllocateUint16Block()){
	    return (uint16 *)nil;
	}
    }
    save = uint16BlockNext;
    uint16BlockNext += num;
    uint16BlockRemain -= num;
    return save;
}

/* BYTEΰγ */
byte *
Record_AllocateByte(uint16 num)
{
    byte *save;

    if (byteBlockRemain < num){
	if (!Record_AllocateByteBlock()){
	    return (byte *)nil;
	}
    }
    save = byteBlockNext;
    byteBlockNext += num;
    byteBlockRemain -= num;
    return save;
}

/* 쥳ɤϿ */
bool
Record_Register(Record *rec)
{
    bool r;

#if 0
    /* ͭä˰̣ʤ */
    rec->len = sizeof(Record) + rec->except_len + rec->done_len
	+ rec->descript_len + rec->link_len;
#endif

    /* 쥳ɤͤ򸡾 */
    if (rec->start_date == 0
	|| rec->start_date > rec->end_date){
	return false;
    }

    /* ǥåɲ */
    r = Index_Insert(rec);
    if (!r){
	return false;
    }

    /* ֤®˸뤿ѿ򥻥å */
    Calendar_GetMonthDayByNo(rec->start_date,
			     &(rec->start_monthNo),
			     &(rec->start_day));
    rec->start_month = rec->start_monthNo % 12 + 1;

    return true;
}

/* 쥳ɤκ */
bool
Record_Delete(Record *rec)
{
    uint16 year, month, day;

    /* õե饰ΩƤ */
    rec->type = RECORD_TYPE_DELETED;

    /* 򹹿 */
    rec->update_user = DB_GetUser();
    rec->update_date = DB_GetDate(&year, &month, &day,
				  &(rec->update_time_h),
				  &(rec->update_time_m));

#if 0
    /* Ĺǡ򣰤 */
    rec->except_len = 0;
    rec->done_len = 0;
    rec->descript_len = 0;
    rec->link_len = 0;
    rec->except = (uint16 *)nil;
    rec->done = (uint16 *)nil;
    rec->descript = (byte *)nil;
    rec->link = (byte *)nil;
#endif

    /* å̵ */
    MonthRec_Invalidate();

    return true;
}

/* 쥳ɤι */
bool
Record_Update(Record *rec, bool invalid)
{
    uint16 year, month, day;

    /* 򹹿 */
    rec->update_user = DB_GetUser();
    rec->update_date = DB_GetDate(&year, &month, &day,
				  &(rec->update_time_h),
				  &(rec->update_time_m));

    /* ɬפʤå̵ */
    if (invalid){
	MonthRec_Invalidate();
    }

    return true;
}

#ifdef DEBUG

/* 쥳ɽ缡ɤ߽Фν */
void
Record_RewindSeq(void)
{
    seqRecordNo = 0;
}

/* 쥳ɽ缡ɤ߽Ф */
Record *
Record_GetSeq(void)
{
    Record *rec;

    if (seqRecordNo >= recordAllocatedNum){
	/* ⤦ʤ */
	return (Record *)nil;
    }

    rec = &(recordBlock[seqRecordNo/RECORD_NUM_IN_BLOCK][seqRecordNo%RECORD_NUM_IN_BLOCK]);
    seqRecordNo++;

    return rec;
}

bool
Record_Dump()
{
    Record *rec;

    Record_RewindSeq();
    while (nil != (rec = Record_GetSeq())){
	switch (rec->type){
	    case RECORD_TYPE_HOLIDAY:
	    printf("HOLIDAY,%d,%05d-%05d\n",
		   rec->len, rec->start_date, rec->end_date);
	    break;

	    default:
	    printf("UNKNOWN,%d\n", rec->len);
	    break;
	}
    }
    return true;
}
#endif
