/*
   record
   Copyright (c) 1997 FURUKAWA Yo.
   $Id: record.h,v 1.1.1.1 1998/12/15 03:42:44 furukawa Exp $
   */

#ifndef _RECORD_H_
#define _RECORD_H_

#include "arch.h"

/* -- macros */

/* types of records */
#define RECORD_TYPE_NONE	0x00
#define RECORD_TYPE_HEAD	0x01
#define RECORD_TYPE_TAIL	0x02
#define RECORD_TYPE_DELETED	0x03
#define RECORD_TYPE_HOLIDAY	0x04
#define RECORD_TYPE_CONTACT	0x05
#define RECORD_TYPE_SCHEDULE	0x06
#define RECORD_TYPE_DEADLINE	0x07
#define RECORD_TYPE_TASK	0x08

#define RECORD_FLAG_NONE	0x00
#define RECORD_USER_ANONYMOUS	0x00
#define RECORD_LABEL_NONE	0x00

#define REPEAT_TYPE_NONE	0x00
#define REPEAT_TYPE_DAILY	0x01
#define REPEAT_TYPE_WEEKLY	0x02
#define REPEAT_TYPE_MONTHLYDAY	0x03
#define REPEAT_TYPE_MONTHLYWEEK	0x04
#define REPEAT_TYPE_YEARLYDAY	0x05
#define REPEAT_TYPE_YEARLYWEEK	0x06

/* record block */
#define RECORD_NUM_IN_BLOCK	(uint16)(ALLOC_BLOCK_SIZE / sizeof(Record))
#define UINT16_NUM_IN_BLOCK	(uint16)(ALLOC_BLOCK_SIZE / sizeof(uint16))
#define BYTE_NUM_IN_BLOCK	(uint16)(ALLOC_BLOCK_SIZE / sizeof(byte))
#define BLOCK_NUM_MAX		48	/* may be too little for servers */

/* -- types */

typedef struct _Task {
    uint16 request_user;		      /* 36  */
    uint8 rank;				      /* 38  */
} Task;

typedef struct _Deadline {
    uint16 request_user;		      /* 36  */
    uint8 end_time_h;			      /* 38 λ  */
    uint8 end_time_m;			      /* 39 λ ʬ */
} Deadline;

typedef struct _Contact {
    uint8 start_time_h;			      /* 36 ϻ  */
    uint8 start_time_m;			      /* 37 ϻ ʬ */
} Contact;

typedef struct _Schedule {
    uint8 start_time_h;			      /* 36 ϻ  */
    uint8 start_time_m;			      /* 37 ϻ ʬ */
    uint8 end_time_h;			      /* 38 λ  */
    uint8 end_time_m;			      /* 39 λ ʬ */
} Schedule;

typedef struct _Holiday {
    uint8 dummy;
} Holiday;

typedef struct _Record {
    uint16 len;				      /* 00 쥳Ĺ */
    uint16 type;			      /* 02  */
    uint16 flag;			      /* 04 ե饰̤ѡ */
    uint32 id;				      /* 06 쥳ɣɣ */
    uint16 user;			      /* 10 桼 */
    uint16 label;			      /* 12 ٥ */
    uint16 start_date;			      /* 14   */
    uint16 start_monthNo;		      /* --  ʾĹ */
    uint16 start_month;			      /* --  ʾĹ */
    uint16 start_day;			      /* --  ʾĹ */
    uint16 end_date;			      /* 16 λ  */
    uint16 update_user;			      /* 18 ǽ 桼 */
    uint16 update_date;			      /* 20 ǽ  */
    uint8 update_time_h;		      /* 22 ǽ   */
    uint8 update_time_m;		      /* 23 ǽ  ʬ */
    uint8 repeat_type;			      /* 24 ֤  */
    uint8 repeat_every;			      /* 25 ֤ ֳ */
    uint8 repeat_arg1;			      /* 26 ֤  */
    uint8 repeat_arg2;			      /* 27 ֤  */
    uint16 except_len;			      /* 28 㳰 Ĺ */
    uint16 done_len;			      /* 30 λ Ĺ */
    uint16 descript_len;		      /* 32  Ĺ */
    uint16 link_len;			      /* 34  Ĺ */
    /* 鶦4bytes */
    union {
	Task task;
	Deadline dead;
	Contact cont;
	Schedule sche;
	Holiday holi;
    } u;
    /* ޤ40bytes6bytesϥ֤ʤ*/
    /* ʸ塤Ĺǡ */
    uint16 *except;
    uint16 *done;
    byte *descript;
    byte *link;
} Record;

/* -- prototypes */

bool
Record_Init(void);
bool
Record_Tidy(void);
Record *
Record_AllocateRecord(uint16 num);
uint16 *
Record_AllocateUint16(uint16 num);
byte *
Record_AllocateByte(uint16 num);
bool
Record_Register(Record *rec);
bool
Record_Delete(Record *rec);

#ifdef DEBUG
void
Record_RewindSeq(void);
Record *
Record_GetSeq(void);
bool
Record_Dump();
#endif

bool
RecordFile_Load(void);
bool
RecordFile_Save(void);

#endif
