/*
   env dialog
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: env.c,v 1.1.1.1 1998/12/15 03:42:46 furukawa Exp $
   */

#include "global.h"
#include "taskmg.h"
#include "taskid.h"
#include "arch.h"
#include "funckey.h"
#include "gendlg.h"
#include "env.h"
#include "calendar.h"
#include "record.h"
#include "month.h"
#include "view.h"
#include "context.h"
#include "util.h"
#include "mg.log"
#include "core.log"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <u-k.h>

/* -- file local macros */
#define REGION_DLG_WIDTH	(600)
#define REGION_DLG_HEIGHT	(192)
#define REGION_DLG_LEFT		(0)
#define REGION_DLG_TOP		(0)
#define REGION_DLG_RIGHT	(REGION_DLG_WIDTH)
#define REGION_DLG_BOTTOM	(REGION_DLG_HEIGHT)

#define REGION_DLG_MARGIN_LEFT	(20)
#define REGION_DLG_MARGIN_TOP	(20)

#define DLG_COLS	(2)
#define DLG_ROWS	(8)
#define FONT_HEIGHT	(16)
#define LINE_INTERVAL	((REGION_DLG_HEIGHT-REGION_DLG_MARGIN_TOP*2)/DLG_ROWS)

#define ENV_COL_1	(REGION_DLG_MARGIN_LEFT)
#define ENV_COL_2	(REGION_DLG_WIDTH/DLG_COLS)

#define ENV_ROW_1	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*0)
#define ENV_ROW_2	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*1)
#define ENV_ROW_3	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*2)
#define ENV_ROW_4	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*3)
#define ENV_ROW_5	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*4)
#define ENV_ROW_6	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*5)
#define ENV_ROW_7	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*6)
#define ENV_ROW_8	(REGION_DLG_MARGIN_TOP+LINE_INTERVAL*7)

#define ENV_MAX_LABEL	5
#define ENV_MAX_OPTION	7

/* -- file local variables */

/* EnvEBhE */
static char *nameEnvWin = "EnvWin";

/* Rg[ */
static HWND hCheckLabel[ENV_MAX_LABEL];
static HWND hCheckOption[ENV_MAX_OPTION];

static char *msgLabel[] = {
    "",
    "P",
    "Q",
    "R",
    "S",
};

static char *msgOption[] = {
    "dONɁuvփWv",
    "x̏l𕪗ނPɌŒ",
    "Ԋ\̊JnXɌŒ",
    "T̊Jnjɂ",
    "yjxɂ",
    "OŎłŃACeI",
    "傫ȃ_CAOg",
};

/* -- file local methods */
static bool
Env_Create(void);
static bool
Env_Destroy(void);
static void
Env_InitControls(void);
static void
Env_TidyControls(void);
static void
Env_Redraw(void);
static bool
Env_LostFocus(WPARAM id, WPARAM win);

/* -- methods */

/* Env */
static bool
Env_Create(void)
{
    int i;
    int x;

    D(mgDPrintf("Env create\n"));

    /* ԕϐ */
    gMode = MODE_ENV;

    /* t@NVL[̐ݒ */
    FuncKey_SetAddMode();		      /* ADDƋ */

    /* xp̃Rg[̍쐬 */
    x = ENV_COL_1 + 16;
    for (i = 0; i < ENV_MAX_LABEL; i++){
	hCheckLabel[i] =
	    mgCreateWindow(BUTTON, (LPSTR)msgLabel[i], 
			   WS_CHILD|WS_VISIBLE|WS_BORDER|WS_TABSTOP|BS_AUTOCHECKBOX,
			   x, ENV_ROW_8,
			   22 + strlen(msgLabel[i]) * 8, 20,
			   ghEnvWin, (HMENU)ID_CHECK_LABEL0 + i,
			   ghTaskInstance);
	if (hCheckLabel[i] == NULL){
	    return false;
	}
	x = x + 22 + strlen(msgLabel[i]) * 8 + 4;
    }

    /* IvVp̃Rg[̍쐬 */
    for (i = 0; i < ENV_MAX_OPTION; i++){
	hCheckOption[i] =
	    mgCreateWindow(BUTTON, (LPSTR)msgOption[i], 
			   WS_CHILD|WS_VISIBLE|WS_BORDER|WS_TABSTOP|BS_AUTOCHECKBOX,
			   ENV_COL_2 + 16, ENV_ROW_2 + LINE_INTERVAL * i,
			   22 + strlen(msgOption[i]) * 8, 20,
			   ghEnvWin, (HMENU)ID_CHECK_OPTION0 + i,
			   ghTaskInstance);
	if (hCheckOption[i] == NULL){
	    return false;
	}
    }

    /* Rg[̏ݒ */
    Env_InitControls();

    /* tH[JẌړ */
    mgSetFocus(hCheckOption[0]);

    return true;
}

/* EnvЕt */
static bool
Env_Destroy(void)
{
    int i;

    D(mgDPrintf("Env destroy\n"));

    for (i = 0; i < ENV_MAX_LABEL; i++){
	mgDestroyWindow(hCheckLabel[i]);
    }

    for (i = 0; i < ENV_MAX_OPTION; i++){
	mgDestroyWindow(hCheckOption[i]);
    }

    /* t@NVL[L */
    FuncKey_SetMainMode();

    gMode = MODE_VIEW;

    return true;
}

/* Rg[̏ݒ */
static void
Env_InitControls(void)
{
    int i;

    for (i = 0; i < ENV_MAX_LABEL; i++){
	if (Context_GetCustomLabel(i)){
	    mgSendMessage(hCheckLabel[i], BM_SETCHECK, 1, 0L);
	}else{
	    mgSendMessage(hCheckLabel[i], BM_SETCHECK, 0, 0L);
	}
    }

    mgSendMessage(hCheckOption[0], BM_SETCHECK,
		  gOpt_PowerOnToday, 0L);
    mgSendMessage(hCheckOption[1], BM_SETCHECK,
		  gOpt_Label1Default, 0L);
    mgSendMessage(hCheckOption[2], BM_SETCHECK,
		  gOpt_TimeTableStartsNine, 0L);
    mgSendMessage(hCheckOption[3], BM_SETCHECK,
		  gOpt_WeeksStartsFromMonday, 0L);
    mgSendMessage(hCheckOption[4], BM_SETCHECK,
		  gOpt_SaturdayIsHoliday, 0L);
    mgSendMessage(hCheckOption[5], BM_SETCHECK,
		  gOpt_SelectItemByPageUpDown, 0L);
    mgSendMessage(hCheckOption[6], BM_SETCHECK,
		  gOpt_LargeDialog, 0L);
}

/* Rg[̔f */
static void
Env_TidyControls(void)
{
    int i;

    for (i = 0; i < ENV_MAX_LABEL; i++){
	Context_SetCustomLabel(i, mgSendMessage(hCheckLabel[i], BM_GETCHECK,
						0, 0L));
    }

    gOpt_PowerOnToday =
	mgSendMessage(hCheckOption[0], BM_GETCHECK, 0, 0L);
    gOpt_Label1Default =
	mgSendMessage(hCheckOption[1], BM_GETCHECK, 0, 0L);
    gOpt_TimeTableStartsNine =
	mgSendMessage(hCheckOption[2], BM_GETCHECK, 0, 0L);
    gOpt_WeeksStartsFromMonday =
	mgSendMessage(hCheckOption[3], BM_GETCHECK, 0, 0L);
    gOpt_SaturdayIsHoliday =
	mgSendMessage(hCheckOption[4], BM_GETCHECK, 0, 0L);
    gOpt_SelectItemByPageUpDown =
	mgSendMessage(hCheckOption[5], BM_GETCHECK, 0, 0L);
    gOpt_LargeDialog =
	mgSendMessage(hCheckOption[6], BM_GETCHECK, 0, 0L);

    Context_ReflectEnvironment();
}

/* EnvEBhEĕ` */
static void
Env_Redraw(void)
{
    char msg[80];
    char taskmg_rev[10], core_rev[10];
    int left, top, right, bottom;

    HDC hDC;
    PAINTSTRUCT ps;

    hDC = mgBeginPaint(ghEnvWin, (LPPAINTSTRUCT)&ps);

    /* g */
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, REGION_DLG_LEFT, REGION_DLG_TOP);
    mgLineTo(hDC, REGION_DLG_RIGHT, REGION_DLG_TOP);
    mgMoveTo(hDC, REGION_DLG_LEFT, REGION_DLG_TOP);
    mgLineTo(hDC, REGION_DLG_LEFT, REGION_DLG_BOTTOM);
    mgMoveTo(hDC, REGION_DLG_LEFT, REGION_DLG_TOP+1);
    mgLineTo(hDC, REGION_DLG_RIGHT, REGION_DLG_TOP+1);
    mgMoveTo(hDC, REGION_DLG_LEFT+1, REGION_DLG_TOP);
    mgLineTo(hDC, REGION_DLG_LEFT+1, REGION_DLG_BOTTOM);
    mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, REGION_DLG_LEFT, REGION_DLG_BOTTOM-1);
    mgLineTo(hDC, REGION_DLG_RIGHT, REGION_DLG_BOTTOM-1);
    mgMoveTo(hDC, REGION_DLG_RIGHT-1, REGION_DLG_TOP);
    mgLineTo(hDC, REGION_DLG_RIGHT-1, REGION_DLG_BOTTOM);
    mgMoveTo(hDC, REGION_DLG_LEFT+1, REGION_DLG_BOTTOM-2);
    mgLineTo(hDC, REGION_DLG_RIGHT, REGION_DLG_BOTTOM-2);
    mgMoveTo(hDC, REGION_DLG_RIGHT-2, REGION_DLG_TOP+1);
    mgLineTo(hDC, REGION_DLG_RIGHT-2, REGION_DLG_BOTTOM);

    /*
       ȊF

       ^XN for MG Rel. 1.10b1
       core:1.11,mgui:1.31,mail:1.20         ݒ
       freemem:204,410bytes                    []yjx
       					       []T̊Jnj
       JX^x\                      []dONōɃWv
         [] []P []Q []R []S
    */

    mgSetTextColor(hDC, CL_BLACK);
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F16_FNT);

    mgSetBkMode(hDC, TRANSPARENT);
    mgSetTextColor(hDC, CL_DKGRAY);
    sprintf(msg, "^XN for MG ");
    mgTextOut(hDC, ENV_COL_1+2, ENV_ROW_1+2, msg, strlen(msg));
    mgSetTextColor(hDC, CL_BLACK);
    sprintf(msg, "^XN for MG / Rel %s", TASKMG_REL);
    mgTextOut(hDC, ENV_COL_1, ENV_ROW_1, msg, strlen(msg));
    mgSetBkMode(hDC, OPAQUE);

    sprintf(msg, "  (c) 1997,98 FURUKAWA Yo.");
    mgTextOut(hDC, ENV_COL_1, ENV_ROW_2, msg, strlen(msg));

    sprintf(core_rev, "%s", &(CORE_REV[11]));
    core_rev[strlen(core_rev)-2] = '\0';
    sprintf(taskmg_rev, "%s", &(TASKMG_REV[11]));
    taskmg_rev[strlen(taskmg_rev)-2] = '\0';
    sprintf(msg, "  Rev: core %s, mgui %s", core_rev, taskmg_rev);
    mgTextOut(hDC, ENV_COL_1, ENV_ROW_3, msg, strlen(msg));

    sprintf(msg, "  Free mem: %ld", freemem_size());
    mgTextOut(hDC, ENV_COL_1, ENV_ROW_4, msg, strlen(msg));

    right = ENV_COL_1 + 8;
    left = right + 16 * 14;
    top = ENV_ROW_7 + 8;
    bottom = top + LINE_INTERVAL * 2;
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, right, top);
    mgLineTo(hDC, right, bottom);
    mgLineTo(hDC, left, bottom);
    mgLineTo(hDC, left, top);
    sprintf(msg, "uJX^vŕ\郉x");
    mgTextOut(hDC, ENV_COL_1, ENV_ROW_7, msg, strlen(msg));

    right = ENV_COL_2 + 8;
    left = right + 16 * 16 + (8);
    top = ENV_ROW_1 + 8;
    bottom = top + LINE_INTERVAL * (ENV_MAX_OPTION+1);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, right, top);
    mgLineTo(hDC, right, bottom);
    mgLineTo(hDC, left, bottom);
    mgLineTo(hDC, left, top);
    sprintf(msg, "ݒ");
    mgTextOut(hDC, ENV_COL_2, ENV_ROW_1, msg, strlen(msg));

    mgEndPaint(ghEnvWin, (LPPAINTSTRUCT)&ps);
}

/* EnvEBhEvV[W */
LRESULT CALLBACK
EnvWinProc(HWND hSelf, UINT msg,
	   WPARAM wParam, LPARAM lParam,
	   POINT LPoint, POINT DPoint)
{
    bool r;

    switch (msg){
	case WM_INITDIALOG:		      /* _CAO */
	ghEnvWin = hSelf;
	r = Env_Create();
	if (r){
	    return 0;			      /*  */
	}else{
	    return -1;			      /* ُ */
	}
	break;

	case WM_PAINT:			      /* ĕ` */
	Env_Redraw();
	return 1;
	break;

	case WM_KEYDOWN:		      /* L[R[h */
	switch (wParam){
	    case VK_RETURN:
	    D(mgDPrintf("register by RETURN key"));
	    Env_TidyControls();
	    mgEndDialog(hSelf, true);
	    return 1;
	    break;

	    case VK_ESCAPE:
	    D(mgDPrintf("got ESCAPE key, which is not out of operation"));
	    break;
	}

	case WM_COMMAND:		      /* R}h */
	switch (wParam){
	    case ID_FUNC_OK:
	    Env_TidyControls();
	    mgEndDialog(hSelf, true);
	    return 1;
	    break;

	    case ID_FUNC_CANCEL:
	    mgEndDialog(hSelf, false);
	    return 1;
	    break;

	    case ID_KEY_ESCAPE:
	    D(mgDPrintf("got ESCAPE key"));
	    mgEndDialog(hSelf, false);
	    return 1;
	    break;

	    case ID_KEY_RETURN:
	    D(mgDPrintf("got RETURN key"));
	    Env_TidyControls();
	    mgEndDialog(hSelf, true);
	    return 1;
	    break;

	    case ID_FUNC_LABEL0:
	    mgSendMessage(hCheckLabel[0], BM_SETCHECK,
			  !mgSendMessage(hCheckLabel[0], BM_GETCHECK, 0, 0L),
			  0L);
	    return 1;
	    break;
	    case ID_FUNC_LABEL1:
	    mgSendMessage(hCheckLabel[1], BM_SETCHECK,
			  !mgSendMessage(hCheckLabel[1], BM_GETCHECK, 0, 0L),
			  0L);
	    return 1;
	    break;
	    case ID_FUNC_LABEL2:
	    mgSendMessage(hCheckLabel[2], BM_SETCHECK,
			  !mgSendMessage(hCheckLabel[2], BM_GETCHECK, 0, 0L),
			  0L);
	    return 1;
	    break;
	    case ID_FUNC_LABEL3:
	    mgSendMessage(hCheckLabel[3], BM_SETCHECK,
			  !mgSendMessage(hCheckLabel[3], BM_GETCHECK, 0, 0L),
			  0L);
	    return 1;
	    break;
	    case ID_FUNC_LABEL4:
	    mgSendMessage(hCheckLabel[4], BM_SETCHECK,
			  !mgSendMessage(hCheckLabel[4], BM_GETCHECK, 0, 0L),
			  0L);
	    return 1;
	    break;
	}
	break;

	case WM_CLOSE:			      /* _CAȌ */
	D(mgDPrintf("got WM_CLOSE"));
	mgEndDialog(hSelf, true);
	return 0;
	break;

	case WM_DESTROY:		      /* _CAO폜 */
	r = Env_Destroy();
	if (r){
	    return 0;			      /*  */
	}else{
	    return -1;			      /* ُ */
	}
	break;
    }

    /* Ȃ */
    return 0;
}
