/*
   file
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: file.c,v 1.1.1.1 1998/12/15 03:42:46 furukawa Exp $
   */

#include "taskmg.h"
#include "taskid.h"
#include "global.h"
#include "arch.h"
#include "file.h"
#include "record.h"
#include "month.h"
#include "view.h"
#include "index.h"
#include "db.h"
#include "gendlg.h"
#include "calendar.h"
#include "context.h"
#include <string.h>
#include <stdio.h>
#include <u-k.h>

/* -- file local variables */

/* -- file local methods */
static bool
File_CSVWriteItem(FILE *fp, Record *rec);

/* -- methods */

/* f[^x[X̕ۑ */
bool
File_Save(HWND win)
{
    D(mgDPrintf("File_Save()"));

    if (gDBDirty)
	RecordFile_Save();
    mgMessageBox(win, "t@Cɕۑ܂", MB_OK);
    return true;
}

/* f[^x[X̓ǂݍ */
bool
File_Merge(HWND win)
{
    char origFN[80], loadFN[80];
    char msg[80];
    bool r;

    D(mgDPrintf("File_Merge()"));

    sprintf(origFN, "%s", DB_GetFileName());

    GenDlg_SetTitle("ǂݍ");
    GenDlg_SetString(origFN);
    r = (bool)mgDialogBoxParam(ghTaskInstance,
			       (LPCSTR)"GenDlg",
			       win,
			       (DLGPROC)GenDlgWinProc,
			       MAKELPARAM(12, GENDLG_STRING));
    if (r){
	D(mgDPrintf("File_Merge() -- got %s", GenDlg_GetString()));
    }else{
	return false;
    }

    sprintf(loadFN, "%s%s", TASK_RECORD_DIR, GenDlg_GetString());
    D(mgDPrintf("load %s", loadFN));
    if (strlen(GenDlg_GetString()) > 12 || NULL == strstr(loadFN, ".rec")){
	mgMessageBox(win, "t@Csł", MB_OK);
	return false;
    }

    sprintf(msg, "%s\nf[^ǂݍ݂܂", GenDlg_GetString());
    r = (bool)mgMessageBox(win, msg, MB_OKCANCEL);
    if (!r){
	return false;
    }

    D(mgDPrintf("load %s", loadFN));

    DB_SetFileName((byte *)loadFN);
    r = RecordFile_Load();
    DB_SetFileName((byte *)origFN);
    
    /* LbV̖ */
    MonthRec_Invalidate();
    View_Redraw();

    if (r){
	mgMessageBox(win, "f[^ǂݍ݂܂", MB_OK);
    }else{
	mgMessageBox(win, "f[^ǂݍ݂\ns܂", MB_OK);
	return false;
    }

    gDBDirty = true;
    return true;
}

/* Ԏw̍폜 */
bool
File_DeleteRegion(HWND win)
{
    bool r;
    uint16 year, month, day, date;
    char msg[80];
    Record *rec;
    uint16 cookie;

    D(mgDPrintf("File_DeleteRegion()"));

    GenDlg_SetTitle("폜(wȑO)");

    r = mgDialogBoxParam(ghTaskInstance,
			 (LPCSTR)"GenDlg",
			 win,
			 (DLGPROC)GenDlgWinProc,
			 (LPARAM)MAKELPARAM(0, GENDLG_DATE));
    if (!r){
	return false;
    }

    date = GenDlg_GetDate();
    Calendar_GetDateByNo(date, &year, &month, &day);

    sprintf(msg, "%dN%d%dȑO\nf[^܂", year, month, day);
    r = (bool)mgMessageBox(win, msg, MB_OKCANCEL);
    if (!r){
	return false;
    }

    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	if (rec->end_date <= date
	    || (rec->type == RECORD_TYPE_TASK && rec->done_len > 0
		&& rec->done[0] <= date)){
	    D(mgDPrintf("delete %u", rec->start_date));
	    Record_Delete(rec);
	    /* TODO: Ԃl̃`FbN */
	}
    }
    
    /* LbV̖ */
    MonthRec_Invalidate();
    View_Redraw();

    mgMessageBox(win, "f[^܂", MB_OK);

    gDBDirty = true;
    return true;
}

/* ACeCSV */
static bool
File_CSVWriteItem(FILE *fp, Record *rec)
{
    uint16 year, month, day, dayWeek, nthWeek, numOfDays;
    char *kanjiNum[] = {
	"", "", "", "O", "l", "",
    };
    int len;
    byte *p;

    D(mgDPrintf("File_CSVWriteItem()"));

    switch (rec->type){
	case RECORD_TYPE_TASK:
	/* 1 */
	fprintf(fp, "TASK,");
	/* 2 */
	fprintf(fp, "\"%s\",", Context_GetLabelName(rec->label));
	/* 3 */
	fprintf(fp, "\"");
	len = rec->descript_len;
	p = rec->descript;
	while (len-- > 0){
	    if (*p == '\n'){
		fprintf(fp, "\\n");
	    }else{
		fwrite(p, 1, 1, fp);
	    }
	    p++;
	}
	fprintf(fp, "\",");
	/* 4 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 5 */
	fprintf(fp, ",");
	/* 6 */
	fprintf(fp, ",");
	/* 7 */
	fprintf(fp, "%d,", rec->u.task.rank);
	/* 8 */
	fprintf(fp, "%s,", ((rec->done_len > 0)?"":""));
	/* 9 */
	fprintf(fp, "\"");
	if (rec->link_len > 0){
	    fprintf(fp, "file:/");
	    fwrite(rec->link, rec->link_len, 1, fp);
	}
	fprintf(fp, "\"\n");
	break;

	case RECORD_TYPE_SCHEDULE:
	/* 1 */
	fprintf(fp, "SCHEDULE,");
	/* 2 */
	fprintf(fp, "\"%s\",", Context_GetLabelName(rec->label));
	/* 3 */
	fprintf(fp, "\"");
	len = rec->descript_len;
	p = rec->descript;
	while (len-- > 0){
	    if (*p == '\n'){
		fprintf(fp, "\\n");
	    }else{
		fwrite(p, 1, 1, fp);
	    }
	    p++;
	}
	fprintf(fp, "\",");
	/* 4 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 5 */
	Calendar_GetDateByNo(rec->end_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 6 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	dayWeek = Calendar_GetDayOfWeekByNo(rec->start_date);
	nthWeek = (day / 7)+((day%7)?1:0);
	numOfDays = Calendar_GetNumOfDays(year, month);
	switch (rec->repeat_type){
	    case REPEAT_TYPE_NONE:
	    fprintf(fp, ",");
	    break;
	    case REPEAT_TYPE_DAILY:
	    if (rec->repeat_every == 1){
		fprintf(fp, ",");
	    }else{
		fprintf(fp, "%d,", rec->repeat_every);
	    }
	    break;
	    case REPEAT_TYPE_WEEKLY:
	    if (rec->repeat_every == 1){
		fprintf(fp, "T%sj,", nameDayWeek[dayWeek]);
	    }else if (rec->repeat_every == 2){
		fprintf(fp, "uT%sj,", nameDayWeek[dayWeek]);
	    }else{
		fprintf(fp, "%dT%sj,", rec->repeat_every, nameDayWeek[dayWeek]);
	    }
	    break;
	    case REPEAT_TYPE_MONTHLYDAY:
	    if (rec->repeat_every == 1){
		fprintf(fp, "%d,", day);
	    }else if (rec->repeat_every == 2){
		fprintf(fp, "u%d,", day);
	    }else{
		fprintf(fp, "%d%d,", rec->repeat_every, day);
	    }
	    break;
	    case REPEAT_TYPE_MONTHLYWEEK:
	    if (numOfDays-day >= 7){
		if (rec->repeat_every == 1){
		    fprintf(fp, "%s%sj,", kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}else if (rec->repeat_every == 2){
		    fprintf(fp, "u%s%sj,", kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%dƑ%s%sj,", rec->repeat_every, kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}
	    }else{
		if (rec->repeat_every == 1){
		    fprintf(fp, "ŏI%sj,", nameDayWeek[dayWeek]);
		}else if (rec->repeat_every == 2){
		    fprintf(fp, "uŏI%sj,", nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%dƍŏI%sj,", rec->repeat_every, nameDayWeek[dayWeek]);
		}
	    }
	    break;
	    case REPEAT_TYPE_YEARLYDAY:
	    if (rec->repeat_every == 1){
		fprintf(fp, "N%d%d,", month, day);
	    }else{
		fprintf(fp, "%dN%d%d,", rec->repeat_every, month, day);
	    }
	    break;
	    case REPEAT_TYPE_YEARLYWEEK:
	    if (numOfDays-day >= 7){
		if (rec->repeat_every == 1){
		    fprintf(fp, "N%d%s%sj,", month, kanjiNum[nthWeek], nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%dN%d%s%sj,", rec->repeat_every, month, kanjiNum[nthWeek], nameDayWeek[dayWeek], rec->repeat_every);
		}
	    }else{
		if (rec->repeat_every == 1){
		    fprintf(fp, "N%dŏI%sj,", month, nameDayWeek[dayWeek]);
		}else{
		    fprintf(fp, "%dN%dŏI%sj,", rec->repeat_every, month, nameDayWeek[dayWeek]);
		}
	    }
	}
	/* 7 */
	fprintf(fp, "%02d:%02d,",
		rec->u.sche.start_time_h,
		rec->u.sche.start_time_m);
	/* 8 */
	fprintf(fp, "%02d:%02d,",
		rec->u.sche.end_time_h,
		rec->u.sche.end_time_m);
	/* 9 */
	fprintf(fp, "\"");
	if (rec->link_len > 0){
	    fprintf(fp, "file:/");
	    fwrite(rec->link, rec->link_len, 1, fp);
	}
	fprintf(fp, "\"\n");
	break;

	case RECORD_TYPE_HOLIDAY:
	/* 1 */
	fprintf(fp, "HOLIDAY,");
	/* 2 */
	fprintf(fp, "\"%s\",", Context_GetLabelName(rec->label));
	/* 3 */
	fprintf(fp, "\"");
	len = rec->descript_len;
	p = rec->descript;
	while (len-- > 0){
	    if (*p == '\n'){
		fprintf(fp, "\\n");
	    }else{
		fwrite(p, 1, 1, fp);
	    }
	    p++;
	}
	fprintf(fp, "\",");
	/* 4 */
	Calendar_GetDateByNo(rec->start_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 5 */
	Calendar_GetDateByNo(rec->end_date, &year, &month, &day);
	fprintf(fp, "%04d/%02d/%02d,", year, month, day);
	/* 6 */
	fprintf(fp, ",");
	/* 7 */
	fprintf(fp, ",");
	/* 8 */
	fprintf(fp, ",");
	/* 9 */
	fprintf(fp, "\"");
	if (rec->link_len > 0){
	    fprintf(fp, "file:/");
	    fwrite(rec->link, rec->link_len, 1, fp);
	}
	fprintf(fp, "\"\n");
	break;
    }
    return true;
}

/* ԎwCSV */
bool
File_CSVWriteRegion(HWND win)
{
    bool r;
    uint16 year, month, day, date;
    char msg[80];
    Record *rec;
    uint16 cookie;
    FILE *fp;

    D(mgDPrintf("File_CSVWriteRegion()"));

    GenDlg_SetTitle(" wȑO");

    r = mgDialogBoxParam(ghTaskInstance,
			 (LPCSTR)"GenDlg",
			 win,
			 (DLGPROC)GenDlgWinProc,
			 (LPARAM)MAKELPARAM(0, GENDLG_DATE));
    if (!r){
	return false;
    }

    date = GenDlg_GetDate();
    Calendar_GetDateByNo(date, &year, &month, &day);

    sprintf(msg, "%dN%d%dȑO\nf[^o܂", year, month, day);
    r = (bool)mgMessageBox(win, msg, MB_OKCANCEL);
    if (!r){
	return false;
    }

    fp = fopen(TASK_CSV_FILE, OPENMODE_WRITE_TEXT);
    if (fp == NULL){
	mgMessageBox(win, "s܂", MB_OK);
	return false;
    }

    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	if (rec->end_date <= date
	    || (rec->type == RECORD_TYPE_TASK && rec->done_len > 0
		&& rec->done[0] <= date)){
	    r = File_CSVWriteItem(fp, rec);
	    if (!r){
		mgMessageBox(win, "s܂", MB_OK);
		fclose(fp);
		return false;
	    }
	}
    }
    
    fclose(fp);

    mgMessageBox(win, "o܂\nobt@C荞݂\nsĂ(task.csv)", MB_OK);
    return true;
}

/* SACeCSV */
bool
File_CSVWriteAll(HWND win)
{
    bool r;
    char msg[80];
    Record *rec;
    uint16 cookie;
    FILE *fp;

    D(mgDPrintf("File_CSVWriteAll()"));

    sprintf(msg, "CSV`ŏo܂");
    r = (bool)mgMessageBox(win, msg, MB_OKCANCEL);
    if (!r){
	return false;
    }

    fp = fopen(TASK_CSV_FILE, OPENMODE_WRITE_TEXT);
    if (fp == NULL){
	mgMessageBox(win, "s܂", MB_OK);
	return false;
    }

    cookie = Index_Rewind();
    while (nil != (rec = Index_FindNext(&cookie))){
	r = File_CSVWriteItem(fp, rec);
	if (!r){
	    mgMessageBox(win, "s܂", MB_OK);
	    fclose(fp);
	    return false;
	}
    }

    fclose(fp);
    
    mgMessageBox(win, "o܂\nobt@C荞݂\nsĂ(task.csv)", MB_OK);
    return true;
}

