/*
   main and its window procedure
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: taskmg.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "db.h"
#include "record.h"
#include "taskmg.h"
#include "taskid.h"
#include "additem.h"
#include "funckey.h"
#include "view.h"
#include "viewfile.h"
#include "context.h"
#include "month.h"
#include "util.h"
#include "coop.h"
#include <string.h>
#include <stdio.h>
#include <malloc.h>
#include <u-k.h>
#include <cmdefwnd.h>

/* -- global variables */

/* -- file local variables */

/* CEBhE */
static char *nameMainWinClass = "TaskMain";

/* -- file local methods */
LRESULT CALLBACK
MainWinProc(HWND, UINT, WPARAM, LPARAM, POINT, POINT);
static bool
MainWin_Initialize(int argc, char *argv[]);
static bool
MainWin_Finalize(void);

/* -- methods */

/* CEBhȄ */
static bool
MainWin_Initialize(int argc, char *argv[])
{
    uint8 hour, min;
    WNDCLASS wndclass;
    IMESTRUCT imeStruct;
    bool r;

    D(mgDPrintf("free mem size: %ld", freemem_size()));

    gTodayDate = DB_GetDate(&gTodayYear, &gTodayMonth, &gTodayDay,
			    &hour, &min);
    gMode = MODE_VIEW;
    gNotUpdateView = false;
    gLabel = 0;
    gCPU = UNI_CPUHIGH;
#if DEBUG
    gDebug = true;
#else
    gDebug = false;
#endif

    /* f[^x[X̏ */
    DB_Init();
    DB_SetMode(DB_MODE_PSEUDO_SERVER);
    DB_SetUser(RECORD_USER_ANONYMOUS);

    /* ReLXg̓ǂݍ */
    if (!Context_Load()){
	Context_Defaults();
    }
    /* x̓ǂݍ */
    Context_InitLabelName();
    Context_LoadLabelName();
    Context_ReflectEnvironment();

#if 0
    /* CPUx̐ݒ */
    MainWin_SetCPUSpeed(gCPU);
    /* NOTE: x̌łȂ΂ȂȂ */
#endif

    /* R[h̓ǂݍ */
    RecordFile_Load();
    gDBDirty = false;

    /* c[Ag */
    Coop_StartFromOther(argc, argv);
    /* ACE */
    Coop_ReadACEFile();

    /* View̏ */
    r = View_Create();
    if (!r){
	return false;
    }

    /* ^XN̓o^ */
    ghTaskInstance = mgCreateTask();

    /* CEChENX̓o^ */
    wndclass.lpszClassName = nameMainWinClass;
    wndclass.lpfnWndProc = (WNDPROC)MainWinProc;
    wndclass.hInstance = ghTaskInstance;
    wndclass.lpszMenuName = RS_MENU_MAIN;
    if (!mgRegisterClass(&wndclass)){
	mgDestroyTask(ghTaskInstance);
	return false;
    }

    /* CEChE̐ */
    ghMainWin = mgCreateMainWindow(ghTaskInstance,
				  (LPCSTR)nameMainWinClass,
				  (LPCSTR)"MAINDLG",
				  (LPCSTR)NULL);
    if (ghMainWin == 0){
	mgUnregisterClass(nameMainWinClass, ghTaskInstance);
	mgDestroyTask(ghTaskInstance);
	return false;
    }

    /* CEChẺ	*/
    mgShowWindow(ghMainWin, SW_SHOW);

    /* CEChË̖̍ĕ` */
    mgUpdateWindow(ghMainWin);

    /* */
    imeStruct.fnc = IME_SETCONTROL;
    imeStruct.wParam = IME_CONTROL_ON;
    r = mgSendIMEMessage(ghMainWin, (LPARAM)&imeStruct);
    if (!r){
	return false;
    }

    /* t@NVL[̏ */
    FuncKey_Initialize();

    return true;
}

/* CEBhE̕Еt */
static bool
MainWin_Finalize(void)
{
    D(mgDPrintf("free mem size: %ld", freemem_size()));

    /* f[^x[X̕Еt */
    if (gDBDirty)
	RecordFile_Save();
    DB_Tidy();
    _heapmin();

    /* ReLXg̕ۑ */
    Context_Save();

    /* CEBhENX̍폜 */
    mgUnregisterClass(nameMainWinClass, ghTaskInstance);

    /* ^XN̍폜 */
    mgDestroyTask(ghTaskInstance);

    D(mgDPrintf("FIN"));

    return true;
}

/* C */
int main(int argc, char *argv[])
{
    MSG msg;

    MainWin_Initialize(argc, argv);

    /* bZ[W[v	*/
    while (mgGetMessage(&msg, (HWND)0)){
	if (!mgTranslateFunction((const MSG FAR*)&msg)){
	    if (!mgTranslateMessage((MSG FAR*)&msg)){
		/* bZ[W̃fBXpb` */
		mgDispatchMessage(&msg);
	    }
	}
    }

    MainWin_Finalize();

    /* I */
    return msg.wParam;
}

/* CEChEvV[W */
LRESULT CALLBACK
MainWinProc(HWND hSelf, UINT msg,
	    WPARAM wParam, LPARAM lParam,
	    POINT LPoint, POINT DPoint)
{
    IMESTRUCT imeStruct;
    int r;

    /* CPUɂ */
    if (msg != WM_TIMER && gCPU != UNI_CPUHIGH){
	MainWin_SetCPUSpeed(UNI_CPUHIGH);
	View_UpdateTitle();
    }

    /* bZ[W */
    switch (msg){
	case WM_CREATE:			      /* EBhE쐬 */
	/* tH[JX */
	mgSetFocus(hSelf);
	/* wiF̐ݒ */
	mgSetWindowBackcolor(hSelf, (COLORREF)CL_LTGRAY);
	/* ^C}̐ݒ */
	mgSetTimer(hSelf, ID_TIMER_VIEW, 300);	/* 0.3b */
	mgSetTimer(hSelf, ID_TIMER_SPEED, 5000); /* 5b */
	/* Wʂɗ */
	return CMDefWindowProc(hSelf, msg, wParam, lParam);
	break;

	case WM_CLOSE:			      /* EBhȄ */
	/* */
	imeStruct.fnc = IME_SETCONTROL;
	imeStruct.wParam = IME_CONTROL_OFF;
	mgSendIMEMessage(ghMainWin, (LPARAM)&imeStruct);

	/* CEChE̔j */
	mgDestroyWindow(hSelf);
	return 0;
	break;

	case WM_DESTROY:		      /* EBhE̕Еt */
	/* View̕Еt */
	View_Destroy();
	/* t@NVL[̍폜 */
	FuncKey_Finilize();
	/* bZ[W[v̏I */
	mgPostQuitMessage();
	/* Wʂɗ */
	return CMDefWindowProc(hSelf, msg, wParam, lParam);
	break;

	case WM_KEYDOWN:		      /* L[R[h */
	switch (wParam){
	    case VK_APCHANGE:		      /* `o؂ւ */
	    gMode = MODE_APCHANGE;
	    mgCreateApChangeWindow();
	    gMode = MODE_VIEW;
	    return 1;
	    break;
	    case VK_MENU:		      /* j[ */
	    r = mgCreateMenuWindow((LPCSTR)NULL,
				   (DWORD)0,
				   MENU_POS_X, MENU_POS_Y,
				   MENU_WIDTH, MENU_HEIGHT,
				   hSelf,
				   (HMENU)NULL,
				   ghTaskInstance);
	    return 1;
	    break;

	    default:			      /* ̑ */
	    /* ݂Viewɗ */
	    r = View_KeyDown(wParam, lParam);
	    if (r){
		/* ς */
		return 1;
	    }
	    break;
	}
	/* ftHgɗ */
	return mgDefWindowProc(hSelf, msg, wParam, lParam);
	break;

	case WM_COMMAND:		      /* R}h */
	if (FuncKey_IsFuncCommand(wParam)){
	    return FuncKey_Dispatch(hSelf, msg, wParam, lParam);
	}
	switch (wParam){
	    default:
	    /* ݂Viewɗ */
	    r = View_Command(hSelf, wParam, lParam);
	    if (r){
		/* ς */
		return 1;
	    }
	}
	break;

	case WM_PAINT:			      /* `揈 */
	D(mgDPrintf("got WM_PAINT at main"));
	if (gMode != MODE_ADD && gMode != MODE_DIALOG){
	    View_Redraw();
	}
	return CMDefWindowProc(hSelf, msg, wParam, lParam);
	break;

	case UWM_HICONDOWN:		      /* n[hACR_E */
	if (wParam == (UINT)HICON_PIM){
	    return mgDefWindowProc(hSelf, msg, wParam,lParam);
	}
	break;

	case UWM_HICONUP:		      /* n[hACRAbv */
	/* neen[hACR^bvꂽd؂ */
	if (wParam == (UINT)HICON_KILL){
	    return mgDefWindowProc(hSelf, msg, wParam,lParam);
	}
	break;

	case WM_TIMER:			      /* ^C}Cxg */
	if (wParam == ID_TIMER_VIEW){
	    /* ݂Viewɗ */
	    r = View_Timer();
	}else if (wParam == ID_TIMER_SPEED){
	    if (gMode == MODE_VIEW && gCPU != UNI_CPULOW){
		MainWin_SetCPUSpeed(UNI_CPULOW);
		View_UpdateTitle();
	    }
	}
	/* Wʂɗ */
	return CMDefWindowProc(hSelf, msg, wParam, lParam);
	break;

	case WM_POWER:
	if (wParam == PWR_SUSPENDRESUME){
	    ViewFile_CheckBattery(true);
	}
	if (wParam == PWR_SUSPENDRESUME
	    && gOpt_PowerOnToday
	    && gMode == MODE_VIEW){
	    /* uvɈړ */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_NOW,
			  MAKELPARAM(0, BN_CLICKED));
	}
	/* Wʂɗ */
	return CMDefWindowProc(hSelf, msg, wParam, lParam);
	break;

	case WM_LBUTTONDOWN:
	case WM_LBUTTONUP:
	case UWM_UPDATETIME:
	/* Wʂɗ */
	return CMDefWindowProc(hSelf, msg, wParam, lParam);
	break;

	default:
	/* ftHgɗ */
	return mgDefWindowProc(hSelf, msg, wParam, lParam);
	break;
    }

    return 0;
}

/* ^Cg̐ݒ */
void
MainWin_SetTitle(char *str)
{
    static CMWNDSTYLE styleWinCommon;
    static char title[60];
    static byte iconBitmap[256];
    static bool first = true;

    /* ACRrbg}bṽ[h */
    if (first){
	mgLoadBitmap(NULL, "taskmg_kb", iconBitmap);
	first = false;
    }

    /* ^Cg */
#if 0
    sprintf(title, "^XN  %s", str);
#else
    sprintf(title, "^XN  %s", str);
#endif

    /* Wʍ쐬 */
    styleWinCommon.hInst = ghTaskInstance;
    styleWinCommon.lpIcon = (LPCSTR)iconBitmap;
    styleWinCommon.lpszTitle = title;
    styleWinCommon.uStyle = CM_WS_CLOCK;
    CMSetWindowStyle(&styleWinCommon);
}

/* CPUx̐ݒ */
void
MainWin_SetCPUSpeed(uint16 speed)
{
    mgSetCpuClock(speed);
    gCPU = speed;
}
