/*
   view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: view.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#define VIEW
#include "arch.h"
#include "global.h"
#include "taskid.h"
#include "taskmg.h"
#include "view.h"
#include "view3m.h"
#include "view5w.h"
#include "view3w.h"
#include "view2w.h"
#include "viewdl.h"
#include "viewws.h"
#include "viewwl.h"
#include "viewbr.h"
#include "viewitem.h"
#include "viewtime.h"
#include "viewfile.h"
#include "viewcal.h"
#include "additem.h"
#include "gendlg.h"
#include "env.h"
#include "calendar.h"
#include "month.h"
#include "info.h"
#include "file.h"
#include "jump.h"
#include "db.h"
#include "context.h"
#include "util.h"
#include "coop.h"
#include <stdio.h>
#include <u-k.h>

/* -- file local macros */

#define DELAYED_UPDATE_DELAY	2
#define VIEW_SHORTCUT

/* -- file local variables */
static uint16 view_timer = 0;

/* -- file local methods */
static bool
View_ChangeSize(void);
static char *
View_GetDialog(void);

/* -- methods */

/* View */
bool
View_Create(void)
{
    view_timer = 0;

    switch (gCurrentView){
	case VIEW_3MONTH:
	return View3M_Create();
	break;
	case VIEW_5WEEK:
	return View5W_Create();
	break;
	case VIEW_3WEEK:
	return View3W_Create();
	break;
	case VIEW_2WEEK:
	return View2W_Create();
	break;
	case VIEW_WEEK_S:
	return ViewWS_Create();
	break;
	case VIEW_WEEK_L:
	return ViewWL_Create();
	break;
	case VIEW_DAY_L:
	return ViewDL_Create();
	break;
	case VIEW_BROWSER:
	return ViewBR_Create();
	break;
    }
    return true;
}

/* ViewЕt */
bool
View_Destroy(void)
{
    RECT rect;

    switch (gCurrentView){
	case VIEW_3MONTH:
	View3M_Destroy();
	break;
	case VIEW_5WEEK:
	View5W_Destroy();
	break;
	case VIEW_3WEEK:
	View3W_Destroy();
	break;
	case VIEW_2WEEK:
	View2W_Destroy();
	break;
	case VIEW_WEEK_S:
	ViewWS_Destroy();
	break;
	case VIEW_WEEK_L:
	ViewWL_Destroy();
	break;
	case VIEW_DAY_L:
	ViewDL_Destroy();
	break;
	case VIEW_BROWSER:
	ViewBR_Destroy();
	break;
    }

    /* ݂̕`̖ */
    rect.left = REGION_USER_LEFT;
    rect.right = REGION_USER_RIGHT;
    rect.top = REGION_USER_TOP;
    rect.bottom = REGION_USER_BOTTOM;
    mgInvalidateRect(ghMainWin, &rect, TRUE);

    return true;
}

/* ^Cg̍XV */
void
View_UpdateTitle(void)
{
    uint16 year, month, day, dayWeek;
    char str[80];

    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    dayWeek = Calendar_GetDayOfWeekByNo(gCursorDate);

    if (!gDebug){
	if (gLabel > 0){
	    sprintf(str, "%4dN%2d%2d(%s) %8s",
		    year, month, day, nameDayWeek[dayWeek],
		    Context_GetLabelName(gLabel));
	}else{
	    sprintf(str, "%4dN%2d%2d(%s)",
		    year, month, day, nameDayWeek[dayWeek]);
	}
    }else{
	sprintf(str, "%4dN%2d%2d(%s) %8s CPU:%s,FREE:%ld",
		year, month, day, nameDayWeek[dayWeek],
		Context_GetLabelName(gLabel),
		((gCPU==UNI_CPUHIGH)?"HIGH":((gCPU==UNI_CPULOW)?"LOW":"MID")),
		freemem_size());
    }
    MainWin_SetTitle(str);
}


/* View^C} */
bool
View_Timer(void)
{
    if (view_timer > 0){
	view_timer--;
	if (view_timer == 0 && gMode == MODE_VIEW){
	    View_UpdateTitle();
	    switch (gCurrentView){
		case VIEW_3MONTH:
		View3M_DelayedRedraw();
		return true;
		break;
		case VIEW_5WEEK:
		View5W_DelayedRedraw();
		return true;
		break;
		case VIEW_3WEEK:
		View3W_DelayedRedraw();
		return true;
		break;
		case VIEW_2WEEK:
		View2W_DelayedRedraw();
		return 0;
		break;
		case VIEW_WEEK_S:
		ViewWS_DelayedRedraw();
		return 0;
		break;
		case VIEW_WEEK_L:
		ViewWL_DelayedRedraw();
		return 0;
		case VIEW_DAY_L:
		ViewDL_DelayedRedraw();
		return 0;
		break;
		case VIEW_BROWSER:
		ViewBR_DelayedRedraw();
		break;
	    }
	}
    }

    /* Ȃ */
    return false;
}

/* View^C} */
bool
View_SetTimer(void)
{
    view_timer = DELAYED_UPDATE_DELAY;
    return true;
}

/* Viewĕ` */
void
View_Redraw(void)
{
#if 0
    mgShowWindow(ghMainWin, SW_HIDE);
#endif

    View_UpdateTitle();

    switch (gCurrentView){
	case VIEW_3MONTH:
	View3M_Redraw();
	break;
	case VIEW_5WEEK:
	View5W_Redraw();
	break;
	case VIEW_3WEEK:
	View3W_Redraw();
	break;
	case VIEW_2WEEK:
	View2W_Redraw();
	break;
	case VIEW_WEEK_S:
	ViewWS_Redraw();
	break;
	case VIEW_WEEK_L:
	ViewWL_Redraw();
	break;
	case VIEW_DAY_L:
	ViewDL_Redraw();
	break;
	case VIEW_BROWSER:
	ViewBR_Redraw();
	break;
    }

#if 0
    mgShowWindow(ghMainWin, SW_SHOW);
    mgUpdateWindow(ghMainWin);
#endif
}

/* L[R[h */
bool
View_KeyDown(WPARAM wParam, LPARAM lParam)
{
    bool r;

    switch (wParam){
	case 'S':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* S */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_ADD_SCHEDULE,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	if (mgGetKeyState(VK_CONTROL)){	      /* CTRL+S */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_MENUITEM_SAVE,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'T':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* T */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_ADD_TASK,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'H':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* H */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_ADD_HOLIDAY,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'L':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* L */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_LINK,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}else if (mgGetKeyState(VK_CONTROL)){ /* CTRL+L */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_NOW,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'U':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* U */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_UNLINK,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'P':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* P */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_ITEMUP,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	if (mgGetKeyState(VK_CONTROL) && mgGetKeyState(VK_SHIFT)){ /* SC+P */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_HOURUP,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	if (mgGetKeyState(VK_CONTROL)){	      /* CTRL+P */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_UP,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'N':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* N */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_ITEMDOWN,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	if (mgGetKeyState(VK_CONTROL) && mgGetKeyState(VK_SHIFT)){ /* SC+N */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_HOURDOWN,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	if (mgGetKeyState(VK_CONTROL)){	      /* CTRL+N */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_DOWN,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'B':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* B */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_BROWSE,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}else if (mgGetKeyState(VK_CONTROL)){ /* CTRL+B */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_LEFT,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'F':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* F */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_FILER,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}else if (mgGetKeyState(VK_CONTROL)){ /* CTRL+F */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_RIGHT,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'M':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* M */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_MAILER,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'X':
	if (mgGetKeyState(VK_CONTROL)){	      /* CTRL+X */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_MENUITEM_CUT,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'C':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* C */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_MONTH,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}else if (mgGetKeyState(VK_CONTROL)){ /* CTRL+C */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_MENUITEM_COPY,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'V':
	if (mgGetKeyState(VK_CONTROL)){	      /* CTRL+V */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_MENUITEM_PASTE,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'D':
	if (mgGetKeyState(VK_CONTROL)){	      /* CTRL+D */
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)VK_DELETE,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;
	case 'J':
	if (mgGetKeyState(VK_CONTROL)){	      /* CTRL+J */
	    switch (gCurrentView){
		case VIEW_3MONTH:
		/* Õr[̂̓Ɉڍs */
		View_Destroy();
		gCurrentView = gPreviousView;
		gPreviousView = VIEW_3MONTH;
		r = View_Create();
		View_Redraw();
		return r;
		break;
		/* gO */
		case VIEW_5WEEK:
		case VIEW_3WEEK:
		case VIEW_2WEEK:
		mgSendMessage(ghMainWin, WM_COMMAND,
			      (WPARAM)ID_FUNC_WEEK, MAKELPARAM(0, BN_CLICKED));
		return true;
		break;
		case VIEW_WEEK_S:
		case VIEW_WEEK_L:
		mgSendMessage(ghMainWin, WM_COMMAND,
			      (WPARAM)ID_FUNC_DAY, MAKELPARAM(0, BN_CLICKED));
		return true;
		break;
		case VIEW_DAY_L:
		case VIEW_BROWSER:
		mgSendMessage(ghMainWin, WM_COMMAND,
			      (WPARAM)ID_FUNC_WEEKS, MAKELPARAM(0, BN_CLICKED));
		return true;
		break;
	    }
	}
	break;

#if 0
	case 'E':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* E */
	    Coop_ExecEditor();
	    return true;
	}
	break;
#endif

	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* 1-8 */
	    Coop_ExecACE(wParam - '1');
	    return true;
	}
	break;

	case VK_RETURN:
	D(mgDPrintf("got RETURN at Common View"));
	if (!mgGetKeyState(VK_CONTROL) && !mgGetKeyState(VK_SHIFT)){ /* RETURN */
	    if (cursor_focus == FOCUS_ITEMLIST){
		/* ̍ڂҏW */
		r = Add_EditItem(gCursorDate, ViewItem_GetCursor());
		if (r){
		    ViewItem_ResetCursor();
		    View_Redraw();
		}
		return true;
	    }else{
		switch (gCurrentView){
		    case VIEW_3MONTH:
		    /* Õr[̂̓Ɉڍs */
		    View_Destroy();
		    gCurrentView = gPreviousView;
		    gPreviousView = VIEW_3MONTH;
		    r = View_Create();
		    View_Redraw();
		    return r;
		    break;
		    /* gO */
		    case VIEW_5WEEK:
		    case VIEW_3WEEK:
		    case VIEW_2WEEK:
		    mgSendMessage(ghMainWin, WM_COMMAND,
				  (WPARAM)ID_FUNC_WEEK, MAKELPARAM(0, BN_CLICKED));
		    return true;
		    break;
		    case VIEW_WEEK_S:
		    case VIEW_WEEK_L:
		    mgSendMessage(ghMainWin, WM_COMMAND,
				  (WPARAM)ID_FUNC_DAY, MAKELPARAM(0, BN_CLICKED));
		    return true;
		    break;
		    case VIEW_DAY_L:
		    case VIEW_BROWSER:
		    mgSendMessage(ghMainWin, WM_COMMAND,
				  (WPARAM)ID_FUNC_WEEKS, MAKELPARAM(0, BN_CLICKED));
		    return true;
		    break;
		}
	    }
	}else if (mgGetKeyState(VK_CONTROL)){ /* CONTROL+RETURN */
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_BROWSE,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	break;

	case VK_BACK:			      /* BS */
	D(mgDPrintf("got BS at Common View"));
	switch (gCurrentView){
	    case VIEW_3MONTH:
	    /* Õr[̂̓Ɉڍs */
	    View_Destroy();
	    gCurrentView = gPreviousView;
	    gPreviousView = VIEW_3MONTH;
	    r = View_Create();
	    View_Redraw();
	    return r;
	    break;
	    case VIEW_BROWSER:
	    View_Destroy();
	    gCurrentView = gPreviousView;
	    gPreviousView = VIEW_BROWSER;
	    r = View_Create();
	    View_Redraw();
	    return r;
	    break;

	    /* tgO */
	    case VIEW_5WEEK:
	    case VIEW_3WEEK:
	    case VIEW_2WEEK:
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_DAY, MAKELPARAM(0, BN_CLICKED));
	    return true;
	    break;
	    case VIEW_WEEK_S:
	    case VIEW_WEEK_L:
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_WEEKS, MAKELPARAM(0, BN_CLICKED));
	    return true;
	    break;
	    case VIEW_DAY_L:
	    mgSendMessage(ghMainWin, WM_COMMAND,
			  (WPARAM)ID_FUNC_WEEK, MAKELPARAM(0, BN_CLICKED));
	    return true;
	    break;
	}
	break;

	case VK_ESCAPE:			      /* ESC */
	D(mgDPrintf("got ESC at Common View"));
	/* `o؂ւ */
	gMode = MODE_APCHANGE;
	mgCreateApChangeWindow();
	gMode = MODE_VIEW;
	return true;
#if 0
	/* Õr[̂̓Ɉڍs */
	if (gPreviousView != gCurrentView){
	    uint16 tmp;
	    View_Destroy();
	    tmp = gCurrentView;
	    gCurrentView = gPreviousView;
	    gPreviousView = tmp;
	    r = View_Create();
	    View_Redraw();
	    return r;
	}
	return false;
#endif
	break;

	case VK_DELETE:			      /* DEL */
	D(mgDPrintf("got DEL at View"));
	mgSendMessage(ghMainWin, WM_COMMAND,
		      (WPARAM)ID_MENUITEM_DELETE,
		      MAKELPARAM(0, BN_CLICKED));
	return true;
	break;

	case VK_SPACE:			      /* SPACE */
	D(mgDPrintf("got SPACE at View"));
	if (cursor_focus == FOCUS_ITEMLIST){
	    /* gO */
	    r = Add_ToggleDoneItem(gCursorDate, ViewItem_GetCursor());
	    if (r){
		ViewItem_ResetCursor();
		View_Redraw();
	    }
	}else{
	    /* Ȃ */
	    return false;
	}
	break;

	case VK_UP:
	if (mgGetKeyState(VK_SHIFT)){
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_HOURUP,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}else if (mgGetKeyState(VK_CONTROL)){
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_ITEMUP,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	/* ӂ͑fʂ */
	break;
	
	case VK_DOWN:
	if (mgGetKeyState(VK_SHIFT)){
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_HOURDOWN,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}else if (mgGetKeyState(VK_CONTROL)){
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_ITEMDOWN,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	/* ӂ͑fʂ */
	break;

	case VK_PRIOR:
	if (gOpt_SelectItemByPageUpDown){
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_ITEMUP,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	/* ӂ͑fʂ */
	break;
	
	case VK_NEXT:
	if (gOpt_SelectItemByPageUpDown){
	    mgSendMessage(ghMainWin, WM_KEYDOWN,
			  (WPARAM)UVK_ITEMDOWN,
			  MAKELPARAM(0, BN_CLICKED));
	    return true;
	}
	/* ӂ͑fʂ */
	break;
	
    }

    /* ĂȂ̂current viewdelegate */
    switch (gCurrentView){
	case VIEW_3MONTH:
	return View3M_KeyDown(wParam, lParam);
	break;
	case VIEW_5WEEK:
	return View5W_KeyDown(wParam, lParam);
	break;
	case VIEW_3WEEK:
	return View3W_KeyDown(wParam, lParam);
	break;
	case VIEW_2WEEK:
	return View2W_KeyDown(wParam, lParam);
	break;
	case VIEW_WEEK_S:
	return ViewWS_KeyDown(wParam, lParam);
	break;
	case VIEW_WEEK_L:
	return ViewWL_KeyDown(wParam, lParam);
	break;
	case VIEW_DAY_L:
	return ViewDL_KeyDown(wParam, lParam);
	break;
	case VIEW_BROWSER:
	return ViewBR_KeyDown(wParam, lParam);
	break;
    }
    return 0;
}

/* R}h */
bool
View_Command(HWND hSelf, WPARAM wParam, LPARAM lParam)
{
    int r = true;
    uint8 hour, min;

    if (HIWORD(lParam) == BN_CLICKED){
	switch (wParam){
	    case ID_MENUITEM_SAVE:
	    File_Save(hSelf);
	    return true;
	    break;
	    case ID_MENUITEM_MERGE:
	    File_Merge(hSelf);
	    return true;
	    break;
	    case ID_MENUITEM_RENDEL:
	    File_DeleteRegion(hSelf);
	    return true;
	    break;
	    case ID_MENUITEM_WRITE_CSV_REGION:
	    File_CSVWriteRegion(hSelf);
	    break;
	    case ID_MENUITEM_WRITE_CSV_ALL:
	    File_CSVWriteAll(hSelf);
	    break;
	    case ID_MENUITEM_OPEN:
	    Info_NotImplemented(hSelf);
	    break;

	    case ID_MENUITEM_DELETE:
	    if (cursor_focus == FOCUS_ITEMLIST){
		/* mF */
		r = mgMessageBox(ghMainWin, "Iꂽڂ\n폜܂",
				 MB_OKCANCEL|MB_DEFBUTTON2);
		if (r == IDOK){
		    /* ̍ڂ */
		    r = Add_DeleteItem(gCursorDate, ViewItem_GetCursor());
		    if (r){
			ViewItem_ResetCursor();
			View_Redraw();
		    }
		}
		return true;
	    }else{
		/* Ȃ */
		return false;
	    }
	    break;
	    case ID_MENUITEM_CUT:
	    if (cursor_focus == FOCUS_ITEMLIST){
		/* ̍ڂJbg */
		r = Misc_CutItem(gCursorDate, ViewItem_GetCursor());
		if (r){
		    ViewItem_ResetCursor();
		    View_Redraw();
		}
	    }
	    return true;
	    break;
	    case ID_MENUITEM_COPY:
	    if (cursor_focus == FOCUS_ITEMLIST){
		/* ̍ڂRs[ */
		r = Misc_CopyItem(gCursorDate, ViewItem_GetCursor());
		if (r){
		    ViewItem_ResetCursor();
		    View_Redraw();
		}
	    }
	    return true;
	    break;
	    case ID_MENUITEM_PASTE:
	    /* ̍ڂy[Xg */
	    r = Misc_PasteItem(gCursorDate);
	    if (r){
		ViewItem_ResetCursor();
		View_Redraw();
	    }
	    return true;
	    break;

	    case ID_MENUITEM_UNDO:
	    Info_NotImplemented(hSelf);
	    break;

	    case ID_FUNC_ADD_SCHEDULE:
	    case ID_MENUITEM_ADD_SCHEDULE:
	    r = mgDialogBoxParam(ghTaskInstance,
				 (LPCSTR)View_GetDialog(),
				 hSelf,
				 (DLGPROC)AddWinProc,
				 (LPARAM)ADDITEM_SCHEDULE);
	    return (bool)r;
	    break;

	    case ID_FUNC_ADD_TASK:
	    case ID_MENUITEM_ADD_TASK:
	    r = mgDialogBoxParam(ghTaskInstance,
				 (LPCSTR)View_GetDialog(),
				 hSelf,
				 (DLGPROC)AddWinProc,
				 (LPARAM)ADDITEM_TASK);
	    return (bool)r;
	    break;

	    case ID_FUNC_ADD_HOLIDAY:
	    case ID_MENUITEM_ADD_HOLIDAY:
	    r = mgDialogBoxParam(ghTaskInstance,
				 (LPCSTR)View_GetDialog(),
				 hSelf,
				 (DLGPROC)AddWinProc,
				 (LPARAM)ADDITEM_HOLIDAY);
	    return (bool)r;
	    break;

	    case ID_FUNC_ENV:
	    case ID_MENUITEM_ENV:
	    r = mgDialogBoxParam(ghTaskInstance,
				 (LPCSTR)"EnvDlg",
				 hSelf,
				 (DLGPROC)EnvWinProc,
				 (LPARAM)0);
	    if (r){
		switch (gCurrentView){
		    case VIEW_3MONTH:
		    break;
		    case VIEW_5WEEK:
		    case VIEW_3WEEK:
		    case VIEW_2WEEK:
		    ViewCal_InitCursor();
		    break;
		    case VIEW_WEEK_S:
		    case VIEW_WEEK_L:
		    case VIEW_DAY_L:
		    ViewTime_InitCursor();
		    break;
		}
		View_Redraw();
	    }
	    return (bool)r;
	    break;

	    case ID_FUNC_JUMP:
	    case ID_FUNC_FIND:
	    case ID_MENUITEM_FIND:
	    case ID_FUNC_FINDNEXT:
	    case ID_MENUITEM_FINDNEXT:
	    return Jump_Command(hSelf, wParam, lParam);
	    break;

	    case ID_FUNC_NOW:
	    gTodayDate = DB_GetDate(&gTodayYear, &gTodayMonth, &gTodayDay,
				    &hour, &min);
	    gCursorDate = gTodayDate;
	    gCursorHour = hour;
	    /* current viewɗ */
	    break;

	    case ID_FUNC_SIZE:
	    case ID_MENUITEM_SIZE:
	    return View_ChangeSize();
	    break;

	    case ID_FUNC_MONTH:
	    case ID_MENUITEM_MONTH:
	    if (gCurrentView != gCurrentViewMonth){
		View_Destroy();
		gPreviousView = gCurrentView;
		gCurrentView = gCurrentViewMonth;
		r = View_Create();
		View_Redraw();
	    }
	    return (bool)r;
	    break;

	    case ID_FUNC_WEEKS:
	    case ID_MENUITEM_WEEKS:
	    if (gCurrentView != gCurrentViewWeeks){
		View_Destroy();
		gPreviousView = gCurrentView;
		gCurrentView = gCurrentViewWeeks;
		r = View_Create();
		View_Redraw();
	    }
	    return (bool)r;
	    break;

	    case ID_FUNC_WEEK:
	    case ID_MENUITEM_WEEK:
	    if (gCurrentView != gCurrentViewWeek){
		View_Destroy();
		gPreviousView = gCurrentView;
		gCurrentView = gCurrentViewWeek;
		r = View_Create();
		View_Redraw();
	    }
	    return (bool)r;
	    break;

	    case ID_FUNC_DAY:
	    case ID_MENUITEM_DAY:
	    if (gCurrentView != gCurrentViewDay){
		View_Destroy();
		gPreviousView = gCurrentView;
		gCurrentView = gCurrentViewDay;
		r = View_Create();
		View_Redraw();
	    }
	    return (bool)r;
	    break;

	    case ID_FUNC_LIST:
	    case ID_MENUITEM_LIST:
	    Info_NotImplemented(hSelf);
	    break;

	    case ID_MENUITEM_HELP:
	    gMode = MODE_HELP;
	    mgUniHelp(hSelf, NULL, 0);
	    gMode = MODE_VIEW;
	    return true;;
	    break;

	    case ID_MENUITEM_ABOUT:
	    Info_DisplayAbout(hSelf);
	    return true;
	    break;

	    case ID_FUNC_SEL_LABEL0:
	    gLabel = 0;
	    MonthRec_SelectLabel(gLabel);
	    if (cursor_focus == FOCUS_ITEMLIST){
		ViewItem_ResetCursor();
	    }
	    View_Redraw();
	    return true;
	    break;
	    case ID_FUNC_SEL_LABEL1:
	    gLabel = 1;
	    MonthRec_SelectLabel(gLabel);
	    if (cursor_focus == FOCUS_ITEMLIST){
		ViewItem_ResetCursor();
	    }
	    View_Redraw();
	    return true;
	    break;
	    case ID_FUNC_SEL_LABEL2:
	    gLabel = 2;
	    MonthRec_SelectLabel(gLabel);
	    if (cursor_focus == FOCUS_ITEMLIST){
		ViewItem_ResetCursor();
	    }
	    View_Redraw();
	    return true;
	    break;
	    case ID_FUNC_SEL_LABEL3:
	    gLabel = 3;
	    MonthRec_SelectLabel(gLabel);
	    if (cursor_focus == FOCUS_ITEMLIST){
		ViewItem_ResetCursor();
	    }
	    View_Redraw();
	    return true;
	    break;
	    case ID_FUNC_SEL_LABEL4:
	    gLabel = 4;
	    MonthRec_SelectLabel(gLabel);
	    if (cursor_focus == FOCUS_ITEMLIST){
		ViewItem_ResetCursor();
	    }
	    View_Redraw();
	    return true;
	    break;
	    case ID_FUNC_SEL_LABEL_CUST:
	    gLabel = 5;
	    Context_SelectCustomLabel();
	    if (cursor_focus == FOCUS_ITEMLIST){
		ViewItem_ResetCursor();
	    }
	    View_Redraw();
	    return true;
	    break;

	    case ID_FUNC_FILER:
	    case ID_MENUITEM_FILER:
	    Coop_ExecFiler();
	    return true;
	    break;

	    case ID_FUNC_MAILER:
	    case ID_MENUITEM_MAILER:
	    Coop_ExecMailer();
	    return true;
	    break;

	    case ID_FUNC_BROWSE:
	    case ID_MENUITEM_BROWSE:
	    if (cursor_focus == FOCUS_ITEMLIST){
		/* Nǂ */
		ViewFile_FollowLink(gCursorDate, ViewItem_GetCursor());
	    }
	    View_Destroy();
	    if (gCurrentView != VIEW_BROWSER){
		gPreviousView = gCurrentView;
		gCurrentView = VIEW_BROWSER;
	    }
	    r = View_Create();
	    View_Redraw();
	    return (bool)r;
	    break;
	    case ID_FUNC_LINK:
	    case ID_MENUITEM_LINK:
	    if (gCurrentView == VIEW_BROWSER
		&& cursor_focus == FOCUS_ITEMLIST){
		/* Nݒ肷 */
		ViewFile_LinkItem(gCursorDate, ViewItem_GetCursor());
		return true;
	    }else{
		/* Ȃ */
		return false;
	    }
	    break;
	    case ID_FUNC_UNLINK:
	    case ID_MENUITEM_UNLINK:
	    if (cursor_focus == FOCUS_ITEMLIST){
		/* N */
		ViewFile_UnlinkItem(gCursorDate, ViewItem_GetCursor());
		return true;
	    }else{
		/* Ȃ */
		return false;
	    }
	    break;
	}
    }

    /* ĂȂ̂current viewdelegate */
    switch (gCurrentView){
	case VIEW_3MONTH:
	return View3M_Command(wParam, lParam);
	break;
	case VIEW_5WEEK:
	return View5W_Command(wParam, lParam);
	break;
	case VIEW_3WEEK:
	return View3W_Command(wParam, lParam);
	break;
	case VIEW_2WEEK:
	return View2W_Command(wParam, lParam);
	break;
	case VIEW_WEEK_S:
	return ViewWS_Command(wParam, lParam);
	break;
	case VIEW_WEEK_L:
	return ViewWL_Command(wParam, lParam);
	break;
	case VIEW_DAY_L:
	return ViewDL_Command(wParam, lParam);
	break;
	case VIEW_BROWSER:
	return ViewBR_Command(wParam, lParam);
	break;
    }
    return 0;
}

/* TCYύX */
static bool
View_ChangeSize(void)
{
    bool r = true;

    /* View̕ύX */
    switch (gCurrentView){
	case VIEW_5WEEK:
	View_Destroy();
	gCurrentView = VIEW_3WEEK;
	gCurrentViewWeeks = VIEW_3WEEK;
	break;
	case VIEW_3WEEK:
	View_Destroy();
	gCurrentView = VIEW_2WEEK;
	gCurrentViewWeeks = VIEW_2WEEK;
	break;
	case VIEW_2WEEK:
	View_Destroy();
	gCurrentView = VIEW_5WEEK;
	gCurrentViewWeeks = VIEW_5WEEK;
	break;
	case VIEW_WEEK_S:
	View_Destroy();
	gCurrentView = VIEW_WEEK_L;
	gCurrentViewWeek = VIEW_WEEK_L;
	break;
	case VIEW_WEEK_L:
	View_Destroy();
	gCurrentView = VIEW_WEEK_S;
	gCurrentViewWeek = VIEW_WEEK_S;
	break;

	default:
	return false;
	break;
    }

    gPreviousView = gCurrentView;
    r = View_Create();
    View_Redraw();
    return r;
}

/* ViewɃ^Ce[u邩 */
bool
View_HasTimeTable(void)
{
    if (gCurrentView == VIEW_WEEK_S
	|| gCurrentView == VIEW_WEEK_L
	|| gCurrentView == VIEW_DAY_L){
	return true;
    }else{
	return false;
    }
}

/* _CAOTCY̎擾 */
static char *
View_GetDialog(void)
{
    if (gCurrentView == VIEW_BROWSER){
	D(mgDPrintf("use SMALL DIALOG"));
	return "AddItemS";
    }else if (gOpt_LargeDialog){
	D(mgDPrintf("use LARGE DIALOG"));
	return "AddItemL";
    }else{
	D(mgDPrintf("use NORMAL DIALOG"));
	return "AddItem";
    }
}
