/*
   day view (large size)
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewdl.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "taskid.h"
#include "view.h"
#include "viewdl.h"
#include "viewitem.h"
#include "viewtime.h"
#include "additem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>

/* -- file local macros */
#define HOURCELL_ROWS		11
#define HOURCELL_COLS		1
#define HOURCELL_WIDTH		(16*10+3)
#define HOURCELL_HEIGHT		(16*1+2)

#define REGION_HOUR_WIDTH	(6*5+2)
#define REGION_HOUR_HEIGHT	(HOURCELL_HEIGHT*HOURCELL_ROWS)
#define REGION_TIMETABLE_WIDTH	(HOURCELL_WIDTH*HOURCELL_COLS)
#define REGION_TIMETABLE_HEIGHT	(HOURCELL_HEIGHT*HOURCELL_ROWS)

#define ITEMCELL_ROWS		10
#define ITEMCELL_COLS		1
#define ITEMCELL_WIDTH		(REGION_USER_WIDTH-REGION_HOUR_WIDTH-REGION_TIMETABLE_WIDTH-2)
#define ITEMCELL_HEIGHT		(16+2)

#define REGION_DATE_WIDTH	(ITEMCELL_WIDTH)
#define REGION_DATE_HEIGHT	(16*1+5)
#define REGION_DATE_LEFT	(REGION_HOUR_WIDTH+REGION_TIMETABLE_WIDTH+2)
#define REGION_DATE_TOP		(REGION_USER_TOP)
#define REGION_ITEMS_WIDTH	(ITEMCELL_WIDTH)
#define REGION_ITEMS_HEIGHT	(ITEMCELL_HEIGHT*ITEMCELL_ROWS)

/* -- file local variables */
static uint16 originDate, originHour;
static int cursorY; 

/* -- file local methods */

/* -- methods */

/* ViewDL */
bool
ViewDL_Create(void)
{
    D(mgDPrintf("ViewDL create\n"));

    /* ViewŗL̕ϐ */
    cursor_focus = FOCUS_TIMETABLE;
    cursor_prev_focus = FOCUS_TIMETABLE;
    ViewItem_Init(REGION_DATE_LEFT, REGION_DATE_TOP,
		  REGION_DATE_WIDTH, REGION_DATE_HEIGHT,
		  REGION_ITEMS_WIDTH, REGION_ITEMS_HEIGHT,
		  ITEMCELL_WIDTH, ITEMCELL_HEIGHT,
		  ITEMCELL_COLS, ITEMCELL_ROWS,
		  F16_FNT, 7+48);
    ViewItem_InitCursor();
    ViewTime_Init(REGION_USER_LEFT+1, REGION_USER_TOP+1,
		  REGION_HOUR_WIDTH, REGION_HOUR_HEIGHT,
		  0, 0,
		  REGION_TIMETABLE_WIDTH, REGION_TIMETABLE_HEIGHT,
		  0, 0,
		  HOURCELL_WIDTH, HOURCELL_HEIGHT,
		  HOURCELL_COLS, HOURCELL_ROWS,
		  0,
		  F16_FNT, 20);
    ViewTime_InitCursor();

    return true;
}

/* ViewDLЕt */
bool
ViewDL_Destroy(void)
{
    D(mgDPrintf("ViewDL destroy\n"));
    return true;
}

/* ViewDLĕ` */
void
ViewDL_Redraw(void)
{
    int left, top, right, bottom;
    HDC hDC;
    PAINTSTRUCT ps;

    hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
    ViewTimeHour_Redraw(hDC);
    ViewTimeWeek_Redraw(hDC);
    ViewItemDate_Redraw(hDC);
    ViewItem_Redraw(hDC);
    ViewTime_InvertCursor(hDC);		      /* ON */
    if (cursor_focus == FOCUS_ITEMLIST)
	ViewItem_InvertCursor(hDC);	      /* ON */

    /* ʒu */
    top = REGION_USER_TOP;
    bottom = REGION_USER_BOTTOM;
    left = REGION_USER_LEFT;
    right = left + REGION_HOUR_WIDTH + REGION_TIMETABLE_WIDTH + 1;

    /* g */
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, left, bottom);
    mgLineTo(hDC, right, bottom);
    mgLineTo(hDC, right, top);
    mgLineTo(hDC, left, top);

    mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
}

/* ViewDLxĕ` */
void
ViewDL_DelayedRedraw(void)
{
    /* NOTTHING TO DO */
}

/* L[R[h */
bool
ViewDL_KeyDown(WPARAM wParam, LPARAM lParam)
{
    HDC hDC;
    PAINTSTRUCT ps;

    switch (wParam){
	case VK_LEFT:			      /*  */
	case UVK_LEFT:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	gCursorDate--;
	ViewTime_Left(hDC);
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* tH[JX^Ce[uɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_TIMETABLE;
	}
	ViewTimeWeek_Redraw(hDC);
	ViewItemDate_Redraw(hDC);
	ViewItem_Redraw(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View_UpdateTitle();
	break;

	case VK_RIGHT:			      /* E */
	case UVK_RIGHT:			      /* E */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	gCursorDate++;
	ViewTime_Right(hDC);
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* tH[JX^Ce[uɈړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_TIMETABLE;
	}
	ViewTimeWeek_Redraw(hDC);
	ViewItemDate_Redraw(hDC);
	ViewItem_Redraw(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	View_UpdateTitle();
	break;

	case VK_UP:			      /*  */
	case UVK_UP:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* J[\ړ */
	if (gCursorHour > 0){
	    gCursorHour--;
	}
	ViewTime_Up(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	/* tH[JXACeXgړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InvertCursor(hDC);
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_TIMETABLE;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case VK_DOWN:			      /*  */
	case UVK_DOWN:			      /*  */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\ */
	if (gCursorHourRange > 1){
	    gCursorHour = gCursorHour + gCursorHourRange - 1;
	    gCursorHourRange = 1;
	}
	/* J[\ړ */
	if (gCursorHour < 23){
	    gCursorHour++;
	}
	ViewTime_Down(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	/* tH[JXACeXgړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InvertCursor(hDC);
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_TIMETABLE;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case UVK_HOURUP:		      /* ԏ */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\k */
	if (gCursorHourRange > 1){
	    gCursorHourRange--;
	}else{
	    if (gCursorHour > 0){
		gCursorHour--;
	    }
	}
	ViewTime_Up(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	/* tH[JXACeXgړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InvertCursor(hDC);
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_TIMETABLE;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case UVK_HOURDOWN:		      /* ԉ */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewTime_InvertCursor(hDC);	      /* OFF */
	/* IJ[\g */
	if (gCursorHour + gCursorHourRange < 24){
	    gCursorHourRange++;
	}
	ViewTime_Down(hDC);
	ViewTime_InvertCursor(hDC);	      /* ON */
	/* tH[JXACeXgړ */
	if (cursor_focus == FOCUS_ITEMLIST){
	    ViewItem_InvertCursor(hDC);
	    ViewItem_InitCursor();
	    cursor_focus = FOCUS_TIMETABLE;
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case UVK_ITEMUP:		      /* ACe */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	if (cursor_focus == FOCUS_TIMETABLE){
	    D(mgDPrintf("focus ITEMLIST"));
	    ViewItem_InvertCursor(hDC);     /* ON */
	    cursor_focus = FOCUS_ITEMLIST;
	}else{
	    ViewItem_InvertCursor(hDC);     /* OFF */
	    ViewItem_Up(hDC);
	    ViewItem_InvertCursor(hDC);     /* ON */
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	case UVK_ITEMDOWN:		      /* ACe */
	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	if (cursor_focus == FOCUS_TIMETABLE){
	    D(mgDPrintf("focus ITEMLIST"));
	    ViewItem_InvertCursor(hDC);     /* ON */
	    cursor_focus = FOCUS_ITEMLIST;
	}else{
	    ViewItem_InvertCursor(hDC);     /* OFF */
	    ViewItem_Down(hDC);
	    ViewItem_InvertCursor(hDC);     /* ON */
	}
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}

/* R}h */
bool
ViewDL_Command(WPARAM wParam, LPARAM lParam)
{
    switch (wParam){
	case ID_FUNC_NOW:
	if (HIWORD(lParam) == BN_CLICKED){
	    ViewTime_InitCursor();
	    ViewItem_InitCursor();
	    View_Redraw();
	}else{
	    return false;
	}
	break;

	default:
	/* Ȃ */
	return false;
	break;
    }

    /*  */
    return true;
}
