/*
   file region of view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewfile.c,v 1.1.1.1 1998/12/15 03:42:48 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskid.h"
#include "taskmg.h"
#include "record.h"
#include "db.h"
#include "view.h"
#include "viewfile.h"
#include "viewitem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>
#include <mbctype.h>

/* -- file local macros */
#define FILE_MAX_LINE		1024
#define FILE_MAX_PATH		(64+13)
#define FILE_MAX_LINEBUF	133

/* -- file local variables */
static int originL, cursorL;
static int region_title_width, region_title_height;
static int region_title_left, region_title_right;
static int region_title_top, region_title_bottom;
static int region_file_width, region_file_height;
static int region_file_left, region_file_right;
static int region_file_top, region_file_bottom;
static int cell_width, cell_height;
static int cell_rows, cell_cols;
static int cell_fontwidth, cell_font;
static int cell_maxlen;

static bool file_is_set = false;
static bool file_is_opened = false;
static FILE *fp;
static int line_num;
static long line_pos[FILE_MAX_LINE];
static uint8 line_len[FILE_MAX_LINE];

static int line_body_start;
static bool file_is_mail;
static bool hasSubject, hasFrom, hasTo, hasDate;
static char mailSubject[FILE_MAX_LINEBUF];
static char mailFrom[FILE_MAX_LINEBUF];
static char mailTo[FILE_MAX_LINEBUF];
static char mailDate[FILE_MAX_LINEBUF];

/* -- file local methods */
static bool
ViewFile_OpenFile(void);
static bool
ViewFile_ScanFile(void);
static void
ViewFile_ScanLineInit(void);
static void
ViewFile_ScanLine(char *line, int len, int no);
static void
ViewFile_ScrollUp(HDC hDC);
static void
ViewFile_ScrollDown(HDC hDC);

/* -- methods */

/* ViewFilȅ */
void
ViewFile_Init(int left, int top,
	      int title_width, int title_height,
	      int file_width, int file_height,
	      int c_width, int c_height,
	      int c_cols, int c_rows,
	      int c_font)
{
    region_title_width = title_width;
    region_title_height = title_height;
    region_title_left = left;
    region_title_top = top;
    region_title_right = region_title_left + region_title_width - 1;
    region_title_bottom = region_title_top + region_title_height - 1;

    region_file_width = file_width;
    region_file_height = file_height;
    region_file_left = left;
    region_file_top = region_title_top + region_title_height;
    region_file_right = region_file_left + region_file_width - 1;
    region_file_bottom = region_file_top + region_file_height - 1;

    cell_width = c_width;
    cell_height = c_height;
    cell_cols = c_cols;
    cell_rows = c_rows;
    cell_font = c_font;

    switch (cell_font){
	case F16_FNT:
	cell_fontwidth = 8;
	break;
	case F12_FNT:
	cell_fontwidth = 6;
	break;
    }

    if (file_is_set){
	ViewFile_OpenFile();
	ViewFile_ScanFile();
    }
}

/* ViewFile̕Еt */
void
ViewFile_Tidy(void)
{
    /* ɊJĂt@C */
    if (file_is_opened){
	fclose(fp);
	D(mgDPrintf("close %s", gSelectedFile));
    }
    file_is_opened = false;
}

/* t@C̐ݒ */
bool
ViewFile_SetFile(char *path)
{
    /* ɊJĂt@C */
    if (file_is_opened){
	fclose(fp);
	D(mgDPrintf("close %s", gSelectedFile));
    }
    file_is_opened = false;

    /* t@Cݒ肷 */
    sprintf(gSelectedFile, path);
    D(mgDPrintf("set %s", gSelectedFile));
    file_is_set = true;

    return true;
}

/* obẽ`FbN */
bool
ViewFile_CheckBattery(bool reopen_test)
{
    if (strncmp(gSelectedFile, "A:", 2) == 0
	|| strncmp(gSelectedFile, "a:", 2) == 0){
	if (!mgCheckBatteryLevel(BATTERY_CARD_RW)){
	    /* dr */
	    mgMessageBox(ghMainWin, "drՂĂ܂", MB_OK);
	    View_Destroy();
	    gPreviousView = VIEW_3WEEK;
	    gCurrentView = VIEW_3WEEK;
	    View_Create();
	    View_Redraw();
	    return false;
	}
	if (reopen_test && file_is_opened){
	    fclose(fp);
	    fp = fopen(gSelectedFile, OPENMODE_READ_TEXT);
	    if (fp == NULL){
		file_is_opened = false;
		file_is_set = false;
		sprintf(gSelectedFile, "");
		/* J[h݂Ȃdr */
		mgMessageBox(ghMainWin, "J[h݂Ȃ\ndrՂĂ܂", MB_OK);
		View_Destroy();
		gPreviousView = VIEW_3WEEK;
		gCurrentView = VIEW_3WEEK;
		View_Create();
		View_Redraw();
		return false;
	    }
	}
    }
    return true;
}

/* t@CJ */
static bool
ViewFile_OpenFile(void)
{
    /* t@CJ */
    D(mgDPrintf("open %s", gSelectedFile));
    if (!ViewFile_CheckBattery(false)){
	return false;
    }
    fp = fopen(gSelectedFile, OPENMODE_READ_TEXT);
    if (fp == NULL){
	file_is_opened = false;
	file_is_set = false;
	sprintf(gSelectedFile, "");
	/* J[h݂Ȃdr */
	mgMessageBox(ghMainWin, "t@C݂Ȃ\nǂݍ߂܂", MB_OK);
	View_Destroy();
	gPreviousView = VIEW_3WEEK;
	gCurrentView = VIEW_3WEEK;
	View_Create();
	View_Redraw();
	return false;
    }
    file_is_opened = true;
}

/* t@C̃`FbN */
bool
ViewFile_CheckFile(void)
{
    if (file_is_set && file_is_opened){
	return true;
    }else{
	return false;
    }
}

/* t@C̃XL */
static bool
ViewFile_ScanFile(void)
{
    int r;
    uint8 col;
    long pos;
    char line[FILE_MAX_LINEBUF];
    bool nextIsMBTrail;

    if (!ViewFile_CheckFile()){
	return false;
    }

    /* ߂ */
    r = fseek(fp, 0, SEEK_SET);
    if (r < 0){
	return false;
    }

    /* t@CXL */
    ViewFile_ScanLineInit();
    D(mgDPrintf("scanning %s ...", gSelectedFile));
    line_num = 0;
    col = 0;
    pos = 0;
    nextIsMBTrail = false;
    line_pos[line_num] = pos;
    while (line_num < FILE_MAX_LINE){
	int c;
	c = fgetc(fp);
	/* t@C̏I */
	if (c == EOF){
	    line_len[line_num] = col;
	    line_num++;
	    break;
	}
	/* s */
	if (c == '\n'){
	    line_len[line_num] = col;
	    line[col] = '\0';
	    ViewFile_ScanLine(line, col, line_num);
	    col = 0;
	    pos++;
	    pos++;
	    line_num++;
	    line_pos[line_num] = pos;
	}else{
	    /* ̑ */
	    line[col] = (char)c;
	    col++;
	    pos++;
	    /* QoCĝPoCgڂQoCgڔ */
	    if (nextIsMBTrail){
		nextIsMBTrail = false;
	    }else{
		if (_ismbblead(c)){
		    nextIsMBTrail = true;
		}
	    }
	    /* \s */
	    if (col >= cell_cols){
		if (nextIsMBTrail){
		    /* Ō̕QoCĝPoCg */
		    col--;
		    pos--;
		    line_len[line_num] = col;
		    line[col] = '\0';
		    ViewFile_ScanLine(line, col, line_num);
		    col = 0;
		    line_num++;
		    line_pos[line_num] = pos;
		    /* ̍sɓ */
		    line[col] = (char)c;
		    col++;
		    pos++;
		}else{
		    /* ʂ̕ */
		    line_len[line_num] = col;
		    line[col] = '\0';
		    ViewFile_ScanLine(line, col, line_num);
		    col = 0;
		    line_num++;
		    line_pos[line_num] = pos;
		}
	    }
	}
    }
    D(mgDPrintf("... done (%d lines, %d bytes)", line_num, pos));
    if (line_body_start < 0){
	line_body_start = 0;
    }
    if (file_is_mail){
	if (line_num > line_body_start + 1){
	    line_body_start++;
	}
	D(mgDPrintf("    file is mail (body starts from line %d)", line_body_start));
	if (hasDate)
	    D(mgDPrintf("        [   Date:] %s", mailDate));
	if (hasTo)
	    D(mgDPrintf("        [     To:] %s", mailTo));
	if (hasFrom)
	    D(mgDPrintf("        [   From:] %s", mailFrom));
	if (hasSubject)
	    D(mgDPrintf("        [Subject:] %s", mailSubject));
    }else{
	D(mgDPrintf("    file is text (body starts from line %d)", line_body_start));
    }
    
    return true;
}

/* s̃XL̏ */
static void
ViewFile_ScanLineInit(void)
{
    line_body_start = -1;
    file_is_mail = false;
    hasSubject = hasFrom = hasTo = hasDate = false;
    mailSubject[0] = '\0';
    mailTo[0] = '\0';
    mailFrom[0] = '\0';
    mailDate[0] = '\0';
}

/* s̃XL */
static void
ViewFile_ScanLine(char *line, int len, int no)
{
    if (line_body_start < 0){
	if (len == 0){
	    if (hasTo && hasFrom && hasDate){
		/* Cł */
		file_is_mail = true;
		line_body_start = no;
	    }else{
		/* Cł͂Ȃ */
		file_is_mail = false;
		line_body_start = 0;
	    }
	}
	
	if (0 == strncmp(line, "Subject:", 8)){
	    hasSubject = true;
	    sprintf(mailSubject, &(line[9]));
	}else if (0 == strncmp(line, "To:", 3)){
	    hasTo = true;
	    sprintf(mailTo, &(line[4]));
	}else if (0 == strncmp(line, "From:", 5)){
	    hasFrom = true;
	    sprintf(mailFrom, &(line[6]));
	}else if (0 == strncmp(line, "Date:", 5)){
	    hasDate = true;
	    sprintf(mailDate, &(line[6]));
	}
    }
}

/* s̕\ */
static void
ViewFile_RedrawLine(HDC hDC, int row, int line_no)
{
    int left, top, right, bottom;
    char line[FILE_MAX_LINEBUF];

    /* ʒu */
    left = region_file_left;
    right = region_file_right;
    top = region_file_top + cell_height * row;
    bottom = top + cell_height - 1;

    /* hԂ */
    mgSetTextColor(hDC, CL_WHITE);
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, left, top, right, bottom);

    /* s̓ǂݍ */
    fseek(fp, line_pos[line_no], SEEK_SET);
    fread(line, 1, line_len[line_no], fp);
    /* TODO: obe[`FbN */

    /* \ */
    mgSetBkMode(hDC, TRANSPARENT);
    mgSetTextColor(hDC, CL_BLACK);
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
    mgTextOut(hDC, left, top, line, line_len[line_no]);
}

/* t@C̍ĕ` */
void
ViewFileFile_Redraw(HDC hDC)
{
    int left, top, right, bottom;
    int i;

    /* ʒu */
    left = region_file_left;
    right = region_file_right;
    top = region_file_top;
    bottom = region_file_bottom;

    /* hԂ */
    mgSetTextColor(hDC, CL_WHITE);
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, left, top, right, bottom);

    /* Pʕ̕` */
    for (i = 0; i < cell_rows && originL + i < line_num; i++){
	ViewFile_RedrawLine(hDC, i, originL + i);
    }
}

/* sXN[Abv */
static void
ViewFile_ScrollUp(HDC hDC)
{
    RECT rect;

    rect.left = region_file_left;
    rect.right = region_file_right;
    rect.top = region_file_top;
    rect.bottom = region_file_top + cell_height * (cell_rows - 1) - 1;
    mgScrollWindow(ghMainWin, 0, cell_height, &rect);

    ViewFile_RedrawLine(hDC, 0, originL);
}

/* sXN[_E */
static void
ViewFile_ScrollDown(HDC hDC)
{
    RECT rect;

    rect.left = region_file_left;
    rect.right = region_file_right;
    rect.top = region_file_top + cell_height;
    rect.bottom = region_file_top + cell_height * cell_rows - 1;
    mgScrollWindow(ghMainWin, 0, -cell_height, &rect);

    ViewFile_RedrawLine(hDC, cell_rows - 1, originL + cell_rows - 1);
}

/* sAbv */
void
ViewFile_Up(HDC hDC)
{
    if (originL > 0){
	originL--;
	ViewFile_ScrollUp(hDC);
    }
}

/* s_E */
void
ViewFile_Down(HDC hDC)
{
    if (originL + cell_rows < line_num){
	originL++;
	ViewFile_ScrollDown(hDC);
    }
}

/* y[WAbv */
void
ViewFile_PageUp(HDC hDC)
{
    originL -= cell_rows;
    if (originL < 0){
	originL = 0;
    }
    ViewFileFile_Redraw(hDC);
}

/* y[W_E */
void
ViewFile_PageDown(HDC hDC)
{
    originL += cell_rows;
    if (originL + cell_rows >= line_num){
	originL = line_num - cell_rows;
    }
    ViewFileFile_Redraw(hDC);
}

/* J[\̔] */
void
ViewFile_InvertCursor(HDC hDC)
{
    hDC = hDC;
}

/* J[\̏ */
void
ViewFile_InitCursor(void)
{
    originL = line_body_start;
    cursorL = line_body_start;
}

/* ^Cg̍ĕ` */
void
ViewFileTitle_Redraw(HDC hDC)
{
    int left, top, right, bottom;

    /* ʒu */
    left = region_title_left;
    right = region_title_right;
    top = region_title_top;
    bottom = region_title_bottom;

    /* hԂ */
    mgSetTextColor(hDC, CL_LTGRAY);
    mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, left, top, right, bottom);

    /* ʃ{^ */
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, left, bottom);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, right, top);
    mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, bottom);
    mgLineTo(hDC, right, bottom);
    mgMoveTo(hDC, right, top);
    mgLineTo(hDC, right, bottom);

    /* ^Cg */
    mgSetBkMode(hDC, TRANSPARENT);
    mgSetTextColor(hDC, CL_BLACK);
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
    if (file_is_mail){
	mgTextOut(hDC, left + 3, top + 3, "Subject: ", 9);
	if (hasSubject){
	    mgTextOut(hDC, left + 3 + cell_fontwidth*9, top + 3,
		      mailSubject, mmin(strlen(mailSubject), 40));
	}
	mgTextOut(hDC, left + 3 + cell_fontwidth*57, top + 3,
		  "Date: ", 6);
	mgTextOut(hDC, left + 3 + cell_fontwidth*63, top + 3,
		  mailDate, mmin(strlen(mailDate), 15));
    }else{
	mgTextOut(hDC, left + 3, top + 3,
		  gSelectedFile, mmin(strlen(gSelectedFile), 70));
    }
}

/* N̐ݒ */
bool
ViewFile_LinkItem(uint16 date, int no)
{
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;
    HDC hDC;
    PAINTSTRUCT ps;

    if (!ViewFile_CheckFile()){
	return false;
    }

    /* R[ho */
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
    rec = monthRec->rec[day][no];

    /* N̐ݒ */
    rec->link_len = strlen(gSelectedFile);
    rec->link = Record_AllocateByte(rec->link_len);
    memcpy(rec->link, gSelectedFile, rec->link_len);

    gDBDirty = true;

    hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
    ViewItem_Redraw(hDC);
    ViewItem_InvertCursor(hDC);	      /* ON */
    mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
    return true;
}

/* N̉ */
bool
ViewFile_UnlinkItem(uint16 date, int no)
{
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;
    HDC hDC;
    PAINTSTRUCT ps;

    /* R[ho */
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
    rec = monthRec->rec[day][no];

    if (rec->link > 0){
	/* N̉ */
	rec->link_len = 0;
	rec->link = nil;

	gDBDirty = true;

	hDC = mgBeginPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
	ViewItem_Redraw(hDC);
	ViewItem_InvertCursor(hDC);	      /* ON */
	mgEndPaint(ghMainWin, (LPPAINTSTRUCT)&ps);
    }

    return true;
}

/* Ñt@C\ */
bool
ViewFile_FollowLink(uint16 date, int no)
{
    uint16 year, month, day;
    MonthRec *monthRec;
    Record *rec;
    char filename[FILE_MAX_PATH];

    /* R[ho */
    Calendar_GetDateByNo(date, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
    rec = monthRec->rec[day][no];

    if (rec->link_len == 0){
	return false;
    }

    memcpy(filename, rec->link, rec->link_len);
    filename[rec->link_len] = '\0';
    ViewFile_SetFile(filename);

    return true;
}

