/*
   timetable region of view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: viewtime.c,v 1.1.1.1 1998/12/15 03:42:49 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "view.h"
#include "viewtime.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <u-k.h>

/* -- file local macros */

/* -- file local variables */
static uint16 originDate, originHour;
static int cursorX, cursorY; 
static int region_hour_width, region_hour_height;
static int region_hour_left, region_hour_right;
static int region_hour_top, region_hour_bottom;
static int region_dayweek_width, region_dayweek_height;
static int region_dayweek_left, region_dayweek_right;
static int region_dayweek_top, region_dayweek_bottom;
static int region_timetable_width, region_timetable_height;
static int region_timetable_left, region_timetable_right;
static int region_timetable_top, region_timetable_bottom;
static int region_day_width, region_day_height;
static int region_day_left, region_day_right;
static int region_day_top, region_day_bottom;
static int cell_width, cell_height;
static int cell_rows, cell_cols;
static int cell_fontwidth, cell_fontheight, cell_font;
static int cell_maxlen;
static int day_rows;

/* -- file local methods */
static void
ViewTimeDayWeek_Redraw(HDC hDC);
static void
ViewTimeTable_Redraw(HDC hDC, int x);
static void
ViewTimeDay_Redraw(HDC hDC, int x);

/* -- methods */

/* ViewTimȅ */
void
ViewTime_Init(int left, int top,
	      int hour_width, int hour_height,
	      int dayweek_width, int dayweek_height,
	      int timetable_width, int timetable_height,
	      int day_width, int day_height,
	      int c_width, int c_height,
	      int c_cols, int c_rows,
	      int d_rows,
	      int c_font, int c_maxlen)
{
    region_hour_width = hour_width;
    region_hour_height = hour_height;
    region_hour_left = left;
    region_hour_top = top + dayweek_height;
    region_hour_right = region_hour_left + region_hour_width - 1;
    region_hour_bottom = region_hour_top + region_hour_height - 1;

    region_dayweek_width = dayweek_width;
    region_dayweek_height = dayweek_height;
    region_dayweek_left = region_hour_right + 1;
    region_dayweek_top = top;
    region_dayweek_right = region_dayweek_left + region_dayweek_width - 1;
    region_dayweek_bottom = region_dayweek_top + region_dayweek_height - 1;

    region_timetable_width = timetable_width;
    region_timetable_height = timetable_height;
    region_timetable_left = region_hour_right + 1;
    region_timetable_top = region_dayweek_bottom + 1;
    region_timetable_right = region_timetable_left + region_timetable_width - 1;
    region_timetable_bottom = region_timetable_top + region_timetable_height - 1;

    region_day_width = day_width;
    region_day_height = day_height;
    region_day_left = region_hour_right + 1;
    region_day_top = region_timetable_bottom + 1;
    region_day_right = region_day_left + region_day_width - 1;
    region_day_bottom = region_day_top + region_day_height - 1;

    cell_width = c_width;
    cell_height = c_height;
    cell_cols = c_cols;
    cell_rows = c_rows;
    cell_font = c_font;
    cell_maxlen = c_maxlen;
    day_rows = d_rows;

    switch (cell_font){
	case F16_FNT:
	cell_fontwidth = 8;
	cell_fontheight = 16;
	break;
	case F12_FNT:
	cell_fontwidth = 6;
	cell_fontheight = 12;
	break;
    }

    /* IvVF^Ce[ůJnXɌŒ */
    if (gOpt_TimeTableStartsNine){
	gCursorHour = 9;
    }
}

/* J[\̏ */
void
ViewTime_InitCursor(void)
{
    if (cell_cols > 1){
	if (gOpt_WeeksStartsFromMonday){
	    /* T͌j */
	    cursorX = (Calendar_GetDayOfWeekByNo(gCursorDate) + 7 - 1) % 7;
	}else{
	    /* T͓j */
	    cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	}
    }else{
	cursorX = 0;
    }
    originDate = gCursorDate - (uint16)cursorX;
    if (gCursorHour + cell_rows > 24){
	originHour = 24 - cell_rows;
	cursorY = gCursorHour - originHour;
    }else{
	originHour = gCursorHour;
	cursorY = 0;
    }
    gCursorHourRange = 1;
}

/* Ԏ̍ĕ` */
void
ViewTimeHour_Redraw(HDC hDC)
{
    int i;
    int left, top, right, bottom;

    /* ʒu */
    top = region_hour_top;
    bottom = region_hour_bottom;
    left = region_hour_left;
    right = region_hour_right;

    /* hԂ */
    mgSetTextColor(hDC, CL_LTGRAY);
    mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, left, top, right, bottom);

    /*  */
    mgSetBkMode(hDC, TRANSPARENT);
    mgSetTextColor(hDC, CL_BLACK);
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|F12_FNT);
    for (i = 0; i < cell_rows; i++){
	char buf[6];
	sprintf(buf, "%2d:00", originHour + i);
	mgTextOut(hDC, left+1, top + cell_height * i, buf, 5);
    }
}

/* j̍ĕ` */
static void
ViewTimeDayWeek_Redraw(HDC hDC)
{
    int i;
    int left, top, right, bottom;
    uint16 year, month, day;
    char str[20];

    top = region_dayweek_top;
    bottom = region_dayweek_bottom;

    /* hԂ */
    mgSetTextColor(hDC, CL_LTGRAY);
    mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
    mgRectangle(hDC, region_dayweek_left, top, region_dayweek_right, bottom);

    for (i = 0; i < cell_cols; i++){
	left = region_dayweek_left + cell_width * i;
	right = left + cell_width - 1;

	/*  */
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
	mgMoveTo(hDC, left, top);
	mgLineTo(hDC, left, bottom);

	/* ʃ{^ */
	mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
	mgMoveTo(hDC, left+1, top);
	mgLineTo(hDC, left+1, bottom);
	mgMoveTo(hDC, left+1, top);
	mgLineTo(hDC, right, top);
	mgSelectStockObject(hDC, CL_DKGRAY|SOLID_PEN|SIZE1_PEN);
	mgMoveTo(hDC, left+2, bottom);
	mgLineTo(hDC, right, bottom);
	mgMoveTo(hDC, right, top);
	mgLineTo(hDC, right, bottom);

	/* j */
	mgSetBkMode(hDC, TRANSPARENT);
	mgSetTextColor(hDC, CL_BLACK);
	mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
	Calendar_GetDateByNo(originDate+i, &year, &month, &day);
	if (gOpt_WeeksStartsFromMonday){
	    /* T͌j */
	    sprintf(str, "%2d  %s", day, nameDayWeek[(i+1)%7]);
	}else{
	    /* T͓j */
	    sprintf(str, "%2d  %s", day, nameDayWeek[i]);
	}
	mgTextOut(hDC, left + (cell_width - strlen(str)*cell_fontwidth) / 2, top+1,
		  str, strlen(str));
    }
}

/* ^Ce[u̕` */
static void
ViewTimeTable_Redraw(HDC hDC, int x)
{
    int left, top, right, bottom;
    uint16 year, month, day;
    MonthRec *monthRec;
    int i;

    /* ʒu */
    left = region_timetable_left + cell_width * x;
    right = region_timetable_left + cell_width * (x + 1) - 1;
    top = region_timetable_top;
    bottom = region_timetable_bottom;

    /* R[ho */
    Calendar_GetDateByNo(originDate +(uint16)x, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* wihԂ */
    mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
    mgSetTextColor(hDC, CL_WHITE);
    mgRectangle(hDC, left, top, right, bottom);

    /* cg */
    mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, left, bottom);

    /* Ԙg */
    for (i = 0; i < cell_rows; i++){
	mgMoveTo(hDC, left, top + cell_height * i);
	mgLineTo(hDC, right, top + cell_height * i);
    }

    /* ACe */
    for (i = 0; i < (int)monthRec->num[day]; i++){
	Record *rec = monthRec->rec[day][i];
	if (rec->type == RECORD_TYPE_SCHEDULE
	    && !(rec->u.sche.end_time_h == 0 && rec->u.sche.end_time_m == 0)){
	    /* Ԃ̂ACe */
	    int16 start_min, end_min;
	    int16 start_y, end_y;
	    bool head_in, tail_in, body_in, desc_in;
	    start_min = rec->u.sche.start_time_h*60 + rec->u.sche.start_time_m;
	    end_min = rec->u.sche.end_time_h*60 + rec->u.sche.end_time_m;
	    start_y = ((start_min - (int16)originHour*60) * cell_height) / 60;
	    end_y = ((end_min - (int16)originHour*60) * cell_height) / 60;
	    /* 蔻 */
	    head_in = tail_in = body_in = desc_in = false;
	    if (start_y < 0 && end_y > region_timetable_height){
		body_in = true;
	    }
	    if (start_y >= 0 && start_y <= region_timetable_height){
		/* Ă */
		head_in = true;
		if (region_timetable_height - start_y > cell_fontheight){
		    /*  */
		    desc_in = true;
		}
	    }else{
		start_y = 0;
	    }
	    if (end_y >= 0 && end_y <= region_timetable_height){
		/* KĂ */
		tail_in = true;
	    }else{
		end_y = region_timetable_height - 1;
	    }
	    /* Ă͕̂` */
	    if (head_in || tail_in || body_in){
		start_y += region_timetable_top;
		end_y += region_timetable_top;
		/* wihԂ */
		mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
		mgSetTextColor(hDC, CL_LTGRAY);
		mgRectangle(hDC, left, start_y, right, end_y);
		/* g` */
		mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
		mgMoveTo(hDC, left, start_y);
		mgLineTo(hDC, left, end_y);
		if (tail_in)
		    mgLineTo(hDC, right, end_y);
		else
		    mgMoveTo(hDC, right, end_y);
		mgLineTo(hDC, right, start_y);
		if (head_in)
		    mgLineTo(hDC, left, start_y);
		else
		    mgMoveTo(hDC, left, start_y);
		if (desc_in){
		    /* ` */
		    mgSetBkMode(hDC, TRANSPARENT);
		    mgSetTextColor(hDC, CL_BLACK);
		    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
		    mgTextOut(hDC, left + 2, start_y + 1, 
			      (char *)rec->descript, mmin((int)rec->descript_len, cell_maxlen));
		}
	    }
	}
    }
}

/* ̍ĕ` */
static void
ViewTimeDay_Redraw(HDC hDC, int x)
{
    int left, top, right, bottom;
    uint16 year, month, day;
    MonthRec *monthRec;
    uint16 y;
    int i;

    /* ʒu */
    left = region_day_left + cell_width * x;
    right = region_day_left + cell_width * (x + 1) - 1;
    top = region_day_top;
    bottom = region_day_bottom;

    /* R[ho */
    Calendar_GetDateByNo(originDate +(uint16)x, &year, &month, &day);
    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

    /* wihԂ */
    if (monthRec->isHoliday[day]){
	/* j͏j */
	mgSelectStockObject(hDC, CL_LTGRAY|SOLID_PEN|SIZE1_PEN);
	mgSetTextColor(hDC, CL_LTGRAY);
    }else{
	/*  */
	mgSelectStockObject(hDC, CL_WHITE|SOLID_PEN|SIZE1_PEN);
	mgSetTextColor(hDC, CL_WHITE);
    }
    mgRectangle(hDC, left, top, right, bottom);

    /* cg */
    mgSelectStockObject(hDC, CL_BLACK|DOT_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, left, bottom);
    /* g */
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN);
    mgMoveTo(hDC, left, top);
    mgLineTo(hDC, right, top);

    /* \̕\ */
    mgSetBkMode(hDC, TRANSPARENT);
    mgSetTextColor(hDC, CL_BLACK);
    mgSelectStockObject(hDC, CL_BLACK|SOLID_PEN|SIZE1_PEN|cell_font);
    for (i = 0, y = 0; i < day_rows && i < (int)monthRec->num[day] && monthRec->rec[day][i]->type != RECORD_TYPE_TASK; i++){
	Record *rec = monthRec->rec[day][i];
	if (rec->type == RECORD_TYPE_HOLIDAY
	    || ((rec->type == RECORD_TYPE_SCHEDULE)
		&& (rec->u.sche.end_time_h == 0 && rec->u.sche.end_time_m == 0))){
	    mgTextOut(hDC, left + 2, top + 2 + y * (cell_fontheight+1), 
		      (char *)(rec->descript),
		      mmin((int)rec->descript_len, cell_maxlen));
	    y++;
	}
    }
}

/* T̍ĕ` */
void
ViewTimeWeek_Redraw(HDC hDC)
{
    int x;

    if (cell_cols > 1){
	ViewTimeDayWeek_Redraw(hDC);
    }
    for (x = 0; x < cell_cols; x++){
	ViewTimeTable_Redraw(hDC, x);
	if (cell_cols > 1){
	    ViewTimeDay_Redraw(hDC, x);
	}
    }
}

/* J[\̔] */
void
ViewTime_InvertCursor(HDC hDC)
{
    RECT rect;
    int topY;

    topY = cursorY - (gCursorHourRange - 1);
    if (topY < 0){
	topY = 0;
    }

    rect.left = region_timetable_left + cell_width * cursorX;
    rect.right = region_timetable_left + cell_width * (cursorX + 1) - 1;
    rect.top = region_timetable_top + cell_height * topY;
    rect.bottom = region_timetable_top + cell_height * (cursorY + 1) - 1;
    mgInvertRect(hDC, &rect);
}

/*  */
void
ViewTime_Up(HDC hDC)
{
    int x;

    if (originHour > 0 && cursorY == 0){
	/* XN[Abv */
	originHour--;
	ViewTimeHour_Redraw(hDC);
	for (x = 0; x < cell_cols; x++){
	    ViewTimeTable_Redraw(hDC, x);
	}
    }else{
	/* J[\Abv */
	cursorY--;
	if (cursorY < 0){
	    cursorY = 0;
	}
    }
}

/*  */
void
ViewTime_Down(HDC hDC)
{
    int x;

    if (originHour + cell_rows < 24 && cell_rows - 1 == cursorY){
	/* XN[_E */
	originHour++;
	ViewTimeHour_Redraw(hDC);
	for (x = 0; x < cell_cols; x++){
	    ViewTimeTable_Redraw(hDC, x);
	}
    }else{
	/* J[\_E */
	cursorY++;
	if (cursorY >= cell_rows){
	    cursorY = cell_rows - 1;
	}
    }
}

/*  */
void
ViewTime_Left(HDC hDC)
{
    if (cell_cols > 1){
	cursorX--;
	if (cursorX < 0){
	    cursorX = cell_cols-1;
	    originDate -= cell_cols;
	    ViewTimeWeek_Redraw(hDC);
	}
    }else{
	originDate = gCursorDate;
    }
}

/* E */
void
ViewTime_Right(HDC hDC)
{
    if (cell_cols > 1){
	cursorX++;
	if (cursorX > cell_cols-1){
	    cursorX = 0;
	    originDate += cell_cols;
	    ViewTimeWeek_Redraw(hDC);
	}
    }else{
	originDate = gCursorDate;
    }
}

/* T */
void
ViewTime_WeekUp(HDC hDC)
{
    originDate -= cell_cols;
    gCursorDate -= cell_cols;
    ViewTimeWeek_Redraw(hDC);
}

/* T */
void
ViewTime_WeekDown(HDC hDC)
{
    originDate += cell_cols;
    gCursorDate += cell_cols;
    ViewTimeWeek_Redraw(hDC);
}

