/*
   context
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: context.c,v 1.2 1998/12/27 16:26:19 furukawa Exp $
   */

#include "global.h"
#include "taskmg.h"
#include "context.h"
#include "view.h"
#include "db.h"
#include "month.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define CONTEXT_VERSION 4
#define CONTINUE_HEADER "#\n# task continuation\n#\n"


/* -- file local variables */
static char labelName[6][16];
static bool labelCustom[5];

static char continue_path[256];
static char label_path[256];
static char default_record_path[256];

/* -- file local methods */

/* -- methods */

static void chop(char *buf)
{
    buf[strlen(buf)-1] = '\0';
}

/* taskǥ쥯ȥκ */
static bool
Context_SetupTaskDirectory(void)
{
    char path[256];
    struct passwd *pwd;
    struct stat sb;

    pwd = getpwuid(getuid());
    if (!pwd){
	fprintf(stderr, "Error:\n:");
	return false; 
    }

    sprintf(path, "%s/.task", pwd->pw_dir);
    if (stat(path, &sb) < 0){
	if (errno == ENOENT){
	    mkdir(path, 0700);
	}else{
	    fprintf(stderr, "Error:\n:");
	    return false;
	}
    }else if (!S_ISDIR(sb.st_mode)){
	fprintf(stderr, "Error: $HOME/.task exists.\n:");
	return false;
    }

    sprintf(continue_path, "%s/.task/%s", pwd->pw_dir, TASK_CONTEXT_FILE);
    sprintf(label_path, "%s/.task/%s", pwd->pw_dir, TASK_LABEL_FILE);
    sprintf(default_record_path, "%s/.task/%s", pwd->pw_dir, TASK_RECORD_FILE);
    return true;
}

/* ƥȤ¸ */
bool
Context_Save(void)
{
    FILE *fp;

    if (!Context_SetupTaskDirectory()){
	return false;
    }

    /* open file */
    fp = fopen(continue_path, OPENMODE_WRITE_TEXT);
    if (NULL == fp){
	return false;
    }

    /* write header */
    fprintf(fp, CONTINUE_HEADER);
    fprintf(fp, "ContinueVersion:        %d\n", CONTEXT_VERSION);
    fprintf(fp, "\n");
    fprintf(fp, "#\n");
    fprintf(fp, "# OPTIONS\n");
    fprintf(fp, "#\n");
    fprintf(fp, "# record file name\n");
    fprintf(fp, "RecordFile:             %s\n", DB_GetFileName());
    fprintf(fp, "# jump the cursor to 'today' on 'power on'\n");
    fprintf(fp, "PowerOnToday:           %d\n", gOpt_PowerOnToday);
    fprintf(fp, "# start weeks from Monday in the calendar view\n");
    fprintf(fp, "WeekStartsFromMonday:   %d\n", gOpt_WeeksStartsFromMonday);
    fprintf(fp, "# start the time table from nine\n");
    fprintf(fp, "TimeTableStartsNine:    %d\n", gOpt_TimeTableStartsNine);
    fprintf(fp, "# do not work on Saturday (^^)\n");
    fprintf(fp, "SaturdayIsHoliday:      %d\n", gOpt_SaturdayIsHoliday);
    fprintf(fp, "# set the label for a new item 'Label1'\n");
    fprintf(fp, "Label1Default:          %d\n", gOpt_Label1Default);
    fprintf(fp, "# select items by 'PageUp' and 'PageDown' key\n");
    fprintf(fp, "SelectItemByPageUpDown: %d\n", gOpt_SelectItemByPageUpDown);
    fprintf(fp, "# use large dialog\n");
    fprintf(fp, "UseLargeDialog:         %d\n", gOpt_LargeDialog);
    fprintf(fp, "# (obsolete)\n");
    fprintf(fp, "SelectedFolder:         %s\n", gSelectedFolder);
    fprintf(fp, "# (obsolete)\n");
    fprintf(fp, "SelectedFile:           %s\n", gSelectedFile);
    fprintf(fp, "# custom label flags\n");
    fprintf(fp, "CustomLabel:            %d %d %d %d %d\n",
	    labelCustom[0],
	    labelCustom[1],
	    labelCustom[2],
	    labelCustom[3],
	    labelCustom[4]);
    fprintf(fp, "\n");
    fprintf(fp, "#\n");
    fprintf(fp, "# variables below holds the cursor position\n");
    fprintf(fp, "#\n");
    fprintf(fp, "CurrentView:            %d\n", gCurrentView);
    fprintf(fp, "CurrentViewMonth:       %d\n", gCurrentViewMonth);
    fprintf(fp, "CurrentViewWeeks:       %d\n", gCurrentViewWeeks);
    fprintf(fp, "CurrentViewWeek:        %d\n", gCurrentViewWeek);
    fprintf(fp, "CurrentViewDay:         %d\n", gCurrentViewDay);
    fprintf(fp, "CurrentViewList:        %d\n", gCurrentViewList);
    fprintf(fp, "CurrentDate:            %d\n", gCursorDate);
    fprintf(fp, "CurrentHour:            %d\n", gCursorHour);
    fprintf(fp, "CurrentLabel:           %d\n", gLabel);

    D(mnwDPrintf("...context file saved"));

    /* close file */
    fclose(fp);
    return true;
}

static int
Context_GetInteger(char *str)
{
    while (*str != ':') str++;
    str++;
    while (*str == ' ' || *str == '\t')	str++;
    return atoi(str);
}

static bool
Context_GetBoolean(char *str)
{
    while (*str != ':')	str++;
    str++;
    while (*str == ' ' || *str == '\t')	str++;
    return atoi(str);
}

static char *
Context_GetString(char *str)
{
    while (*str != ':')	str++;
    str++;
    while (*str == ' ' || *str == '\t')	str++;
    chop(str);
    return str;
}

static void
Context_GetLabel(char *str)
{
    while (*str != ':')	str++;
    str++;
    while (*str == ' ' || *str == '\t')	str++;
    labelCustom[0] = atoi(&(str[0]));
    labelCustom[1] = atoi(&(str[2]));
    labelCustom[2] = atoi(&(str[4]));
    labelCustom[3] = atoi(&(str[6]));
    labelCustom[4] = atoi(&(str[8]));
    D(mnwDPrintf("CustomLabel: %d %d %d %d %d",
		 labelCustom[0],
		 labelCustom[1],
		 labelCustom[2],
		 labelCustom[3],
		 labelCustom[4]));
}

/* ƥȤɤ߹ */
bool
Context_Load(void)
{
    FILE *fp;
    char buf[256];
    int version;

    D(mnwDPrintf("context file loading..."));
    
    if (!Context_SetupTaskDirectory()){
	return false;
    }

    /* ǥեͤ */
    Context_Defaults();

    /* open file */
    fp = fopen(continue_path, OPENMODE_READ_TEXT);
    if (NULL == fp){
	return false;
    }

    while (NULL != fgets(buf, 256, fp)){
	if (buf[0] == '\n' || buf[0] == '#')
	    continue;
	if (0 == strncmp(buf, "ContinueVersion:", 16)){
	    version = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "RecordFile:", 11)){
	    DB_SetFileName((byte *)Context_GetString(buf));
	}else if (0 == strncmp(buf, "PowerOnToday:", 13)){
	    gOpt_PowerOnToday = Context_GetBoolean(buf);
	}else if (0 == strncmp(buf, "WeekStartsFromMonday:", 21)){
	    gOpt_WeeksStartsFromMonday = Context_GetBoolean(buf);
	}else if (0 == strncmp(buf, "TimeTableStartsNine:", 20)){
	    gOpt_TimeTableStartsNine = Context_GetBoolean(buf);
	}else if (0 == strncmp(buf, "SaturdayIsHoliday:", 18)){
	    gOpt_SaturdayIsHoliday = Context_GetBoolean(buf);
	}else if (0 == strncmp(buf, "Label1Default:", 14)){
	    gOpt_Label1Default = Context_GetBoolean(buf);
	}else if (0 == strncmp(buf, "SelectItemByPageUpDown:", 23)){
	    gOpt_SelectItemByPageUpDown = Context_GetBoolean(buf);
	}else if (0 == strncmp(buf, "UseLargeDialog:", 15)){
	    gOpt_LargeDialog = Context_GetBoolean(buf);
	}else if (0 == strncmp(buf, "CurrentView:", 12)){
	    gCurrentView = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CurrentViewMonth:", 17)){
	    gCurrentViewMonth = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CurrentViewWeeks:", 17)){
	    gCurrentViewWeeks = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CurrentViewWeek:", 16)){
	    gCurrentViewWeek = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CurrentViewDay:", 15)){
	    gCurrentViewDay = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CurrentViewDay:", 15)){
	    gCurrentViewList = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CurrentDate:", 12)){
	    gCursorDate = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CurrentHour:", 12)){
	    gCursorHour = Context_GetInteger(buf);
	    gCursorHourRange = 1;
	}else if (0 == strncmp(buf, "CurrentLabel:", 13)){
	    gLabel = Context_GetInteger(buf);
	}else if (0 == strncmp(buf, "CustomLabel:", 12)){
	    Context_GetLabel(buf);
	}else{
	    chop(buf);
	    mnwDPrintf("%s <- no match", buf);
	}
    }

    gPreviousView = VIEW_DAY_L;

    /* close file */
    fclose(fp);
    D(mnwDPrintf("... done"));
    return true;
}

/* ƥȤΥǥե */
bool
Context_Defaults(void)
{
    uint8 hour, min;

    D(mnwDPrintf("setting defaults..."));

    DB_SetFileName((byte *)default_record_path);
    gCurrentView = VIEW_3WEEK;
    gCurrentViewMonth = VIEW_3MONTH;
    gCurrentViewWeeks = VIEW_2WEEK;
    gCurrentViewWeek = VIEW_WEEK_S;
    gCurrentViewDay = VIEW_DAY_L;
    gCurrentViewList = VIEW_LIST_S;
    gPreviousView = VIEW_3WEEK;
    gTodayDate = DB_GetDate(&gTodayYear, &gTodayMonth, &gTodayDay,
			    &hour, &min);
    gCursorDate = gTodayDate;
    gCursorHour = hour;
    gCursorHourRange = 1;

    gLabel = 0;
    labelCustom[0] = 1;
    labelCustom[1] = 1;
    labelCustom[2] = 1;
    labelCustom[3] = 1;
    labelCustom[4] = 1;
    gOpt_PowerOnToday = false;
    gOpt_WeeksStartsFromMonday = false;
    gOpt_TimeTableStartsNine = false;
    gOpt_SaturdayIsHoliday = false;
    gOpt_Label1Default = false;
    gOpt_SelectItemByPageUpDown = false;
    gOpt_LargeDialog = false;
    gOpt_SaveDBOnDirty = true;

    gSelectedFolder[0] = '\0';
    gSelectedFile[0] = '\0';

    return true;
}

/* ٥̾ν */
bool
Context_InitLabelName(void)
{
    sprintf(labelName[0], "    ");
    sprintf(labelName[1], " ʬࣱ ");
    sprintf(labelName[2], " ʬࣲ ");
    sprintf(labelName[3], " ʬࣳ ");
    sprintf(labelName[4], " ʬࣴ ");
    sprintf(labelName[5], "");
    return true;
}

/* ٥̾ɤ߹ */
bool
Context_LoadLabelName(void)
{
    int i;
    FILE *fp;
    char buf[80];

    D(mnwDPrintf("label file loading... %s", label_path));

    /* open file */
    fp = fopen(label_path, OPENMODE_READ_TEXT);
    if (NULL == fp){
	D(mnwDPrintf("label file not found."));
	return false;
    }

    for (i = 1; i <= 4; i++){
	if (NULL == fgets(buf, sizeof(buf), fp)){
	    return false;
	}
	chop(buf);
	if (strlen(buf) > 0){
	    buf[8] = '\0';
	    Context_SetLabelName(i, buf);
	}
    }

    /* close file */
    fclose(fp);
    return true;
}

/* ٥̾ */
bool
Context_SetLabelName(int i, char *name)
{
    if (i > 0 && i <= 4){
	switch(strlen(name)){
	    case 0:
	    case 1:
	    case 2:
	    sprintf(labelName[i], "   %-5s", name);
	    break;
	    case 3:
	    case 4:
	    sprintf(labelName[i], "  %-6s", name);
	    break;
	    case 5:
	    case 6:
	    sprintf(labelName[i], " %-7s", name);
	    break;
	    case 7:
	    case 8:
	    default:
	    sprintf(labelName[i], "%-8s", name);
	    break;
	}
	labelName[i][8] = '\0';
	return true;
    }else{
	return false;
    }
}

/* ٥̾μ */
char *
Context_GetLabelName(int i)
{
    if (i >= 0 && i <= 5){
	return labelName[i];
    }else{
	return "";
    }
}

/* ٥Υå */
void
Context_SetCustomLabel(int label, bool flag)
{
    labelCustom[label] = flag;
}

/* ٥μ */
bool
Context_GetCustomLabel(int label)
{
    return labelCustom[label];
}

/* ٥ */
void
Context_SelectCustomLabel(void)
{
    D(mnwDPrintf("Context_SelectCustomLabel()"));
    MonthRec_ConfigSetLabel(0, labelCustom[0]);
    MonthRec_ConfigSetLabel(1, labelCustom[1]);
    MonthRec_ConfigSetLabel(2, labelCustom[2]);
    MonthRec_ConfigSetLabel(3, labelCustom[3]);
    MonthRec_ConfigSetLabel(4, labelCustom[4]);
}

/* Ķ */
void
Context_ReflectEnvironment(void)
{
    D(mnwDPrintf("Reflecting Environment..."));

    if (gLabel < 5){
	MonthRec_SelectLabel(gLabel);
    }else{
	Context_SelectCustomLabel();
    }

    MonthRec_ConfigIsSaturdayHoliday(gOpt_SaturdayIsHoliday);
}
