/*
   main and its window procedure
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: taskmg.c,v 1.3 1999/01/04 13:48:40 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "db.h"
#include "record.h"
#include "taskmg.h"
#include "taskid.h"
#include "additem.h"
#include "view.h"
#include "context.h"
#include "month.h"
#include "util.h"
#include "funckey.h"
#include "file.h"
#include <string.h>
#include <stdio.h>
#include "mnw.h"

/* -- global variables */
static byte taskmg_icon[] = {
#include "taskmg.ico"
};

/* -- file local methods */
static mnwWindow gMainWinDummy;
static bool
MainWin_Callback(mnwWindow *self, mnwEvent *event);
static bool
Main_Initialize(int argc, char *argv[]);
static bool
Main_Finalize(void);

/* -- methods */

/* ᥤ */
int main(int argc, char *argv[])
{
    mnwEvent *event;

    Main_Initialize(argc, argv);

    /* ᥤ󥤥٥ȥ롼 */
    while ((event = mnwEvent_GetNext(true)) != nil){
	if (!mnwEvent_Translate(event)){
	    D(mnwPrintEvent(event));
	    mnwEvent_Dispatch(event);
	}
    }

    Main_Finalize();

    /* ｪλ */
    return 0;
}

/*  */
static bool
Main_Initialize(int argc, char *argv[])
{
    uint8 hour, min;
    mnwRect rect;
    mnwEvent e;

    gTodayDate = DB_GetDate(&gTodayYear, &gTodayMonth, &gTodayDay,
			    &hour, &min);
    gMode = MODE_VIEW;
    gNotUpdateView = false;
    gLabel = 0;
#if DEBUG
    gDebug = true;
#else
    gDebug = false;
#endif

    /* ǡ١ν */
    DB_Init();
    DB_SetMode(DB_MODE_PSEUDO_SERVER);
    DB_SetUser(RECORD_USER_ANONYMOUS);

    /* ƥȤɤ߹ */
    Context_Load();
    /* ٥̾ɤ߹ */
    Context_InitLabelName();
    Context_LoadLabelName();
    Context_ReflectEnvironment();

    /* 쥳ɤɤ߹ */
    RecordFile_Load();
    gDBDirty = false;
    gDBMadeBackup = false;
    
    /* ɥƥν */
    mnwSetup();

    /* طʤɤĤ֤ */
    mnwRect_SetXYWH(&rect, 0, 0, 640, 240);
    mnwSetForeColor(kColor_LtGray);
    mnwFillRect(&rect);

    /* 롼ȥɥβĻ벽	*/
    mnwEvent_SetAll(&e, kEvent_Show, nil, 0, 0);
    mnwEvent_SendTo(&e, gRootWindow);
    mnwEvent_SetAll(&e, kEvent_SetPixmap, taskmg_icon, sizeof(taskmg_icon), 0);
    mnwEvent_SendTo(&e, gRootWindow);

    /* ᥤ󥦥ɥ */
    gMainWin = &gMainWinDummy;
    mnwRect_SetXYWH(&rect, 0, 20, 640, 200);
    mnwWindow_Register(gMainWin, MainWin_Callback, nil, &rect);

    /* ᥤ󥦥ɥβĻ벽	*/
    mnwEvent_SetFocusWindow(gMainWin);
    mnwEvent_SetAll(&e, kEvent_Show, nil, 0, 0);
    mnwEvent_SendTo(&e, gMainWin);

    /* ᥤ󥦥ɥ̵ΰκ */
    /* TODO: */

    /* IMEν */
    /* TODO: */

    /* ե󥯥󥭡ν */
    FuncKey_Initialize();
    FuncKey_SetMainMode();

    /* SJISEUCؤѴ */
    if (argc > 1 && 0 == strcmp("-sjistoeuc", argv[1])){
	mnwEvent_SetAll(&e, kEvent_Command, nil, ID_FUNC_CONVERT_SJIS, 0);
	mnwEvent_SendTo(&e, gMainWin);
    }

    return true;
}

/* դ */
static bool
Main_Finalize(void)
{
    /* ե󥯥󥭡κ */
    FuncKey_Finilize();

    /* ǡ١դ */
    if (gDBDirty){
	File_MakeBackup();	
	RecordFile_Save();
    }
    DB_Tidy();

    /* ƥȤ¸ */
    Context_Save();

    /* ᥤ󥦥ɥκ */
    /* TODO: */

    /* ɥƥդ */
    mnwTidy();

    D(mnwDPrintf("FIN"));

    return true;
}

/* ᥤ󥦥ɥץ */
static bool
MainWin_Callback(mnwWindow *self, mnwEvent *event)
{
    int r;
    mnwEvent e;

    /* å */
    switch (event->type){
	case kEvent_Create:		      /* ɥ */
	mnwWindow_Callback(self, event);      /* super */
	/* Viewν */
	View_Create();
	/* եʬ */
	mnwEvent_SetFocusWindow(self);
	/* طʿ */
	mnwEvent_SetAll(&e, kEvent_SetBackColor, nil, kColor_LtGray, 0);
	mnwEvent_SendTo(&e, self);
	/* ޤ */
#ifdef DEBUG
	mnwEvent_SetTimer(ID_TIMER_VIEW, self, 2000);
#else
	mnwEvent_SetTimer(ID_TIMER_VIEW, self, 500);
#endif
	mnwEvent_SetTimer(ID_TIMER_CLOCK, gRootWindow, 10000);
	/* ɸ̤ή */
	/* TODO: */
	return true;
	break;

	case kEvent_Destroy:		      /* ɥդ */
	mnwWindow_Callback(self, event);      /* super */
	/* Viewդ */
	View_Destroy();
	/* å롼פνλ */
	mnwEvent_SetAll(&e, kEvent_Quit, nil, 0, 0);
	mnwEvent_Post(&e);
	/* ɸ̤ή */
	/* TODO: */
	return true;
	break;

	case kEvent_KeyDown:		      /* ɽ */
	/* ߤViewή */
	r = View_KeyDown(event);
	if (r){
	    /* Ѥ */
	    return true;
	}
	/* ǥեȤή */
	/* TODO: */
	return true;
	break;

	case kEvent_Command:		      /* ޥɽ */
	/* ߤViewή */
	r = View_Command(self, event);
	if (r){
	    /* Ѥ */
	    return true;
	}
	return false;
	break;

	case kEvent_Redraw:		      /*  */
	D(mnwDPrintf("got kEvent_Redraw at main"));
	if (gMode != MODE_ADD && gMode != MODE_DIALOG){
	    View_Redraw();
	}
	/* ǥեȤή */
	/* TODO: */
	return true;
	break;

	case kEvent_Timer:		      /* ޥ٥ */
	if (event->param1 == ID_TIMER_VIEW){
	    /* ߤViewή */
	    r = View_Timer();
	}
	/* ɸ̤ή */
	/* TODO: */
	return true;
	break;

	default:
	mnwWindow_Callback(self, event);      /* super */
	return true;
	break;
    }

    /* NOTREACHED */
    return mnwWindow_Callback(self, event);   /* super */
}

/* ȥ */
void
MainWin_SetTitle(char *str)
{
    mnwEvent e;
    static char title[80];

    sprintf(title, "  %s", str);
    mnwEvent_SetAll(&e, kEvent_SetText, title, 0, 0);
    mnwEvent_SendToDirect(&e, gRootWindow);
    mnwEvent_SetAll(&e, kEvent_Redraw, nil, 0, 0);
    mnwEvent_SendTo(&e, gRootWindow);
}
