/*
   3 months view
   Copyright (c) 1997,98 FURUKAWA Yo.
   $Id: view3m.c,v 1.2 1998/12/27 16:26:23 furukawa Exp $
   */

#include "arch.h"
#include "global.h"
#include "calendar.h"
#include "month.h"
#include "taskmg.h"
#include "taskid.h"
#include "view.h"
#include "view3m.h"
#include "viewitem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* -- file local macros */
#define CALENDARS		3

#define DAYCELL_COLS		7
#define DAYCELL_ROWS		6
#define DAYCELL_WIDTH		26
#define DAYCELL_HEIGHT		20
#define DAYWEEKCELL_COLS	7
#define DAYWEEKCELL_WIDTH	(DAYCELL_WIDTH)
#define DAYWEEKCELL_HEIGHT	20
#define MONTHCELL_COLS		1
#define MONTHCELL_WIDTH		(DAYCELL_WIDTH*DAYCELL_COLS)
#define MONTHCELL_HEIGHT	20
#define CALENDAR_WIDTH		(DAYCELL_WIDTH*DAYCELL_COLS)
#define CALENDAR_HEIGHT		(DAYCELL_HEIGHT*DAYCELL_ROWS+DAYWEEKCELL_HEIGHT+MONTHCELL_HEIGHT)
#define CALENDAR_INTERVAL	((REGION_USER_WIDTH-(CALENDAR_WIDTH*CALENDARS))/(CALENDARS+1))

#define REGION_CAL1_WIDTH	(CALENDAR_WIDTH)
#define REGION_CAL1_HEIGHT	(CALENDAR_HEIGHT)
#define REGION_CAL1_LEFT	(CALENDAR_INTERVAL)
#define REGION_CAL1_TOP		(REGION_USER_TOP+(REGION_USER_HEIGHT-CALENDAR_HEIGHT)/2)
#define REGION_CAL1_RIGHT	(REGION_CAL1_LEFT+REGION_CAL1_WIDTH-1)
#define REGION_CAL1_BOTTOM	(REGION_CAL1_TOP+REGION_CAL1_HEIGHT-1)
#define REGION_CAL2_WIDTH	(CALENDAR_WIDTH)
#define REGION_CAL2_HEIGHT	(CALENDAR_HEIGHT)
#define REGION_CAL2_LEFT	(CALENDAR_INTERVAL*2+CALENDAR_WIDTH*1)
#define REGION_CAL2_TOP		(REGION_USER_TOP+(REGION_USER_HEIGHT-CALENDAR_HEIGHT)/2)
#define REGION_CAL2_RIGHT	(REGION_CAL2_LEFT+REGION_CAL2_WIDTH-1)
#define REGION_CAL2_BOTTOM	(REGION_CAL2_TOP+REGION_CAL2_HEIGHT-1)
#define REGION_CAL3_WIDTH	(CALENDAR_WIDTH)
#define REGION_CAL3_HEIGHT	(CALENDAR_HEIGHT)
#define REGION_CAL3_LEFT	(CALENDAR_INTERVAL*3+CALENDAR_WIDTH*2)
#define REGION_CAL3_TOP		(REGION_USER_TOP+(REGION_USER_HEIGHT-CALENDAR_HEIGHT)/2)
#define REGION_CAL3_RIGHT	(REGION_CAL3_LEFT+REGION_CAL3_WIDTH-1)
#define REGION_CAL3_BOTTOM	(REGION_CAL3_TOP+REGION_CAL3_HEIGHT-1)

/* -- file local variables */
static uint16 originMonth;
static int cursorM, cursorX, cursorY;
static int region_left[3], region_top[3];

/* -- file local methods */
static void
View3M_RedrawDayWeek(void);
static void
View3M_RedrawCalendars(void);
static void
View3M_RedrawMonth(int i);
static void
View3M_InvertCursor(void);

/* -- methods */

/* View3M */
bool
View3M_Create(void)
{
    uint16 year, month, day;

    D(mnwDPrintf("View3M create\n"));

    /* Viewͭѿ */
    Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
    cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
    cursorY = ((day+5) - cursorX) / 7;
    cursorM = 1;
    originMonth = Calendar_GetMonthNo(year, month) - 1;
    cursor_focus = FOCUS_CALENDAR;
    cursor_prev_focus = FOCUS_CALENDAR;
    ViewItem_InitCursor();

    region_left[0] = REGION_CAL1_LEFT;
    region_left[1] = REGION_CAL2_LEFT;
    region_left[2] = REGION_CAL3_LEFT;
    region_top[0] = REGION_CAL1_TOP;
    region_top[1] = REGION_CAL2_TOP;
    region_top[2] = REGION_CAL3_TOP;

    return true;
}

/* View3Mդ */
bool
View3M_Destroy(void)
{
    D(mnwDPrintf("View3M destroy\n"));
    return true;
}

/* View3M */
void
View3M_Redraw(void)
{
    View3M_RedrawDayWeek();
    View3M_RedrawCalendars();
    View3M_InvertCursor();		      /* ON */
}

/* View3Mٱ */
void
View3M_DelayedRedraw(void)
{
    /* NOTHING TO DO */
}

/*  */
static void
View3M_RedrawDayWeek(void)
{
    int i, dayW;
    int left, top, right, bottom;
    mnwRect rect;

    for (i = 0; i < CALENDARS; i++){
	top = region_top[i] + MONTHCELL_HEIGHT;
	bottom = top + DAYWEEKCELL_HEIGHT - 1;

	for (dayW = 0; dayW < DAYCELL_COLS; dayW++){
	    left = region_left[i] + DAYCELL_WIDTH * dayW;
	    right = left + DAYCELL_WIDTH - 1;

	    /* ̥ܥ */
	    mnwRect_SetXYXY(&rect, left, top, right, bottom);
	    mnwDrawFrame3D(&rect);

	    /*  */
	    mnwSetForeColor(kColor_Black);
	    mnwSetFont(kFont_16);
	    mnwDrawText(left + (DAYWEEKCELL_WIDTH - 16) / 2, top + 2,
		     nameDayWeek[dayW], 2);
	}
    }
}

/*  */
static void
View3M_RedrawCalendars(void)
{
    int i;
    for (i = 0; i < CALENDARS; i++){
	View3M_RedrawMonth(i);
    }
}

/*  */
static void
View3M_RedrawMonth(int i)
{
    int left, top, right, bottom;
    int x, y;
    MonthRec *monthRec;
    uint16 year, month, day, date;
    char *nameMonth[13] = {
	"",
	"", "", "", "", "", "", 
	"", "", "", "", "", "", 
    };
    char daystr[3];
    mnwRect rect;

    /* 쥳ɼФ */
    monthRec = MonthRec_GetByMonthNo(originMonth + i);
    date = monthRec->start_date;
    Calendar_GetDateByNo(date, &year, &month, &day);

    /*  */
    left = region_left[i];
    right = left + MONTHCELL_WIDTH - 1;
    top = region_top[i];
    bottom = top + MONTHCELL_HEIGHT - 1;

    /* طɤĤ֤ */
    mnwSetForeColor(kColor_LtGray);
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /* ̥ܥ */
    mnwDrawFrame3D(&rect);

    /*  */
    mnwSetForeColor(kColor_Black);
    mnwSetFont(kFont_16);
    mnwDrawText(left + (MONTHCELL_WIDTH - strlen(nameMonth[month])*8) / 2, top+2,
	     nameMonth[month], strlen(nameMonth[month]));

    /*  */
    top = region_top[i] + MONTHCELL_HEIGHT + DAYWEEKCELL_HEIGHT;
    bottom = top + DAYCELL_HEIGHT*DAYCELL_ROWS - 1;

    /* طɤĤ֤ */
    mnwSetForeColor(kColor_LtGray);
    mnwRect_SetXYXY(&rect, left, top, right, bottom);
    mnwFillRect(&rect);

    /*  */
    x = Calendar_GetDayOfWeekByNo(monthRec->start_date);
    y = 0;
    for (day = 1; day <= monthRec->numOfDays; day++){
	/*  */
	left = region_left[i] + DAYCELL_WIDTH * x;
	right = left + DAYCELL_WIDTH - 1;
	top = region_top[i] + MONTHCELL_HEIGHT + DAYWEEKCELL_HEIGHT + DAYCELL_HEIGHT * y;
	bottom = top + DAYCELL_HEIGHT - 1;

	/* طɤĤ֤ */
	if (x == 0 || monthRec->isHoliday[day]){
	    mnwSetForeColor(kColor_DkGray);
	    mnwRect_SetXYXY(&rect, left, top, right, bottom);
	    mnwFillRect(&rect);
	    mnwSetForeColor(kColor_White);
	}else if (date == gTodayDate){
	    mnwSetForeColor(kColor_White);
	    mnwRect_SetXYXY(&rect, left, top, right, bottom);
	    mnwFillRect(&rect);
	    mnwSetForeColor(kColor_Black);
	    mnwDrawRect(&rect);
	}else{
	    mnwSetForeColor(kColor_White);
	    mnwRect_SetXYXY(&rect, left, top, right, bottom);
	    mnwFillRect(&rect);
	    mnwSetForeColor(kColor_Black);
	}

	/*  */
	mnwSetFont(kFont_16);
	sprintf(daystr, "%2d", day);
	mnwDrawText(left + (DAYCELL_WIDTH - 16) / 2, top + 2,
		 daystr, 2);

	date++;
	x++;
	if (x == DAYCELL_COLS){
	    x = 0;
	    y++;
	}
    }
}

/* ȿž */
static void
View3M_InvertCursor(void)
{
    mnwRect rect;

    mnwRect_SetXYXY(&rect,
		    region_left[cursorM] + DAYCELL_WIDTH * cursorX,
		    region_top[cursorM] + MONTHCELL_HEIGHT + DAYWEEKCELL_HEIGHT + DAYCELL_HEIGHT * cursorY,
		    region_left[cursorM] + DAYCELL_WIDTH * cursorX + DAYCELL_WIDTH - 1,
		    region_top[cursorM] + MONTHCELL_HEIGHT + DAYWEEKCELL_HEIGHT + DAYCELL_HEIGHT * cursorY + DAYCELL_HEIGHT - 1);
    mnwInvertRect(&rect);
}

/* ɽ */
bool
View3M_KeyDown(mnwEvent *event)
{
    uint16 year, month, day, prev_month;

    switch (event->param1){
	case kKey_LEFT:			      /*  */
	case kUKey_LEFT:		      /*  */
	View3M_InvertCursor();		      /* OFF */
	gCursorDate--;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (day == Calendar_GetNumOfDays(year, month)){
	    /* ǽ */
	    if (cursorM == 0){
		originMonth--;
		View3M_RedrawCalendars();
	    }else{
		cursorM--;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor();		      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_RIGHT:		      /*  */
	case kUKey_RIGHT:		      /*  */
	View3M_InvertCursor();		      /* OFF */
	gCursorDate++;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (day == 1){
	    /* ǽ */
	    if (cursorM == 2){
		originMonth++;
		View3M_RedrawCalendars();
	    }else{
		cursorM++;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor();		      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_UP:			      /*  */
	case kUKey_UP:			      /*  */
	View3M_InvertCursor();		      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &prev_month, &day);
	gCursorDate -= 7;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (month != prev_month){
	    /* ǽ */
	    if (cursorM == 0){
		originMonth--;
		View3M_RedrawCalendars();
	    }else{
		cursorM--;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor();		      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_DOWN:			      /*  */
	case kUKey_DOWN:		      /*  */
	View3M_InvertCursor();		      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &prev_month, &day);
	gCursorDate += 7;
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	if (month != prev_month){
	    /* ǽ */
	    if (cursorM == 2){
		originMonth++;
		View3M_RedrawCalendars();
	    }else{
		cursorM++;
	    }
	}
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor();		      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_PRIOR:		      /*  */
	View3M_InvertCursor();		      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	originMonth -= 3;
	View3M_RedrawCalendars();
	year = 1900 + ((originMonth + cursorM) / 12);
	month = (originMonth + cursorM) % 12 + 1;
	if (day > Calendar_GetNumOfDays(year, month)){
	    day = Calendar_GetNumOfDays(year, month);
	}
	gCursorDate = Calendar_GetDayNo(year, month, day);
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor();		      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	case kKey_NEXT:			      /*  */
	View3M_InvertCursor();		      /* OFF */
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	originMonth += 3;
	View3M_RedrawCalendars();
	year = 1900 + ((originMonth + cursorM) / 12);
	month = (originMonth + cursorM) % 12 + 1;
	if (day > Calendar_GetNumOfDays(year, month)){
	    day = Calendar_GetNumOfDays(year, month);
	}
	gCursorDate = Calendar_GetDayNo(year, month, day);
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	View3M_InvertCursor();		      /* ON */
	/* ٱ䥢åץǡ */
	View_SetTimer();
	break;

	default:
	/* ʤ */
	return false;
	break;
    }

    /*  */
    return true;
}

/* ޥɽ */
bool
View3M_Command(mnwEvent *event)
{
    uint16 year, month, day;

    switch (event->param1){
	case ID_FUNC_NOW:
	Calendar_GetDateByNo(gCursorDate, &year, &month, &day);
	cursorX = Calendar_GetDayOfWeekByNo(gCursorDate);
	cursorY = ((day+5) - cursorX) / 7;
	cursorM = 1;
	originMonth = Calendar_GetMonthNo(year, month) - 1;
	View_Redraw();
	break;

	default:
	/* ʤ */
	return false;
	break;
    }

    /*  */
    return true;
}
