/*
   Schedule and Task Viewer for command line
   Copyright (c) 1997 FURUKAWA Yo.
   */

#include "arch.h"
#include "record.h"
#include "index.h"
#include "month.h"
#include "db.h"
#include "calendar.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* -- macros */
#define MODE_CALENDAR		0x00
#define MODE_LIST_TODO		0x01
#define MODE_LIST_SCHEDULE	0x02

/* -- file local variables */
static uint16 gTodayDate, gTodayYear, gTodayMonth, gTodayDay;
static uint16 gMode;
static char *gRecordFile = "task.rec";

/* -- file local methods */
static int
display_calendar(void);
static int
display_list_todo(void);
static int
display_list_schedule(void);
static int
getoption(int argc, char *argv[]);
static int
finalize(void);
static int
initialize(void);

/* -- methods */

/* ꥹȷǥɽ */
static int
display_list_todo(void)
{
    MonthRec *monthRec;
    char buf[1024];
    uint16 i;

    printf("%u/%u/%u\n", gTodayYear, gTodayMonth, gTodayDay);

    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(gTodayYear, gTodayMonth));

    for (i = 0; i < monthRec->num[gTodayDay]; i++){
	switch (monthRec->rec[gTodayDay][i]->type){
	    case RECORD_TYPE_TASK:
	    if (monthRec->rec[gTodayDay][i]->done_len == 0
		|| (monthRec->rec[gTodayDay][i]->done_len > 0
		    && monthRec->rec[gTodayDay][i]->done[0] == gTodayDate)){
		strncpy(buf, 
			monthRec->rec[gTodayDay][i]->descript,
			monthRec->rec[gTodayDay][i]->descript_len);
		buf[monthRec->rec[gTodayDay][i]->descript_len] = '\0';
		printf("TODO : %s\n", buf);
	    }
	    break;
	}
    }

    return 0;
}

/* ꥹȷǥ塼ɽ */
static int
display_list_schedule(void)
{
    MonthRec *monthRec;
    char buf[1024];
    uint16 i;

    printf("%u/%u/%u\n", gTodayYear, gTodayMonth, gTodayDay);

    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(gTodayYear, gTodayMonth));

    for (i = 0; i < monthRec->num[gTodayDay]; i++){
	switch (monthRec->rec[gTodayDay][i]->type){
	    case RECORD_TYPE_SCHEDULE:
	    strncpy(buf,
		    monthRec->rec[gTodayDay][i]->descript,
		    monthRec->rec[gTodayDay][i]->descript_len);
	    buf[monthRec->rec[gTodayDay][i]->descript_len] = '\0';
	    printf("SCHEDULE %2u:%02u-%2u%02u : %s\n",
		   monthRec->rec[gTodayDay][i]->u.sche.start_time_h,
		   monthRec->rec[gTodayDay][i]->u.sche.start_time_m,
		   monthRec->rec[gTodayDay][i]->u.sche.end_time_h,
		   monthRec->rec[gTodayDay][i]->u.sche.end_time_m,
		   buf);
	    break;

	    case RECORD_TYPE_CONTACT:
	    strncpy(buf,
		    monthRec->rec[gTodayDay][i]->descript,
		    monthRec->rec[gTodayDay][i]->descript_len);
	    buf[monthRec->rec[gTodayDay][i]->descript_len] = '\0';
	    printf("SCHEDULE %2u:%02u : %s\n",
		   monthRec->rec[gTodayDay][i]->u.sche.start_time_h,
		   monthRec->rec[gTodayDay][i]->u.sche.start_time_m,
		   buf);
	    break;

	    case RECORD_TYPE_HOLIDAY:
	    strncpy(buf,
		    monthRec->rec[gTodayDay][i]->descript,
		    monthRec->rec[gTodayDay][i]->descript_len);
	    buf[monthRec->rec[gTodayDay][i]->descript_len] = '\0';
	    printf("HOLIDAY : %s\n",
		   buf);
	    break;
	}
    }

    return 0;
}

/* ɽ */
static int
display_calendar(void)
{
    uint16 origin;
    int week, dayW;
    uint16 i;

    for (i = 0; i < (78 - (strlen(Calendar_GetNameOfMonth(gTodayMonth)) + 5)) / 2; i++){
	printf(" ");
    }
    printf("%s %4u\n", Calendar_GetNameOfMonth(gTodayMonth), gTodayYear);
    printf("------------------------------------------------------------------------------\n");
    printf("|   Sun    |   Mon    |   Tue    |   Wed    |   Thu    |   Fri    |   Sat    |\n");
    printf("------------------------------------------------------------------------------\n");

    /* Sunday of the last week */
    origin = gTodayDate - Calendar_GetDayOfWeekByNo(gTodayDate) - 7;

    for (week = 0; week < 4; week++){
	uint16 date = origin + week * 7;
	uint16 max_disp = 0;
	uint16 numDW[7];
	uint16 p[7];

	for (dayW = 0; dayW < 7; dayW++){
	    uint16 year, month, day;
	    MonthRec *monthRec;

	    /* print dates */
	    Calendar_GetDateByNo(date, &year, &month, &day);
	    if (date == gTodayDate){
		printf("[%2u]       ", day);
	    }else{
		printf("|%2u|       ", day);
	    }

	    /* get max */
	    monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));
	    numDW[dayW] = 0;
	    for (i = 0; i < monthRec->num[day]; i++){
		if (monthRec->rec[day][i]->type == RECORD_TYPE_SCHEDULE
		    || monthRec->rec[day][i]->type == RECORD_TYPE_CONTACT
		    || monthRec->rec[day][i]->type == RECORD_TYPE_HOLIDAY){
		    numDW[dayW]++;
		}
	    }
	    if (max_disp < numDW[dayW]){
		max_disp = numDW[dayW];
	    }
	    p[dayW] = 0;

	    date++;
	}
	printf("|\n");


	for (i = 0; i < max_disp; i++){
	    date = origin + week * 7;
	    for (dayW = 0; dayW < 7; dayW++){
		uint16 year, month, day;
		MonthRec *monthRec;
		
		Calendar_GetDateByNo(date, &year, &month, &day);
		monthRec = MonthRec_GetByMonthNo(Calendar_GetMonthNo(year, month));

		if (numDW[dayW] > 0){
		    char buf[11];
		    bool disp = false;

		    while (!disp){
			switch (monthRec->rec[day][p[dayW]]->type){
			    case RECORD_TYPE_SCHEDULE:
			    case RECORD_TYPE_CONTACT:
			    case RECORD_TYPE_HOLIDAY:
			    strncpy(buf,
				    monthRec->rec[day][p[dayW]]->descript,
				    min(monthRec->rec[day][p[dayW]]->descript_len, 10));
			    buf[min(monthRec->rec[day][p[dayW]]->descript_len, 10)] = '\0';
			    printf("|%-10s", buf);
			    disp = true;
			    numDW[dayW]--;
			    break;
			}
			p[dayW]++;
		    }
		}else{
		    printf("|          ");
		}

		date++;
	    }
	    printf("|\n");
	}

	printf("------------------------------------------------------------------------------\n");
    }

    return 0;
}

/*  */
static int
initialize(void)
{
    uint8 hour, min;

    DB_Init();
    DB_SetMode(DB_MODE_PSEUDO_SERVER);
    DB_SetUser(RECORD_USER_ANONYMOUS);
    DB_SetFileName((byte *)gRecordFile);

    /* get today */
    gTodayDate = DB_GetDate(&gTodayYear, &gTodayMonth, &gTodayDay,
			    &hour, &min);

    return 0;
}

/* դ */
static int
finalize(void)
{
    DB_Tidy();

    return 0;
}

/* ץμ */
static int
getoption(int argc, char *argv[])
{
    int i;

    /* default */
    gMode = MODE_CALENDAR;

    for (i = 1; i < argc; i++){
	switch (argv[i][1]){
	    case 'c':
	    gMode = MODE_CALENDAR;
	    break;

	    case 't':
	    gMode = MODE_LIST_TODO;
	    break;

	    case 's':
	    gMode = MODE_LIST_SCHEDULE;
	    break;

	    case 'f':
	    gRecordFile = argv[++i];
	    break;

	    case 'h':
	    printf("Simple Task Viewer for testing. Copyright (c) 1997,98 FURUKAWA Yo.\n");
	    printf("% taskview [option]\n");
	    printf("       -h             help\n");
	    printf("       -f [taskfile]  change task record file\n");
	    printf("       -t             display list of tasks\n");
	    printf("       -s             display list of schedules\n");
	    printf("       -c             display calendar\n");
	    exit(1);
	    break;

	    default:
	    break;
	}
    }

    return 0;
}

/* ᥤ */
int main(int argc, char *argv[])
{
    int r;

    r = getoption(argc, argv);
    if (r < 0)
	return 1;

    r = initialize();
    if (r < 0)
	return 1;

    RecordFile_Load();
    switch (gMode){
	case MODE_LIST_TODO:
	display_list_todo();
	break;
	case MODE_LIST_SCHEDULE:
	display_list_schedule();
	break;
	case MODE_CALENDAR:
	default:
	display_calendar();
	break;
    }

    r = finalize();
    if (r < 0)
	return 1;

    return 0;
}
